#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
import CDTK.Tools.Conversion as conv
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************
#! /usr/bin/env python

import numpy as np

from CDTK.Dynamics.IniConSampler import IniConSampler as ics
from CDTK.Dynamics.SimulationBox import SimulationBox as sbo
import CDTK.Tools.Conversion as conv
import CDTK.Dynamics.QChemInterface as qintf

# QCHEM input parameters
symbols = ['H','H']
qintf.symbols = symbols

qintf.input = qintf.load_input("NMODE_QCHEM.INP")
qintf.charge = 0
qintf.multiplicity = 1

# length unit [AA] 
x0 = np.array([  0.000000,    0.000000,    0.370700,
                 0.000000,    0.000000,   -0.370700 ]) / conv.au2an


m = np.array([1.0, 1.0, 1.0,
              1.0, 1.0, 1.0])*conv.am2au

def getFuncNMode():
    def f(x):
        e = qintf.runQchemNMode(x)
        return e
    return f
normMode = getFuncNMode()

# compute the modes
NMode = normMode(x0)

# Harmoic modes 
FileNMode = "nmodes.list"
qintf.save_qchem(FileNMode, NMode)

