#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
import CDTK.Tools.Conversion as conv
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************
#!/usr/bin/env python

import numpy as np

from CDTK.Dynamics.IniConSampler import IniConSampler as ics
from CDTK.Dynamics.SimulationBox import SimulationBox as sbo
import CDTK.Tools.Conversion as conv
import CDTK.Dynamics.QChemInterface as qintf

# QCHEM input parameters
symbols = ['O','H','H','O','H','H','O','H','H','O','H','H','O','H','H','O','H','H','H']
qintf.symbols = symbols
qintf.input = """
$rem
SYM_IGNORE         TRUE
MEM_TOTAL          2000
MEM_STATIC         190
exchange           HF
scf_final_print    1
correlation        none
basis              6-31g 
$end
"""

qintf.charge = 1
qintf.multiplicity = 1

# Length from Q-Chem in [AA]. convert to [au] to be used in QDTK
# which is then reconverted into [AA] in QchemInterface

# Length unit in [au]
x0 = np.array([  1.1801600,   3.4344667,  -1.8980165,               
                 1.5401720,   3.8667987,  -2.6721816,              
                 1.3928694,   2.5093009,  -2.0191541,               
                -1.1757719,   2.4288418,  -0.5428235,               
                -2.0992339,   2.2658649,  -0.7338935,             
                -0.8864320,   3.0163815,  -1.2406277,
                -0.1444868,   3.5356247,   1.8274297,
                -0.7708170,   3.2464453,   1.1641522,
                -0.5802549,   4.2655988,   2.2668559,
                 3.1378438,   1.4502764,   0.8167638,
                 3.2678596,   1.0384463,   1.6707797,
                 2.4894311,   0.8980730,   0.3803194,
                 1.1249471,   0.8033389,  -1.0646388,
                 0.2179337,   0.9498513,  -0.7968478,
                 1.1053162,  -0.0317033,  -1.5317259,
                 2.0473551,   3.7328877,   0.5467274,
                 2.5688815,   2.9279289,   0.7262748,
                 1.8263017,   3.8080990,  -0.4007241,
                 1.2454260,   3.7668616,   1.1016518 ]) / conv.au2an 

m = np.array([16.0,16.0,16.0,
               1.0, 1.0, 1.0,
               1.0, 1.0, 1.0,
              16.0,16.0,16.0,
               1.0, 1.0, 1.0,
               1.0, 1.0, 1.0,
              16.0,16.0,16.0,
               1.0, 1.0, 1.0,
               1.0, 1.0, 1.0,
              16.0,16.0,16.0,
               1.0, 1.0, 1.0,
               1.0, 1.0, 1.0,
              16.0,16.0,16.0,
               1.0, 1.0, 1.0,
               1.0, 1.0, 1.0,
              16.0,16.0,16.0,
               1.0, 1.0, 1.0,
               1.0, 1.0, 1.0,
               1.0, 1.0, 1.0 ]) * conv.am2au

def getFuncEn():
    def f(x):
        e = qintf.runQchem(x)
        return e
    return f
potentialE = getFuncEn()

s = ics(x0,potentialE,
        RefEnergy = -456.3818543976, 
        TEMP = 300,
        M = m,
        NSAMPLES = 50)

rate,ratio = s.burnin(steps = 500,
                      increment = 50,
                      targetRate = 0.4)


s.sample()


