from __future__ import print_function
#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
import CDTK.Tools.Conversion as conv
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************
#!/usr/bin/env python

import numpy as np

from CDTK.Dynamics.IniConSampler import IniConSampler as ics
from CDTK.Dynamics.SimulationBox import SimulationBox as sbo
import CDTK.Tools.Conversion as conv
import CDTK.PotentialModels.DoubleACM as dacm

x0 = np.array([ -10.5 ])
m = np.array([ 2000.0 ]) # in a.u.

doubleacm = dacm.DoubleACM(natoms=1,ndim=1,nstates=2)
potent = doubleacm.getPotentialFunction()
cplmat = doubleacm.getCplMatrixElement()

def getFuncEn():
    def f(x):
        e, w = potent(x)
        return e[0] # initializing from lowest potential
    return f
potentialE = getFuncEn()

print(ics.__doc__)
s = ics(x0,potentialE,
        RefEnergy = 0.00, 
        TEMP = 300.0,
        M = m,
        NSAMPLES = 1,
        ndim = 1)

rate,ratio = s.burnin(steps = 500,
                      increment = 50,
                      targetRate = 0.4)

s.sample()
