#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
from CDTK.Dynamics.SimulationBox import SimulationBox as sbo
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************
#!/usr/bin/env python

import sys
import numpy as np

from CDTK.Dynamics.IniConSampler import IniConSampler as ics
from CDTK.Dynamics.SimulationBox import SimulationBox as sbo
from CDTK.Dynamics.SurfaceHoppingFS import SurfaceHoppingFS as shfs
import CDTK.Tools.Conversion as conv
import MDiabats as po

# model input parameters
symbols = ['X']

filename = sys.argv[1]
istate   = int(sys.argv[2])
nstates  = int(sys.argv[3])

m = np.array([2000.0],float)

po = po.MultDiabats(natoms=1,ndim=1,nstates=10,masses=m)
potent = po.getPotentialFunction()
cplmat = po.getCplMatrixElement()

# Initialization of Surface-Hopping module
def getFuncEn():
    def f(x):
        e = potent(x, returnAll=True)
        return e
    return f
potential = getFuncEn()

def getFuncD():
    def f(x, state_block):
        d = cplmat(x, state_block)
        return d
    return f
cplmatrix = getFuncD()

# Load trajectory (initial conditions) from file
sbox = sbo().load(filename)
sbox.ndim = 1
sbox.islogfile = True
sbox.atomSymbols = symbols
sbox.sh_method = 'FS' # fewest switching method
# _ed and _ob are not saved
sbox._has_ed = False
sbox._has_ob = False
sbox._ed = None
sbox._ob = None 
# reset initial position
randnum = np.random.random()
DX = 0.3
X_init = -2.0 + DX * randnum
sbox.X = np.array([X_init],float)
# reset initial velocity
V_init = 0.2
sbox.V = np.array([V_init],float)

fs_cntrl1 = shfs(initial_state=istate,
                 nstates=nstates,
                 n_q=10, 
                 external_E=potential,
                 external_D=cplmatrix,
                 cplVD_method='explicit_d_matrix',
                 HFKoopmans_method=True,
                 cplBlock_radius=5)

sbox.add_surface_hopping_cntrl(fs_cntrl1)

sbox.integrate(tfinal=100.0,dt=0.5)
sbox.save(fname='sb_final')
sbox.traj2VTF()


