from __future__ import print_function
#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************
#!/usr/bin/env python
# Example interface to xmolecule
#
#

import sys
import re
import numpy as np


from CDTK.Dynamics.SimulationBox import SimulationBox as sbo
from CDTK.Dynamics.ObservableCharge  import ObservableCharge  as obs
from pyxmol import *
from CDTK.Dynamics.populationdynamic import PopDyn

AMU2AU=1822.877332827


XM=xmol()
atoms=['O','H','H']
masses=[16.0,1.0,1.0]
pos=np.array([
   0.0000000000,        0.0000000000,        0.0000000000,   
   0.0000000000,       -1.4309301866,        1.1083919440,   
   0.0000000000,        1.4309301866,        1.1083919440   
])


THRESHOLD=1e-4
def getFuncMCharge():
    def f(state):
        MCharge=XM.mullikencharge()
        return(MCharge)
    return f
mcharge = getFuncMCharge()
def getFuncP():
    def f():
        process=XM.process()
        return(process)
    return f
process = getFuncP()
def getFuncgocc():
    def f():
        occ=XM.get_occ()
        return(occ)
    return f
gocc = getFuncgocc()
def setFuncsocc():
    def f(occ):
        XM.set_occ(occ)
    return f
socc = setFuncsocc()
def setFuncreadchkpt():
    def f():
        XM.read_chkpt()
    return f
readchkpt = setFuncreadchkpt()
def setFuncwritechkpt():
    def f():
        XM.write_chkpt()
    return f
writechkpt = setFuncwritechkpt()
def getFuncEn():
    def f(x):
        global pos
        print("#t= %F  energy" % sb.TIME)
        print("#t= %F  gradient" % sb.TIME)
        if( sum( (pos[i]-x[i])*(pos[i]-x[i]) for i in range(0,len(x)) )  > THRESHOLD*THRESHOLD ):
            pos=x
            XM.set_geom(x.tolist())
        E=XM.energy
        return(E)
    return f
potential = getFuncEn()
def getFuncGr():
    def f(x):
        global pos
        print("#t= %F  gradient" % sb.TIME)
        if( sum( (pos[i]-x[i])*(pos[i]-x[i]) for i in range(0,len(x)) )  > THRESHOLD*THRESHOLD ):
            pos=x
            XM.set_geom(x.tolist())
        G=XM.gradient()
        # the gradient of xmolecule has numerical errors
        # this substracts the sum force from the atoms
        Natom=len(G)/3
        for i in [0,1,2]:
            Gsum=sum( [G[j*3+i] for j in range(0,Natom)])
            for j in range(0,Natom):
                G[j*3+i]=G[j*3+i]-Gsum/Natom

            
        return np.array(G)
    return f
gradients = getFuncGr()





XM.init()
pd=PopDyn(
    external_P=process,
    external_getocc=gocc,
    external_setocc=socc,
    external_write_chkpt=writechkpt,
    external_read_chkpt=readchkpt
    )
ob_charge=obs(external_CHARGE=mcharge )

if True:
    sb=sbo()
    sb.X=pos
    sb.V=np.array([0. for a in atoms for i in 1,2,3 ])
    sb.M=np.array([m*AMU2AU for m in masses for i in 1,2,3])
    sb.atomSymbols=atoms
    sb.state=1
#    sb.removeCenterMassMotion()

#    occ=XM.get_occ()
#    occ[0]=1
#    XM.set_occ(occ)
else:
    sb=sbo().load('sb_20160818_5f827896d1ae.rst')
    pd.read_chkpt('sb_20160818_5f827896d1ae.rst.xmol')
    sb.isrestart = True


sb.external_E= potential
sb.external_G= gradients

#sb.add_population_dynamics_cntrl(pd)
sb.add_observable_cntrl(ob_charge)

sb.islogfile=True
print("#Starting MD integration")

sb.integrate(tfinal=sb.TIME+2000.,dt=10.0)
sb.save(fname='sb_final')

#sb.traj2VTF()
