#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
#*  Oriol Vendrell, DESY, <oriol.vendrell@desy.de>
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************

"""
Some miscellaneous tools
"""

# memory monitoring
import resource
from sys import platform
from resource import getpagesize
PAGESIZE = getpagesize()

def using(point=""):
    usage = resource.getrusage(resource.RUSAGE_SELF)
    utime = usage[0]
    stime = usage[1]
    maxrss = usage[2]
    currss = usage[3]
    if platform == "linux":
        with open("/proc/self/statm") as f:
            currss = int(f.read().split()[1]) * PAGESIZE / 1024.
    return "%25s %15.2f %15.2f %15.2f %15.2f\n" % (point,utime,stime,
                maxrss/1024.0, currss/1024.0 )

def init():
    with open("mem.log",'w') as fmem:
        fmem.write("%25s %15s %15s %15s %15s\n"%("step","usertime(s)","systime(s)","maxrss(mb)","currss(mb)"))

def log(s):
    with open("mem.log",'a') as fmem:
        fmem.write(using(s))



        