#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
#*  Oriol Vendrell, DESY, <oriol.vendrell@desy.de>
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************

import numpy as np
import matplotlib.pyplot as plt
from CDTK.Tools.Mathematics import FourierTransform

Nsegments = 5000
tinit = -10.0
tend = 10.0
dt = (tend - tinit)/Nsegments

def function(x):
    w1 = 0.2
    freq1 = 9.0
    w2 = 0.24
    freq2 = 20.0
    w3 = 0.37
    freq3 = 45.0
    w4 = 0.19
    freq4 = 63.0
    y = w1 * np.cos(freq1 * x) + w2 * np.cos(freq2 * x) + w3 * np.cos(freq3 * x) + w4 * np.cos(freq4 * x)
    return y

t_line = np.linspace(tinit, tend, Nsegments+1)
y_line = function(t_line)

f_line, FTy_line = FourierTransform(y_line, dt, method='simps', odamp=1, t_init=tinit, npoints=2001, f_range=(0.0, 50.0))

plt.plot(f_line, np.abs(FTy_line))
plt.xlim(0.0, 100)
plt.show()

