#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
#*  Oriol Vendrell, DESY, <oriol.vendrell@desy.de>
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************

import numpy as np
from CDTK.Dynamics.SimulationBox import SimulationBox as sb
from CDTK.Dynamics.Models.testModels import tetrahedron as efunc
import CDTK.Tools.Conversion as cv

xl = [1,1,1,-1,-1,1,-1,1,-1,1,-1,-1]
x0 = np.array(xl,float) / np.sqrt(8.0) * 2.0
v0 = np.zeros(len(x0),float)
m = np.ones(len(x0),float) * 12.0 * cv.am2au

x0[0] = 3.0
x0[1] = 1.0
x0[2] = 0.75

def c1(x):
    xx=x.copy()
    xx.shape=(4,3)
    r34 = xx[2] - xx[3]
    return np.linalg.norm(r34) - 1.06*cv.an2au

def c2(x):
    xx=x.copy()
    xx.shape=(4,3)
    r23 = xx[1] - xx[2]
    return np.linalg.norm(r23) - 1.06*cv.an2au

def c3(x):
    xx=x.copy()
    xx.shape=(4,3)
    r24 = xx[1] - xx[3]
    return np.linalg.norm(r24) - 1.06*cv.an2au



system = sb(X=x0,
            V=v0,
            M=m,
            atomSymbols=['C','C','C','C'],
            external_E=efunc)
#           constraints=[c1,c2,c3])

system.integrate(tfinal=5000.0,dt=10.0)
print(system.TrajV[-1])
print(system.TotE[2])
print(system.TotE[-1])
print(abs(system.TotE[2]-system.TotE[-1])/system.TotE[2]*100)

