#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
#*  Oriol Vendrell, DESY, <oriol.vendrell@desy.de>
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************

import sys
import numpy as np
from CDTK.Dynamics.SimulationBox import SimulationBox as sb
from CDTK.Dynamics.IniConSampler import IniConSampler
import CDTK.Tools.Conversion as cv


k = 100.0 * cv.ic2au
d0 = 1.2 * cv.an2au

def v(x):
    xx=x.copy()
    xx.shape=(2,3)
    r = np.linalg.norm(xx[0]-xx[1])
    return 0.5*k*(r-d0)**2

x0 = np.array([0.0,0.0,0.0,0.0,0.0,1.0],float) * cv.an2au
v0 = np.zeros(6,float)
M = np.array([79.90,79.90,79.90,79.90,79.90,79.90],float) * cv.am2au # Bromine 2

sampler = IniConSampler(x0,v,T=300.0,DIST='boltzmann',M=M,NSAMPLES=5000)

sampler.burnin(steps=2000)
sampler.sample(savefiles=False,saveX0V0=True)

ncol  = sampler._log['ncol']
ntot  = sampler._log['ntot']
nabs  = sampler._log['NABS']
ratio = sampler._log['ratio']
sys.stdout.write('N Collected: %i\n' % (ncol,))
sys.stdout.write('N Calls (collection): %i\n' % (ntot,))
sys.stdout.write('N Calls (absolute): %i\n' % (nabs,))
sys.stdout.write('Acceptance ratio: %6.3f\n' % (ratio,))
sys.stdout.write('Step size: %6.3f\n' % (sampler._optimalStepNorm,))


