#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
#*  Oriol Vendrell, DESY, <oriol.vendrell@desy.de>
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************
#! /usr/bin/env python

import os
import unittest
import numpy as np
import random
import sys
import math
from StringIO import StringIO

import Bin.job_submit as js

class jobsubmitTestCase(unittest.TestCase):
    def setUp(self):
        pass

    def test_start(self):
        """ 
        Testing the parsing of a hessian
        """

        # basic
        sys.argv = ['dummy', '-c', 'xpyder -i muh', '-n', 'blah.job']

        # for redirecting output
        saved_stdout = sys.stdout
        try:
            out = StringIO()
            sys.stdout = out
            js.start()
            output = out.getvalue().strip()
        finally:
            sys.stdout = saved_stdout

        # first compare output
        compare_str = '>Queue: quick\n >Executing: /usr/local/bin/cpquick_submit blah.job "*"  "*"'

        self.assertEqual(output, compare_str)

        compare_file = ["#!/bin/sh\n", "\n", "#$ -V\n", "\n", "ulimit -s 8192; ulimit -a; \n", "\n", "xpyder -i muh\n", "\n", "# This script was sent to queue with command line\n", '# > /usr/local/bin/cpquick_submit blah.job "*"  "*"\n']

        # compare created job file
        with open('blah.job', 'r') as inf:
            i = 0
            for line in inf:
                self.assertEqual(line, compare_file[i])
                i += 1

        os.system('rm blah.job')

        # all options
        sys.argv = ['dummy', '-c', 'xpyder -i muh', '-q', 'mid', '-n', 'blah.job', '-s', '1000', '-l', 'r_molcas r_python27',  '-i', 'miep', '-o', 'wuff', '--cpu', '10', '--mem', '5']

        # for redirecting output
        saved_stdout = sys.stdout
        try:
            out = StringIO()
            sys.stdout = out
            js.start()
            output = out.getvalue().strip()
        finally:
            sys.stdout = saved_stdout

        # first compare output
        compare_str = '>Queue: mid\n >Executing: /usr/local/bin/cpmid_submit blah.job "miep"  "wuff"'

        self.assertEqual(output, compare_str)

        compare_file = ["#!/bin/sh\n", "\n", "#$ -V\n", "#$ -l r_molcas\n", "#$ -l r_python27\n","\n", "#$ -l r_maxcpu=10\n", "export OMP_NUM_THREADS=10\n", "#$ -l r_maxmem=5\n", "ulimit -s 1000; ulimit -a; \n", "\n", "xpyder -i muh\n", "\n", "# This script was sent to queue with command line\n", '# > /usr/local/bin/cpmid_submit blah.job "miep"  "wuff"\n']

        # compare created job file
        with open('blah.job', 'r') as inf:
            i = 0
            for line in inf:
                self.assertEqual(line, compare_file[i])
                i += 1

        os.system('rm blah.job')

        return

def getSuite():
    suite = unittest.TestSuite()

    suite.addTest(jobsubmitTestCase('test_start'))

    return suite

if __name__ == "__main__":
    unittest.main()

