#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
#*  Oriol Vendrell, DESY, <oriol.vendrell@desy.de>
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************
#! /usr/bin/env python

import os
import unittest
import numpy as np
import random
import sys
import math
from io import StringIO

import Bin.xmodel as xmodel

class xmodelTestCase(unittest.TestCase):
    def setUp(self):
        pass

    def test_start(self):
        """ 
        Testing the starting of model calculations.
        """

        # seed random number generator
        np.random.seed(0)
        random.seed(0)

        # write input file here
        inf = open('input.in', 'w')
        inf.write("$inicond \n")
        inf.write("pos = -5.0 \n")
        inf.write("vel = 0.1 \n")
        inf.write("$end \n")
        inf.write("$system \n")
        inf.write("model = TullyA \n")
        inf.write("$end \n")
        inf.write("$trajectory \n")
        inf.write("dt = 1.0 \n")
        inf.write("tf = 3.0 \n")
        inf.write("$end \n")
        inf.write("$quantum \n")
        inf.write("type = fssh \n")
        inf.write("nstates = 2 \n")
        inf.write("istate = 1 \n")
        inf.write("$end \n")
        inf.close()

        # basic
        sys.argv = ['dummy', '-i', 'input.in', '-d', 'model_test']

        xmodel.start()
                  
        ## TODO check all files

        compare_file = [" 0.000000   0.00000  0.00000   1.00000  0.00000\n", " 1.000000   0.00000  0.00000   1.00000  0.00000\n", " 2.000000   0.00000  0.00000   1.00000  0.00000\n", " 3.000000   0.00000  0.00000   1.00000  0.00000\n"]
        # compare created files
        with open('run_model_test/C.log', 'r') as inf:
            i = 0
            for line in inf:
                self.assertEqual(line, compare_file[i])
                i += 1

        compare_file = [" 0.000000    10.000000      0.009997     10.009997\n", " 1.000000    10.000001      0.009996     10.009997\n", " 2.000000    10.000001      0.009995     10.009997\n", " 3.000000    10.000002      0.009995     10.009997\n"]
        # compare created files
        with open('run_model_test/E.log', 'r') as inf:
            i = 0
            for line in inf:
                self.assertEqual(line, compare_file[i])
                i += 1

        compare_file = ["   0    0      0.0000   -0.01999329    0.00000000\n", "   0    1      0.0000    0.00000000    0.00000000\n", "   1    0      0.0000    0.00000000   -0.00000000\n", "   1    1      0.0000    0.00000000    0.00000000\n", "   0    0      1.0000   -0.01999271    0.00000000\n", "   0    1      1.0000    0.00000000    0.00000000\n", "   1    0      1.0000    0.00000000   -0.00000000\n", "   1    1      1.0000   -0.00000058    0.00000000\n", "   0    0      2.0000   -0.01999203    0.00000000\n", "   0    1      2.0000    0.00000000    0.00000000\n", "   1    0      2.0000    0.00000000   -0.00000000\n", "   1    1      2.0000   -0.00000127    0.00000000\n", "   0    0      3.0000   -0.01999122    0.00000000\n", "   0    1      3.0000    0.00000000    0.00000000\n", "   1    0      3.0000    0.00000000   -0.00000000\n", "   1    1      3.0000   -0.00000207    0.00000000\n"]
        # compare created files
        with open('run_model_test/NAC.log', 'r') as inf:
            i = 0
            for line in inf:
                self.assertEqual(line, compare_file[i])
                i += 1

        compare_file = ["  0.000000   0.00000   1.00000   1.00000\n", "  1.000000   0.00000   1.00000   1.00000\n", "  2.000000   0.00000   1.00000   1.00000\n", "  3.000000   0.00000   1.00000   1.00000\n"]
        # compare created files
        with open('run_model_test/P.log', 'r') as inf:
            i = 0
            for line in inf:
                self.assertEqual(line, compare_file[i])
                i += 1

        compare_file = [" 0.000000    -5.000000\n", " 1.000000    -4.900000\n", " 2.000000    -4.800000\n", " 3.000000    -4.700000\n"]
        # compare created files
        with open('run_model_test/R.log', 'r') as inf:
            i = 0
            for line in inf:
                self.assertEqual(line, compare_file[i])
                i += 1

        compare_file = [" 0.000000     1\n", " 1.000000     1\n", " 2.000000     1\n", " 3.000000     1\n"]
        # compare created files
        with open('run_model_test/S.log', 'r') as inf:
            i = 0
            for line in inf:
                self.assertEqual(line, compare_file[i])
                i += 1

        compare_file = ["-         0.000  0.5488     1    1 False False 0.000 -0.000\n", "-         0.024  0.7152     1    1 False False 0.000 -0.000\n", "-         0.048  0.6028     1    1 False False 0.000 -0.000\n"]
        # compare created files
        with open('run_model_test/Switch.log', 'r') as inf:
            i = 0
            for line in inf:
                self.assertEqual(line, compare_file[i])
                i += 1

        compare_file = ["     0.0000    -0.00999665    0.00999665\n", "     1.0000    -0.00999606    0.00999606\n", "     2.0000    -0.00999538    0.00999538\n", "     3.0000    -0.00999458    0.00999458\n"]
        # compare created files
        with open('run_model_test/V_ad.log', 'r') as inf:
            i = 0
            for line in inf:
                self.assertEqual(line, compare_file[i])
                i += 1

        compare_file = [" 0.000000     0.100000\n", " 1.000000     0.100000\n", " 2.000000     0.100000\n", " 3.000000     0.100000\n"]
        # compare created files
        with open('run_model_test/V.log', 'r') as inf:
            i = 0
            for line in inf:
                self.assertEqual(line, compare_file[i])
                i += 1

        compare_file = []
        # compare created files
        with open('run_model_test/Warnings.log', 'r') as inf:
            i = 0
            for line in inf:
                self.assertEqual(line, compare_file[i])
                i += 1


        os.system('rm -rf input.in run_model_test')

        return

def getSuite():
    suite = unittest.TestSuite()

    suite.addTest(xmodelTestCase('test_start'))

    return suite

if __name__ == "__main__":
    unittest.main()

