#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019
#*  Oriol Vendrell, DESY, <oriol.vendrell@desy.de>
#*
#*  Copyright (C) 2020 - 2022
#*  Ludger Inhester, DESY, <ludger.inhester@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************

import unittest
import os
import sys
import test_GamessUSInterface
#import test_MolcasInterface
import test_XMoleculeInterface
import test_WrapperInterface

import test_Trajectory
import test_Trajectory_SH
import test_IniConSampler
import test_MCElectronDynamics
import test_Embedding
import test_NormalModesAnalysis

import test_analyze
import test_coord
import test_getHessian
import test_runall
#import test_jobsubmit
#import test_xmodel
#import test_xsample
#import test_xpyder


if __name__ == '__main__':
    suite = unittest.TestSuite()

    ## Electronic strucuture interfaces
    suite.addTest(test_GamessUSInterface.getSuite())
    #suite.addTest(test_MolcasInterface.getSuite())
    suite.addTest(test_XMoleculeInterface.getSuite())
    suite.addTest(test_WrapperInterface.getSuite())
    
    
    ## dynamics modules
    suite.addTest(test_Trajectory.getSuite())
    suite.addTest(test_Trajectory_SH.getSuite())
    suite.addTest(test_IniConSampler.getSuite())
    suite.addTest(test_MCElectronDynamics.getSuite())
    suite.addTest(test_Embedding.getSuite())

    ## Input/Output and main programs

    ## Tools
    suite.addTest(test_NormalModesAnalysis.getSuite())

    ## Bins
    suite.addTest(test_analyze.getSuite())
    suite.addTest(test_coord.getSuite())
    suite.addTest(test_getHessian.getSuite())
    suite.addTest(test_runall.getSuite())
    #suite.addTest(test_jobsubmit.getSuite())
    #suite.addTest(test_xmodel.getSuite())
    #suite.addTest(test_xpyder.getSuite())
    #suite.addTest(test_xsample.getSuite())



    ## Run all tests
    runner = unittest.TextTestRunner(verbosity=3)
    result = runner.run(suite)

    os.system("rm -rf trj* *nao* *conf* *.dat *.inp *.log")
    os.system("rm -rf *.molden* *Orb* *.status *xml *err")

    if result.wasSuccessful():
        print("All test passed !")
        sys.exit(os.EX_OK)
    else:
        print("Something went wrong:")
        print(result.errors)
        sys.exit(os.EX_SOFTWARE)


