#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
#*  Oriol Vendrell, DESY, <oriol.vendrell@desy.de>
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************

import unittest
import numpy as np

from CDTK.Tools import Mathematics as mat

class MathematicsTestCase(unittest.TestCase):
    def setUp(self):
        self.x = np.arange(0.0, 10.0, 0.01)
        self.y = mat.GaussianFunction(self.x, xbar=5.0, std=0.5, norm=False) * 3.0

    def test_moments(self):

        m, w, h = mat.gaussian_moments(self.x, self.y, mode='area')

        self.assertAlmostEqual(m, 5.0, delta=0.01)
        self.assertAlmostEqual(w, 0.5, delta=0.01)
        self.assertAlmostEqual(h, 3.0, delta=0.01)



# Add tests above to the suite definition:
def getSuite():
    suite = unittest.TestSuite()
    suite.addTest(NormalModesTestCase('test_moments'))
    return suite


if __name__ == "__main__":
    unittest.main()
