#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
#*  Oriol Vendrell, DESY, <oriol.vendrell@desy.de>
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************

import unittest
import numpy as np
import CDTK.Tools.NormalModeAnalysis as nm
import CDTK.Tools.Conversion as cv


class NormalModesTestCase(unittest.TestCase):
    def setUp(self):
        self.Hc = np.array(
[
[ 0.825112, 0.000000, 0.000000,-0.412556, 0.000000, 0.280671,-0.412556, 0.000000,-0.280671],
[ 0.000000,-0.000016, 0.000000, 0.000000, 0.000008, 0.000000, 0.000000, 0.000008, 0.000000],
[ 0.000000, 0.000000, 0.439070, 0.205418, 0.000000,-0.219536,-0.205418, 0.000000,-0.219536],
[-0.412556, 0.000000, 0.205418, 0.443615, 0.000000,-0.243044,-0.031058, 0.000000, 0.037625],
[ 0.000000, 0.000008, 0.000000, 0.000000,-0.000010, 0.000000, 0.000000, 0.000002, 0.000000],
[ 0.280671, 0.000000,-0.219536,-0.243044, 0.000000, 0.205233,-0.037625, 0.000000, 0.014303],
[-0.412556, 0.000000,-0.205418,-0.031058, 0.000000,-0.037625, 0.443615, 0.000000, 0.243044],
[ 0.000000, 0.000008, 0.000000, 0.000000, 0.000002, 0.000000, 0.000000,-0.000010, 0.000000],
[-0.280671, 0.000000,-0.219536, 0.037625, 0.000000, 0.014303, 0.243044, 0.000000, 0.205233]
]
)

        self.M = np.array([15.99491,15.99491,15.99491,
                            1.0078 , 1.0078 , 1.0078 ,
                            1.0078 , 1.0078 , 1.0078 ])*cv.am2au

    def test_evals(self):
        evals,evecs = nm.normal_modes_from_HessianCart(self.Hc,self.M)
        self.assertAlmostEqual(evals[-1]*cv.au2ic,4145.70,delta=1.0)


# Add tests above to the suite definition:
def getSuite():
    suite = unittest.TestSuite()
    suite.addTest(NormalModesTestCase('test_evals'))
    return suite


if __name__ == "__main__":
    unittest.main()
