#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
#*  Oriol Vendrell, DESY, <oriol.vendrell@desy.de>
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************
#! /usr/bin/env python

import os
import unittest
import numpy as np
import random
import sys
import math
import CDTK.an_coord as coord

class coordTestCase(unittest.TestCase):
    def setUp(self):
        pass

    def test_start(self):
        """ 
        Testing the calculation of distances.
        """

        t = np.array([0.0, 2.0, 4.0, 5.0])
        R = np.array([[0.0, 0.0, 0.0, 0.0, 0.0, 1.0], [0.0, 0.0, 0.0, 0.0, 0.1, 1.0], [0.2, -0.1, 0.1, 0.0, 0.2, 1.0], [0.3, -0.2, -0.3, -0.1, 0.1, 1.2]])
        with open('R.log', 'w') as fout:
            for i in range(len(t)):
                fout.write('{0: >12.3f}  '.format(t[i]))
                for j in range(len(R[i])):
                    fout.write('{0: >12.3f}  '.format(R[i,j]))
                fout.write('\n')

        d_compare = np.array([1.0, np.sqrt(1.01), np.sqrt(0.94), np.sqrt(2.5)])

        sys.argv = ['dummy', '-a', '1', '-b', '2']

        coord.start()

        with open('coord_1_2', 'r') as fin:
            i = 0
            for line in fin:
                vals = line.split()
                
                self.assertAlmostEqual(float(vals[0]), t[i], delta=1e-10)
                self.assertAlmostEqual(float(vals[1]), d_compare[i], delta=1e-5)
                i += 1                                             

        os.system('rm coord_1_2')

        return

def getSuite():
    suite = unittest.TestSuite()

    suite.addTest(coordTestCase('test_start'))

    return suite

if __name__ == "__main__":
    unittest.main()

