#*  **************************************************************************
#*
#*  CDTK, Chemical Dynamics Toolkit
#*  A modular system for chemical dynamics applications and more
#*
#*  Copyright (C) 2011, 2012, 2013, 2014, 2015, 2016
#*  Oriol Vendrell, DESY, <oriol.vendrell@desy.de>
#*
#*  Copyright (C) 2017, 2018, 2019
#*  Ralph Welsch, DESY, <ralph.welsch@desy.de>
#*
#*  Copyright (C) 2020, 2021, 2022, 2023
#*  Ludger Inhester, DESY, ludger.inhester@cfel.de
#*
#*  This file is part of CDTK.
#*
#*  CDTK is free software: you can redistribute it and/or modify
#*  it under the terms of the GNU General Public License as published by
#*  the Free Software Foundation, either version 3 of the License, or
#*  (at your option) any later version.
#*
#*  This program is distributed in the hope that it will be useful,
#*  but WITHOUT ANY WARRANTY; without even the implied warranty of
#*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#*  GNU General Public License for more details.
#*
#*  You should have received a copy of the GNU General Public License
#*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#*
#*  **************************************************************************
#! /usr/bin/env python

import os
import unittest
import numpy as np
import random
import sys
import math
from io import StringIO
#from StringIO import StringIO
import CDTK.an_getHessian as getHes

class getHessianTestCase(unittest.TestCase):
    def setUp(self):
        pass

    def test_start(self):
        """ 
        Testing the parsing of a hessian
        """


        hess_compare = np.random.rand(9,9)
        datfile = open('hess.dat', 'w')
        datfile.write("DUMMY LINE\n" + " $HESS\n" + "DUMMY LINE\n")

        for i in range(9):
            datfile.write(str(i) + ' 1 ')
            for j in range(5):
                datfile.write("{0:.12E} ".format(hess_compare[i][j]))
            datfile.write('\n' + str(i) + ' 2 ')
            for j in range(5,9):
                datfile.write("{0:.12E} ".format(hess_compare[i][j]))
            datfile.write('\n')

        datfile.write(" $END \n")
        datfile.close()

        sys.argv = ['dummy', '-H', 'hess.dat', '-t', 'gamess']

        # for redirecting output
        saved_stdout = sys.stdout
        try:
            out = StringIO()
            sys.stdout = out
            getHes.start()
            output = out.getvalue().strip()
        finally:
            sys.stdout = saved_stdout

        compare_str = ''
        for hrow in hess_compare:
            for h in hrow:
                compare_str += ' {0: >16.12e}'.format(h)
            compare_str += '\n'

        self.assertEqual(' ' + output + '\n', compare_str)

        return

def getSuite():
    suite = unittest.TestSuite()

    suite.addTest(getHessianTestCase('test_start'))

    return suite

if __name__ == "__main__":
    unittest.main()

