#!/bin/bash
 
# Script updated by Zoltan Jurek on 20.06.2022
 
# Tested with gromacs-2018.8-1.el7.x86_64 (CentOS 7)
 
# For help simply run the script without any arguments.
 
VERBOSE=-v
#VERBOSE=
 
###########################################################################
###########################################################################
###########################################################################
 
#####(\u202f\u1d54\ufe21\u202f\u035c\u0296\u202f\u1d54\ufe20 ) *Before executing bash script you may create a tmp folder and move bash script in the tmp folder for execution. EX, mkdir tmp
#########\u0295\u202f\U0001f441\ufe0f\u202f\u035c\u0296\u202f\U0001f441\ufe0f\u202f\u0294 *Please visit tutorial website http://www.mdtutorials.com/gmx/lysozyme/08_MD.html and https://www.gromacs.org/Downloads. The gmx commands are copied and modified from there.
#########\u0295\u202f\U0001f441\ufe0f\u202f\u035c\u0296\u202f\U0001f441\ufe0f\u202f\u0294 *Please download from my driver to run the calculation. You will have all *.mdp parameter files with the proper file names. Regarding downloading please send
#########\u0295\u202f\U0001f441\ufe0f \u202f\u035c\u0296\u202f\U0001f441\ufe0f \u202f\u0294 me a message(rishat.dilmurat@gmail.com) or I may put it somewhere convenient place. You may also download a relatively different version from the first tutorial website.
#########\u0295\u202f\U0001f441\ufe0f \u202f\u035c\u0296\u202f\U0001f441\ufe0f \u202f\u0294 After downloading, please modify both file names and parameters for water molecules.
#########\u0295\u202f\U0001f441\ufe0f\u202f\u035c\u0296\u202f\U0001f441\ufe0f\u202f \u0294 *finally execute the bash file. ./create_water.sh
  
# you may always change the size of the box(as example here is 3nm) and number of molecules in the box.
# you may change water to the other solvent
 
SCRIPT=$(readlink -f $0)
ROOT=`pwd`
 
STEP=0
 
boxa=3 ; boxb=3 ; boxc=3
OPT_CUTOFF=1.0
OPT_T=300
OPT_p=1.0
OPT_NSTEPS_E=50000
OPT_NSTEPS_T=50000
OPT_NSTEPS_p=50000
OPT_NSTEPS_X=500000
OPT_SALT=0

###########################################################################
###########################################################################
###########################################################################
 
function f_step_done()
{
    STEP=$(( STEP + 1 ))
    if [ -z "$VERBOSE" ] ; then echo "Step done: $STEP" ; return ; fi
    echo
    echo "##########################################"
    (echo "@@@"f_step_done ; nl -ba -s"@@@"  $SCRIPT ) | grep -m $((STEP+1)) "@@@"f_step_done$ -B99999 | tac | grep -m2 "@@@"f_step_done$ -B99999 | tac | cut -d\# -f1 | grep -v ^$ | grep -v "@@@"$ | grep -v "@@@"f_step_done$ | sed s/"@@@"/\ /
    echo "##########################################"
    echo
    read -p"Step done: $STEP" -n1
    echo
}
 
###########################################################################
 
function f_print_miniEn_mdp()
{
# based on http://www.mdtutorials.com/gmx/lysozyme/Files/minim.mdp
echo "
; minim.mdp - used as input into grompp to generate em.tpr
; Parameters describing what to do, when to stop and what to save
integrator  = steep         ; Algorithm (steep = steepest descent minimization)
emtol       = 1000.0        ; Stop minimization when the maximum force < 1000.0 kJ/mol/nm
emstep      = 0.01          ; Minimization step size
nsteps      = $OPT_NSTEPS_E         ; Maximum number of (minimization) steps to perform
 
; Parameters describing how to find the neighbors of each atom and how to calculate the interactions
nstlist         = 1         ; Frequency to update the neighbor list and long range forces
cutoff-scheme   = Verlet    ; Buffered neighbor searching
ns_type         = grid      ; Method to determine neighbor list (simple, grid)
coulombtype     = PME       ; Treatment of long range electrostatic interactions
rcoulomb        = $OPT_CUTOFF       ; Short-range electrostatic cut-off
rvdw            = $OPT_CUTOFF       ; Short-range Van der Waals cut-off
pbc             = xyz       ; Periodic Boundary Conditions in all 3 dimensions
"
}
 
###########################################################################
 
function f_print_nvt_mdp()
{
# based on http://www.mdtutorials.com/gmx/lysozyme/Files/nvt.mdp
echo "
title                   = liquid water NVT equilibration
; define                  = -DPOSRES  ; position restrain the protein
; Run parameters
integrator              = md        ; leap-frog integrator
nsteps                  = $OPT_NSTEPS_T     ; 2 * 50000 = 100 ps
dt                      = 0.002     ; 2 fs
; Output control
nstxout                 = 500       ; save coordinates every 1.0 ps
nstvout                 = 500       ; save velocities every 1.0 ps
nstenergy               = 500       ; save energies every 1.0 ps
nstlog                  = 500       ; update log file every 1.0 ps
; Bond parameters
continuation            = no        ; first dynamics run
constraint_algorithm    = lincs     ; holonomic constraints
constraints             = h-bonds   ; bonds involving H are constrained
lincs_iter              = 1         ; accuracy of LINCS
lincs_order             = 4         ; also related to accuracy
; Nonbonded settings
cutoff-scheme           = Verlet    ; Buffered neighbor searching
ns_type                 = grid      ; search neighboring grid cells
nstlist                 = 10        ; 20 fs, largely irrelevant with Verlet
rcoulomb                = $OPT_CUTOFF       ; short-range electrostatic cutoff (in nm)
rvdw                    = $OPT_CUTOFF       ; short-range van der Waals cutoff (in nm)
DispCorr                = EnerPres  ; account for cut-off vdW scheme
; Electrostatics
coulombtype             = PME       ; Particle Mesh Ewald for long-range electrostatics
pme_order               = 4         ; cubic interpolation
fourierspacing          = 0.16      ; grid spacing for FFT
; Temperature coupling is on
tcoupl                  = V-rescale             ; modified Berendsen thermostat
;tc-grps                 = Protein Non-Protein   ; two coupling groups - more accurate
;tau_t                   = 0.1     0.1           ; time constant, in ps
;ref_t                   = 300     300           ; reference temperature, one for each group, in K
tc-grps                 = System   ; two coupling groups - more accurate
tau_t                   = 0.1                ; time constant, in ps
ref_t                   = $OPT_T                ; reference temperature, one for each group, in K
; Pressure coupling is off
pcoupl                  = no        ; no pressure coupling in NVT
; Periodic boundary conditions
pbc                     = xyz       ; 3-D PBC
; Velocity generation
gen_vel                 = yes       ; assign velocities from Maxwell distribution
gen_temp                = $OPT_T       ; temperature for Maxwell distribution
gen_seed                = -1        ; generate a random seed
"
}
 
 
###########################################################################
 
function f_print_npt_mdp()
{
# based on http://www.mdtutorials.com/gmx/lysozyme/Files/npt.mdp
echo "
title                   = liquid water NPT equilibration
; define                  = -DPOSRES  ; position restrain the protein
; Run parameters
integrator              = md        ; leap-frog integrator
nsteps                  = $OPT_NSTEPS_p     ; 2 * 50000 = 100 ps
dt                      = 0.002     ; 2 fs
; Output control
nstxout                 = 500       ; save coordinates every 1.0 ps
nstvout                 = 500       ; save velocities every 1.0 ps
nstenergy               = 500       ; save energies every 1.0 ps
nstlog                  = 500       ; update log file every 1.0 ps
; Bond parameters
continuation            = yes       ; Restarting after NVT
constraint_algorithm    = lincs     ; holonomic constraints
constraints             = h-bonds   ; bonds involving H are constrained
lincs_iter              = 1         ; accuracy of LINCS
lincs_order             = 4         ; also related to accuracy
; Nonbonded settings
cutoff-scheme           = Verlet    ; Buffered neighbor searching
ns_type                 = grid      ; search neighboring grid cells
nstlist                 = 10        ; 20 fs, largely irrelevant with Verlet scheme
rcoulomb                = $OPT_CUTOFF       ; short-range electrostatic cutoff (in nm)
rvdw                    = $OPT_CUTOFF       ; short-range van der Waals cutoff (in nm)
DispCorr                = EnerPres  ; account for cut-off vdW scheme
; Electrostatics
coulombtype             = PME       ; Particle Mesh Ewald for long-range electrostatics
pme_order               = 4         ; cubic interpolation
fourierspacing          = 0.16      ; grid spacing for FFT
; Temperature coupling is on
tcoupl                  = V-rescale             ; modified Berendsen thermostat
;tc-grps                 = Protein Non-Protein   ; two coupling groups - more accurate
;tau_t                   = 0.1     0.1           ; time constant, in ps
;ref_t                   = 300     300           ; reference temperature, one for each group, in K
tc-grps                 = System   ; two coupling groups - more accurate
tau_t                   = 0.1                ; time constant, in ps
ref_t                   = $OPT_T                ; reference temperature, one for each group, in K
; Pressure coupling is on
pcoupl                  = Parrinello-Rahman     ; Pressure coupling on in NPT
pcoupltype              = isotropic             ; uniform scaling of box vectors
tau_p                   = 2.0                   ; time constant, in ps
ref_p                   = $OPT_p                   ; reference pressure, in bar
compressibility         = 4.5e-5                ; isothermal compressibility of water, bar^-1
refcoord_scaling        = com
; Periodic boundary conditions
pbc                     = xyz       ; 3-D PBC
; Velocity generation
gen_vel                 = no        ; Velocity generation is off
"
}
 
###########################################################################
 
function f_print_prdc_mdp()
{
# based on http://www.mdtutorials.com/gmx/lysozyme/Files/md.mdp
echo "
title                   = liquid water NPT equilibration
; Run parameters
integrator              = md        ; leap-frog integrator
nsteps                  = $OPT_NSTEPS_X    ; 2 * 500000 = 1000 ps (1 ns)
dt                      = 0.002     ; 2 fs
; Output control
nstxout                 = 1000      ; every 2 ps output of coordinates and
nstvout                 = 1000      ; velocity
nstfout                 = 0         ; nstvout, and nstfout
nstenergy               = 5000      ; save energies every 10.0 ps
nstlog                  = 5000      ; update log file every 10.0 ps
nstxout-compressed      = 5000      ; save compressed coordinates every 10.0 ps
compressed-x-grps       = System    ; save the whole system
; Bond parameters
continuation            = yes       ; Restarting after NPT
constraint_algorithm    = lincs     ; holonomic constraints
constraints             = h-bonds   ; bonds involving H are constrained
lincs_iter              = 1         ; accuracy of LINCS
lincs_order             = 4         ; also related to accuracy
; Neighborsearching
cutoff-scheme           = Verlet    ; Buffered neighbor searching
ns_type                 = grid      ; search neighboring grid cells
nstlist                 = 10        ; 20 fs, largely irrelevant with Verlet scheme
rcoulomb                = $OPT_CUTOFF       ; short-range electrostatic cutoff (in nm)
rvdw                    = $OPT_CUTOFF       ; short-range van der Waals cutoff (in nm)
; Electrostatics
coulombtype             = PME       ; Particle Mesh Ewald for long-range electrostatics
pme_order               = 4         ; cubic interpolation
fourierspacing          = 0.16      ; grid spacing for FFT
; Temperature coupling is on
tcoupl                  = V-rescale             ; modified Berendsen thermostat
;tc-grps                 = Protein Non-Protein   ; two coupling groups - more accurate
;tau_t                   = 0.1     0.1           ; time constant, in ps
;ref_t                   = 300     300           ; reference temperature, one for each group, in K
tc-grps                 = System   ; two coupling groups - more accurate
tau_t                   = 0.1                ; time constant, in ps
ref_t                   = $OPT_T                ; reference temperature, one for each group, in K
; Pressure coupling is on
pcoupl                  = Parrinello-Rahman     ; Pressure coupling on in NPT
pcoupltype              = isotropic             ; uniform scaling of box vectors
tau_p                   = 2.0                   ; time constant, in ps
ref_p                   = $OPT_p                   ; reference pressure, in bar
compressibility         = 4.5e-5                ; isothermal compressibility of water, bar^-1
; Periodic boundary conditions
pbc                     = xyz       ; 3-D PBC
; Dispersion correction
DispCorr                = EnerPres  ; account for cut-off vdW scheme
; Velocity generation
gen_vel                 = no        ; Velocity generation is off
"
}
 
###########################################################################
 
function f_help()
{
    echo "Usage:
  `basename $0` [OPTIONS] <CMD>
 
where <CMD> is one of the subcommands
  sys-sol sys-top sys-addsalt  min-mdp min-mpp min-run   nvt-mdp nvt-mpp nvt-run nvt-plt
  npt-mdp npt-mpp npt-run npt-plt   pro-mdp pro-mpp pro-run
 
OPTIONS:
  -a       <BOXA>     # nm
  -b       <BOXB>     # nm
  -c       <BOXC>     # nm
  -abc     <BOXABC>   # nm
  -T       <T>        # K
  -p       <P>        # bar
  -cutoff  <CUTOFF>   # nm
  -nstepE  <NSTEPE>
  -nstepT  <NSTEPT>
  -nstepp  <NSTEPp>
  -nstepX  <NSTEPX>
  -salt    <concentration> # in Mol/L of NaCl
 
Default values:
  BOXA=BOXB=BOXC=$boxa , CUTOFF=$OPT_CUTOFF , T=$OPT_T , p=$OPT_p
  NSTEPE=$OPT_NSTEPS_E , NSTEPT=$OPT_NSTEPS_T , NSTEPp=$OPT_NSTEPS_p , NSTEPX=$OPT_NSTEPS_X, salt=$OPT_SALT
 
Simple run:
  `basename $0` [OPTIONS] all
so that all subcommands are executed consecutively.
"
exit
}
 
###########################################################################
###########################################################################
###########################################################################
 
#++++++.1.++++++ Create water n the box #
#box with number of water molecules inside the box(number of the molecules are automatically created)#
#3-point model (e.g. SPC, SPC/E or TIP3P) it will work with specify -cs spc216.gro which
#will take this file from the gromacs/share/top
 
if [ -z "$*" ] ; then f_help ; exit ; fi
 
D_SOL=01-sol
D_MIN=02-min
D_NVT=03-nvt
D_NPT=04-npt
D_PRO=05-pro
LOGFILE=99-log
 
 
while [ ! -z "$1" ] ; do
    case "x$1" in
        x-h|-help|x--help) f_help ; exit ;;
        x-abc) boxa=$2 ; boxb=$2 ; boxc=$2 ; shift 1 ;;
        x-a)   boxa=$2 ; shift 1 ;;
        x-b)   boxb=$2 ; shift 1 ;;
        x-c)   boxc=$2 ; shift 1 ;;
        x-T)   OPT_T=$2 ; shift 1 ;;
        x-p)   OPT_p=$2 ; shift 1 ;;
        x-cutoff)   OPT_CUTOFF=$2 ; shift 1 ;;
        x-nstepE)   OPT_NSTEPS_E=$2 ; shift 1 ;;
        x-nstepT)   OPT_NSTEPS_T=$2 ; shift 1 ;;
        x-nstepp)   OPT_NSTEPS_p=$2 ; shift 1 ;;
        x-nstepX)   OPT_NSTEPS_X=$2 ; shift 1 ;;
        x-salt)   OPT_SALT=$2 ; shift 1 ;;
        xall)  CMD="$0   -a $boxa -b $boxb -c $boxc -T $OPT_T -p $OPT_p -cutoff $OPT_CUTOFF -nstepE $OPT_NSTEPS_E -nstepT $OPT_NSTEPS_T -nstepp $OPT_NSTEPS_p -nstepX $OPT_NSTEPS_X -salt $OPT_SALT sys-sol sys-top sys-addsalt min-mdp min-mpp min-run nvt-mdp nvt-mpp nvt-run nvt-plt npt-mdp npt-mpp npt-run npt-plt pro-mdp pro-mpp pro-run"
               echo >> $LOGFILE
               echo "############################################################################" >> $LOGFILE
               pwd >> $LOGFILE
               echo "$CMD" >> $LOGFILE
               echo >> $LOGFILE
               $CMD
               exit
               ;;
        *) break ;;
    esac
    shift 1
done
 
 
 
while [ ! -z "$1" ] ; do
 
    T0=`date +%s`
 
    case "x$1" in
 
    xsolvate|xsys-sol)
        D=$D_SOL ; mkdir -pv $D ; cd $D
        gmx solvate -cs spc216.gro -box $boxa $boxb $boxc -o water
    ;;
     
    # you may insert the number of molecules (as you want) in the given box as following instructions.
    # For that please create an example.gro file. The example.gro file should be as follows.
    # please just copy-paste the following example and save as example.gro.
    # #title (Generated by gmx solvate)
    # #number of atom (1)
    # GMX coordinates(nm) for .gro file
    # size of the box (nm)
     
    #****************************for instance*******************************************
    #echo Generated by gmx solvate > example.gro
    #echo 3 >> example.gro
    #echo 1SOL OW 1006 1.300 0.453 2.553 >> example.gro
    #echo 1SOL HW1 1007 1.241 0.533 2.557 >> example.gro
    #echo 1SOL HW2 1008 1.315 0.418 2.646 >> example.gro
    #echo 3.00000 3.00000 3.00000 >> example.gro
    #************************************************************************************
    #number of water molecules mol=200
    #gmx insert-molecules -ci example.gro -nmol $mol -box $boxa $boxb $boxc -o ${mol}_box.gro -seed 100
      
    #++++++.1 continue.++++++ Create topology file for water molecules 'topol.top'#
    # echo 15 choose option 15 with force field 'OPLS-AA/L all-atom force field (2001 aminoacid dihedrals)'
    # You may immediately see the difference between water.gro and water.out.gro files.
    
    xtopol|xsys-top)
        D=$D_SOL ; mkdir -pv $D ; cd $D
        echo 15 | gmx pdb2gmx -f water.gro -o water.out.gro -water spce
    ;;

    xsys-addsalt)
        D=$D_SOL ; mkdir -pv $D ; cd $D
        f_print_miniEn_mdp > tmp.mdp
        gmx grompp -f tmp.mdp -c water.out.gro -p topol.top -o tmp.tpr
        echo 2 | gmx genion -s tmp.tpr -o water.out.gro -p topol.top -pname NA -nname CL -conc ${OPT_SALT} -rmin 0.4
    ;;

    #_________________________________REAL_BUSINESS=MD_calculation_____________________________________________#
    #++++++.2.++++++ Energy Minimization#
    # we have water.out.gro and topol.top, find *.mdp parameter file on web http://www.mdtutorials.com/gmx/lysozyme/05_EM.html
    # please read Gromacs manual booklet from Gromacs web
     
    xmin-mdp)
        D=$D_MIN ; mkdir -pv $D ; cd $D
        f_print_miniEn_mdp > miniEn.mdp
        #cp $VERBOSE water.out.gro miniEn.mdp topol.top minEn
    ;;
      
     
    xmin-mpp)
        D=$D_MIN ; mkdir -pv $D ; cd $D
        gmx grompp -f miniEn.mdp -c ../$D_SOL/water.out.gro -p ../$D_SOL/topol.top -o miniEn.tpr
    ;;
 
 
    xmin-run)
        D=$D_MIN ; mkdir -pv $D ; cd $D
        gmx mdrun -v -deffnm miniEn
    ;;

      
    #++++++.2.1++++++ nvt calculation#
    # find nvt.mdp file in the $web1 or download from the drive. plot temperature_VS_time from temperature.xvg
      
    xnvt-mdp)
        D=$D_NVT ; mkdir -pv $D ; cd $D
        f_print_nvt_mdp > nvt.mdp
    ;;
 
  
    xnvt-mpp)
        #cp $VERBOSE minEn/miniEn.gro nvt/water.minEn.gro
        #cp $VERBOSE minEn/topol.top nvt/
        D=$D_NVT ; mkdir -pv $D ; cd $D
        gmx grompp -f nvt.mdp -c ../$D_MIN/miniEn.gro -r ../$D_MIN/miniEn.gro -p ../$D_SOL/topol.top -o water_nvt.tpr
    ;;
 
 
    xnvt-run)
        D=$D_NVT ; mkdir -pv $D ; cd $D
        gmx mdrun -deffnm water_nvt
    ;;
     
      
    xnvt-plt)
        D=$D_NVT ; mkdir -pv $D ; cd $D
        echo 9 | gmx energy -f water_nvt.edr -o temperature.xvg
    ;;
          
          
        #++++++.2.1++++++ npt calculation#
        # find npt.mdp file in the $web1 or download from the drive. plot pressure_VS_time from pressure.xvg
          
    xnpt-mdp)
        D=$D_NPT ; mkdir -pv $D ; cd $D
        f_print_npt_mdp > npt.mdp
    ;;
 
  
    xnpt-mpp)
        D=$D_NPT ; mkdir -pv $D ; cd $D
        #cp $VERBOSE topol.top npt.mdp npt/
        #cp $VERBOSE nvt/water_nvt.gro npt/
        #cp $VERBOSE nvt/water_nvt.cpt npt/
        gmx grompp -f npt.mdp -c ../$D_NVT/water_nvt.gro -r ../$D_NVT/water_nvt.gro -t ../$D_NVT/water_nvt.cpt -p ../$D_SOL/topol.top -o water_npt.tpr
    ;;
 
 
    xnpt-run)
        D=$D_NPT ; mkdir -pv $D ; cd $D
        gmx mdrun -deffnm water_npt
    ;;
         
  
    xnpt-plt)
        D=$D_NPT ; mkdir -pv $D ; cd $D
        echo 11 | gmx energy -f water_npt.edr -o pressure.xvg
    ;;
          
          
        #++++++.2.3++++++ Production_MD calculation#
        #gmx grompp -f prdc.mdp -c water_npt.gro -t water_npt.cpt -p topol.top -o water_prdc.tpr
        #gmx mdrun -deffnm water_prdc
          
          
    xpro-mdp)
        D=$D_PRO ; mkdir -pv $D ; cd $D
        f_print_prdc_mdp > prdc.mdp
    ;;
 
 
    xpro-mpp)
        D=$D_PRO ; mkdir -pv $D ; cd $D
        #cp $VERBOSE topol.top prdc.mdp prdc/
        #cp $VERBOSE npt/water_npt.gro prdc/
        #cp $VERBOSE npt/water_npt.cpt prdc/
        gmx grompp -f prdc.mdp -c ../$D_NPT/water_npt.gro -t ../$D_NPT/water_npt.cpt -p ../$D_SOL/topol.top -o water_prdc.tpr
    ;;
 
 
    xpro-run)
        D=$D_PRO ; mkdir -pv $D ; cd $D
        gmx mdrun -deffnm water_prdc
        # output last snapshot (t=1000ps)
        echo '0
        ' | gmx trjconv -s water_prdc.tpr -f water_prdc.trr -o water_sample.trr -b 1000 -pbc mol
        echo '0
        ' |gmx trjconv -s water_prdc.tpr -f water_prdc.trr -o water_sample.gro -b 1000 -pbc mol
    ;;
 
    
    esac
 
    if [ $? != 0 ] ; then echo "We stop due to error at stage $1 ." ; exit ; fi
      
    cd "$ROOT"
 
    T1=`date +%s`
 
    echo "`date +%Y.%m.%d-%H.%M.%S`  -  $1:   "$((T1-T0)) | tee -a $LOGFILE
 
    shift 1
 
done

