#!/bin/sh

SYSTEM="C";  Z=6        # carbon
PE=12000                # unit: eV

XATOM_EXE=../../src/xatom               # newly-compiled version to be tested
XATOM_OPT="-Z $Z -pcs -PE $PE -decay"   # default setup as much as possible
XATOM_DATA_FILE="${SYSTEM}_process_${PE}eV.dat"
[ -f $XATOM_DATA_FILE ] && rm -f $XATOM_DATA_FILE  # start from empty

DIR_REFERENCE="results_v0.2"

FILTER_EXE=../../src/show_essence
ACCURACY="-d 6"                 # option for "show_essence"
TEMP_NEW="temporary_file.new"
TEMP_OLD="temporary_file.old"

max_2p=$(( $Z - 2 - 2 ))
#
# NOTE: seq [start number] [increment] [end number]
#
for n_1s in `seq 2 -1 0` ; do           # occupation number in 1s
  for n_2s in `seq 2 -1 0` ; do         # occupation number in 2s
    for n_2p in `seq $max_2p -1 0` ; do # occupation number in 2p
      n_elec=$(( $n_1s + $n_2s + $n_2p ))
      [ $n_elec -eq 0 ] && continue
      charge=$(( $Z - $n_elec ))
      config="1s${n_1s}_2s${n_2s}_2p${n_2p}"
      FILE="${SYSTEM}_${charge}+_${config}.out"
      echo -n "Testing for $FILE ... "
      $XATOM_EXE $XATOM_INPUT $XATOM_OPT -config $config -dat $XATOM_DATA_FILE -o $FILE
      $FILTER_EXE $ACCURACY $FILE > $TEMP_NEW
      $FILTER_EXE $ACCURACY "$DIR_REFERENCE/$FILE" > $TEMP_OLD
      diff $TEMP_NEW $TEMP_OLD
      echo "done"
    done
  done
done

rm -f $TEMP_NEW $TEMP_OLD
