#!/bin/bash
# -----------------------------------------------------------------------------
# This script runs test calculations using "run_all.sh" for Ne at 2000 eV
# and compares with the previous results using "show_essence."
# Please make sure those parameters (Ne at 2000 eV) in the "run_all.sh."
# -----------------------------------------------------------------------------

SYSTEM="Ne";  Z=10              # neon
yes_run_all=true                # true: enforce to run, false: skip to run
RUN_ALL_EXE=./run_all.sh
if [ "$yes_run_all" = true ]; then 
  echo "Run $RUN_ALL_EXE at the beginning"
  $RUN_ALL_EXE
  echo ""
fi

FILTER_EXE=../../src/show_essence
DIR_REFERENCE="results_v0.2"    # previously calculated values
DIGIT=6                         # requested accuracy: 6 digits after "."
OPT="-d $DIGIT"                 # option for "show_essence"
TEMP_NEW="temporary_file.new"
TEMP_OLD="temporary_file.old"
echo "Essential data (orbital energies, cross sections, and rates)"
echo "are compared with data in ${DIR_REFERENCE}"
echo "up to ${DIGIT} digits after the decimal point."
echo ""

#
# NOTE: seq [start number] [increment] [end number]
#
count=0
max_2p=$(( $Z - 2 - 2 ))
for n_1s in `seq 2 -1 0` ; do                   # occupation number in 1s
  for n_2s in `seq 2 -1 0` ; do                 # occupation number in 2s
    for n_2p in `seq $max_2p -1 0` ; do         # occupation number in 2p
      n_elec=$(( $n_1s + $n_2s + $n_2p ))
      [ $n_elec -eq 0 ] && continue
      charge=$(( $Z - $n_elec ))
      config="1s${n_1s}_2s${n_2s}_2p${n_2p}"
      FILE="${SYSTEM}_${charge}+_${config}.out" # filename for new output
      FILE_OLD="$DIR_REFERENCE/$FILE"           # filename for old output
      echo -n "Testing for $FILE ... "
      $FILTER_EXE $OPT $FILE     > $TEMP_NEW    # essential data for new
      $FILTER_EXE $OPT $FILE_OLD > $TEMP_OLD    # essential data for old
      RESULT=`diff $TEMP_NEW $TEMP_OLD`         # comparison b/w old and new
      if [ "$RESULT" == "" ]; then
        echo "passed"
      else
        echo "different"
        echo "$RESULT"
        let "count++"
      fi
    done
  done
done

rm -f $TEMP_NEW $TEMP_OLD

echo ""
if [ $count == 0 ]; then
  echo "All files are successfully passed."
else
  echo "$count file(s) have the difference from ${DIR_REFERENCE}!"
fi
