#!/bin/sh
# -----------------------------------------------------------------------------
# This script runs XATOM dynamics calculations with three different excutables.
# - xatom_req: direct rate-equation solver, which requires an atomic data file
#              Ref) Phys. Rev. A 83, 033402 (2011)
# - xatom_mc:  Monte Carlo rate-eq. solver, which requires an atomic data file
#              Ref) Nature Photon. 6, 858 (2012); Phys. Rev. A 85, 063415 (2012)
# - xatom_mc_fly: Monte Carlo on the fly, which does not need any data file
#              Ref) Phys. Rev. Lett. 110, 173005 (2013)
#
# NOTE: The atomic data file is specified by $DATA, 
#       which is pre-calculated using "../Ne/run_all.sh"
#       with a series of XATOM static calculations (for all possible config.)
#       If you don't have $DATA, go to "../Ne/" and "run_all.sh" over there.
#
# Since the x-ray fluence and the pulse duration are not specified 
# for the first two examples, they will use the default values 
# (probably F=1E12 ph/micron^2 and 100 fs FWHM). Please check them out
# in the output files to confirm those x-ray beam parameters.
# -----------------------------------------------------------------------------

EXEDIR=../../src
DATA=../Ne/Ne_process_2keV.dat

# example for neutral Ne (ground configuration) at 2 keV
OPT="-Z 10 -PE 2000"
$EXEDIR/xatom_req    $OPT -dat $DATA -o Ne_2keV_N1E12_GND.req.out
$EXEDIR/xatom_mc     $OPT -dat $DATA -o Ne_2keV_N1E12_GND.mc.out
$EXEDIR/xatom_mc_fly $OPT            -o Ne_2keV_N1E12_GND.mc_fly.out

# example for core-ionized Ne^+ at 2 keV
OPT="-Z 10 -PE 2000 -config 1s1_2s2_2p5"
$EXEDIR/xatom_req    $OPT -dat $DATA -o Ne_2keV_N1E12_1s1_1s2_2p5.req.out
$EXEDIR/xatom_mc     $OPT -dat $DATA -o Ne_2keV_N1E12_1s1_1s2_2p5.mc.out
$EXEDIR/xatom_mc_fly $OPT            -o Ne_2keV_N1E12_1s1_1s2_2p5.mc_fly.out

# example for decay process of Ne double-core-hole state (w/o any x-ray pulse)
OPT="-Z 10 -config 1s0_2s2_2p5 -fluence 0"
$EXEDIR/xatom_req    $OPT -dat $DATA -o Ne_decay_1s0.req.out
$EXEDIR/xatom_mc     $OPT -dat $DATA -o Ne_decay_1s0.mc.out
$EXEDIR/xatom_mc_fly $OPT            -o Ne_decay_1s0.mc_fly.out
