!----------------------------------------------------------------------------
! RSE.f90
!----------------------------------------------------------------------------
! Copyright (C) 2010,2023 Deutsches Elektronen-Synchrotron DESY,
!                         a research centre of the Helmholtz Association.
! Authors: 2010,2023 Sang-Kil Son <sangkil.son@cfel.de>
! License: GPL-3.0-or-later
!
! This file is part of XATOM.
!
! XATOM is free software: you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software
! Foundation, either version 3 of the License, or (at your option) any later
! version.
!
! XATOM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! XATOM. If not, see <https://www.gnu.org/licenses/>.
!----------------------------------------------------------------------------
! Radial Schoedinger equation solver (needed for rk4.f)
!   originally written by Robin Santra (RadialWaveFunctions/Continuum/)
!   slightly modified by Son, Sang-Kil (including SP)
!   again modified by Son, Sang-Kil (including F, instead of SP)
!----------------------------------------------------------------------------
      subroutine RSE(r,n,ndot,l,energy,F)
      use wavefunction

      implicit none

      integer neq
      parameter(neq = 2) 
      real*8 r
      real*8 n(neq),ndot(neq)
      real*8 l,energy
      !real*8 rr(441),rV(441),b(441),c(441),d(441)
      real*8 rpot !,seval
      type(WF_data) :: F

C      common /potpar/ rr,rV,b,c,d

C      pot = seval(441,r,rr,rV,b,c,d)
      !
      ! since F%SP_pot contains r * pot, we need a special care for
      ! extrapolation if a constant potential is needed for plasma calc.
      ! also see unbound.f
      !
      if ( r <= F%rmax ) then                           ! interpolation
        rpot = Spline_interpolate( F%SP_pot, r )        ! rpot means r * pot
      else                                              ! extrapolation
        !
        ! "-F%q" is the end value of Spline_interpolate( SP, r=F%rmax )
        ! - for isolated atom calc., potential should be -F%q / r, 
        !   so rpot = -F%q / r * r = -F%q, which should be constant
        ! - for plasma calc., potential should be constant if muffin-tin
        !   approximation or external environmental potential is applied
        !
        if ( F%yes_flat_tail ) then     ! plasma case (flat pot. tail)
          rpot = -F%q / F%rmax * r
        else                            ! isolated atom case: -F%q constant
          rpot = -F%q
        end if
      end if

      ndot(1) = n(2) 

      ndot(2) = l*(l+1.d0)/r + 2.d0*rpot
      ndot(2) = ndot(2)/r - 2.d0*energy
      ndot(2) = ndot(2)*n(1)

      return
      end

