!----------------------------------------------------------------------------
! atom.f90
!----------------------------------------------------------------------------
! Copyright (C) 2004      Sang-Kil Son
! Copyright (C) 2010-2018 Deutsches Elektronen-Synchrotron DESY,
!                         a research centre of the Helmholtz Association.
! Authors: 2010-2018 Sang-Kil Son <sangkil.son@cfel.de>
! License: GPL-3.0-or-later
!
! This file is part of XATOM.
!
! XATOM is free software: you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software
! Foundation, either version 3 of the License, or (at your option) any later
! version.
!
! XATOM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! XATOM. If not, see <https://www.gnu.org/licenses/>.
!----------------------------------------------------------------------------
! Atomic data for the DFT calculation
! originally extracted from OCTOPUS, http://www.tddft.org/programs/octopus/
! updated from NIST, https://www.nist.gov/pml/atomic-weights-and-isotopic-compositions-relative-atomic-masses
!   written by Son, Sang-Kil in Nov. 2004 - Dec. 2004
!   rewritten by Son, Sang-Kil in Jun. 2018
!
! - ATOMIC_NAME(1...109)
! - ATOMIC_MASS(1...94)
!----------------------------------------------------------------------------
module atom
  use constants
  implicit none
  character(len=2), parameter :: ATOMIC_NAME(109) = (/ &
    'H ','He', &                                                  ! period 1
    'Li','Be','B ','C ','N ','O ','F ','Ne', &                    ! period 2
    'Na','Mg','Al','Si','P ','S ','Cl','Ar', &                    ! period 3
    'K ','Ca','Sc','Ti','V ','Cr','Mn','Fe','Co','Ni','Cu','Zn', &
                                       'Ga','Ge','As','Se','Br','Kr', &  ! 4
    'Rb','Sr','Y ','Zr','Nb','Mo','Tc','Ru','Rh','Pd','Ag','Cd', &
                                       'In','Sn','Sb','Te','I ','Xe', &  ! 5
    'Cs','Ba', &
         'La','Ce','Pr','Nd','Pm','Sm','Eu', &
         'Gd','Tb','Dy','Ho','Er','Tm','Yb', &
              'Lu','Hf','Ta','W ','Re','Os','Ir','Pt','Au','Hg', &
                                       'Tl','Pb','Bi','Po','At','Rn', &  ! 6
    'Fr','Ra', &
         'Ac','Th','Pa','U ','Np','Pu','Am', &
         'Cm','Bk','Cf','Es','Fm','Md','No', &
              'Lr','Rf','Db','Sg','Bh','Hs','Mt' /)               ! period 7
  real(long), parameter :: ATOMIC_MASS(94) = (/ &
    1.00784, &          ! 1  H 
    4.002602, &         ! 2  He
    6.938, &            ! 3  Li
    9.01218315, &       ! 4  Be
    10.806, &           ! 5  B 
    12.0096, &          ! 6  C 
    14.00643, &         ! 7  N 
    15.99903, &         ! 8  O 
    18.998403163, &     ! 9  F 
    20.1797, &          ! 10 Ne
    22.98976928, &      ! 11 Na
    24.304, &           ! 12 Mg
    26.9815385, &       ! 13 Al
    28.084, &           ! 14 Si
    30.973761998, &     ! 15 P 
    32.059, &           ! 16 S 
    35.446, &           ! 17 Cl
    39.948, &           ! 18 Ar
    39.0983, &          ! 19 K 
    40.078, &           ! 20 Ca
    44.955908, &        ! 21 Sc
    47.867, &           ! 22 Ti
    50.9415, &          ! 23 V 
    51.9961, &          ! 24 Cr
    54.938044, &        ! 25 Mn
    55.845, &           ! 26 Fe
    58.933194, &        ! 27 Co
    58.6934, &          ! 28 Ni
    63.546, &           ! 29 Cu
    65.38, &            ! 30 Zn
    69.723, &           ! 31 Ga
    72.630, &           ! 32 Ge
    74.921595, &        ! 33 As
    78.971, &           ! 34 Se
    79.901, &           ! 35 Br
    83.798, &           ! 36 Kr
    85.4678, &          ! 37 Rb
    87.62, &            ! 38 Sr
    88.90584, &         ! 39 Y 
    91.224, &           ! 40 Zr
    92.90637, &         ! 41 Nb
    95.95, &            ! 42 Mo
    98., &              ! 43 Tc
    101.07, &           ! 44 Ru
    102.90550 , &       ! 45 Rh
    106.42, &           ! 46 Pd
    107.8682, &         ! 47 Ag
    112.414, &          ! 48 Cd
    114.818, &          ! 49 In
    118.710, &          ! 50 Sn
    121.760, &          ! 51 Sb
    127.60, &           ! 52 Te
    126.90447, &        ! 53 I 
    131.293, &          ! 54 Xe
    132.90545196, &     ! 55 Cs
    137.327, &          ! 56 Ba
    138.905, &          ! 57 La
    140.116, &          ! 58 Ce
    140.90766, &        ! 59 Pr
    144.242, &          ! 60 Nd
    145., &             ! 61 Pm
    150.36, &           ! 62 Sm
    151.964, &          ! 63 Eu
    157.25, &           ! 64 Gd
    158.92535, &        ! 65 Tb
    162.500, &          ! 66 Dy
    164.93033, &        ! 67 Ho
    167.259, &          ! 68 Er
    168.93422, &        ! 69 Tm
    173.054, &          ! 70 Yb
    174.9668, &         ! 71 Lu
    178.49, &           ! 72 Hf
    180.947, &          ! 73 Ta
    183.84, &           ! 74 W 
    186.207, &          ! 75 Re
    190.23, &           ! 76 Os
    192.217, &          ! 77 Ir
    195.084, &          ! 78 Pt
    196.966569, &       ! 79 Au
    200.592, &          ! 80 Hg
    204.382, &          ! 81 Tl
    207.2, &            ! 82 Pb
    208.98040, &        ! 83 Bi
    209., &             ! 84 Po
    210., &             ! 85 At
    222., &             ! 86 Rn
    223., &             ! 87 Fr
    226., &             ! 88 Ra
    227., &             ! 89 Ac
    232.0377, &         ! 90 Th
    231.03588, &        ! 91 Pa
    238.02891, &        ! 92 U 
    237., &             ! 93 Np
    244. /)             ! 94 Pu
contains

function get_atomic_number( s ) result (n)
  integer :: n
  character(len=2), intent(in) :: s
  character(len=2) :: atom
  character(len=*), parameter :: uppercase = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
  integer :: i
  atom = s
  if ( index( uppercase, atom(2:2) ) /= 0 ) &
    atom(2:2) = char( ichar( s(2:2) ) - ichar( "A" ) + ichar( "a" ) )
  do i = 1, size(ATOMIC_NAME)
    if ( atom .eq. ATOMIC_NAME(i) ) then
      n = i
      return
    end if
  end do
  write(STDERR,*) "There is no atomic name like " // trim(atom) // "."
  stop
end function get_atomic_number

end module atom
