!----------------------------------------------------------------------------
! random_num.f90
!----------------------------------------------------------------------------
! Copyright (C) 2016 Deutsches Elektronen-Synchrotron DESY,
!                    a research centre of the Helmholtz Association.
! Authors: 2016 Sang-Kil Son <sangkil.son@cfel.de>
!          2016 Koudai Toyota
! License: GPL-3.0-or-later
!
! This file is part of XATOM.
!
! XATOM is free software: you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software
! Foundation, either version 3 of the License, or (at your option) any later
! version.
!
! XATOM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! XATOM. If not, see <https://www.gnu.org/licenses/>.
!----------------------------------------------------------------------------
! random generator: intrinsic fortran routine or MKL routine
! - MKL routines are problematic, and intrinsic one looks okay.
! - random_seed( put=seed(1:1) ) gives compile errors with gfortran44.
!   to fix it, use the following... but it's awfully slow!
!   why do you want to use gfortran44 if ifort is available?
!----------------------------------------------------------------------------
module random_num
  use constants
  implicit none
contains
  subroutine initialize_random()
#if defined(__INTEL_COMPILER) || defined(__INTEL_LLVM_COMPILER)
    integer :: icount, seed(1)
    call system_clock( icount )
    seed(1) = icount      ! initialize random generator using a timing seed
    call random_seed( put=seed(1:1) )
#elif defined( __GNUC__)
    !
    ! no particular reason why 37 is given below... 
    ! but it might be from gfortran 4.8 example:
    ! https://gcc.gnu.org/onlinedocs/gcc-4.8.0/gfortran/RANDOM_005fSEED.html
    !
    integer :: icount, n, i
    integer, allocatable :: seed(:)
    call random_seed( size=n )
    allocate( seed(n) )
    call system_clock( icount )
    seed(:) = icount + 37 * (/ (i - 1, i = 1, n) /)
    call random_seed( put=seed )
    deallocate( seed )
#else
    #error "compiler not identified" 
#endif
  end subroutine initialize_random

  function random() result (r)
    real(long) :: r, x(1)
    call random_number( x(1:1) )
    r = x(1)
  end function random

!----------------------------------------------------
! Random number generator from MKL routines: NOT USED
!----------------------------------------------------
! you need to specify the following at the beginning
!  include 'mkl_vsl.fi'
!  use MKL_VSL_TYPE
!  use MKL_VSL
!
!subroutine initialize_random( stream )
!  type(VSL_STREAM_STATE), intent(inout) :: stream
!  integer :: seed, errcode
!  integer :: brng = VSL_BRNG_MT19937
!  call system_clock( seed )
!  errcode = vslnewstream( stream, brng, seed )
!  integer :: icount, i, n
!  integer, allocatable :: seed(:)
!end subroutine initialize_random

!function random( stream ) result (r)
!  type(VSL_STREAM_STATE), intent(inout) :: stream
!  real(long) :: r, x(1), a = 5.0_long, sigma = 2.0_long
!  integer :: errcode
!  integer :: method = VSL_METHOD_DGAUSSIAN_ICDF
!  errcode = vdrnggaussian( method, stream, 1, x, a, sigma )
!  call random_number( x(1:1) )
!  r = x(1)
!end function random

end module random_num
