!----------------------------------------------------------------------------
! rk4.f
!----------------------------------------------------------------------------
! Copyright (C) 2010,2023 Deutsches Elektronen-Synchrotron DESY,
!                         a research centre of the Helmholtz Association.
! Authors: 2010,2023 Sang-Kil Son <sangkil.son@cfel.de>
! License: GPL-3.0-or-later
!
! This file is part of XATOM.
!
! XATOM is free software: you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software
! Foundation, either version 3 of the License, or (at your option) any later
! version.
!
! XATOM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! XATOM. If not, see <https://www.gnu.org/licenses/>.
!----------------------------------------------------------------------------
! Runge-Kutta method solver (needed for unbound.f and requires RSE.f)
!   originally written by Robin Santra (RadialWaveFunctions/Continuum/)
!   slightly modified by Son, Sang-Kil (including SP)
!   again modified by Son, Sang-Kil (including F, instead of SP)
!----------------------------------------------------------------------------
      subroutine rk4(u0,r0,uf,rf,l,energy,F)
      use wavefunction

      implicit none

      integer neq
      parameter (neq = 2)
      real*8 u0(neq),uf(neq) 
      real*8 r0,rc,rf,dr
      real*8 l,energy
      real*8 work(neq,5)
      type(WF_data) :: F
      integer i

      rc = 0.5d0*(r0 + rf)
      dr = rf - r0

      call RSE(r0,u0,work(1,1),l,energy,F)

      do i = 1, neq
         work(i,5) = u0(i) + 0.5d0*dr*work(i,1)
      end do

      call RSE(rc,work(1,5),work(1,2),l,energy,F)

      do i = 1, neq
         work(i,5) = u0(i) + 0.5d0*dr*work(i,2)
      end do
 
      call RSE(rc,work(1,5),work(1,3),l,energy,F)

      do i = 1, neq
         work(i,5) = u0(i) + dr*work(i,3)
      end do

      call RSE(rf,work(1,5),work(1,4),l,energy,F)

      do i = 1, neq
         work(i,5) = work(i,1) + 2.d0*work(i,2) 
     &        + 2.d0*work(i,3) + work(i,4)
         uf(i) = u0(i) + work(i,5)*dr/6.d0
      end do

      return 
      end
