#!/bin/sh

#Some useful commmands to prevent from segmentation fault or missing libraries
ulimit -s unlimited
. /opt/intel/2011/bin/ifortvars.sh intel64

EXE=../../xatom

OUTPUT_FILE_WH="waller-hartree.dat"
OUTPUT_FILE_SPECTRUM="spectrum.dat"
OUTPUT_FILE_DDSCS="DDSCS.dat"

Z=6
PE=10000

OPTION_WH="-Z $Z -PE $PE -compton -output $OUTPUT_FILE_WH"  
OPTION_SPECTRUM="-Z $Z -PE $PE -compton spectrum -dr 0.05 -compton_d_omega 20 -compton_n_omega 20 -compton_theta 60 -compton_rmax 450 -output $OUTPUT_FILE_SPECTRUM"

touch carbon.in
OPTION_DDSCS="-input carbon.in -Z $Z -PE $PE -compton DDSCS -dr 0.05 -compton_d_omega 20 -compton_n_omega 20 -compton_d_theta 5 -compton_n_theta 10 -compton_rmax 450 -output $OUTPUT_FILE_DDSCS"

echo "THIS SCRIPT GENERATES AND TESTS OUTPUT FOR THE COMPTON MODULE!" 

echo ""
echo "WALLER-HARTREE method: Now generating test data!"
echo "This should take approx. 1sec." 
$EXE $OPTION_WH
echo "Comparing output to test data:"
echo ""
echo "Checking waller-hartree.dat and test_waller-hartree.test "
diff -s -BI '^#' waller-hartree.dat test_waller-hartree.test || \
echo "set terminal x11 1 persist; plot 'test_waller-hartree.test' u 1:(\$3/\$2) w lp t 'TEST-DATA Waller-Hartree', 'waller-hartree.dat' u 1:(\$3/\$2) w lp t 'New generated data Waller-Hartree'" | gnuplot
echo ""

echo "SPECTRUM calulation: Now generating test data!"
echo "This should take approx. 10 sec." 
$EXE $OPTION_SPECTRUM
echo "Comparing output to test data:"
echo ""
echo "Checking spectrum.dat and test_spectrum.test "
diff -s -BI '^#' $OUTPUT_FILE_SPECTRUM test_spectrum.test || \
echo "set terminal x11 2 persist; plot 'test_spectrum.test' u 1:4 w lp t 'TEST-DATA Spectrum', 'spectrum.dat' u 1:4 w lp t 'New generated data Spectrum'" | gnuplot
echo ""

echo "DDSCS calculation: Now generating test data!"
echo "This should take approx. 20 sec."


$EXE $OPTION_DDSCS
echo "Comparing output to test data:"
echo ""
echo "Checking test_DDSP_integrated_energy_total.test "
diff -s -BI '^#' carbon.ddspi_tot test_DDSP_integrated_energy_total.test || \
echo "set terminal x11 3 persist; plot 'test_DDSP_integrated_energy_total.test' u 1:4 w lp t 'TEST-DATA DDSCS', 'carbon.ddspi_tot' u 1:4 w lp t 'New generated data DDSCS'" | gnuplot
echo ""
echo "Checking test_DDSP_integrated_energy_1s.test"
diff -s -BI '^#' carbon.ddspi_1s  test_DDSP_integrated_energy_1s.test
echo ""
echo "Checking test_DDSP_integrated_energy_2s.test"
diff -s -BI '^#' carbon.ddspi_2s  test_DDSP_integrated_energy_2s.test
echo ""
echo "Checking test_DDSP_integrated_energy_2p.test"
diff -s -BI '^#' carbon.ddspi_2p  test_DDSP_integrated_energy_2p.test
echo ""
echo "Checking test_DDSP_energy_angle_1s.test"
diff -s -BI '^#' carbon.ddsp_1s  test_DDSP_energy_angle_1s.test
echo ""
echo "Checking test_DDSP_energy_angle_2s.test"
diff -s -BI '^#' carbon.ddsp_2s  test_DDSP_energy_angle_2s.test
echo ""
echo "Checking DDSP_energy_angle_2p.dat and test_DDSP_energy_angle_2p.test"
diff -s -BI '^#' carbon.ddsp_2p  test_DDSP_energy_angle_2p.test
echo ""

rm carbon.ixs
rm carbon.ixs2

echo "TEST COMPTON MODULE FINISHED"



