!----------------------------------------------------------------------------
! tindex.f90
!----------------------------------------------------------------------------
! Copyright (C) 2014 Deutsches Elektronen-Synchrotron DESY,
!                    a research centre of the Helmholtz Association.
! Authors: 2014 Otfried Geffert
! License: GPL-3.0-or-later
!
! This file is part of XATOM.
!
! XATOM is free software: you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software
! Foundation, either version 3 of the License, or (at your option) any later
! version.
!
! XATOM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! XATOM. If not, see <https://www.gnu.org/licenses/>.
!----------------------------------------------------------------------------
module tindex

use constants
!integer ,parameter ::long = selected_real_kind(16,99)! was 16,99 before using pg

integer :: position_at(0:64)

contains


recursive subroutine mid_number(lo,hi,order,inlevel)
  integer, intent(in) :: lo,hi
  integer, intent(inout) :: order(:)
  integer, intent(in) :: inlevel
  integer :: mid, shift, level

  level = inlevel
  level = level+1
  shift = 2**level

  mid = lo + (hi-lo)/2
  position_at(level) = position_at(level)+1
  pos = position_at(level)
  order(pos) = mid


  !write(*,*) lo,hi,mid

  if( (mid-1-lo) >= 1 ) then
    !write(*,*) 'calling mid_number left'
    call  mid_number(lo,mid, order(1+shift:), level)
  end if 

  if( (hi-1-mid) >= 1 ) then 
    !write(*,*) 'calling mid_number right'
    call  mid_number(mid,hi, order(1+shift:), level)
  end if

end subroutine

function divide2_index_order(lo,hi) result(order)
 integer, intent(in) :: lo,hi
  integer :: order(lo:hi)
  integer :: level

  position_at(:) = 0 

  ! divide2 order - only first two are determined, lo, hi
  level = -1
  order(lo) = lo
  if( hi .gt. lo ) order(lo+1) = hi
  if( hi .gt. lo+1 ) then
      !write(*,*) 'calling mid_number 0 left'
      call  mid_number(lo,hi,order(lo+2:hi),level)
  end if
end function

function linear_index_order(lo,hi) result(order)
 integer, intent(in) :: lo,hi
  integer :: order(lo:hi)
  integer :: k
  ! linear order - simplest case
  do k=lo,hi
    order(k) = k
  end do
end function

function random_index_order(lo,hi) result(order)
 integer, intent(in) :: lo,hi
  integer :: order(lo:hi)
  logical :: done(lo:hi)
  integer :: k,t,howmany,cnt
  real(long) :: rn
  ! linear order - simplest case
  !do k=lo,hi
  !  order(k) = k
  !end do

  ! random order - only first two are determined, lo, hi
  order(lo) = lo
  if( hi .gt. lo ) order(lo+1) = hi
  if( hi .gt. lo+1 ) then
    done(:) = .false.
    done(lo) = .true.
    done(hi) = .true.
    howmany = hi-lo+1 
    howmany = howmany -2!-2 caused by first two
    t=lo+2
    cnt = 0
    random: do 
      cnt = cnt+1
      call random_number(rn)
      k = int( (rn*real(howmany))+0.5 )
      !write(*,*) 'k=',k
      k = k+ lo+1 !add the offset
      if( k.gt.hi ) cycle !too high k

      if( .not. done(k) ) then !hit a free index
        done(k)  = .true.
        order(t) = k 
        t = t+1
        if( t.gt.hi ) exit random
      end if 
    end do random
    !write(*,*) 'cnt=',cnt
  end if
end function

end module

!program indextest
!  use index
!  integer :: lo, hi, k
!  !integer, allocatable, dimension(:) :: order
!  integer, dimension(1:130) :: order
!  lo = 1
!  hi = 130
!  !allocate(order(hi))
!
!  !order = random_index_order(lo,hi)
!  !write(*,*) order
!
!  !order = linear_index_order(lo,hi)
!  !write(*,*) order
!
!  write(*,*) "calling divide2_index_order()"
!  order = divide2_index_order(lo,hi)
!  write(*,*) order
!end program
