!----------------------------------------------------------------------------
! track_elec_config.f90
!----------------------------------------------------------------------------
! Copyright (C) 2016 Deutsches Elektronen-Synchrotron DESY,
!                    a research centre of the Helmholtz Association.
! Authors: 2016 Koudai Toyota
! License: GPL-3.0-or-later
!
! This file is part of XATOM.
!
! XATOM is free software: you can redistribute it and/or modify it under the
! terms of the GNU General Public License as published by the Free Software
! Foundation, either version 3 of the License, or (at your option) any later
! version.
!
! XATOM is distributed in the hope that it will be useful, but WITHOUT ANY
! WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
! A PARTICULAR PURPOSE. See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with
! XATOM. If not, see <https://www.gnu.org/licenses/>.
!----------------------------------------------------------------------------
module track_elec_config
  use constants
  use read_param
  use process
  implicit none

  type :: track_config
     logical :: track_res_started
     logical :: track_cs_started
  end type track_config
contains
  subroutine open_track_res( P, tec )
    type(Param), intent(in) :: P
    type(track_config), intent(inout) :: tec
    
    tec%track_res_started = .false.
    open( P%unit_track_res, file="track_res", action="write")

  end subroutine open_track_res

  subroutine track_res( P, tag, orbital, t, tec )
    type(Param), intent(in) :: P
    character(len=1), intent(in) :: tag
    integer, intent(in) :: orbital(:,:)
    real(long), intent(in) :: t
    type(track_config), intent(inout) :: tec
    integer :: charge

    if ( tag == "E" .and. .not. tec%track_res_started ) then
       tec%track_res_started = .true.
    endif

    if ( tec%track_res_started ) then
       if ( P%yes_relativity ) then
          charge = P%Z - sum( orbital(:,4) )
          write(P%unit_track_res,'(es10.3,1x,i2,1x,a,1x,a)')t, charge, tag, trim(config_rel_to_string( orbital ))
       else
          charge = P%Z - sum( orbital(:,3) )
          write(P%unit_track_res,'(es10.3,1x,i2,1x,a,1x,a)')t, charge, tag, trim(config_to_string( orbital ))
       endif
    endif
  end subroutine track_res

  subroutine close_track_res( P )
    type(Param), intent(in) :: P

    close( P%unit_track_res )

  end subroutine close_track_res

  subroutine open_track_cs( P, tec )
    type(Param), intent(in) :: P
    type(track_config), intent(inout) :: tec
    
    tec%track_cs_started = .false.
    open( P%unit_track_cs, file="track_cs", action="write")

  end subroutine open_track_cs

  subroutine track_cs( P, tag, orbital, t, tec )
    type(Param), intent(in) :: P
    character(len=1), intent(in) :: tag
    integer, intent(in) :: orbital(:,:)
    real(long), intent(in) :: t
    type(track_config), intent(inout) :: tec
    integer :: charge

    ! photo ionization from neutral state
    if ( .not. tec%track_cs_started) then 
       tec%track_cs_started = .true.
    endif
    if ( P%yes_relativity ) then
       charge = P%Z - sum( orbital(:,4) )
       write(P%unit_track_cs,'(es10.3,1x,i2,1x,a,1x,a)')t, charge, tag, trim(config_rel_to_string( orbital ))
    else
       charge = P%Z - sum( orbital(:,3) )
       write(P%unit_track_cs,'(es10.3,1x,i2,1x,a,1x,a)')t, charge, tag, trim(config_to_string( orbital ))
    endif
  end subroutine track_cs

  subroutine close_track_cs( P )
    type(Param), intent(in) :: P

    close( P%unit_track_cs )

  end subroutine close_track_cs


end module track_elec_config
