///
///  XMDYN - a computer simulation tool for modeling x-ray induced dynamics of matter
///
///  Copyright (C) 2012-2024 Deutsches Elektronen-Synchrotron DESY,
///                          a research centre of the Helmholtz Association.
///  Email: xraypac@mail.desy.de 
/// 
///  This file is part of XMDYN, which is distributed under the terms of the GNU General Public License version 3.
///  See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
///
///  Authors: Zoltan Jurek, Center for Free-Electron Laser Science, DESY
///


#define LSTRLTH 4096
#define MSTRLTH 2048
#define STRLTH 1024
#define SSTRLTH 64
#define COMMENTCARACTER '#'

#define __FUNCTION_NAME__   __FUNCTION__  


#define QUOTE(name) #name
#define STR(macro) QUOTE(macro)


/* __FUNCTION__  seems to work without c99 */


#define XMDYN_ALLOCATE_MEM(x) xmdyn_allocate_mem( x , __FILE__ , __LINE__  ,  __FUNCTION_NAME__ )
#define XMDYN_DEALLOCATE_MEM(x) xmdyn_deallocate_mem( x )

#define XMDYN_GET_MALLOC_INFO(x,s,y) xmdyn_get_malloc_info( x , s , y , __FILE__ , __FUNCTION_NAME__ )

extern int xmdyn_get_meminfo( char *comm ) ;
extern int xmdyn_print_malloc_info( char *misc , int step , int substep , char *fname , const char *func , int xline ) ;
extern int xmdyn_get_malloc_info( char *info , int len_info , char *misc , char *fname , const char *func ) ;
extern void* my_malloc( int a_size , void *a_idstr ) ;
extern int error_exit(char *message,double num,int returnvalue);

extern void* xmdyn_allocate_mem(int nb, char *a_file , int a_line , const char *func ) ;
extern void xmdyn_deallocate_mem( void *p ) ;
int xmdyn_list_allocated_mem( char *list ) ;
extern void xmdyn_print_mem_usage( void ) ;

extern int xmdyn_remove_element_x(void **array, int sizeOfArray, int indexToRemove , char *xtype ) ;



#define ANSI_COLOR_RED     "\x1b[31m"
#define ANSI_COLOR_GREEN   "\x1b[32m"
#define ANSI_COLOR_YELLOW  "\x1b[33m"
#define ANSI_COLOR_BLUE    "\x1b[34m"
#define ANSI_COLOR_MAGENTA "\x1b[35m"
#define ANSI_COLOR_CYAN    "\x1b[36m"
#define ANSI_COLOR_RESET   "\x1b[0m"

#define TERM_COLOR_red     "\e[0;31m"
#define TERM_COLOR_RED     "\e[1;31m"
#define TERM_COLOR_RESET   "\e[0m"

#define LOG_RED(P,X) fprintf(P,"%s %s %s",TERM_COLOR_red,X,TERM_COLOR_RESET)


