///
///  XMDYN - a computer simulation tool for modeling x-ray induced dynamics of matter
///
///  Copyright (C) 2012-2024 Deutsches Elektronen-Synchrotron DESY,
///                          a research centre of the Helmholtz Association.
///  Email: xraypac@mail.desy.de 
/// 
///  This file is part of XMDYN, which is distributed under the terms of the GNU General Public License version 3.
///  See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
///
///  Authors: Zoltan Jurek, Center for Free-Electron Laser Science, DESY
///


#define maxZvalue 100

//###################################################

// This is a small `dictionary`
enum g_option_string
{
    // general
    YES ,
    NO , 
    OK ,
    ERROR ,
    NONE,
    DUMMY,

    // impact ionization
    FORWARD ,
    ISOTROP ,
    RESTRICTED ,
    FULL,

    // DATABASE
    EADL_TAB ,
    XATOM_TAB ,
    XATOM_ONTHEFLY ,

    ATOMDATA_USE_TAB,
    ATOMDATA_USE_TREE ,

    FROM_FILE ,
    CALC ,
    LOG,
    XFILE,

    SCREEN ,
    NOSCREEN,
    LOGFILE ,
    SCREEN_AND_LOGFILE ,

    DATABASE,
    DATADIR,

    RUNNING,
    DONE,
    PAUSE,
    PAUSE100,
    INTERRUPTED,
    EXIT_AT_NEXT_SNP,
    EXIT_AT_NEXT_STEP,
    EXIT_FORCE_SAVE_AT_NEXT_STEP ,

    DECOUPLED ,
    COUPLED ,

    COULOMB,
    COULOMB_ATOMS,

    SIMPLE_01,
    CH_MGR_TOP_BOTTOM,
    CH_MGR_BOTTOM_TOP,
    CH_MGR_RANDOM_SHELL,
    CH_MGR_AR_3SP_TO_3SP,

    DEF_SAMPLE,
    EXT_SAMPLE,
    SNP1_SAMPLE,
    SNP2_SAMPLE,

    FLUSH,
    NOFLUSH,

    STORE,
    RESTORE,

    DEFAULT,

    X
} ;


//###################################################


struct  g_struct_inputs {


        char inputfile[STRLTH];
	char runnumber[STRLTH];

	int N;
	int Z;
	int E;
	int Eextra;

	double DT;
	int nstep;
	int nstepdummy1;

	double Ephoton;
	double Nphoton;
	double diameter;
	double flu_max ;
	double T_1;
	double T_2;
	double T0;
	double xrsd_delay;
	double xrsd_ratio_2toT;

	int tsfactor;

	char beamtype[LSTRLTH];

	int foto;
	char progress_target[STRLTH];
	int progress_dump;
	char paramsout[STRLTH]; // TODO: remove it.

        char snp_input[LSTRLTH] ;
        int *snp ;
        int  snp_dim ;

        char snp_full_input[LSTRLTH] ;
        int *snp_full ;
        int  snp_full_dim ;
        double chkp_dt ;

        char snp_E_input[LSTRLTH] ;
        int *snp_E ;
        int  snp_E_dim ;

	double lattice_const;
	double lattice_devi;

	double potencrb;
	double potencrb2;
	double rstarte;

	char timestepOK[SSTRLTH];
	char photoionOK[SSTRLTH];
	char relaxflOK[SSTRLTH];
	char relaxauOK[SSTRLTH];
	char scatteringOK[SSTRLTH];
	char elscattOK[SSTRLTH];
	char elimpionOK[SSTRLTH];
	char anizotropy[STRLTH];
	char impdistcountOK[SSTRLTH];
	char energycalc[SSTRLTH];

	char bindE[STRLTH];

	int qphoto;
	int qauger;
	int qsecphoto;
	int qsecauger;
	int qsecsec;
	int qfion;
	int qatom;
	int dqatomphotoion;
	int dqatomauger;
	int dqatomsec;
	int dqatomfion;

	int randseed;

	char r1input[STRLTH];
	char v1input[STRLTH];
	char etypeinput[STRLTH];
	char qinput[STRLTH];
	char minput[STRLTH];
	char rootdir[STRLTH];
	char datadir[STRLTH];
	char dbasedir[STRLTH];

	double sprob;
	double pprob;

	double phiin;
	double thetain;

        char coulomb_calc_method[STRLTH] ;


	char int_method[STRLTH];
	char tselectrons;
	char fixatomsOK;

	int maxscattertry;
	double maximpdist;
	char impioncriterium[STRLTH];
	char inel_dsdW;
        char impion_model[STRLTH] ;
        double impion_scale ;

	int deloctest_nn;
	char deloctest_comm;
	int deloctest_ninterp;

	char avaltype[STRLTH];
	int centeratom;

	int onelastsnp ;
	int fotomomentsdim;
	char fotofromfile;
	char printmessage;
	int daemon;
	int startindex;
  	int started;
	int initfoto;
	char kiirasprec[STRLTH];
	char savev3;

	int print_buff_store_size;
	
	char cachedatadir[STRLTH];
	char si_type[STRLTH];
	int  si_model ;
        char si_dsig_dome ; 
        int  si_restricted ; 

	char   instrecOK[SSTRLTH];
	double maxrecdist;
	char   ireccond;
	int    irecDT;
	double irecperiods;
        char   irec_method[STRLTH] ;
        char   irec_switchon[STRLTH] ;
        char irec_misc[LSTRLTH] ;

	char no_realspace_dyn[STRLTH];

	double maxcachetime;


	char extraopts[LSTRLTH];
	int step;
	double subDT;

        char misc_opt [LSTRLTH] ;

        enum g_option_string one_photoion ;
        enum g_option_string  quantumproc_coupled  ;

        char xatom_energies[STRLTH] ;


  char nots;
  char nosec;
  char noall;
  char fixts;

        char  atomic_data_source_string[STRLTH] ;
        char  atomic_data_eval_string[STRLTH]   ;
        char  xatom_onthefly_savedir_string[STRLTH] ;
        char  xatom_version[LSTRLTH] ;
        char  xatom_commandline_opt[LSTRLTH] ; 

        int   dbase_xatom_version ;
        char  dbase_types[LSTRLTH] ;

        enum g_option_string  atomic_data_source ;
        enum g_option_string  atomic_data_eval   ;
        enum g_option_string  xatom_onthefly_savedir ;

        enum g_option_string  xatom_silent ;
        enum g_option_string  xatom_load_silent ;
        enum g_option_string  xatom_calc_silent ;

        enum g_option_string  classdyn_partsel ;


        char format_loadsave[STRLTH] ;
        char txtbin_loadsave[STRLTH] ;

        char sample_dir[LSTRLTH] ;

        char input_version[STRLTH] ;
        char xmdyn_version[STRLTH] ;

        char shortcut_comm[LSTRLTH] ;
        int  shortcut_ref ;  // if 0 <= : on, with this reference ; -1: off

        enum g_option_string event_dest ;


} ;



// #############################################
// #############################################
// #############################################



struct g_struct_xatom_data 
{

    int Z ;
    char *str;
    long long int id_present ;

    int  max_orbit ;
    int  NE ;
    int *NE_orbit ;

    double  E ;

    double *r ;

    double *B ;
    double *K ;
    double *V ;

    double *ph_E ;
    double *ph_sigma ;

    int num_fl ;
    int (*fl_orbit)[2] ;
    double *fl_E ;
    double *fl_w ;

    int num_au ;
    int (*au_orbit)[3] ;
    double *au_E ;
    double *au_w ;
    
    int num_fij ;
    double *fij ;               /* osc. strength */

    int num_sample ;
    double *q_sample ;           /* sampleing in Q-space */
    double *f0 ;                 /* form factor values */
    double fprime ;
    double fdprime ;
    double fdprime_pcs ;
    double *ixs ;

    void *parent ;

    double B_rec ;
    double Ecrit_cl2q ;
    double *dE_recomb ;

} ;

// #############################################

struct SHELL_STRUCT{
  int e_from;
  int e_to;
  int NE;                                                  // changes 
} ;

// -----------------------------------

struct ATOMS_STRUCT{
  int *Z;
  int maxshellnum;                                         // changes 
  int e_from;
  int e_to;
  struct SHELL_STRUCT *shell;

  int NE;                                                  // changes 

  struct g_struct_xatom_data *atomic_econfig ;             // changes 
} ;

// #############################################

struct SHELLDATA_STRUCT{
  double NE;
} ;

// -----------------------------------

struct ATOMDATA_STRUCT {
  int maxshellnum;
  struct SHELLDATA_STRUCT *shell;
};


// #############################################
// #############################################
// #############################################

struct ENERGY_VALUES_STRUCT{
  double Eintotal;
  double Eintotal_th;
  double Einph;
  double Einfl;
  double Einau;
  double Einsi;
  double Einrec;
  double Einfi;
  double Einchmig;
  double Einitial;
  double totalpotE;
  double Elost;
  double E0 ;
  double Work ;
  double KE ;
  double Ebound ;
  double Efl_photon;
};


// #############################################
// #############################################
// #############################################

struct g_struct_system_data {

    double (*r1)[3]  ;                                     // changes 
    double (*r2)[3]  ;                                     // changes 
    double (*v1)[3]  ;                                     // changes 
    double (*v2)[3]  ;                                     // changes 
    double (*prev_r2)[3]  ;                                // changes 
    double *m  ;
    double *qd  ;                                          // changes 
    int *qi  ;                                             // changes 
    int *qi_corr  ;                                        // changes 
    char *ptype  ;                                         // changes 
    int *Z  ;
    long long int *uid ;                                   // changes 

    int *motherion ;                                       // changes 
} ;

// #############################################
// #############################################
// #############################################


extern struct  g_struct_inputs  g_inputs;

extern struct ATOMS_STRUCT *g_atomic_data;

extern struct ATOMDATA_STRUCT *ATOMDATA;

extern struct ENERGY_VALUES_STRUCT g_ENERGY_VALUES;

extern struct g_struct_system_data       g_system      ;


