///
///  XMDYN - a computer simulation tool for modeling x-ray induced dynamics of matter
///
///  Copyright (C) 2012-2024 Deutsches Elektronen-Synchrotron DESY,
///                          a research centre of the Helmholtz Association.
///  Email: xraypac@mail.desy.de 
/// 
///  This file is part of XMDYN, which is distributed under the terms of the GNU General Public License version 3.
///  See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
///
///  Authors: Zoltan Jurek, Center for Free-Electron Laser Science, DESY
///


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include <stdarg.h>

#include "common.h"

#include "globalconsts.h"
#include "filetests.h"
#include "xmdyn_io.h"

#include "tests.h"



char DEBUGFILE[STRLTH]="./debugfile.txt";


//############################## FUNCTION #################################

int write_to_debugfile(int i,double d,char ch,char type,char s){
  static FILE *fp;

  switch (s){
   case 'o':
    fp=fopen(DEBUGFILE,"w");
    printf("%s debugfile is opened.\n",DEBUGFILE);
    break;
   case 'c':
    fclose(fp);
    printf("%s debugfile is closed.\n  Press enter to continue. ",DEBUGFILE);
    getchar();
    break;
   default:
    switch (type) {
     case 'i':
      fprintf(fp,"%d ",i);
      break;
     case 'd':
      fprintf(fp,"%le ",d);
      break;
     case 'c':
      fprintf(fp,"%c ",ch);
      break;
    }
  }

  return 0 ;
}



//############################## FUNCTION #################################

void testsuite_drawrandom_general( double (*actdistr)(double,double*), double *params, int n , char *fname , int drawid ) {
  FILE *fp;
  int ii ;
  double xx ; 

  fp = fopen( fname , "w" ) ;
  for ( ii=0 ; ii<n ; ii++ ) {
    xx = drawrandomdistr( actdistr , params , drawid , 0 );
    fprintf( fp , "%le " , xx ) ;
  }
  fclose( fp ) ;
  printf("File: %s\n", fname );
}



#include "mc/quantumproc.h"
#include "mc/pheff.h"
#include "secel.h"


//############################## FUNCTION #################################

void testsuite_drawrandom_test( void ) {
  double *params , params_pheff_phi[2] , params_pheff_th[2] , 
         params_secel_th[4] ,params_secel_E[8] ;
  int n = 10000 , drawid ;
  double (*distr)(double,double*) ;

  //  #  #  #  #  #  #  #  #  #  #
  
  params_pheff_phi[1] = 0 ;
  
  params_pheff_th[1] = 1./6.8 ;

  params_secel_th[1] = 0.1 ;

  params_secel_E[1] = 1  ;
  params_secel_E[2] = 10 ;
  params_secel_E[3] = 10 ;
  params_secel_E[4] = 100 ;
  params_secel_E[5] = 1 ;
  params_secel_E[6] = 1 ;

  //  #  #  #  #  #  #  #  #  #  #
 
#if 1
  params = params_pheff_th ;
  distr = pheff_xth_distr ;
  drawid = 2 ;
#endif
  //  #  #  #  #  #  #  #  #  #  #

  testsuite_drawrandom_general( distr , params , n , "./xxxxx.txt" , drawid );

  printf("Comment 'testsuite_drawrandom_test' to get gid of this test.\n") ;
  exit( 0 ) ;

}


//############################## FUNCTION #################################

void testsuite_inel_cross_section( char *a_impion_model ) 
{

    int atomid  ; //numE
    double  xE  ;
    int ii  , kk;
    FILE *fp ;
    char testfile[LSTRLTH]  , Zstr[STRLTH]="" , numstr[STRLTH]="" ;
    int E0_tab[4] = {1  ,  100,  1000,  10000 } , 
        dE_tab[4] = {1  ,  10 ,  100 ,  1000  } ,
        nE_tab[4] = {100,  90 ,  90  ,  10    } ,
        tab_dim = 4 ;

    atomid = 0 ;

    strcpy( testfile , "./xxxxx.dat" ) ;
    sprintf( Zstr , "%02d" , g_system.Z[atomid] ) ;
    sprintf( numstr , "%05d" , atomid ) ;
    strcpy( testfile , "" ) ;
    strcat(strcat(strcat(
           strcat(strcat(strcat(strcat( testfile , "impion-crossect-" ) , 
                  a_impion_model ) , "-Z") , Zstr ) ,
           "-atom" ) , numstr ) , ".dat" ) ;
printf("SSS %s\n" , testfile);
    printf("Atom pos: %le %le %le\n" ,  g_system.r1 [atomid][0] ,  g_system.r1 [atomid][1] ,  g_system.r1 [atomid][2] ) ;


    fp = fopen ( testfile , "w" ) ; 
    for ( kk=0; kk<tab_dim ; kk++ )
    {
        for ( ii = 0 ; ii < nE_tab[kk] ; ii++ ) 
        {
            xE = E0_tab[kk]* g_phys_math_const.e1  + dE_tab[kk] * ii* g_phys_math_const.e1  ;
            fprintf( fp , "%le %le\n" , xE , eimpion_sigma_total_as_KE( atomid , xE )     ) ;
        }
    }

    fclose( fp ) ;

}




