title: Command-Line Arguments
author: Ludger Inhester
###Command-Line Arguments

| Parameter | Value | Meaning |
| --------- | ----- | ------- |
| occ, occupation | string of integers (0,1, or 2) | occupation numbers (0,1, or 2) for each molecular orbital  |
| multp | integer  | spin multiplicity  |
| basissetoverlaplim | float  | upper threshold at which basis functions with larger overlap are projected out  |
| qmmm | boolean  |  option for qmmm calculation: if true, skips calculating repulsion between point charges |
| basis_string | comma-separated list of numerical atomic filenames   | specifying for each atom a numerical atomic orbital (this is alternative to the specification via the geometry)  |
| angstrom, angs | boolean  | specifies that units in molecular geometry are in angstrom  |
| gaussian_basis, gto | filename(s) with basis set specification or basis set name  | GTO basis set specification. Either basis set if specified, or a comma-separated list is given that specifies for each atom an individual basis set|
| gaussian_basis2, gto2 | filename(s) with basis set specification or basis set name  | GTO basis set specification. only relevant for project_basis_gto  |
|  | | |
| scattering | boolean  | triggers elastic scattering calculation |
| compton | boolean  | triggers Compton scattering (Waller-Hartree-approximation) calculation  |
| ionization_dynamics | boolean  | ?  |
| auger | boolean   | triggers Auger-Meitner decay calculation  |
| fluorescence, fluo | boolean   | triggers calculation of fluorescence rates  |
| absorption | boolean  | triggers calculation of oscillator strengths  |
| pcs | boolean   | triggers calculation of core-level photoionization cross sections |
| final_hole | MO_indices  | specifies the molecular orbitals from where absorption / photoionization should be considered   |
| shakeup, shake_up, shake | boolean  |  triggers shake-up analysis (has to be combined with option -pcs) |
| process_ci | boolean  | triggers calculation of absorption, photoionization, fluorescence, and Auger-Meitner decay calculations based on configuration interaction |
| process_cis | boolean  | triggers calculation of photoionization, fluorescence, and Auger-Meitner decay calculations based on configuration interaction with additional excitations |
| process_ci_relaxed | boolean  | triggers calculation of absorption, photoionization, fluorescence, and Auger-Meitner decay calculations based on configuration interaction and using a seperate set of final state orbitals read from the filename wf_relaxed.dat  |
| | | |
| rmax | float  | the maximum radius for the atomic grids, default 50 a.u.  |
| lmax | integer | maximum angular momentum for each atomic grid, default 10  |
| | | |
| direct | boolena  | triggers on-the-fly calculation of two-electron integrals  |
| x_type| string  | exchange functional  |
| c_type| string  | correlation functional |
| lmax_mp | integer  | maximum angular momebtnum at which the multipole expansion around each atom center at which the Coulomb interaction is truncated |
| scf_max_iter | integer  | maximum number of SCF iterations, default 100 |
| scf_energy_conv | float | SCF energy conversion criterion, default 1e-6 a.u. |
| scf_density_conv | float | SCF density conversion criterion, default 1e-6 a.u.  |
| scf_mixing | float | mixing coefficient for mixing new iteration with previous iteration  |
| diis | boolean   | triggers direct inversion of interative subspace during SCF, default True  |
| cphfdiis | boolkean  | triggers direct inversion of interative subspace during CPHF, default True  |
| levelshift | boolean  | triggers levelshift during SCF  |
| switchtodiis |   |   |
| switchtorstrct |   |   |
| switchlevelshiftoff |   |   |
| rstrct | boolean  | triggers restriction of MO change during SCF  |
| mom | boolean  | switches on the Maximum-Overlap-Method  |
| momtype | 1, 2, or 3  | choose MOM variant, default 3  |
| openshell | boolean | specifies the the electronic structure calculation should be treated as openshell configuration  |
| rohf_variant | 1, 2, or 3  | specifies ROHF variant: 1=Roothaan (default), 2=Davidson (e.g. as in molpro), 3=Guest&Saunders  |
| soscf | boolean  | triggers second-order SCF, default false |
| soscf_start |   |   |
| f_occ, frac_occ, frac_occupation | comma-separated array of floats  | specify fractional occupation numbers for each orbital  |
| freeze | MO indices  | list of orbital indices to be frozen during SCF iterations |
| sym | boolean  | triggers symmetry |
| state_irrep | integer| index of the irreducible representation of the electronic state |
| irrep_mo | comma-separated list of integers  | indicates for each orbital the index of its irreducible representation (to be used with sym=True)   |
| localize | comma-separated list of orbital indices  | triggers orbital localization  |
| fix_nuc | comma-separated list of orbital indices, colon, comma-separated list of atom indices  | re-groups orbitals to be dominantly localted on a specified atom  |
| active_space_mo | comma-separated list of orbital indices  | specifies active orbital space  |
| average_occ_mo | colon-separated groups of comma-separated list of orbital indices  | specifies that fractional occupation numbers are used that average the occupation numbers for groups of orbitals  |
| levelshiftdynamic | float  | triggers dynamic levelshifting, value is initial level shift value |
| diis_nvec | integer  | size of DIIS subspace  |
| mo_fid_ini | boolean  | triggers calculation of fidelity to initial guess orbitals  |
| population| boolean  | triggers population analysis  |
| stab, stability | boolean  | triggers stability analysis  |
| unrestricted | boolean | triggers unrestricted SCF calculation  |
| hf | boolean  | triggers Hartree-Fock calculation. default False   |
| cis | boolean | triggers CIS excited state calculation. default False   |
| state | integer | state index  |
| nstates | integer | number of states to calculate   |
| dump_ints | boolean  | triggers the dump of all transformed two-electron integrals  |
| Nvoxel | integer  | total number of voxels Nx*Ny*Nz  for cube file generation, default 1000000 |
| density_cube | boolean  | triggers generation of cube file with electron density |
| spin_density_cube | boolean | triggers generation of cube file with spin density  |
| mo_cube | list of MO indices | triggers generation of cube file with orbitals  |
| t_ev| float  | triggers finite-temperature SCF, value is temperature in eV  |
| t_k, temp_k, temperature_k | float| triggers finite-temperature SCF, value is temperature in K  |
| ncore, n_core | integer  | overrides the number of core-level orbitals to be considered  |
| valgs_occ | boolean  | triggers the calculation of electronic structure using Koopmann's theorem, i.e., orbitals are calculated for the closed valence shell and orbital energies are substracted|
| gradient | boolean | triggers calculation of gradients  |
| orbital_energy_gradient | boolean  | triggers calculation of orbital-energy gradients  |
| | | |
| eps_auger | float  | threshold for the calculation of Auger decay rates  |
| eps_teint | float  | theshold for the calculation of two-electron integrals, default 1e-9  |
| eps_overlap, eps |float   | truncation for evaluation of matrix element integrals (only relevant for numerical atomic orbitals), default 1e-6  |
| eps_single |float   | truncation for evaluation of matrix element integrals diagonal elements (only relevant for numerical atomic orbitals), default 1e-6  |
| eps_00, eps0 | float  | truncation of the Hartree potential. A ratio threshold for for Q_A / r (only relevant for numerical atomic orbitals), default 1e-2  |
| eps_lm, eps1 | float  |  truncation of the Hartree potential. A ratio threshood for  d_lm / d_00 (only relevant for numerical atomic orbitals), default 5e-5 |
| eps_r, eps2 | float  |  truncation of the Hartree potential. A ratio threshold for d_lm(r) / d_lm (only relevant for numerical atomic orbitals), default 1e-5 |
| r_cutoff | float  |  strict cut-off radius for the Hartree potenital calculation, default 0 (no truncation) |
| | | |
| o | filename  | specifies output filename  |
| v, verbose | boolean  | triggers verbose output  |
| print_level | integer  | sets print output level  |
| nmo_print | integer  | number of molecular orbitals to print  |
| rini | boolean  | reads guess orbitals from file wf.dat  |
| coreini | boolean  | triggers initial guess from bare nuclear electrostatic potential   |
| sini | boolean  | triggers saving molecular orbitals into file wf.dat  |
| rinish | boolean  | triggers reading numerical atomic orbitals for the guess orbitals from file basis.dat |
| molden | filename  | triggers generating a molden output file containing information on the molecular orbitals |
