title: Example
author: Ludger Inhester
## Examples

### Water with nao, HFS, and Auger-Meitner rates

We generate some numerical atomic orbitals
```shell
xatom -s O -nao -latter off -file_nao O.nao -N 100
xatom -s H -nao -latter off -file_nao H.nao -N 100
```

These commands invoke an atomic electronic structure calculation for atomic oxygen and hydrogen.
The options "-nao" and "-file_nao O.nao" are used to store the radial orbital function in the file O.nao.
A total number of 100 radial grid points is used ("-N 100") and we do not emply the Latter correction.
For more details, please consider the xatom manual.


The input file specifies the geometry and the numberical atomic orbital basis set
```
 O         0.000000000000    -0.125549913844     0.000000000000    2s1p   O.nao
 H        -1.423436754553     0.996284121013     0.000000000000    1s     H.nao
 H         1.423436754553     0.996284121013     0.000000000000    1s     H.nao
```

The command
```shell
xmolecule input.in -o output.out
```
calculates neutral ground state water employing the Hartree-Fock-Slater method. The output of the calculation is printed in the file output.out.
If "-o output.out" is omitted, calculation results are printed (standard out).

We generate now numerical atomic orbitals for core ionized oxygen:
```shell
xatom -s O -nao -latter off -file_nao O1s.nao -N 100 -hole 1s1 -transition auger -file_auger O1s.auger
```
The additional option "-hole 1s1" invokes the computation of a K hole state.
By using the option "-transition auger -file_auger O1s.auger" additional information of Auger radial matrix elements are printed into the file O1s.auger.
These values are used by xmolecule to compute Auger transitions. 

We employ those in the input file and add options for the molecular electronic configuration and trigger Auger-Meitner decay calculations:
```
 O         0.000000000000    -0.125549913844     0.000000000000    2s1p   O1s.nao
 H        -1.423436754553     0.996284121013     0.000000000000    1s     H.nao
 H         1.423436754553     0.996284121013     0.000000000000    1s     H.nao
auger=yes
occ=12222
```

The command
```shell
xmolecule input.in -o output.out
```
calculates now the electronic structure for the core-ionized water molecule and calculated Auger-Meitner decay rates.


### Water with gaussian-type orbital basis set, Hartree-Fock, and Auger-Meitner rates

The input file specifies the geometry and the numerical atomic orbital basis set
```
 O         0.000000000000    -0.125549913844     0.000000000000  
 H        -1.423436754553     0.996284121013     0.000000000000  
 H         1.423436754553     0.996284121013     0.000000000000  
 gto=6-31G
 HF=yes
```

```shell
xmolecule input.in -o output.out
```
makes a Hartree-Fock calculation on water.

With
```
xatom -silent -latter off -s O -hole 1s1 -N  100 -nao -file_nao O.nao \
 -transition auger -file_auger O.auger
```
we provide the Auger-Meitner Matrix elements and numeric atomic orbitals for the oxygen atom.

Running
```shell
xmolecule input.in -occ 12222 -auger
```
calculates the core ionized state of water using restricted open-shell Hartree Fock and calculates Auger-Meitner processes (energies are based on orbital energy differences)



Running
```shell
xmolecule input.in -occ 12222 -auger -fluorescence -process_CIS -nstates 100 -o output.out
```
computes Auger-Meitner and fluorescence decays employing configuration interaction based on a three-valence-hole-one-particle scheme.



### K-shell Photoionization of Carbon Monoxide
```shell
xatom -N 50 -rmax 30. -latter off -s C -nao -file_nao C.nao \
 -transition transitiondipole -file_tdipole C.tdipole -PE_start 10 -PE_end 1000 -PE_dE 2.0
xatom -N 50 -rmax 30. -latter off -s O -nao -file_nao O.nao \
 -transition transitiondipole -file_tdipole O.tdipole -PE_start 10 -PE_end 1000 -PE_dE 2.0
```
In the atomic electronic structure calculations, we now provide the additional command line parameters "-file_tdipole O.tdipole -PE_start 10 -PE_end 1000 -PE_dE 2.0".
They invoke the computation of transition diplole matrix elements for the atom employing an energy grid for the electronic continuum starting at 10 eV up to 1000 eV with 2.0 eV steps. These values have to be adapted to cover the expected molecular photoelectron energy.

input file:
```
C 0.0 0.0 0.0   2s1p C.nao
O 0.0 0.0 1.128 2s1p O.nao
unit=angstrom
gto=6-31G
```

The command
```
xmolecule input.in -HF -pcs -PE 1000 -o output.out
```
computes photoionization cross sections at photonenergy 1000eV based on orbital energies.


The command

```
xmolecule input.in -HF -pcs -PE 1000 -process_CIS
```

computes K-shell photoionization cross sections at photonenergy 1000eV based on configuration interaction employing a one-core-hole--one-valence-hole--one-virtual-particle scheme.

The command

```
xmolecule input.in -HF -CIS -gradient -state 2 -nstates 10
```

computes the second excited states employing configuration-interaction singles,
its gradients and non-adiabatic couplings to the other 10 states


The command

```
xmolecule input.in -HF -CIS -state 2 -absorption 
```

computes the absorption of the second excited CIS states employing configuration-interaction singles.




