title: File formats
author: Ludger Inhester
## File formats

### Numerical atomic orbitals
The file ATOMTYPE.nao contains numerical atomic orbitals.
The file contains a header with the first two lines being

```
          NGRID   LGRID        RMAX
          NUM_ATOM_ORB
```
- NGRID is the number of radial grid points

- LGRID is a grid parameters, that is usually 1.0

- RMAX is the maximum radial grid position

- NUM_ATOM_ORB is the number of atomic orbitals in this file

The header is followed by NUM_ATOM_ORB lines.
Each line contains three integers describing the principal, the angular quantum number, and the occupation number for this atomic subshell, specifying the
electronic configuration for which these radial wave function were calculated.

```
np           l           occupation
```

Next, a line follows containing two integers,

```
LMAX NEV
```

describing how many angular momentum components and radial functions per angular component are
given in this file. The values LMAX and NEV are also explained above.
This is followed by the values of the atomic orbitals at each grid point in the following scheme:

- outer loop l=1..lmax
- middle loop j=1..nev
- inner loop k=1..NGRID

The data is read by the following loop:
```fortran
   do l = 0, lmax
     do j = 1, nev
       read (IO_BASIS, *) A%u_AO(:, k)
     end do
   end do
```



### Auger-Meitner radial integrals
The file ATOMTYPE.auger contains Auger-Meitner radial integrals of type
$$R^{(\lambda)}(a,b,c,k) =  \int_0^\infty dr \int_0^\infty dr' \phi_a(r) \phi_b(r') \frac{r_<^\lambda}{r_>^{\lambda+1}}  \phi_c(r) \phi_k(r'), $$
where \(\phi_a(r)\) and \(\phi_b(r)\) are the radial atomic wave functions of two valence orbitals,
\(\phi_c(r)\) is the radial atomic wave functions of a core orbital, and \(\phi_k(r)\) is a radial atomic wave function of a continuum orbital.

Each line in the file contains one radial integrals according to the following scheme:

 \(\lambda\)  \(n_a\) \(l_a\)  \(n_b\) \(l_b\)  \(n_c\) \(l_c\)  \(l_k\) \(R^{(\lambda)}(a,b,c,k)\)  \(E_k\) #   \(\chi_\text{Auger}\)

- \(n_a\) principal quantum number of valence orbital \(a\) (integer)
- \(l_a\) angular momentum of valence orbital \(a\) (integer)
- \(n_b\) principal quantum number of valence orbital \(b\) (integer)
- \(l_b\) angular momentum of valence orbital \(b\) (integer)
- \(n_c\) principal quantum number of core orbital \(c\) (integer)
- \(l_c\) angular momentum of core orbital \(c\) (integer)
- \(l_k\) angular momentum of continuum \(k\) (integer)
- \(E_k\) energy of the atomic continuum, i.e., energy of the atomic Auger-Meitner electron (float)
- \(\chi_\text{Auger}\) (optional) scattering phase of the atomic continuum wave function (float)

### Transition dipoles
The file ATOMTYPE.tdipole contains transition-dipole integrals of type
$$ t(a,k) =  \int_0^\infty dr \phi_a(r) r  \phi_k(r), $$
where \(\phi_a(r)\) is the wave function for a bound orbital with quantum numbers \(n_a,l_a\), and
\(\phi_k(r)\) is a wave function for an unbound continuum orbital with radial momentum \(k\) and angular momentum \(l_k\).

The first line specifies the number of triples combining different values of quantum numbers \(n_a, l_a, l_k\).
The next line containts for each triple the quantum numbers (all integers), i.e.,

\(n_{a1}\) \(l_{a1}\) \(l_{k1}\) \(n_{a2}\) \(l_{a2}\) \(l_{k2}\) ... \(n_{aN}\) \(l_{aN}\) \(l_{kN}\)

Next, there is a line for each considered photoelectron energy with the values of the respective transition dipole matrix elements

\(E\) \(t(a1,k1)\) \(t(a2,k2)\) ... \(t(aN,kN)\)

where \(E\) is the continuum energy in eV.


### Format of Gaussian basis set specification

The gaussian basis set data is specified in the same format as for the Gaussian program.
Basis set specifications can be downloaded from the [Basis Set Exhange database](https://www.basissetexchange.org/).

