title: Usage
author: Ludger Inhester
## Usage of xmolecule

Remember to set the required environment variables (see [Installation](installation.html)).

xmolecule is called by providing an input file:
``xmolecule input_file``
Any parameters can be set using command line arguments as "-arg1=value" or you can alternatively add the options in the input file with a line "arg1=value"

### Specification of the molecular geometry

The input file specifies the molecular geometry. This is done in xyz format.

**Atoms**:
Specify each atom in a line:

```
 AtomType1  x1 y1 z1
 AtomType2  x2 y2 z2
 ...
```

where AtomType is the label of the atomic element, e.g., "H","He","Li","B","C","N","O","F", ...

x, y, z are the cartesian positions (by default in atomic units)

If you use a numerical atomic orbitals, you can specify the specific radial basis function.

```
AtomType  x  y  z  basisString  basisFile
```

basisString is a string like '3s2p', specifying the number of radial functions per angular momentum l.

**Point charges**:
To incorporate point charges to the calculation add lines with atom type "X"

```
X  x  y  z  charge
```

where charge is the charge value (float) of the point charge.

**Gaussian charge distribution**: To incorporate spatially spreaded charge distribution to the calculation, add lines with an atom type "XG"

```
 XG  x  y  z  charge  zeta
```

where charge is the integrated charge value (float), and
zeta (float) is the exponent of the Gaussian.

### How to generate numerical atomic orbitals

The radial values for numberical atomic orbitals are saved in a .nao file format.
Numerical atomic orbitals are generated via xatom

```
xatom -s ATOMTYPE -conf ATOMIC_CONFIGURATION -silent -nev NEV \
-lmax LMAX -latter off -rm RMTHRESHOLD -N NGRID -rmax RMAX \
-nao yes -file_nao ATOMTYPE.nao \
-transition auger -file_auger ATOMTYPE.auger  \
-transition transitiondipole -file_tdipole ATOMTYPE.tdipole \
-PE_start PESTART -PE_end PEEND -dE PESTEP
```

**Parameters:**

- ATOMIC_CONFIGURATION

  specifies the atomic configuration for which the numerical atomic orbital is built. Example: 1s2_2s2_2p2 for neutral carbon.
  
- NEV

  specifies the number of radial functions for each angular momentum.
  Example: NEV=2 for a minimal basis set for carbon
  
- LMAX

  specifies the maximum angular momentum
  Example: lmax=1 for a minimal basis set for carbon

- RMTHRESHOLD:

  The bound radial wavefunction should be smaller than this at the maximum radial grid point
  Example: 1e-6

- NGRID:

  Numer of radial grid points.
  Example: 30

- RMAX:

  Maximum radial grid point in a.u.
  Example: 50.0


- PESTART:

  Lowest considered photoelectron energy for photoionization in eV.
  Example: 1000

- PEEND:

  Highest considered photoelectron energy for photoionization in eV.
  Example: 5000

- PESTEP:

  Step with which to scan the photoelectron energy in eV.
  Example: 1.0


