#!/bin/bash

source run_examples.source
source 0001202.convergence_via_intermediate.source
rev_angles=$(get_last_num "$(${xmol_angles} -xatom_path ${xatom_path} | grep 'Rev.')")
script_dir=/home/michael/XMOLECULE/examples/CO/scf_failure_analysis
calc_dir=calculation/convergence2

cd $calc_dir

setup_file=$(pwd)/maketransitions_setup.sh
source $setup_file
cp $ref_dir/input.in .
input_file=$(pwd)/input.in
echo "setup_file=$setup_file"
echo "input_file=$input_file"


#                           0       1       2       3       4       5       6       7       8       9     
            occ_list=(1001002 1001002 1001002 0001202 0001202 0001202 1001102 1001102 1000202 1000202 )
      parentocc_list=(1011002 2001002 1001102 0011202 1001202 0001212 1002102 1101102 1001202 1100202 )
parentocc_final_list=(1011002 1011002 1011002 0011202 0011202 0011202 1002102 1002102 1001202 1001202 )

# redo inconsistent case 0001212 to 0001202
# #+BEGIN_EXAMPLE
#    -4,7,10 - yes no no 0.0000 0.0000 
#    +4,7,10 - yes no yes 0.9474 1.0000 
# #+END_EXAMPLE


echo "### Get output files for subspace 4,7,10 without and with orthogonal subspace"
{
    j=5
    occ=${occ_list[$j]}
    parentocc=${parentocc_list[$j]}
    parentocc_final=${parentocc_final_list[$j]}
    set_occ_array; set_p_occ_array
    c="freeze"

    cd $occ

    for executable in "$xmolecule_path" "$xmol_angles"; do
        if [ "$executable" == "$xmolecule_path" ]; then
            out_file=output.${rev}.${c}.${parentocc}
            dat_file="subspace_convergence.${parentocc}.xmolecule.dat" 
            rev_executable=$rev
        elif [ "$executable" == "$xmol_angles" ]; then
            out_file=output.${rev_angles}.${c}.${parentocc}.angles
            dat_file="subspace_convergence.${parentocc}.xmol_angles.dat" 
            rev_executable=$rev_angles
        fi


        echo "# $4,7,10$"
        subspace1="4,7,10"
        freeze_list1="1,2,3,5,6,8,9"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="no"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate
        cp $out_file ${out_file}.4710

        echo "# $4,7,10$; prepend orthogonal"
        subspace1="4,7,10"
        freeze_list1="1,2,3,5,6,8,9"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="yes"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate
        cp $out_file ${out_file}.4710o

    done

    cd ..
}

echo "### Compare output with orthogonal subspace calculation"
{
    j=5
    occ=${occ_list[$j]}
    parentocc=${parentocc_list[$j]}
    parentocc_final=${parentocc_final_list[$j]}
    c="freeze"

    cd $occ

    for executable in "$xmolecule_path" "$xmol_angles"; do
        if [ "$executable" == "$xmolecule_path" ]; then
            out_file=output.${rev}.${c}.${parentocc}
        elif [ "$executable" == "$xmol_angles" ]; then
            out_file=output.${rev_angles}.${c}.${parentocc}.angles
        fi
        cp ${out_file}.4710o $out_file
    done

    diff -u subspace_convergence.${parentocc}.{xmolecule,xmol_angles}.dat

    # Compare based on important quantities ############################
    # get intermediate and final overlap 
    grep -n -v "#" output.${rev}.freeze.${parentocc} output.${rev_angles}.freeze.${parentocc}.angles

    # check for number of steps till convergence
    grep -n "converged after" output.${rev}.freeze.${parentocc} output.${rev_angles}.freeze.${parentocc}.angles

    # check for case that didn't converge
    grep -n --context=3 "NOT converged" output.${rev}.freeze.${parentocc} output.${rev_angles}.freeze.${parentocc}.angles

    # compare output for subspace based on SCF step info
    grep "# SCFITER:" output.${rev}.freeze.${parentocc}
    sed -n "270,$ p" output.${rev}.freeze.${parentocc} | grep "# SCFITER:" | less
    sed -n "270,774 p" output.${rev}.freeze.${parentocc} | grep "# SCFITER:" | less

    grep "# SCFITER:" output.${rev_angles}.freeze.${parentocc}.angles
    sed -n "730,2951 p" output.${rev_angles}.freeze.${parentocc}.angles | grep "# SCFITER:" | less
    sed -n "730,2951 p" output.${rev_angles}.freeze.${parentocc}.angles | grep "# SCFITER:" | less

    diff -u <(sed -n "270,774 p" output.${rev}.freeze.${parentocc} | grep "# SCFITER:") \
            <(sed -n "730,2951 p" output.${rev_angles}.freeze.${parentocc}.angles | grep  "# SCFITER:") \
        | less

    diff -y <(sed -n "270,774 p" output.${rev}.freeze.${parentocc} | grep "# SCFITER:") \
            <(sed -n "730,2951 p" output.${rev_angles}.freeze.${parentocc}.angles | grep "# SCFITER:") \
        | less

    # compare output for orthogonal subspace ###################
    sed -n "1,270 p" output.${rev}.freeze.${parentocc} | grep "# SCFITER:" | less
    sed -n "1,730 p" output.${rev_angles}.freeze.${parentocc}.angles | grep "# SCFITER:" | less
    diff -u <(sed -n "1,270 p" output.${rev}.freeze.${parentocc} | grep "# SCFITER:") \
            <(sed -n "1,730 p" output.${rev_angles}.freeze.${parentocc}.angles | grep "# SCFITER:") \
        | less

    # remove all angles specific output
    grep -v -f ../../remove_angles_output.grep output.${rev_angles}.freeze.${parentocc}.angles
    # get line number of overlap
    grep -v -f ../../remove_angles_output.grep output.${rev_angles}.freeze.${parentocc}.angles | grep -n -v "#"

    # orthonormal subspace 
    sed -n "1,730 p" output.${rev_angles}.freeze.${parentocc}.angles \
        | grep -v -f ../../remove_angles_output.grep \
        | less

    # compare output files based on file to file basis
    diff -u <(sed -n "1,270 p" output.${rev}.freeze.${parentocc}) \
            <(sed -n "1,730 p" output.${rev_angles}.freeze.${parentocc}.angles | grep -v -f ../../remove_angles_output.grep) \
        | less

    # compare output for subspace ##############################
    diff -u <(sed -n "270,774 p" output.${rev}.freeze.${parentocc}) \
            <(sed -n "730,2951 p" output.${rev_angles}.freeze.${parentocc}.angles | grep -v -f ../../remove_angles_output.grep) \
        | less

    cd ..
}

echo "### Compare output without orthogonal subspace calculation"
{
    j=5
    occ=${occ_list[$j]}
    parentocc=${parentocc_list[$j]}
    parentocc_final=${parentocc_final_list[$j]}
    c="freeze"

    cd $occ

    for executable in "$xmolecule_path" "$xmol_angles"; do
        if [ "$executable" == "$xmolecule_path" ]; then
            out_file=output.${rev}.${c}.${parentocc}
        elif [ "$executable" == "$xmol_angles" ]; then
            out_file=output.${rev_angles}.${c}.${parentocc}.angles
        fi
        cp ${out_file}.4710 $out_file
    done

    diff -u subspace_convergence.${parentocc}.{xmolecule,xmol_angles}.dat

    # get intermediate and final overlap 
    grep -n -v "#" output.${rev}.freeze.${parentocc} output.${rev_angles}.freeze.${parentocc}.angles

    # check for number of steps till convergence
    grep -n "converged after" output.${rev}.freeze.${parentocc} 
    grep -v -f ../../remove_angles_output.grep output.${rev_angles}.freeze.${parentocc}.angles | grep -n "converged after"

    # compare output for subspace based on SCF step info
    diff -u <(grep "# SCFITER:" output.${rev}.freeze.${parentocc}) \
            <(grep "# SCFITER:" output.${rev_angles}.freeze.${parentocc}.angles) \
        | less

    # compare output files based on file to file basis
    diff -u output.${rev}.freeze.${parentocc} \
            <(grep -v -f ../../remove_angles_output.grep output.${rev_angles}.freeze.${parentocc}.angles) \
        | less


    cd ..
}

