echo "Assume you have run the script maketransitions.sh to calculate all possible ionization states of CO. Some important files from the calculations are in the subdir transitions as the file notconvergedlist.txt. It lists all the combinations of parent occupation/scf starting guess and occupation for which scf didn't converge with eiter a standard method (DIIS) or a fall back option (smooth). The fall back option reduces the occupation in fractions of electrons between parent occupation and occupation. In some cases DIIS didn't converge, in less cases also smooth didn't converge. One of those cases is 1001102 -> 1001002. We would like to investigate this scf failure in more detail."

#### Should be sourced in to run_examples.sh ####
# else you have to locate run_examples.source and source it.
# source run_examples.source

# General definitions
occ=1001002
parentocc=1001102
parentocc_final=1011002
set_occ_array; set_p_occ_array
wf_file=../1001102/wf.dat

############################################################
# Prepare separate directory for analysis
if ! [ -d ${parentocc}to${occ} ]; then mkdir ${parentocc}to${occ}; fi
cd ${parentocc}to${occ}
ln -s ../{C,O}.nao .
write_input_file

cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat

# Use rotate_orbitals between MO 4 and 7 using the above MO for f=1.0 to calculate the 1-dim energy surface for 
# the given fractional occupations
# (data periodic at pi/2 <-> -pi/2.)
# fractions=(1.00 0.90 0.80 0.70 0.60 0.50 0.40 0.30 0.20 0.13 0.12 0.11 0.10 0.09 0.08 0.00)
fractions=(1.00 0.50 0.30 0.20 0.10 0.00)
options_rotate_1angle="-RIni -RIniSh -orb1 4 -orb2 7"
plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"

cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat
for i in ${fractions[@]}; do
    out_file=scan_rotation.${parentocc}.f=${i}.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    set_frac_occ $i
    ${rotate_orbitals_path} input.in -occ ${occ} -f_occ ${frac_occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} -f_occ ${frac_occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
done
{
    cp ../wf.${parentocc_final}.dat wf.dat; cp ../basis.${parentocc_final}.dat basis.dat
    out_file=scan_rotation.${parentocc}.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} -RIni -RIniSh -orb1 4 -orb2 7 -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5 -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
}
# !script for occ_switched is missing!

# Use rotate_orbitals between MO 7 and 10 using the above MO to calculate the 1-dim energy surface for 
# the given fractional occupations
fractions=(0.00)
options_rotate_1angle="-RIni -RIniSh -orb1 7 -orb2 10"
plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"

cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat
for i in ${fractions[@]}; do
    out_file=scan_rotation.${parentocc}.f=${i}.orbitals7_10.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    set_frac_occ $i
    ${rotate_orbitals_path} input.in -occ ${occ} -f_occ ${frac_occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc)  -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} -f_occ ${frac_occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
done


# Calculate splines of the 1-dim energy surfaces above and make plots
fractions=(1.00 0.90 0.80 0.70 0.60 0.50 0.40 0.30 0.20 0.13 0.12 0.11 0.10 0.09 0.08 0.00)

# splines of total energy
cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat
for i in ${fractions[@]}; do
    data_file=scan_rotation.${parentocc}.f=${i}.dat
    ${script_dir}/interpolate_total_energy.py "$data_file" "${data_file/.dat/.spline}"
    # get derivatives from Fock matrix and calculate splines of them
    grep "HMAT" $data_file | sed -r "s/# HMAT:  $//" | sed -r ':a;N;$!ba;s/[\n]# HMAT://g' | sed -r ';s/(\-?[0-9]+[.][0-9]*)/:\1:/37;s/.*:(.*):.*/\1/;' > ${data_file/.out/.derivative}
    head -n 1 ${data_file/.out/.derivative} >> ${data_file/.out/.derivative}
    mv ${data_file/.out/.derivative} ${data_file/.out/.derivative_temp}
    paste ${data_file/.out/.dat} ${data_file/.out/.derivative_temp} > ${data_file/.out/.derivative}
    rm ${data_file/.out/.derivative_temp}
    ${script_dir}/interpolate_total_energy.py ${data_file/.out/.derivative} ${data_file/.out/.derivative.spline} 0 2
    # plot
    gnuplot -e "data_file='${data_file}';spline_file='${data_file/.dat/.spline}';der_file='${data_file/.dat/.derivative}';der_spline_file='${data_file/.dat/.derivative.spline}';out_file='${data_file/.dat/.spline.png}';" ${script_dir}/interpolate_total_energy.plot
done
{
    cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat
    data_file=scan_rotation.${parentocc}.dat
    ${script_dir}/interpolate_total_energy.py "$data_file" "${data_file/.dat/.spline}"
    # get derivatives from Fock matrix and calculate splines of them
    grep "HMAT" $data_file | sed -r "s/# HMAT:  $//" | sed -r ':a;N;$!ba;s/[\n]# HMAT://g' | sed -r ';s/(\-?[0-9]+[.][0-9]*)/:\1:/37;s/.*:(.*):.*/\1/;' > ${data_file/.out/.derivative}
    head -n 1 ${data_file/.out/.derivative} >> ${data_file/.out/.derivative}
    mv ${data_file/.out/.derivative} ${data_file/.out/.derivative_temp}
    paste ${data_file/.out/.dat} ${data_file/.out/.derivative_temp} > ${data_file/.out/.derivative}
    rm ${data_file/.out/.derivative_temp}
    ${script_dir}/interpolate_total_energy.py ${data_file/.out/.derivative} ${data_file/.out/.derivative.spline} 0 2
    # plot
    gnuplot -e "data_file='${data_file}';spline_file='${data_file/.dat/.spline}';der_file='${data_file/.dat/.derivative}';der_spline_file='${data_file/.dat/.derivative.spline}';out_file='${data_file/.dat/.spline.png}';" ${script_dir}/interpolate_total_energy.plot    
}

# Use rotate_orbitals with two angles
fractions=(0.0)
plot_file_2angles="${script_dir}/gnuplot/scan_rotation_2angles.plot"
# options_rotate_2angles="-RIni -RIniSh -angle_step 0.1 -angle_max 1.6 -orb1 3 -orb2 4 -angle_step2 0.1 -angle_max2 1.6 -orb3 4 -orb4 7"
options_rotate_2angles="-RIni -RIniSh -angle_step 0.1 -angle_max 1.6 -orb1 7 -orb2 10 -angle_step2 0.1 -angle_max2 1.6 -orb3 4 -orb4 7"

cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat
for i in ${fractions[@]}; do
    out_file=scan_rotation.2angles.${parentocc}.f=${i}.dat
    echo "# wf_file=../${wf_file}" > ${out_file}
    set_frac_occ $i
    ${rotate_orbitals_path} input.in -occ ${occ} -f_occ ${frac_occ} $options_rotate_2angles  | tee ${out_file}
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.dat/.png}';xmax=1.6;xshift=0.0" ${plot_file_2angles}
done

# Calculate trajectories with MOM, always starting at a prepared angle.
# (modified xmolecule version for this special purpose)
# (MOs besides 4 and 7 are frozen to solution at f=1.0)
angle_steps=40
fractions=(1.00 0.50 0.20 0.00)
plot_file="${script_dir}/gnuplot/SCF_angles.plot"
plot_file_trajectory="${script_dir}/gnuplot/trajectory.plot"
plot_file_trajectory_occ_switched="${script_dir}/gnuplot/trajectory_occ_switched.plot"

for ((step=0; step<$angle_steps; step++)); do
    angle_ini=$(calc_angle $step $angle_steps "-$pi/2.0" $pi)
    if ! [ -d "angle=${angle_ini}" ]; then 
        mkdir "angle=${angle_ini}"
    fi
    cd "angle=${angle_ini}"
    write_input_file
    cp ../{C,O}.nao .
    if ! [ -f wf.dat ]; then     
        cp ../../${wf_file} wf.dat; cp ../../${wf_file/wf/basis} basis.dat
        $rotate_orbitals_path  input.in -occ $occ -RIni -RIniSh -angle $angle_ini -orb1 4 -orb2 7 -SIni 
    fi
    for i in ${fractions[@]}; do 
        set_frac_occ $i 
        out_file=SCF_angles.${parentocc}.f=${i}.out
        # if [ -f $out_file ]; then continue; fi
        $xmol_angles input.in -occ ${occ} -f_occ ${frac_occ} -MOM -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -population -xatom_path=${xatom_path} -freeze 1,2,3,5,6,8,9,10 -ref ../../$wf_file -ref_basis ../../${wf_file/wf/basis} | tee $out_file 
        printf "# %8s %10s %10s %10s\n" "phi_n" "E_n" "Olap_n" "MOM" > ${out_file/.out/.dat}
        paste \
            <(grep -f <(echo -e "SCFINFO: E_tot\nANGLE47_Ref") $out_file | egrep -oE "\-?[0-9]+[.][0-9]*" | xargs -n3 printf "%10.4f %10.4f %10.4f  \n") \
            <(grep -f <(echo -e "SCFITER\nMOM: swapping") $out_file | tail -n +2 | sed -r ':a;N;$!ba;s/# MOM:[^\n]*\n# SCFITER:[^\n]*/1/g' | sed "s/#.*/0/") \
            > ${out_file/.out/.dat};
        # plot SCF iteration vs angle
        gnuplot -e "input_file='${out_file/.out/.dat}'; output_file='${out_file/.out/.png}'; fraction=${i};" $plot_file
        # plot trajectory in angle vs energy plot
        gnuplot -e "graph_file='../scan_rotation.${parentocc}.f=${i}.out'; trajectory_file='${out_file/.out/.dat}'; out_file='trajectory.${parentocc}.f=${i}.png';" $plot_file_trajectory
    done
    cd ..
done
# No MOM
for ((step=0; step<$angle_steps; step++)); do
    angle_ini=$(calc_angle $step $angle_steps "-$pi/2.0" $pi)
    if ! [ -d "angle=${angle_ini}" ]; then 
        mkdir "angle=${angle_ini}"
    fi
    cd "angle=${angle_ini}"
    write_input_file
    cp ../{C,O}.nao .
    if ! [ -f wf.dat ]; then 
        cp ../../${wf_file} wf.dat; cp ../../${wf_file/wf/basis} basis.dat
        $rotate_orbitals_path  input.in -occ $occ -RIni -RIniSh -angle $angle_ini -orb1 4 -orb2 7  -SIni
    fi
    for i in ${fractions[@]}; do 
        set_frac_occ $i 
        out_file=SCF_angles.noMOM.${parentocc}.f=${i}.out
        $xmol_angles input.in -occ ${occ} -f_occ ${frac_occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -population -xatom_path=${xatom_path} -freeze 1,2,3,5,6,8,9,10 -ref ../../$wf_file -ref_basis ../../${wf_file/wf/basis} | tee $out_file 
        printf "# %8s %10s %10s %10s\n" "phi_n" "E_n" "Olap_n" "MOM" > ${out_file/.out/.dat}
        paste \
            <(grep -f <(echo -e "SCFINFO: E_tot\nANGLE47_Ref") $out_file | egrep -oE "\-?[0-9]+[.][0-9]*" | xargs -n3 printf "%10.4f %10.4f %10.4f  \n") \
            <(grep -f <(echo -e "SCFITER\nMOM: swapping") $out_file | tail -n +2 | sed -r ':a;N;$!ba;s/# MOM:[^\n]*\n# SCFITER:[^\n]*/1/g' | sed "s/#.*/0/") \
            >> ${out_file/.out/.dat};
        # plot SCF iteration vs angle
        gnuplot -e "input_file='${out_file/.out/.dat}'; output_file='${out_file/.out/.png}'; fraction=${i};" $plot_file
        # plot trajectory in angle vs energy plot
        gnuplot -e "graph_file='../scan_rotation.${parentocc}.f=${i}.out'; trajectory_file='${out_file/.out/.dat}'; out_file='trajectory.noMOM.${parentocc}.f=${i}.png';" $plot_file_trajectory
    done
    cd ..
done
# No MOM; occupation switched
for ((step=0; step<$angle_steps; step++)); do
    angle_ini=$(calc_angle $step $angle_steps "-$pi/2.0" $pi)
    if ! [ -d "angle=${angle_ini}" ]; then 
        mkdir "angle=${angle_ini}"
    fi
    cd "angle=${angle_ini}"
    write_input_file
    cp ../{C,O}.nao .
    if ! [ -f wf.dat ]; then 
        cp ../../${wf_file} wf.dat; cp ../../${wf_file/wf/basis} basis.dat
        $rotate_orbitals_path  input.in -occ $occ -RIni -RIniSh -angle $angle_ini -orb1 4 -orb2 7  -SIni
    fi
    occ=1002001; set_occ_array; set_p_occ_array
    for i in ${fractions[@]}; do 
        set_frac_occ $i 
        out_file=SCF_angles.noMOM.occ_switched.${parentocc}.f=${i}.out
#         $xmol_angles input.in -occ ${occ} -f_occ ${frac_occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -population -xatom_path=${xatom_path} -freeze 1,2,3,5,6,8,9,10 -ref ../../$wf_file -ref_basis ../../${wf_file/wf/basis} | tee $out_file 
#         printf "# %8s %10s %10s %10s\n" "phi_n" "E_n" "Olap_n" "MOM" > ${out_file/.out/.dat}
#         paste \
#             <(grep -f <(echo -e "SCFINFO: E_tot\nANGLE47_Ref") $out_file | egrep -oE "\-?[0-9]+[.][0-9]*" | xargs -n3 printf "%10.4f %10.4f %10.4f  \n") \
#             <(grep -f <(echo -e "SCFITER\nMOM: swapping") $out_file | tail -n +2 | sed -r ':a;N;$!ba;s/# MOM:[^\n]*\n# SCFITER:[^\n]*/1/g' | sed "s/#.*/0/") \
#             >> ${out_file/.out/.dat};
#         # plot SCF iteration vs angle
        gnuplot -e "input_file='${out_file/.out/.dat}'; output_file='${out_file/.out/.png}'; fraction=${i};" $plot_file
        # plot trajectory in angle vs energy plot
#        gnuplot -e "graph_file='../scan_rotation.occ_switched.1001102.f=${i}.out'; trajectory_file='${out_file/.out/.dat}'; out_file='trajectory.noMOM.occ_switched.${parentocc}.f=${i}.png';" $plot_file_trajectory_occ_switched
        gnuplot -e "graph_file='../scan_rotation.occ_switched.1001102.f=${i}.out'; trajectory_file='${out_file/.out/.dat}'; out_file='trajectory.noMOM.occ_switched.${parentocc}.f=${i}.png';" $plot_file_trajectory
    done
    occ=1001002; set_occ_array; set_p_occ_array
    cd ..
done
# SOSCF
# cut off output file after the section which contains line "# switch soscf off"
# to only look at the SOSCF steps
for ((step=0; step<$angle_steps; step++)); do
    angle_ini=$(calc_angle $step $angle_steps "-$pi/2.0" $pi)
    if ! [ -d "angle=${angle_ini}" ]; then 
        mkdir "angle=${angle_ini}"
    fi
    cd "angle=${angle_ini}"
    write_input_file
    cp ../{C,O}.nao .
    if ! [ -f wf.dat ]; then 
        cp ../../${wf_file} wf.dat; cp ../../${wf_file/wf/basis} basis.dat
        $rotate_orbitals_path  input.in -occ $occ -RIni -RIniSh -angle $angle_ini -orb1 4 -orb2 7  -SIni
    fi
    for i in ${fractions[@]}; do 
        set_frac_occ $i 
        out_file=SCF_angles.SOSCF.${parentocc}.f=${i}.out
        $xmol_angles input.in -occ ${occ} -f_occ ${frac_occ} -NODIIS -RIni -RIniSh -population -xatom_path=${xatom_path} -freeze 1,2,3,5,6,8,9,10 -SOSCF -ref ../../$wf_file -ref_basis ../../${wf_file/wf/basis} | tee $out_file 
        printf "# %8s %10s %10s %10s\n" "phi_n" "E_n" "Olap_n" "MOM" > ${out_file/.out/.dat}
        sed_cmd=':a;N;$!ba;s/(.*'$(cat $out_file | sed -r ':a;N;$!ba;s/^.*# switch soscf off//' | grep 'SCFITER' | head -n 1)').*/\1/'
        paste \
            <(cat $out_file | sed -r "$sed_cmd" | grep -f <(echo -e "SCFINFO: E_tot\nANGLE47_Ref") | egrep -oE "\-?[0-9]+[.][0-9]*" | xargs -n3 printf "%10.4f %10.4f %10.4f  \n") \
            <(cat $out_file | sed -r "$sed_cmd" | grep -f <(echo -e "SCFITER\nMOM: swapping") | tail -n +2 | sed -r ':a;N;$!ba;s/# MOM:[^\n]*\n# SCFITER:[^\n]*/1/g' | sed "s/#.*/0/") \
            > ${out_file/.out/.dat};
        # # plot SCF iteration vs angle
        gnuplot -e "input_file='${out_file/.out/.dat}'; output_file='${out_file/.out/.png}'; fraction=${i};" $plot_file
        # # plot trajectory in angle vs energy plot
        gnuplot -e "graph_file='../scan_rotation.${parentocc}.f=${i}.out'; trajectory_file='${out_file/.out/.dat}'; out_file='trajectory.SOSCF.${parentocc}.f=${i}.png';" $plot_file_trajectory
    done
    cd ..
done
# # No MOM; accuracy
# for ((step=0; step<$angle_steps; step++)); do
#     angle_ini=$(calc_angle $step $angle_steps "-$pi/2.0" $pi)
#     if ! [ -d "angle=${angle_ini}" ]; then 
#         mkdir "angle=${angle_ini}"
#     fi
#     cd "angle=${angle_ini}"
#     write_input_file
#     cp ../{C,O}.nao .
#     if ! [ -f wf.dat ]; then 
#         $rotate_orbitals_path  input.in -occ $occ -RIni -RIniSh -angle $angle_ini -orb1 4 -orb2 7  -SIni
#     fi
#     for i in ${fractions[@]}; do 
#         set_frac_occ $i 
#         out_file=SCF_angles.noMOM.accuracy.${parentocc}.f=${i}.out
#         $xmol_angles input.in -occ ${occ} -f_occ ${frac_occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -population -xatom_path=${xatom_path} -freeze 1,2,3,5,6,8,9,10 -ref ../../$wf_file -ref_basis ../../${wf_file/wf/basis} -scf_energy_conv 1E-11 -scf_density_conv 1E-11 | tee $out_file 
#         printf "# %8s %10s %10s %10s\n" "phi_n" "E_n" "Olap_n" "MOM" > ${out_file/.out/.dat}
#         paste \
#             <(grep -f <(echo -e "SCFINFO: E_tot\nANGLE47_Ref") $out_file | egrep -oE "\-?[0-9]+[.][0-9]*" | xargs -n3 printf "%10.4f %10.4f %10.4f  \n") \
#             <(grep -f <(echo -e "SCFITER\nMOM: swapping") $out_file | tail -n +2 | sed -r ':a;N;$!ba;s/# MOM:[^\n]*\n# SCFITER:[^\n]*/1/g' | sed "s/#.*/0/") \
#             >> ${out_file/.out/.dat};
#         # plot SCF iteration vs angle
#         gnuplot -e "input_file='${out_file/.out/.dat}'; output_file='${out_file/.out/.png}'; fraction=${i};" $plot_file
#         # plot trajectory in angle vs energy plot
#         gnuplot -e "graph_file='../scan_rotation.${parentocc}.f=${i}.out'; trajectory_file='${out_file/.out/.dat}'; out_file='trajectory.noMOM.accuracy.${parentocc}.f=${i}.png';" $plot_file_trajectory
#     done
#     cd ..
# done
# # No DIIS; No MOM
# for ((step=0; step<$angle_steps; step++)); do
#     angle_ini=$(calc_angle $step $angle_steps "-$pi/2.0" $pi)
#     if ! [ -d "angle=${angle_ini}" ]; then 
#         mkdir "angle=${angle_ini}"
#     fi
#     cd "angle=${angle_ini}"
#     write_input_file
#     cp ../{C,O}.nao .
#     if ! [ -f wf.dat ]; then 
#         $rotate_orbitals_path  input.in -occ $occ -RIni -RIniSh -angle $angle_ini -orb1 4 -orb2 7  -SIni
#     fi
#     for i in ${fractions[@]}; do 
#         set_frac_occ $i 
#         out_file=SCF_angles.noDIIS.noMOM.${parentocc}.f=${i}.out
#         $xmol_angles input.in -occ ${occ} -f_occ ${frac_occ} -NODIIS -RIni -RIniSh -population -xatom_path=${xatom_path} -freeze 1,2,3,5,6,8,9,10 -ref ../../$wf_file -ref_basis ../../${wf_file/wf/basis} | tee $out_file 
#         paste \
#             <(grep -f <(echo -e "SCFINFO: E_tot\nANGLE47_Ref") $out_file | egrep -oE "\-?[0-9]+[.][0-9]*" | xargs -n3 printf "%10.4f %10.4f %10.4f  \n") \
#             <(grep -f <(echo -e "SCFITER\nMOM: swapping") $out_file | tail -n +2 | sed -r ':a;N;$!ba;s/# MOM:[^\n]*\n# SCFITER:[^\n]*/1/g' | sed "s/#.*/0/") \
#             > ${out_file/.out/.dat};
#         # plot SCF iteration vs angle
#         gnuplot -e "input_file='${out_file/.out/.dat}'; output_file='${out_file/.out/.png}'; fraction=${i};" $plot_file
#         # plot trajectory in angle vs energy plot
#         gnuplot -e "graph_file='../scan_rotation.${parentocc}.f=${i}.out'; trajectory_file='${out_file/.out/.dat}'; out_file='trajectory.noDIIS.noMOM.${parentocc}.f=${i}.png';" $plot_file_trajectory
#     done
#     cd ..
# done
# create softlinks to directories for fractional occupation
angle_steps=40
fractions=(1.00 0.50 0.20 0.00)
for i in ${fractions[*]}; do 
    if ! [ -d "f=$i" ]; then mkdir "f=$i"; fi
    cd "f=$i"
    for ((step=0; step<$angle_steps; step++)); do
        angle_ini=$(calc_angle $step $angle_steps "-$pi/2.0" $pi)
        for png_file in SCF_angles.{,noMOM.,noMOM.occ_switched.,SOSCF.}${parentocc}.f=${i}.png \
                        trajectory.{,noMOM.,noMOM.occ_switched.,SOSCF.}${parentocc}.f=${i}.png; do
            if ! [ -f ../angle\=${angle_ini}/$png_file ]; then continue; fi
            ln_file=${png_file/f\=$i.png/angle\=$angle_ini.png}
            if [ -f $ln_file ]; then rm $ln_file; fi
            ln -s ../angle\=${angle_ini}/${png_file} ${ln_file}
        done
        # # ln SCF_angles plot
        # png_file=SCF_angles.${parentocc}.f=${i}.png
        # ln_file=${png_file/f\=$i.png/angle\=$angle_ini.png}
        # if [ -f $ln_file ]; then rm $ln_file}; fi
        # ln -s ../angle\=${angle_ini}/${png_file} ${ln_file}
        # # ln trajectory plot
        # png_file=trajectory.${parentocc}.f=${i}.png
        # ln_file=${png_file/f\=$i.png/angle\=$angle_ini.png}
        # if [ -f $ln_file ]; then rm $ln_file}; fi
        # ln -s ../angle\=${angle_ini}/${png_file} $ln_file
    done
    cd ..
done

# Collect convergence data and plot 
fractions=(0.00)
plot_file=${script_dir}/gnuplot/convergence_data.plot

for i in ${fractions[@]}; do
    convergence_data_file="SCF_conv.f=${i}.dat"
#    convergence_data_file="SCF_conv.f=${i}.wf.f=0.2.dat"
    printf "# %10s %10s %10s\n" "phi_0" "phi_f" "converged" > $convergence_data_file
    out_file=SCF_angles.${parentocc}.f=${i}.out
    for dir in angle*; do
        # angle_ini=$(get_nth_num "$dir" 1)
        if ! [ -f "${dir}/${out_file}" ]; then continue; fi
        angle_ini=$(get_nth_num "$(head -n 2 ${dir}/${out_file/.out/.dat})" 1)
        if [ -z "$(grep 'SCF converged' ${dir}/${out_file})" ]; then
            printf "  %10.4f %10.4f %10i \n" $angle_ini 0.0 0 >> $convergence_data_file
        else
            angle_final=$(get_nth_num "$(tail -n 1 ${dir}/${out_file/.out/.dat})" 1)
            printf "  %10.4f %10.4f %10i \n" $angle_ini $angle_final 1 >> $convergence_data_file
        fi
    done
done
# No MOM
for i in ${fractions[@]}; do
    convergence_data_file="SCF_conv.noMOM.f=${i}.dat"
#    convergence_data_file="SCF_conv.noMOM.f=${i}.wf.f=0.2.dat"
    printf "# %10s %10s %10s\n" "phi_0" "phi_f" "converged" > $convergence_data_file
    out_file=SCF_angles.noMOM.${parentocc}.f=${i}.out
    for dir in angle*; do
        # angle_ini=$(get_nth_num "$dir" 1)
        if ! [ -f "${dir}/${out_file}" ]; then continue; fi
        angle_ini=$(get_nth_num "$(head -n 2 ${dir}/${out_file/.out/.dat})" 1)        
        if [ -z "$(grep 'SCF converged' ${dir}/${out_file})" ]; then
            printf "  %10.4f %10.4f %10i \n" $angle_ini 0.0 0 >> $convergence_data_file
        else
            angle_final=$(get_nth_num "$(tail -n 1 ${dir}/${out_file/.out/.dat})" 1)
            printf "  %10.4f %10.4f %10i \n" $angle_ini $angle_final 1 >> $convergence_data_file
        fi
    done
done
# No MOM; occupation switched
for i in ${fractions[@]}; do
    convergence_data_file="SCF_conv.noMOM.occ_switched.f=${i}.dat"
#    convergence_data_file="SCF_conv.noMOM.occ_switched.f=${i}.wf.f=0.2.dat"
    printf "# %10s %10s %10s\n" "phi_0" "phi_f" "converged" > $convergence_data_file
    out_file=SCF_angles.noMOM.occ_switched.${parentocc}.f=${i}.out
    for dir in angle*; do
        # angle_ini=$(get_nth_num "$dir" 1)
        if ! [ -f "${dir}/${out_file}" ]; then continue; fi
        angle_ini=$(get_nth_num "$(head -n 2 ${dir}/${out_file/.out/.dat})" 1)        
        if [ -z "$(grep 'SCF converged' ${dir}/${out_file})" ]; then
            printf "  %10.4f %10.4f %10i \n" $angle_ini 0.0 0 >> $convergence_data_file
        else
            angle_final=$(get_nth_num "$(tail -n 1 ${dir}/${out_file/.out/.dat})" 1)
            printf "  %10.4f %10.4f %10i \n" $angle_ini $angle_final 1 >> $convergence_data_file
        fi
    done
done
# SOSCF
for i in ${fractions[@]}; do
    convergence_data_file="SCF_conv.SOSCF.f=${i}.dat"
    printf "# %10s %10s %10s\n" "phi_0" "phi_f" "converged" > $convergence_data_file
    out_file=SCF_angles.SOSCF.${parentocc}.f=${i}.out
    for dir in angle*; do
        # angle_ini=$(get_nth_num "$dir" 1)
        if ! [ -f "${dir}/${out_file}" ]; then continue; fi
        angle_ini=$(get_nth_num "$(head -n 1 ${dir}/${out_file/.out/.dat})" 1)        
        sed_cmd=':a;N;$!ba;s/(.*'$(cat ${dir}/$out_file | sed -r ':a;N;$!ba;s/^.*# switch soscf off//' | grep 'SCFITER' | head -n 1)').*/\1/'
        n_i=$(cat ${dir}/$out_file | sed -r "$sed_cmd" | egrep -E "SCFITER: +[0-9]+" | wc -l)
        if [ ${n_i} -eq 100 ]; then
            printf "  %10.4f %10.4f %10i \n" $angle_ini 0.0 0 >> $convergence_data_file
        else
            angle_final=$(get_nth_num "$(tail -n 1 ${dir}/${out_file/.out/.dat})" 1)
            printf "  %10.4f %10.4f %10i \n" $angle_ini $angle_final 1 >> $convergence_data_file
        fi
    done
done
for i in ${fractions[@]}; do
    # spline_file="scan_rotation.${parentocc}.f=${i}.spline"
    # gnuplot -e "out_file='SCF_conv.f=${i}.png';angle_file1='SCF_conv.f=${i}.dat';angle_file2='SCF_conv.noMOM.f=${i}.dat';angle_file3='SCF_conv.noDIIS.noMOM.f=${i}.dat';angle_file4='SCF_conv.SOSCF.f=${i}.dat';spline_file='${spline_file}';" ${script_dir}/convergence_data.plot
        gnuplot -e "out_file='SCF_conv.f=${i}.png';angle_file1='SCF_conv.f=${i}.dat';angle_file2='SCF_conv.noMOM.f=${i}.dat';angle_file3='SCF_conv.noDIIS.noMOM.f=${i}.dat';angle_file4='SCF_conv.SOSCF.f=${i}.dat';" $plot_file
done
# # No MOM; accuracy
# for i in ${fractions[@]}; do
#     convergence_data_file="SCF_conv.noMOM.accuracy.f=${i}.dat"
#     printf "# %10s %10s %10s\n" "phi_0" "phi_f" "converged" > $convergence_data_file
#     out_file=SCF_angles.noMOM.accuracy.${parentocc}.f=${i}.out
#     for dir in angle*; do
#         # angle_ini=$(get_nth_num "$dir" 1)
#         if ! [ -f "${dir}/${out_file}" ]; then continue; fi
#         angle_ini=$(get_nth_num "$(head -n 2 ${dir}/${out_file/.out/.dat})" 1)        
#         if [ -z "$(grep 'SCF converged' ${dir}/${out_file})" ]; then
#             printf "  %10.4f %10.4f %10i \n" $angle_ini 0.0 0 >> $convergence_data_file
#         else
#             angle_final=$(get_nth_num "$(tail -n 1 ${dir}/${out_file/.out/.dat})" 1)
#             printf "  %10.4f %10.4f %10i \n" $angle_ini $angle_final 1 >> $convergence_data_file
#         fi
#     done
# done
# # No DIIS no MOM
# for i in ${fractions[@]}; do
#     convergence_data_file="SCF_conv.noDIIS.noMOM.f=${i}.dat"
#     printf "# %10s %10s %10s\n" "phi_0" "phi_f" "converged" > $convergence_data_file
#     out_file=SCF_angles.noDIIS.noMOM.${parentocc}.f=${i}.out
#     for dir in angle*; do
#         # angle_ini=$(get_nth_num "$dir" 1)
#         if ! [ -f "${dir}/${out_file}" ]; then continue; fi
#         angle_ini=$(get_nth_num "$(head -n 1 ${dir}/${out_file/.out/.dat})" 1)        
#         if [ -z "$(grep 'SCF converged' ${dir}/${out_file})" ]; then
#             printf "  %10.4f %10.4f %10i \n" $angle_ini 0.0 0 >> $convergence_data_file
#         else
#             angle_final=$(get_nth_num "$(tail -n 1 ${dir}/${out_file/.out/.dat})" 1)
#             printf "  %10.4f %10.4f %10i \n" $angle_ini $angle_final 1 >> $convergence_data_file
#         fi
#     done
# done
# print graph of convergence data for each fraction 

############################################################
# Get MO energies from converged solution 

# initial solution
angle=0.00
fractions=(1.00 0.90 0.80 0.70 0.60 0.50 0.40 0.30 0.20 0.13 0.12) 
data_file="MO_energies_initial_solution.dat"
echo "# fraction vs. MO energies" > $data_file

for i in ${fractions[@]}; do
    out_file="angle=${angle}/SCF_angles.${parentocc}.f=${i}.out"
    grep  "# SCFINFO: MO eigenvalues" $out_file | tail -n 2 | sed ':a;N;$!ba;s/\n# SCFINFO: MO eigenvalues: unocc//g' | sed "s/# SCFINFO: MO eigenvalues: occ/$i/" >> $data_file
done
gnuplot -e "xmin=1.0;xmax=0.0;ymin=-8.0;ymax=-4.0;data_file=\"${data_file}\";out_file=\"${data_file/.dat/.png}\";" ${script_dir}/MO_energies.plot

# final solution
angle=0.94
fractions=(0.30 0.20 0.13 0.12 0.13 0.12 0.11 0.10 0.09 0.08 0.00)
data_file="MO_energies_final_solution.dat"
echo "# fraction vs. MO energies" > $data_file

for i in ${fractions[@]}; do
    out_file="angle=${angle}/SCF_angles.${parentocc}.f=${i}.out"
    grep  "# SCFINFO: MO eigenvalues" $out_file | tail -n 2 | sed ':a;N;$!ba;s/\n# SCFINFO: MO eigenvalues: unocc//g' | sed "s/# SCFINFO: MO eigenvalues: occ/$i/" >> $data_file
done
gnuplot -e "xmin=1.0;xmax=0.0;ymin=-8.0;ymax=-4.0;data_file=\"${data_file}\";out_file=\"${data_file/.dat/.png}\";" ${script_dir}/MO_energies.plot

# Get total energy from converged solution
############################################################
# Get total energy vs fraction starting at different guesses

# initial solution ##########
fractions=(1.00 0.90 0.80 0.70 0.60 0.50 0.40 0.30 0.20 0.18 0.15) 
wf_file="../wf.${parentocc}.f=1.0.dat"
basis_file=${wf_file/wf/basis}
out_file_suffix="ini.out"
data_file="total_energies.${parentocc}.ini.dat"
xmol_angles_options="-MOM -NODIIS -SwitchToDIIS 0.01"
calc_total_energies_of_fractions
collect_total_energies_of_fractions
# initial solution all frozen but 3,4,7
fractions+=(0.12)
out_file_suffix="ini_frozen.out"
data_file="total_energies.${parentocc}.ini_frozen.dat"
xmol_angles_options="-MOM -NODIIS -SwitchToDIIS 0.01 -freeze 1,2,5,6,8,9,10"
calc_total_energies_of_fractions
collect_total_energies_of_fractions
# initial solution all frozen but 3,4,7 no MOM
fractions+=(0.10 0.00)
out_file_suffix="ini_frozen_noMOM.out"
data_file="total_energies.${parentocc}.ini_frozen_noMOM.dat"
xmol_angles_options="-NODIIS -SwitchToDIIS 0.01 -freeze 1,2,5,6,8,9,10"
calc_total_energies_of_fractions
collect_total_energies_of_fractions
# initial solution all frozen but 3,4,7,10
fractions=(1.00 0.90 0.80 0.70 0.60 0.50 0.40 0.30 0.20 0.10 0.00)
out_file_suffix="ini_frozen2.out"
data_file="total_energies.${parentocc}.ini_frozen.dat"
xmol_angles_options="-MOM -NODIIS -SwitchToDIIS 0.01 -freeze 1,2,5,6,8,9"
calc_total_energies_of_fractions
collect_total_energies_of_fractions
# initial solution all frozen but 3,4,7,10 no MOM
fractions=(1.00 0.90 0.80 0.70 0.60 0.50 0.40 0.30 0.20 0.10 0.00)
out_file_suffix="ini_frozen2_noMOM.out"
data_file="total_energies.${parentocc}.ini_frozen_noMOM.dat"
xmol_angles_options="-NODIIS -SwitchToDIIS 0.01 -freeze 1,2,5,6,8,9"
calc_total_energies_of_fractions
collect_total_energies_of_fractions



# final solution ##########
fractions=(0.50 0.40 0.30 0.20 0.10 0.00) 
wf_file="../wf.${parentocc_final}.dat"
basis_file=${wf_file/wf/basis}
out_file_suffix="fin.out"
data_file="total_energies.${parentocc}.fin.dat"
xmol_angles_options="-MOM -NODIIS -SwitchToDIIS 0.01"
calc_total_energies_of_fractions
collect_total_energies_of_fractions
# final solution all frozen but 3,4,7
out_file_suffix="fin_frozen.out"
data_file="total_energies.${parentocc}.fin_frozen.dat"
xmol_angles_options="-MOM -NODIIS -SwitchToDIIS 0.01 -freeze 1,2,5,6,8,9,10"
calc_total_energies_of_fractions
collect_total_energies_of_fractions
# final solution all frozen but 3,4,7 no MOM
out_file_suffix="fin_frozen_noMOM.out"
data_file="total_energies.${parentocc}.fin_frozen_noMOM.dat"
xmol_angles_options="-NODIIS -SwitchToDIIS 0.01 -freeze 1,2,5,6,8,9,10"
calc_total_energies_of_fractions
collect_total_energies_of_fractions

# global minimum ##########
angle_ini=1.10
wf_file=wf.1001102.f=1.0.dat
cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat
$rotate_orbitals_path  input.in -occ $occ -RIni -RIniSh -angle $angle_ini -orb1 4 -orb2 7 -file_wf ../$wf_file -SIni

fractions=(1.00 0.90 0.80 0.70 0.60 0.50 0.40 0.30 0.20 0.10 0.00) 
wf_file=wf.dat
basis_file=basis.dat
out_file_suffix="global_min.out"
data_file="total_energies.${parentocc}.global_min.dat"
xmol_angles_options="-MOM -NODIIS -SwitchToDIIS 0.01"
calc_total_energies_of_fractions
collect_total_energies_of_fractions

# convergence after freeze #################################
i=0.00
wf_file=wf.1001102.f=1.0.dat

# convergence from saddle point ######
# MOM converges to saddle point if started at \alpha=0.00 with freeze
cp $wf_file wf.dat
${xmol_angles} input.in -occ ${occ} -NODIIS -RIni -RIniSh -SIni -LevelShift 0.2 -SwitchToDIIS 0.01 -population -xatom_path=/home/michael/XATOM/src/xatom -MOM -freeze 1,2,3,5,6,8,9,10
# wf close to initial?
${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref ../../$parentocc/wf.dat -ref_basis ../../$parentocc/basis.dat
# WF_overlap  0.970938949999984
# Converges with MOM without freeze?
${xmol_angles} input.in -occ ${occ} -NODIIS -RIni -RIniSh -SIni -LevelShift 0.2 -SwitchToDIIS 0.01 -population -xatom_path=/home/michael/XATOM/src/xatom -MOM
# not converged!

# convergence from min ###############
# MOM converges to minimum if started at \alpha=1.10 with freeze
# (would converge even without freeze)
cp $wf_file wf.dat
angle_ini=1.10
$rotate_orbitals_path  input.in -occ $occ -RIni -RIniSh -angle $angle_ini -orb1 4 -orb2 7 -file_wf $wf_file -SIni
${xmol_angles} input.in -occ ${occ} -NODIIS -RIni -RIniSh -SIni -LevelShift 0.2 -SwitchToDIIS 0.01 -population -xatom_path=/home/michael/XATOM/src/xatom -MOM -freeze 1,2,3,5,6,8,9,10 -ref $wf_file
# converged to angle -1.14 with reference to $wf_file 
${xmol_angles} input.in -occ ${occ} -NODIIS -RIni -RIniSh -SIni -LevelShift 0.2 -SwitchToDIIS 0.01 -population -xatom_path=/home/michael/XATOM/src/xatom -MOM 
# converges!
${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref ../../$parentocc/wf.dat -ref_basis ../../$parentocc/basis.dat
# WF_overlap  0.468623486959872     
${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat
# WF_overlap -0.462338088279706     
# stationary point neither close to initial nor final solution

# convergence from max ###############
# MOM converges to maximum if started at \alpha=
cp $wf_file wf.dat
angle_ini=-0.80
$rotate_orbitals_path  input.in -occ $occ -RIni -RIniSh -angle $angle_ini -orb1 4 -orb2 7 -file_wf $wf_file -SIni
${xmol_angles} input.in -occ ${occ} -NODIIS -RIni -RIniSh -SIni -LevelShift 0.2 -SwitchToDIIS 0.01 -population -xatom_path=/home/michael/XATOM/src/xatom -MOM -freeze 1,2,3,5,6,8,9,10 -ref $wf_file
# converged close to final?
${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat
# WF_overlap  0.926590606600396     
# Yes!
${xmol_angles} input.in -occ ${occ} -NODIIS -RIni -RIniSh -SIni -LevelShift 0.2 -SwitchToDIIS 0.01 -population -xatom_path=/home/michael/XATOM/src/xatom -MOM 
${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat
# WF_overlap  0.999999999999974     
# stationary point final solution
############################################################
# try to first optimize 7,10 and then 4,7 
executable="$xmol_angles"
rev_executable=$rev_angles
converged="-"
olap_int1=""
olap_int2=""
olap_fin=""
cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat

c="freeze4,7"
out_file=output.${rev_angles}.${c}.${parentocc}.angles
freeze_list1="1,2,3,5,6,8,9,10"
${executable} input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -freeze $freeze_list1      -SIni  | tee $out_file
olap_int1=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0")
echo $olap_int1

fractions=(0.00)
options_rotate_1angle="-RIni -RIniSh -orb1 4 -orb2 7"
plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
for i in ${fractions[@]}; do
    out_file=scan_rotation.${parentocc}.f=${i}.offset4,7.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    set_frac_occ $i
    ${rotate_orbitals_path} input.in -occ ${occ} -f_occ ${frac_occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} -f_occ ${frac_occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
done

c="freeze7,10"
out_file=output.${rev_angles}.${c}.${parentocc}.angles
freeze_list1="1,2,3,4,5,6,8,9"
${executable} input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -freeze $freeze_list1      -SIni  | tee $out_file
olap_int2=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0")
echo $olap_int2

fractions=(0.00)
options_rotate_1angle="-RIni -RIniSh -orb1 4 -orb2 7"
plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
for i in ${fractions[@]}; do
    out_file=scan_rotation.${parentocc}.f=${i}.offset4,7-7,10.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    set_frac_occ $i
    ${rotate_orbitals_path} input.in -occ ${occ} -f_occ ${frac_occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} -f_occ ${frac_occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
done

c="freeze-final"
out_file=out_file=output.${rev_angles}.${c}.${parentocc}.angles
${executable} input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -MOM -SIni | tee $out_file

fractions=(0.00)
options_rotate_1angle="-RIni -RIniSh -orb1 4 -orb2 7"
plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
for i in ${fractions[@]}; do
    out_file=scan_rotation.${parentocc}.f=${i}.offset7,10.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    set_frac_occ $i
    ${rotate_orbitals_path} input.in -occ ${occ} -f_occ ${frac_occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} -f_occ ${frac_occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
done

############################################################
# go back into directory $occ
cd ..



############################################################
# Old stuff

# # initial solution
# fractions=(1.00 0.90 0.80 0.70 0.60 0.50 0.40 0.30 0.20 0.18 0.15) 
# wf_file=wf.${parentocc}.f=1.0.dat

# cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat
# for i in ${fractions[@]}; do 
#     set_frac_occ $i 
#     out_file=SCF_angles.${parentocc}.f=${i}.ini.out
#     $xmol_angles input.in -occ ${occ} -f_occ ${frac_occ} -MOM -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh | tee $out_file
# done

# data_file="total_energies.${parentocc}.initial_solution.dat"
# echo "# fraction vs. total energy" > $data_file
# for i in ${fractions[@]}; do
#     out_file="SCF_angles.${parentocc}.f=${i}.ini.out"
#     if ! [ -f $out_file ]; then continue; fi
#     if [ -n "$(grep 'NOT converged' $out_file)" ]; then 
#         echo "${i}   NaN" | tee -a $data_file
#     else
# #        grep "SCFITER" $out_file | tail -n 1 | sed -r "s/# SCFITER: +[0-9]+ +(\-?[0-9]+[.][0-9]*).*/${i}   \1/" | tee -a $data_file
#         grep "#   Total Energy   =" $out_file | sed -r "s/#   Total Energy   = +(\-?[0-9]+[.][0-9]*).*/${i}  \1/" | tee -a $data_file
#     fi
# done

# # final solution
# fractions=(0.50 0.40 0.30 0.20 0.10 0.00) 
# wf_file=wf.${parentocc_final}.dat

# cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat
# for i in ${fractions[@]}; do 
#     set_frac_occ $i 
#     out_file=SCF_angles.${parentocc}.f=${i}.fin.out
#     $xmol_angles input.in -occ ${occ} -f_occ ${frac_occ} -MOM -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh | tee $out_file
# done

# data_file="total_energies.${parentocc}.final_solution.dat"
# echo "# fraction vs. total energy" > $data_file
# for i in ${fractions[@]}; do
#     out_file="SCF_angles.${parentocc}.f=${i}.fin.out"
#     if ! [ -f $out_file ]; then continue; fi
#     if [ -n "$(grep 'NOT converged' $out_file)" ]; then 
#         echo "${i}   NaN" | tee -a $data_file
#     else
# #        grep "SCFITER" $out_file | tail -n 1 | sed -r "s/# SCFITER: +[0-9]+ +(\-?[0-9]+[.][0-9]*).*/${i}   \1/" | tee -a $data_file
#         grep "#   Total Energy   =" $out_file | sed -r "s/#   Total Energy   = +(\-?[0-9]+[.][0-9]*).*/${i}  \1/" | tee -a $data_file
#     fi
# done

# # global minimum solution
# fractions=(1.00 0.90 0.80 0.70 0.60 0.50 0.40 0.30 0.20 0.10 0.00) 
# angle_ini=1.10
# wf_file=wf.${parentocc}.f=1.0.dat

# cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat
# $rotate_orbitals_path  input.in -occ $occ -RIni -RIniSh -angle $angle_ini -orb1 4 -orb2 7 -file_wf ../$wf_file -SIni
# for i in ${fractions[@]}; do 
#     set_frac_occ $i 
#     out_file=SCF_angles.${parentocc}.f=${i}.global_min.out
#     $xmol_angles input.in -occ ${occ} -f_occ ${frac_occ} -MOM -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh | tee $out_file
# done

# data_file="total_energies.${parentocc}.global_min.dat"
# echo "# fraction vs. total energy" > $data_file
# for i in ${fractions[@]}; do
#     out_file="SCF_angles.${parentocc}.f=${i}.global_min.out"
#     if ! [ -f $out_file ]; then continue; fi
#     if [ -n "$(grep 'NOT converged' $out_file)" ]; then 
#         echo "${i}   NaN" | tee -a $data_file
#     else
# #        grep "SCFITER" $out_file | tail -n 1 | sed -r "s/# SCFITER: +[0-9]+ +(\-?[0-9]+[.][0-9]*).*/${i}   \1/" | tee -a $data_file
#         grep "#   Total Energy   =" $out_file | sed -r "s/#   Total Energy   = +(\-?[0-9]+[.][0-9]*).*/${i}  \1/" | tee -a $data_file
#     fi
# done

# # initial solution all frozen but 4,7
# fractions=(1.00 0.90 0.80 0.70 0.60 0.50 0.40 0.30 0.20 0.18 0.15 0.12) 
# wf_file=wf.${parentocc}.f=1.0.dat

# cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat
# for i in ${fractions[@]}; do 
#     set_frac_occ $i 
#     out_file=SCF_angles.${parentocc}.f=${i}.ini47.out
#     $xmol_angles input.in -occ ${occ} -f_occ ${frac_occ} -MOM -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -freeze 1,2,3,5,6,8,9,10 | tee $out_file
# done

# data_file="total_energies.${parentocc}.initial_solution47.dat"
# echo "# fraction vs. total energy" > $data_file
# for i in ${fractions[@]}; do
#     out_file="SCF_angles.${parentocc}.f=${i}.ini47.out"
#     if ! [ -f $out_file ]; then continue; fi
#     if [ -n "$(grep 'NOT converged' $out_file)" ]; then 
#         echo "${i}   NaN" | tee -a $data_file
#     else
# #        grep "SCFITER" $out_file | tail -n 1 | sed -r "s/# SCFITER: +[0-9]+ +(\-?[0-9]+[.][0-9]*).*/${i}   \1/" | tee -a $data_file
#         grep "#   Total Energy   =" $out_file | sed -r "s/#   Total Energy   = +(\-?[0-9]+[.][0-9]*).*/${i}  \1/" | tee -a $data_file
#     fi
# done

# # final solution all frozen but 4,7
# fractions=(0.50 0.40 0.30 0.20 0.10 0.00) 
# wf_file=wf.${parentocc_final}.dat

# cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat
# for i in ${fractions[@]}; do 
#     set_frac_occ $i 
#     out_file=SCF_angles.${parentocc}.f=${i}.fin47.out
#     $xmol_angles input.in -occ ${occ} -f_occ ${frac_occ} -MOM -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh | tee $out_file
# done

# data_file="total_energies.${parentocc}.final_solution47.dat"
# echo "# fraction vs. total energy" > $data_file
# for i in ${fractions[@]}; do
#     out_file="SCF_angles.${parentocc}.f=${i}.fin47.out"
#     if ! [ -f $out_file ]; then continue; fi
#     if [ -n "$(grep 'NOT converged' $out_file)" ]; then 
#         echo "${i}   NaN" | tee -a $data_file
#     else
# #        grep "SCFITER" $out_file | tail -n 1 | sed -r "s/# SCFITER: +[0-9]+ +(\-?[0-9]+[.][0-9]*).*/${i}   \1/" | tee -a $data_file
#         grep "#   Total Energy   =" $out_file | sed -r "s/#   Total Energy   = +(\-?[0-9]+[.][0-9]*).*/${i}  \1/" | tee -a $data_file
#     fi
# done

# # global minimum solution all frozen but 4,7
# fractions=(1.00 0.90 0.80 0.70 0.60 0.50 0.40 0.30 0.20 0.10 0.00) 
# angle_ini=1.10
# wf_file=wf.${parentocc}.f=1.0.dat

# cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat
# $rotate_orbitals_path  input.in -occ $occ -RIni -RIniSh -angle $angle_ini -orb1 4 -orb2 7 -file_wf ../$wf_file -SIni
# for i in ${fractions[@]}; do 
#     set_frac_occ $i 
#     out_file=SCF_angles.${parentocc}.f=${i}.global_min47.out
#     $xmol_angles input.in -occ ${occ} -f_occ ${frac_occ} -MOM -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh | tee $out_file
# done

# data_file="total_energies.${parentocc}.global_min47.dat"
# echo "# fraction vs. total energy" | tee $data_file
# for i in ${fractions[@]}; do
#     out_file="SCF_angles.${parentocc}.f=${i}.global_min47.out"
#     if ! [ -f $out_file ]; then continue; fi
#     if [ -n "$(grep 'NOT converged' $out_file)" ]; then 
#         echo "${i}   NaN" | tee -a $data_file
#     else
#         # grep "SCFITER" $out_file | tail -n 1 | sed -r "s/# SCFITER: +[0-9]+ +(\-?[0-9]+[.][0-9]*).*/${i}   \1/" | tee -a $data_file
#         grep "#   Total Energy   =" $out_file | sed -r "s/#   Total Energy   = +(\-?[0-9]+[.][0-9]*).*/${i}  \1/" | tee -a $data_file
#     fi
# done
