#!/bin/bash

# start in scf_failure_analysis base directory
if [ -f run_examples.source ]; then
    source run_examples.source
    source 0001202.convergence_via_intermediate.source
    rev_angles=$(get_last_num "$(${xmol_angles} -xatom_path ${xatom_path} | grep 'Rev.')")
    script_dir=/home/michael/XMOLECULE/examples/CO/scf_failure_analysis
    calc_dir=calculation/xmolecule_r2652/compare_convergence_smooth
else
    echo "'run_examples.source' wasn't found." 1>&2
    if ! [ "$0" == "bash" ] && ! [  "$0" == "-bash" ] && ! [  "$0" == "/bin/bash" ]; then
        exit
    fi
fi

cd $calc_dir

setup_file=$(pwd)/maketransitions_setup.sh
source $setup_file
input_file=$(pwd)/input.in
echo "setup_file=$setup_file"
echo "input_file=$input_file"


#                           0       1       2       3       4       5       6       7       8       9     
            occ_list=(1001002 1001002 1001002 0001202 0001202 0001202 1001102 1001102 1000202 1000202 )
      parentocc_list=(1011002 2001002 1001102 0011202 1001202 0001212 1002102 1101102 1001202 1100202 )
parentocc_final_list=(1011002 1011002 1011002 0011202 0011202 0011202 1002102 1002102 1001202 1001202 )

for (( j=0; j<${#occ_list[@]}; j++ )); do
    occ=${occ_list[j]}
    parentocc=${parentocc_list[j]}
    parentocc_final=${parentocc_final_list[j]}
    set_occ_array; set_p_occ_array
    echo "### $parentocc to $occ"
    cd $occ

    echo "### determinantal overlap of initial to final"
    cp_parent_data
    get_last_num "$(${overlap_path} input.in -occ ${occ} -RIni -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat)" > olap.ini.${parentocc}.${occ}

    echo "### orbital overlap matrix of initial to final"
    out_file=output.${rev_angles}.OREF.ini.${parentocc}.angles
    cp_parent_data
    $xmol_angles input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -freeze 1,2,3,4,5,6,7,8,9,10 -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat > $out_file

    echo "### initial Fock matrix"
    out_file=output.${rev_angles}.fock.ini.${parentocc}.angles
    cp_parent_data
    $xmol_angles input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -freeze 1,2,3,4,5,6,7,8,9,10 > $out_file

# out_file=output.${rev}.${c}.${parentocc}
    if ! [ -f output.${rev}.cmd5.${parentocc} ]; then
        echo "### calculation of fallback smooth with xmolecule"
        cd ..
        set_atom_conf 
        link_basis_and_atomic_transitions 
        calculate_ionized_conf_smooth
        cd $occ
    fi
#   calculate_parentocc_occ

    cd ..
done

echo "### Check convergence via intermediate configuration $I$ calculated in a subspace of the orbitals"
for (( j=0; j<${#occ_list[@]}; j++ )); do
    occ=${occ_list[$j]}
    parentocc=${parentocc_list[$j]}
    parentocc_final=${parentocc_final_list[$j]}
    set_occ_array; set_p_occ_array
    c="freeze"

    cd $occ

    for executable in "$xmolecule_path" "$xmol_angles"; do
        if [ "$executable" == "$xmolecule_path" ]; then
            out_file=output.${rev}.${c}.${parentocc}
            dat_file="subspace_convergence.${parentocc}.xmolecule.dat" 
            rev_executable=$rev
        elif [ "$executable" == "$xmol_angles" ]; then
            out_file=output.${rev_angles}.${c}.${parentocc}.angles
            dat_file="subspace_convergence.${parentocc}.xmol_angles.dat" 
            rev_executable=$rev_angles
        fi

        printf "# %20s %20s %20s %20s %20s %20s %20s \n" '$subspace1' '$subspace2' '$orthogonal_subspace' '$subspace_MOM' '$converged' '$olap_int' '$olap_fin' | tee $dat_file

        echo "# $3,7$; noMOM in subspace"
        subspace1="3,7"
        freeze_list1="1,2,4,5,6,8,9,10"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="no"
        converged=""
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# $3,7$; prepend orthogonal"
        subspace1="3,7"
        freeze_list1="1,2,4,5,6,8,9,10"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="yes"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# $3,4$; noMOM in subspace"
        subspace1="3,4"
        freeze_list1="1,2,5,6,7,8,9,10"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="no"
        converged=""
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# $3,4$; prepend orthogonal"
        subspace1="3,4"
        freeze_list1="1,2,5,6,7,8,9,10"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="yes"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# $4,7$"
        subspace1="4,7"
        freeze_list1="1,2,3,5,6,8,9,10"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="no"
        converged=""
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# $4,7$; prepend orthogonal"
        subspace1="4,7"
        freeze_list1="1,2,3,5,6,8,9,10"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="yes"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# $7,10$; "
        subspace1="7,10"
        freeze_list1="1,2,3,4,5,6,8,9"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="no"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# $7,10$; prepend orthogonal"
        subspace1="7,10"
        freeze_list1="1,2,3,4,5,6,8,9"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="yes"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate
        
        echo "# $3,7,10$"
        subspace1="3,7,10"
        freeze_list1="1,2,4,5,6,8,9"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="no"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# $3,7,10$; prepend orthogonal"
        subspace1="3,7,10"
        freeze_list1="1,2,4,5,6,8,9"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="yes"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# $4,7,10$"
        subspace1="4,7,10"
        freeze_list1="1,2,3,5,6,8,9"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="no"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# $4,7,10$; prepend orthogonal"
        subspace1="4,7,10"
        freeze_list1="1,2,3,5,6,8,9"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="yes"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# first $7,10$, then $4,7$"
        subspace1="7,10"
        freeze_list1="1,2,3,4,5,6,8,9"
        subspace_MOM="no"
        subspace2="4,7"
        freeze_list2="1,2,3,5,6,8,9,10"
        orthogonal_subspace="no"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# first $7,10$, then $4,7$; prepend orthogonal"
        subspace1="7,10"
        freeze_list1="1,2,3,4,5,6,8,9"
        subspace_MOM="no"
        subspace2="4,7"
        freeze_list2="1,2,3,5,6,8,9,10"
        orthogonal_subspace="yes"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# first $4,7$, then $7,10$"
        subspace1="4,7"
        freeze_list1="1,2,3,5,6,8,9,10"
        subspace_MOM="no"
        subspace2="7,10"
        freeze_list2="1,2,3,4,5,6,8,9"
        orthogonal_subspace="no"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# first $4,7$, then $7,10$; prepend orthogonal"
        subspace1="4,7"
        freeze_list1="1,2,3,5,6,8,9,10"
        subspace_MOM="no"
        subspace2="7,10"
        freeze_list2="1,2,3,4,5,6,8,9"
        orthogonal_subspace="yes"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# first $7,10$, then $3,7$"
        subspace1="7,10"
        freeze_list1="1,2,3,4,5,6,8,9"
        subspace_MOM="no"
        subspace2="3,7"
        freeze_list2="1,2,4,5,6,8,9,10"
        orthogonal_subspace="no"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# first $7,10$, then $3,7$; prepend orthogonal"
        subspace1="7,10"
        freeze_list1="1,2,3,4,5,6,8,9"
        subspace_MOM="no"
        subspace2="3,7"
        freeze_list2="1,2,4,5,6,8,9,10"
        orthogonal_subspace="yes"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# first $3,7$, then $7,10$"
        subspace1="3,7"
        freeze_list1="1,2,4,5,6,8,9,10"
        subspace_MOM="no"
        subspace2="7,10"
        freeze_list2="1,2,3,4,5,6,8,9"
        orthogonal_subspace="no"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# first $3,7$, then $7,10$; prepend orthogonal"
        subspace1="3,7"
        freeze_list1="1,2,4,5,6,8,9,10"
        subspace_MOM="no"
        subspace2="7,10"
        freeze_list2="1,2,3,4,5,6,8,9"
        orthogonal_subspace="yes"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate

        echo "# $3,4,7,10$"
        subspace1="3,4,7,10"
        freeze_list1="1,2,5,6,8,9"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="no"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate


        echo "# $3,4,7,10$; prepend orthogonal"
        subspace1="3,4,7,10"
        freeze_list1="1,2,5,6,8,9"
        subspace_MOM="no"
        subspace2="-"
        freeze_list2="-"
        orthogonal_subspace="yes"
        converged="-"
        olap_int=""
        olap_fin=""
        convergence_via_intermediate
    done
    cd ..

done &> log

### check for differences in convergence via intermediate
for (( j=0; j<${#occ_list[@]}; j++ )); do
    occ=${occ_list[j]}
    parentocc=${parentocc_list[j]}
    parentocc_final=${parentocc_final_list[j]}
    set_occ_array; set_p_occ_array
    echo "### $parentocc to $occ: Check convergence via intermediate"
    cd $occ
    
    for file in subspace_convergence.${parentocc}.xmol{_angles,ecule}.dat; do
        if ! [ -f $file ]; then
            echo "${file} does not exist." 1>&2
            break
        fi
    done
    diff -u subspace_convergence.${parentocc}.xmol{_angles,ecule}.dat

    cd ..
done | less

### check for differences in convergence and converged fractions
for (( j=0; j<${#occ_list[@]}; j++ )); do
    occ=${occ_list[j]}
    parentocc=${parentocc_list[j]}
    parentocc_final=${parentocc_final_list[j]}
    set_occ_array; set_p_occ_array
    echo "### $parentocc to $occ: Check convergence and converged fractions"
    cd $occ
    
    for c in cmd cmd5; do
        # out_file=output.${rev}.${c}.${parentocc}
        # if [ -n "$(grep 'NOT converged' $out_file)" ]; then
        #     echo "Calculation in $out_file didn't converge."
        # fi
        # out_file=output.${rev_angles}.${c}.${parentocc}.angles
        # if [ -n "$(grep 'NOT converged' $out_file)" ]; then
        #     echo "Calculation in $out_file didn't converge."
        # fi    
        diff -u <(grep 'NOT converged' output.${rev}.${c}.${parentocc}) \
                <(grep 'NOT converged' output.${rev_angles}.${c}.${parentocc}.angles)
    done

    echo "### check converged fractions"
    c=cmd5
    # for out_file in output.${rev}.${c}.${parentocc} output.${rev_angles}.${c}.${parentocc}.angles; do
    #     echo "${out_file}:"
    #     check_convergence_fractions
    # done
    diff -u <(out_file=output.${rev}.${c}.${parentocc}; check_convergence_fractions) \
            <(out_file=output.${rev_angles}.${c}.${parentocc}.angles; check_convergence_fractions)
    
    cd ..
done | less

### check for differences in overlap of initial to final determinantal overlap
for (( j=0; j<${#occ_list[@]}; j++ )); do
    occ=${occ_list[j]}
    parentocc=${parentocc_list[j]}
    parentocc_final=${parentocc_final_list[j]}
    set_occ_array; set_p_occ_array
    echo "### $parentocc to $occ: Check determinantal overlap."
    cd $occ

    c=cmd
    out_file=output.${rev}.${c}.${parentocc}
    if [ -n "$(grep 'NOT converged' $out_file)" ]; then
        echo "Calculation in $out_file didn't converge."
    else
        if [ -f olap.${c}.${parentocc}.${occ} ] && [ -f olap.${c}.${parentocc}.${occ}.angles ]; then
            diff olap.${c}.${parentocc}.${occ}{"",.angles}        
        else
            for file in olap.${c}.${parentocc}.${occ}{"",.angles}; do
                if ! [ -f $file ]; then
                    echo "${file} does not exist." 1>&2
                fi
            done
        fi
    fi
    
    cd ..
done

### check for differences in results.txt between two calculation folders
occ_list=(1001002 0001202 1001102 1000202)

# xmolecule
suffixes=("" "")

directories=(redo_calculation redo_calculation_tmux)
out_file="compare_term_tmux.log"
compare_calculation_results | tee ${out_file}

directories=(redo_calculation redo_calculation_ansi-term)
out_file="compare_term_ansi-term.log"
compare_calculation_results | tee ${out_file}

directories=(redo_calculation_tmux redo_calculation_ansi-term)
out_file="compare_tmux_ansi-term.log"
compare_calculation_results | tee ${out_file}

# angles
suffixes=(".angles" ".angles")

directories=(redo_calculation redo_calculation_tmux)
out_file="compare_term_tmux.log"
compare_calculation_results | tee ${out_file}

directories=(redo_calculation redo_calculation_ansi-term)
out_file="compare_term_ansi-term.log"
compare_calculation_results | tee ${out_file}

directories=(redo_calculation_tmux redo_calculation_ansi-term)
out_file="compare_tmux_ansi-term.log"
compare_calculation_results | tee ${out_file}

### compare to calculation in '${ref_dir}'
occ_list=(1001002 0001202 1001102 1000202)
directories=(${ref_dir})

# xmolecule
directories=(${ref_dir})
suffixes=("" "")
for directory in redo_calculation redo_calculation_tmux redo_calculation_ansi-term; do    
    directories[1]=$directory
    compare_calculation_results
    echo
done | less

# angles
directories=(${ref_dir})
suffixes=("" ".angles")
for directory in redo_calculation redo_calculation_tmux redo_calculation_ansi-term; do    
    directories[1]=$directory
    compare_calculation_results
    echo
done | less
