# Gnuplot script to plot spline vs data and also derivatives

# spline_file="transitions/1001002/scan_rotation.1001102.f=0.20.spline"
# data_file="transitions/1001002/scan_rotation.1001102.f=0.20.dat"
# out_file="transitions/1001002/scan_rotation.1001102.f=0.20.spline.png"
# der_file="scan_rotation.1001102.f=0.20.derivative"
# der_spline_file="scan_rotation.1001102.f=0.20.derivative.spline"

set xrange [-pi/2:pi/2]
set ytics format "%.2f"

# set terminal pngcairo size 1440,640
set terminal pngcairo size 2880,1280
set output out_file

set multiplot layout 2,3 columnsfirst
unset key

# spline and data
set ylabel "Total energy and its spline [H]"

plot data_file u (-$1):2 lt 3, \
     spline_file u (-$1):2 with line lt 1

unset ylabel

# first derivative
set ylabel "first derivative of spline [H/rad]"
set arrow 1 from -pi/2,0 to pi/2,0 nohead
set xlabel "phi [rad]"

plot der_file u (-$1):(2*$3) lt 3 , \
     spline_file u (-$1):(-$3) with line lt 1 

unset arrow 1
unset ylabel
unset xlabel

# second derivative
set ylabel "second derivative of spline [H/rad^2]"
set arrow 1 from -pi/2,0 to pi/2,0 nohead

plot spline_file u (-$1):4 with line lt 1, \
     der_spline_file u (-$1):(-2*$3) with lines lt 2

# plot   der_spline_file u (-$1):(-2*$3) with lines

unset arrow 1
unset ylabel

# third derivative
set ylabel "third derivative of spline [H/rad^3]"
set xlabel "phi [rad]"
set arrow 1 from -pi/2,0 to pi/2,0 nohead

plot spline_file u (-$1):(-$5) with line lt 1 , \
     der_spline_file u (-$1):(2*$4) with lines lt 2

unset arrow 1
unset ylabel
unset xlabel


# convergence factor and first derivative

set xlabel "phi [rad]"
set arrow 1 from -pi/2, second 0 to pi/2, second 0 nohead
set ytics nomirror
set y2tics
set yrange [0:10]
set ylabel "convergence factor 0.5*abs(f''/f')"
set y2label "first derivative [H/rad]"

plot spline_file u (-$1):(abs($5/$4)/2) w lines lt 1, \
     "" u (-$1):(-$3) w lines axis x1y2 

unset arrow 1
unset xlabel
unset y2tics
unset yrange
unset ylabel
unset y2label

# end of plot
unset multiplot
