### in case file is loaded in gnuplot
# reset

if(!exists("input_file")) { 
  print "variable input_file not given -> exit"
  exit                      
} 
if(!exists("output_file")) { 
  print "variable output_file not given -> exit"
  exit                      
} 

set terminal pngcairo size 640,640
set output output_file

### no legend of linestyles
unset key
### x and y labels
set xlabel "angle between orb 3 and 4 [rad]"
set ylabel "angle between orb 4 and 7 [rad]"

### draw contour on xy plane
### set contour {base | surface | both}
set contour base

### view as map on xy plane
### set view map {scale <scale>}
set view map

### no surface drawn; in view map the surface is only seen as a grid
unset surface

### get stats of z values; z_min and z_max are set here
stats input_file u 3 name "z" nooutput

### get stats in different regions of plot using ranges
### stats {<ranges>} 'filename' {matrix | using N{:M}} {name 'prefix'} {{no}output}
### this doesn't work out, because the range can only be used for a column; instead use sth like
### (haven't found a way to compare the values from different statnames, because I haven't found
###  a way of printing variables when the name is dynamically generated as is the case with stats)
# x_min=-1.6;x_max=1.6;steps=10;x_int=(x_max-x_min)/steps
# do for [i=0:steps-1] {
# x_0=x_min+i*x_int
# statname=sprintf("stat%i",i)
# stats "scan_rotation.1001102.2angles.out" u ($2< x_0+x_int && $2 > x_0 ? $1 : 1/0):3 name statname nooutput
# }

### textbox for labels without borders and with opaque background 
set style textbox opaque margins  0.5,  0.5 noborder

### style of contourlabels
### set cntrlabel {format "format"} {font "font"} {start <int>} {interval <int>} {onecolor}
### start is the linesegement at which label is drawn, interval is the repetion rate
### (this is important for readibility)
set cntrlabel  format '%8.3g' font ',7' # start 18 interval 20

### contourparameter levels: set numer of contourlines or values of contourlines
### set cntrparam levels { auto {<n>} | <n> | discrete <z1> {,<z2>{,<z3>...}} | incremental <start>, <incr> {,<end>}
# set cntrparam levels 20
# set cntrparam levels incremental  -37.2,0.25,-35
# set cntrparam levels discrete z_max-0.01, -35.2, -35.5, -36., -36.5, -37., -37.5, -37.7, z_min+0.01
# step final
# set cntrparam levels discrete z_max-0.01, -35.5, -36., -36.5, -37., -37.5, z_min+0.01
# step 1.0
# set cntrparam levels discrete z_max-0.01, -44.0, -43.5, -43.0, -42.5, -42.0, z_min+0.01
# step 0.5
# set cntrparam levels discrete z_max-0.01, -41.0, -40.5, -40.0, -39.5, -39.0, z_min+0.01
# step 0.2
#
# step 0.0
# 
# splot input_file with lines, \
#       input_file with labels boxed

#set xrange [-pi/2:pi/2]
#set yrange [-pi/2:pi/2]
set xrange [-1.6:1.6]
set yrange [-1.6:1.6]
set xtics out
set ytics out
set size square 

# substract z_max
# set cntrparam levels discrete -0.01, -0.05, -0.1, -0.2, -0.3, -0.5, -1.0, -1.5, -2.0, -2.5, z_min-z_max+0.01, z_min-z_max+0.05
# set cntrparam levels discrete -0.01, -0.05, -0.1, -0.2, -0.3, -0.5, -1.0, -1.5, -2.0, -2.3, -2.4, -2.45, -2.5, -2.55, -2.56, -2.57, -2.575, -2.6, -2.7, z_min-z_max+0.01

if(!exists("contour_lines")) {
# contour_lines=" -0.01, -0.05, -0.1, -0.2, -0.3, -0.5, -1.0, -1.5, -2.0, -2.5"
# contour_lines=" -0.01 -0.05 -0.1 -0.2 -0.3 -0.5 -1.0 -1.5 -2.0 -2.5 "
  contour_lines=" -0.01, -0.05, -0.1, -0.2, -0.3, -0.5, -1.0, -1.5, -2.0, -2.5"
}
contour_lines=contour_lines.sprintf(" %f %f %f %f %f",z_min-z_max+0.01, z_min-z_max+0.05, z_min-z_max+0.1, z_min-z_max+0.2, z_min-z_max+0.3, z_min-z_max+0.5)
print contour_lines

set for [i=1:words(contour_lines)] cntrparam levels discrete word(contour_lines, i)
show cntrparam 

splot input_file u 1:2:($3-z_max) with lines, \
      input_file u 1:2:($3-z_max) with labels boxed
