#!/bin/bash
#----------------------------------------------------------------------------
# setup file - takes computer specific data like the paths to xmolecule and other programs
#            and variables that are molecule specific to setup calculation with
#            maketransitions.sh
#
#### to adapt to other molecules, you will have to change
#### the following variables:
#      occ_neutral - to the occupation of the neutral molecule     
#      atoms - array of atom names
#      atoms_conf_neutral - array of neutral configurations of each atom, like 1s2_2s2_2p2 for C
#            (K-hole configurations, like 1s1 and 1s0, are generated from this)
#      atoms_Kshell_MO_index - array of index's of molecular orbital which is equivalent to
#              atoms K-shell orbital
#      atoms_options - array of options used to calculate atoms with xatom
#### the following functions might have to be changed/extended in maketransitions_functions.sh
#      set_atom_conf - function that uses the arrays atoms_Kshell_MO_index and atoms_conf_neutral and
#            atoms and the string occ to set the array atom_conf 
#            (negative Kshell_MO_index if the atoms_conf_neutral value is not supposed to be changed)
#      calculate_basis_and_atomic_transitions and link_basis_and_atomic_transitions -
#            cannot differentiate between two atoms of the same species
#### the following functions included in this setup file are still CO specific:
#      exclude_occ_due_to_sym - function that chooses which occupations
#            can be excluded from calculation (in case of CO because of
#            symmetry of Pi-orbitals)
#
# $Id$
#----------------------------------------------------------------------------

# paths to executables (please do not use ~ in double quotes)
xmolecule_path="/home/michael/XMOLECULE/src/xmolecule"
overlap_path="/home/michael/XMOLECULE/src/overlap"
rotate_orbitals_path="/home/michael/XMOLECULE/src/rotate_orbitals"
xatom_path="/home/michael/XATOM/src/xatom"

# options for calling xmolecule
input_file="input.in"
# occupation of neutral molecule
occ_neutral=2222222
# options for neutral molecule
optionsneutral=" -DIIS -SIni  -population"
# optionsneutral=" -DIIS -SIni  -population"
# first try to converge with these options
optionsionized=" -NODIIS -RIni -RIniSh -SIni -MOM -LevelShift 0.2 -SwitchToDIIS 0.01  -population "
# fall back options 
optionsionized0=" -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -SIni -MOM   -population "                  
# Process data (comment if unnecessary)
PHOTON_ENERGY=540
options_transitions="$optionsionized -pcs -fluorescence -auger -PE $PHOTON_ENERGY"

### arrays specifying options for atoms
atoms=("C" "O")
atoms_conf_neutral=("1s2_2s2_2p2" "1s2_2s2_2p4")
atoms_Kshell_MO_index=(1 0) # counting of MO index starts with 0
# options for calling xatom for each atom in array atoms
atoms_options[0]="-N 100 -lmax 3 -rmax 40 -latter false -silent -nao -transition auger -transition transitiondipole -PE_start 10 -PE_end 600 -dE 1" 
atoms_options[1]="-N 100 -lmax 3 -rmax 40 -latter false -silent -nao -transition auger -transition transitiondipole -PE_start 10 -PE_end 600 -dE 1 "

exclude_occ_due_to_sym() {
    ################################################################################
    # mol. orbitals (MOs) 5 and 6 of CO are degenerate
    # therefore regarding the subspace of all possible occupations
    # of MOs 5 and 6 out of 9 possibilities we can exclude 4
    ### returns: ###################################################################
    # returns 2 if occupation should be included
    # returns 0 if it should be excluded
    ################################################################################
    local occupation    
    if [ -z ${1+x} ]
    then
        echo "Pass ${FUNCNAME[0]} occupation string as argument -> exit"
        return 1
    fi
    occupation=$1

    if [ ${occupation:4:1} -lt 2 ] && [ ${occupation:5:1} -gt 0 ]; then
        return 0; # unix yes
    else
        return 2; # unix no
    fi    
}
# exclude_occ_due_to_sym() {
#     ################################################################################
#     # implementation if no symmetry should be used
#     ### returns: ###################################################################
#     # returns 2 if occupation should be included
#     # returns 0 if it should be excluded
#     ################################################################################
#     # local occupation    
#     # if [ -z ${1+x} ]
#     # then
#     #     echo "Pass ${FUNCNAME[0]} occupation string as argument -> exit"
#     #     return 1
#     # fi
#     # occupation=$1
#     return 2
# }

################################################################################
# The following variables are optional
# 
# reference values for CO and acceptible deviation for tests
# states origin of reference value
# value of acceptible deviation chosen 
stats_text=" are reference values for CO"
stats_total=(5016 0) # Ludger
stats_DIIS=(4996 10) # Ludger
stats_DIIS_small=(149 10) # Ludger, ambigous value was 151
stats_DIIS_improved=(113 10) # calculated with Rev. 2693
stats_DIIS_improved_big=(56 10) # Ludger
stats_DIIS_not_converged=(20 5) # Ludger
stats_DIIS_not_converged_smooth=(15 5) # Ludger
stats_DIIS_not_converged_smooth_big=(1 5) # Ludger
stats_smooth_converged=(160 10) # calculated with Rev. 2693
stats_smooth_not_converged=(8 5) # calculated with Rev. 2693
stats_not_converged=(5 3) # Ludger
stats_different_energies=(61 20) # calculated with Rev. 2693
stats_converged_wrong=(79 20) # calculated with Rev. 2693
stats_overlap_small=(108 20) # calculated with Rev. 2693 and by summing values from Ludger


