#!/bin/bash

#### Should be sourced in to run_examples.sh ####
# else you have to locate run_examples.source and source it:
# source run_examples.source

cd calculation/failure_analysis-angles_3481/
setup_file=$(pwd)/maketransitions_setup.sh
source $setup_file
input_file=$(pwd)/input.in
echo "setup_file=$setup_file"
echo "input_file=$input_file"

occ=1001002
parentocc=1001102
parentocc_final=1011002
set_occ_array; set_p_occ_array
wf_file=../1001102/wf.dat
cd $occ/${parentocc}to${occ}/

############################################################
# plot energy with 2 angles between 7,10 and 4,7
fractions=(1.0 0.0)
plot_file_2angles="${script_dir}/gnuplot/scan_rotation_2angles.plot"
options_rotate_2angles="-RIni -RIniSh -angle_step 0.1 -angle_max 1.6 -orb1 7 -orb2 10 -angle_step2 0.05 -angle_max2 1.6 -orb3 4 -orb4 7"

cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat
for i in ${fractions[@]}; do
    out_file=scan_rotation.2angles.${parentocc}.f=${i}.dat
    echo "# wf_file=../${wf_file}" > ${out_file}
    set_frac_occ $i
    ${rotate_orbitals_path} input.in -occ ${occ} -f_occ ${frac_occ} $options_rotate_2angles  | tee ${out_file}
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.dat/.png}';xmax=1.6;xshift=0.0" ${plot_file_2angles}
done

# adjust contourlines
i=1.0
out_file=scan_rotation.2angles.${parentocc}.f=${i}.dat
contour_lines=" -0.01, -0.05, -0.1, -0.2, -0.3, -0.5, -1.0, -1.5, -1.9, -1.95, -1.955, -1.9575, -1.99 "
gnuplot -e "input_file='${out_file}';output_file='${out_file/.dat/.png}';xmax=1.6;xshift=0.0;contour_lines='${contour_lines}';" ${plot_file_2angles}

i=0.0
out_file=scan_rotation.2angles.${parentocc}.f=${i}.dat
contour_lines=" -0.01, -0.05, -0.1, -0.2, -0.3, -0.5, -1.0, -1.5, -2.0, -2.3, -2.4, -2.45, -2.5, -2.55, -2.56, -2.57, -2.575, -2.6, -2.7 "
gnuplot -e "input_file='${out_file}';output_file='${out_file/.dat/.png}';xmax=1.6;xshift=0.0;contour_lines='${contour_lines}';" ${plot_file_2angles}

############################################################
# try to first optimize 4,7 with orthogonal subspace first
executable="$xmol_angles"
rev_executable=$rev_angles
converged="-"
olap_ort1=""
olap_int1=""
olap_ort2=""
olap_int2=""
olap_fin=""
cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat

c="freeze4,7"
out_file=output.${rev_angles}.${c}.${parentocc}.angles
orthogonal_subspace="yes"
subspace1="4,7"
freeze_list1="1,2,3,5,6,8,9,10"

if [ "$orthogonal_subspace" == "yes" ]; then
    ${executable} input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -freeze $subspace1 -MOM -SIni
    olap_ort1=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0")
fi 
${executable} input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -freeze $freeze_list1      -SIni  | tee $out_file
olap_int1=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0")

# Use rotate_orbitals with one angle
# {
#     options_rotate_1angle="-RIni -RIniSh -orb1 4 -orb2 7"
#     plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
#     out_file=scan_rotation.${parentocc}.offset4,7.out
#     echo "# wf_file=../${wf_file}" > ${out_file}
#     ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
#     ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
#     grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
#     grep -v "#" $out_file > ${out_file/.out/.dat}
#     # 1d total energy surface
#     gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
# }
# # Use rotate_orbitals with two angles
{
    plot_file_2angles="${script_dir}/gnuplot/scan_rotation_2angles.plot"
    options_rotate_2angles="-RIni -RIniSh -angle_step 0.1 -angle_max 1.6 -orb1 7 -orb2 10 -angle_step2 0.1 -angle_max2 1.6 -orb3 4 -orb4 7"

    out_file=scan_rotation.2angles.${parentocc}.offset4,7.dat
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_2angles  | tee ${out_file}
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.dat/.png}';xmax=1.6;xshift=0.0" ${plot_file_2angles}
}

c="freeze-final"
out_file=out_file=output.${rev_angles}.${c}.${parentocc}.angles
${executable} input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -MOM -SIni | tee $out_file
olap_final=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0")

# Use rotate_orbitals with one angle
# {
#     options_rotate_1angle="-RIni -RIniSh -orb1 4 -orb2 7"
#     plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
#     out_file=scan_rotation.${parentocc}.offset4,7-final.out
#     echo "# wf_file=../${wf_file}" > ${out_file}
#     ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
#     ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
#     grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
#     grep -v "#" $out_file > ${out_file/.out/.dat}
#     # 1d total energy surface
#     gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
# }
# Use rotate_orbitals with two angles
{
    plot_file_2angles="${script_dir}/gnuplot/scan_rotation_2angles.plot"
    options_rotate_2angles="-RIni -RIniSh -angle_step 0.1 -angle_max 1.6 -orb1 7 -orb2 10 -angle_step2 0.1 -angle_max2 1.6 -orb3 4 -orb4 7"

    out_file=scan_rotation.2angles.${parentocc}.offset4,7-final.dat
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_2angles  | tee ${out_file}
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.dat/.png}';xmax=1.6;xshift=0.0" ${plot_file_2angles}
}
echo "overlap to final after orthogonal subspace1    : $olap_ort1"
echo "overlap to final after            subspace1    : $olap_int1"
echo "overlap to final after unrestricted calculation: $olap_final"

############################################################
# try 7,10, then 4,7 with orthogonal subspace first
executable="$xmol_angles"
rev_executable=$rev_angles
converged="-"
olap_ort1=""
olap_int1=""
olap_ort2=""
olap_int2=""
olap_fin=""
cp ../$wf_file wf.dat; cp ../${wf_file/wf/basis} basis.dat

# Use rotate_orbitals with one angle
# {
#     options_rotate_1angle="-RIni -RIniSh -orb1 4 -orb2 7"
#     plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
#     out_file=scan_rotation.${parentocc}.offset0.orbitals4,7.out
#     echo "# wf_file=../${wf_file}" > ${out_file}
#     ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
#     ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
#     grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
#     grep -v "#" $out_file > ${out_file/.out/.dat}
#     # 1d total energy surface
#     gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
# }
# Use rotate_orbitals with one angle
# {
#     options_rotate_1angle="-RIni -RIniSh -orb1 7 -orb2 10"
#     plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
#     out_file=scan_rotation.${parentocc}.offset0.orbitals7,10.out
#     echo "# wf_file=../${wf_file}" > ${out_file}
#     ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
#     ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
#     grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
#     grep -v "#" $out_file > ${out_file/.out/.dat}
#     # 1d total energy surface
#     gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
# }
# # Use rotate_orbitals with two angles
# {
#     plot_file_2angles="${script_dir}/gnuplot/scan_rotation_2angles.plot"
#     options_rotate_2angles="-RIni -RIniSh -angle_step 0.1 -angle_max 1.6 -orb1 7 -orb2 10 -angle_step2 0.1 -angle_max2 1.6 -orb3 4 -orb4 7"
# 
#     out_file=scan_rotation.2angles.${parentocc}.offset0.dat
#     ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_2angles  | tee ${out_file}
#     gnuplot -e "input_file='${out_file}';output_file='${out_file/.dat/.png}';xmax=1.6;xshift=0.0" ${plot_file_2angles}
# }
 
##########
c="freeze7,10-"
out_file=output.${rev_angles}.${c}.${parentocc}.angles
orthogonal_subspace="yes"
subspace1="4,7,10"
freeze_list1="1,2,3,4,5,6,8,9"

if [ "$orthogonal_subspace" == "yes" ]; then
    ${executable} input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -freeze $subspace1 -MOM -SIni
    olap_ort1=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0")
fi 
# Use rotate_orbitals with one angle
{
    options_rotate_1angle="-RIni -RIniSh -orb1 4 -orb2 7"
    plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
    out_file=scan_rotation.${parentocc}.orthogonal4,7,10.orbitals4,7.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
}
# Use rotate_orbitals with one angle
{
    options_rotate_1angle="-RIni -RIniSh -orb1 7 -orb2 10"
    plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
    out_file=scan_rotation.${parentocc}.orthogonal4,7,10.orbitals7,10.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
}
# # Use rotate_orbitals with two angles
{
    plot_file_2angles="${script_dir}/gnuplot/scan_rotation_2angles.plot"
    options_rotate_2angles="-RIni -RIniSh -angle_step 0.1 -angle_max 1.6 -orb1 7 -orb2 10 -angle_step2 0.1 -angle_max2 1.6 -orb3 4 -orb4 7"

    out_file=scan_rotation.2angles.${parentocc}.orthogonal4,7,10.dat
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_2angles  | tee ${out_file}
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.dat/.png}';xmax=1.6;xshift=0.0" ${plot_file_2angles}
}

${executable} input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -freeze $freeze_list1      -SIni  | tee $out_file
olap_int2=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0")
echo $olap_int2

# Use rotate_orbitals with one angle
{
    options_rotate_1angle="-RIni -RIniSh -orb1 4 -orb2 7"
    plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
    out_file=scan_rotation.${parentocc}.freeze7,10-.orbitals4,7.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
}
# Use rotate_orbitals with one angle
{
    options_rotate_1angle="-RIni -RIniSh -orb1 7 -orb2 10"
    plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
    out_file=scan_rotation.${parentocc}.freeze7,10-.orbitals7,10.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
}
# # Use rotate_orbitals with two angles
{
    plot_file_2angles="${script_dir}/gnuplot/scan_rotation_2angles.plot"
    options_rotate_2angles="-RIni -RIniSh -angle_step 0.1 -angle_max 1.6 -orb1 7 -orb2 10 -angle_step2 0.1 -angle_max2 1.6 -orb3 4 -orb4 7"

    out_file=scan_rotation.2angles.${parentocc}.freeze7,10-.dat
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_2angles  | tee ${out_file}
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.dat/.png}';xmax=1.6;xshift=0.0" ${plot_file_2angles}
}

##########
c="freeze7,10-freeze4,7"
out_file=output.${rev_angles}.${c}.${parentocc}.angles
orthogonal_subspace="yes"
subspace1="4,7,10"
freeze_list1="1,2,3,5,6,8,9,10"

if [ "$orthogonal_subspace" == "yes" ]; then
    ${executable} input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -freeze $subspace1 -MOM -SIni
    olap_ort2=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0")
fi 
${executable} input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -freeze $freeze_list1      -SIni  | tee $out_file
olap_int2=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0")

# Use rotate_orbitals with one angle
{
    options_rotate_1angle="-RIni -RIniSh -orb1 4 -orb2 7"
    plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
    out_file=scan_rotation.${parentocc}.freeze7,10-4,7.orbitals4,7.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
}
# Use rotate_orbitals with one angle
{
    options_rotate_1angle="-RIni -RIniSh -orb1 7 -orb2 10"
    plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
    out_file=scan_rotation.${parentocc}.freeze7,10-4,7.orbitals7,10.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
}
# Use rotate_orbitals with two angles
{
    plot_file_2angles="${script_dir}/gnuplot/scan_rotation_2angles.plot"
    options_rotate_2angles="-RIni -RIniSh -angle_step 0.1 -angle_max 1.6 -orb1 7 -orb2 10 -angle_step2 0.1 -angle_max2 1.6 -orb3 4 -orb4 7"

    out_file=scan_rotation.2angles.${parentocc}.freeze7,10-4,7.dat
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_2angles  | tee ${out_file}
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.dat/.png}';xmax=1.6;xshift=0.0" ${plot_file_2angles}
}

##########
c="freeze-final"
out_file=out_file=output.${rev_angles}.${c}.${parentocc}.angles
${executable} input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -MOM -SIni | tee $out_file
olap_final=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0")

# Use rotate_orbitals with one angle
{
    options_rotate_1angle="-RIni -RIniSh -orb1 4 -orb2 7"
    plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
    out_file=scan_rotation.${parentocc}.freeze-final.orbitals4,7.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
}
# Use rotate_orbitals with one angle
{
    options_rotate_1angle="-RIni -RIniSh -orb1 7 -orb2 10"
    plot_file_1angle="${script_dir}/gnuplot/scan_rotation.plot"
    out_file=scan_rotation.${parentocc}.freeze-final.orbitals7,10.out
    echo "# wf_file=../${wf_file}" > ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle $(echo "scale=10; -$pi/2.0" | bc) -verbose  | tee -a ${out_file}
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_1angle -angle_step 0.1 -angle_max 1.5  -verbose | tee -a ${out_file}
    grep -v "#" $out_file | sed -n "s/-1.57/ 1.57/p" >> $out_file
    grep -v "#" $out_file > ${out_file/.out/.dat}
    # 1d total energy surface
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.out/.png}';xmax=1.6;xshift=0.0" ${plot_file_1angle}
}
# Use rotate_orbitals with two angles
{
    plot_file_2angles="${script_dir}/gnuplot/scan_rotation_2angles.plot"
    options_rotate_2angles="-RIni -RIniSh -angle_step 0.1 -angle_max 1.6 -orb1 7 -orb2 10 -angle_step2 0.1 -angle_max2 1.6 -orb3 4 -orb4 7"

    out_file=scan_rotation.2angles.${parentocc}.freeze-final.dat
    ${rotate_orbitals_path} input.in -occ ${occ} $options_rotate_2angles  | tee ${out_file}
    gnuplot -e "input_file='${out_file}';output_file='${out_file/.dat/.png}';xmax=1.6;xshift=0.0" ${plot_file_2angles}
}
echo "overlap to final after orthogonal subspace1    : $olap_ort1"
echo "overlap to final after            subspace1    : $olap_int1"
echo "overlap to final after orthogonal subspace2    : $olap_ort2"
echo "overlap to final after            subspace2    : $olap_int2"
echo "overlap to final after unrestricted calculation: $olap_final"


