#!/bin/bash

if [ -f run_examples.source ]; then
    source run_examples.source
    rev_angles=$(get_last_num "$(${xmol_angles} -xatom_path ${xatom_path} | grep 'Rev.')")
    script_dir=/home/michael/XMOLECULE/examples/CO/scf_failure_analysis
else
    echo "'run_examples.source' wasn't found." 1>&2
    if ! [ "$0" == "bash" ] && ! [  "$0" == "-bash" ] && ! [  "$0" == "/bin/bash" ]; then
        echo no
    fi
fi

############################################################
if ! [ -d "calculation" ]; then mkdir calculation; fi
cd calculation
# Get the setup file and the input file, which specifiy details of the CO calculation.
# cp $ref_dir/setup.sh maketransitions_setup.sh
cp $maketransitions_dir/maketransitions_setup.sh .
setup_file=$(pwd)/maketransitions_setup.sh
source $setup_file
cp $ref_dir/input.in .
input_file=$(pwd)/input.in
echo "setup_file=$setup_file"
echo "input_file=$input_file"

# xmolecule: retrieving parent data and repetition of all parentocc to occ calculations:
occ_list=(1001002 0001202 1001102 1000202)

for (( j=0; j<${#occ_list[@]}; j++ )); do
    occ=${occ_list[j]}
    # we determine all parents and create directories including xmolecule wf and basis files from ref_dir.
    get_parents
    i=0
    while ! [ -z ${parents[$i]+x} ]; do
        if exclude_occ_due_to_sym ${parents[$i]}; then 
            unset parents[$i]
        fi
        let i+=1
    done
    parents=(${parents[@]})

    for ((i=0; i<${#parents[@]}; i++)); do          
        parentocc=${parents[$i]}
        if ! [ -d "$parentocc" ]; then mkdir $parentocc; fi
        cp $ref_dir/$parentocc/{wf.dat,basis.dat} $parentocc
    done

    # we want to redo the calculation for this occupation! 

    # For this we need the atomic basis functions:
    calculate_basis_and_atomic_transitions

    # Now we will run the calculation of occ from all parents:
    set_atom_conf 
    link_basis_and_atomic_transitions 
    for ((i=0; i<${#parents[@]}; i++)); do          
        parentocc=${parents[$i]}

        calculate_parentocc_occ
        # choose best method
        analyze_parentocc_occ 
    done
    # choose best parent
    analyze_results_all_parents
done

# xmol_angles: repetition of all parentocc to occ calculations
occ_list=(1001002 0001202 1001102 1000202)

for (( j=0; j<${#occ_list[@]}; j++ )); do
    occ=${occ_list[j]}

    # we determine all parents and create directories including xmolecule wf and basis files from ref_dir.
    get_parents
    i=0
    while ! [ -z ${parents[$i]+x} ]; do
        if exclude_occ_due_to_sym ${parents[$i]}; then 
            unset parents[$i]
        fi
        let i+=1
    done
    parents=(${parents[@]})

    for ((i=0; i<${#parents[@]}; i++)); do          
        parentocc=${parents[$i]}
        if ! [ -d "$parentocc" ]; then mkdir $parentocc; fi
        cp $ref_dir/$parentocc/{wf.dat,basis.dat} $parentocc
    done

    # we want to redo the calculation for this occupation! 

    # For this we need the atomic basis functions:
    calculate_basis_and_atomic_transitions

    # Now we will run the calculation of occ from all parents:
    set_atom_conf 
    link_basis_and_atomic_transitions 
    cd $occ
    for ((i=0; i<${#parents[@]}; i++)); do          
        parentocc=${parents[$i]}
        echo "Calculating ${occ} with parent occ ${parentocc}"

        calculate_parentocc_occ_including_olap_mat > /dev/null        
        # choose best method
        analyze_parentocc_occ_angles
    done
    cd ..
    # choose best parent
    analyze_results_all_parents_angles
done

# Combine standard analysis for all cases:
            occ_list=(1001002 1001002 1001002 0001202 0001202 0001202 1001102 1001102 1000202 1000202 1000202 )
      parentocc_list=(1011002 2001002 1001102 0011202 1001202 0001212 1002102 1101102 1001202 2000202 1100202 )
parentocc_final_list=(1011002 1011002 1011002 0011202 0011202 0011202 1002102 1002102 2000202 2000202 2000202 )

# occ_list=(1001002)
# parentocc_list=(1011002)
# parentocc_final_list=(1011002)

for (( j=0; j<${#occ_list[@]}; j++ )); do
    occ=${occ_list[j]}
    parentocc=${parentocc_list[j]}
    parentocc_final=${parentocc_final_list[j]}

    cd $occ

    echo "${parentocc}->${occ}"

    # With xmolecule 
    # calculate_parentocc_occ > /dev/null
    cd ..
    calculate_ionized_conf_DIIS > /dev/null
    calculate_ionized_conf_smooth > /dev/null
    cd $occ
    c=cmd 
    out_file=output.${rev}.${c}.${parentocc}
    if [ -n "$(grep 'NOT converged' $out_file)" ]; then
        echo "Calculation with c=$c with xmolecule didn't converge."
    fi
    c=cmd5
    out_file=output.${rev}.${c}.${parentocc}
    if [ -n "$(grep 'NOT converged' $out_file)" ]; then
        echo "Calculation with c=$c with xmolecule didn't converge."

        last_converged=$(sed '/\# \*\*\* NOT CONVERGED \*\*\*/q' $out_file | grep 'finish f=' | tail -n 1)
        if [ -n "$last_converged" ]; then
            echo "Last converged step was ${last_converged/finish /}"
        else
            echo "Calculation didn't converge for f=1.0."
        fi
    fi
    
    # With modified xmolecule "angles"
    calculate_parentocc_occ_including_olap_mat > /dev/null
    c=cmd 
    out_file=output.${rev_angles}.${c}.${parentocc}.angles
    if [ -n "$(grep 'NOT converged' $out_file)" ]; then
        echo "Calculation with c=$c with xmol_angles didn't converge."
    fi
    c=cmd5
    out_file=output.${rev_angles}.${c}.${parentocc}.angles
    if [ -n "$(grep 'NOT converged' $out_file)" ]; then
        echo "Calculation with c=$c with xmol_angles didn't converge."

        last_converged=$(sed '/\# \*\*\* NOT CONVERGED \*\*\*/q' $out_file | grep 'finish f=' | tail -n 1)
        if [ -n "$last_converged" ]; then
            echo "Last converged step was ${last_converged/finish /}"
            echo "Overlap matrice to reference:"
            sed "/${last_converged}/q" output.${rev_angles}.${c}.${parentocc}.angles | grep "OREF"| tail -n 11 | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*.*" | tee ${out_file/output/OREF}
            echo "Groups of orbitals that mix, with a relative overlap square >= 0.005"
            ${script_dir}/olap_mat_to_subspace.py ${out_file/output/OREF} 0.005
        else
            echo "Calculation didn't converge for f=1.0."
        fi
    fi

    # freeze all but 3,4,7,10

    # xmolecule
    freeze_list="1,2,5,6,8,9"
    # calculate_fallback_freeze > /dev/null
    c="freeze"
    out_file=output.${rev}.${c}.${parentocc}
    cp_parent_data
    {
        echo "Step freeze:" 

        ${xmolecule_path} input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -freeze ${freeze_list} -SIni 

        if [ -n "$(grep 'NOT converged' $out_file)" ]; then 
            echo "Calculation not converged."
        else
            save_basis_wf
            echo "Step final:" 

            ${xmolecule_path} input.in -occ ${occ} -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -SIni -MOM  
            if [ -z "$(grep 'NOT converged' $out_file)" ]; then 
                olap_ini=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref ../${parentocc}/wf.dat -ref_basis  ../${parentocc}.basis.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0")
                olap_fin=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0" )
                echo "Overlap from final optimization (ini) (fin):   $olap_ini    $olap_fin"
                # echo "Overlap ini: $olap_ini"
                # echo "Overlap fin: $olap_fin"
            fi

            # echo "Overlap from freeze:"
            cp basis.${c}.${parentocc}.dat basis.dat; cp wf.${c}.${parentocc}.dat wf.dat
            # echo -n "Overlap ini:"
            # echo -n "Overlap fin:"
            olap_ini=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref ../${parentocc}/wf.dat -ref_basis ../${parentocc}/basis.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0")
            olap_fin=$(${overlap_path} input.in -occ ${occ} -RIni -RIniSh -ref wf.${parentocc_final}.dat -ref_basis basis.${parentocc_final}.dat | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" || echo "0.0")
            echo "Overlap from freeze optimization (ini) (fin):   $olap_ini    $olap_fin"
        fi
    } | tee $out_file > /dev/null
    c=freeze
    out_file=output.${rev}.${c}.${parentocc}
    if [ -n "$(grep 'NOT converged' $out_file)" ]; then
        echo "Calculation with c=$c with xmolecule didn't converge."
    else
        olap_fin=$(grep "Overlap from final optimization (ini) (fin):" $out_file | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" | tail -1)
        echo "olap_fin=$olap_fin"
        if [ $(echo "1-$olap_fin*$olap_fin < 0.1" | bc) == 1 ]; then
            echo "Calculation with c=$c with xmolecule converged to right solution."
        else
            echo "Calculation with c=$c with xmolecule converged, but not to right solution."
        fi
    fi    
    
    # angles 
    freeze_list="1,2,5,6,8,9"
    calculate_fallback_freeze > /dev/null
    c=freeze
    out_file=output.${rev_angles}.${c}.${parentocc}.angles
    if [ -n "$(grep 'NOT converged' $out_file)" ]; then
        echo "Calculation with c=$c with xmol_angles didn't converge."
    else
        olap_fin=$(grep "Overlap from final optimization (ini) (fin):" $out_file | egrep -oE "\-?[0-9]+[.][0-9]*((e|E)(\-|\+)?)?[0-9]*" | tail -1)
        echo "olap_fin=$olap_fin"
        if [ $(echo "1-$olap_fin*$olap_fin < 0.1" | bc) == 1 ]; then
            echo "Calculation with c=$c with xmol_angles converged to right solution."
        else
            echo "Calculation with c=$c with xmol_angles converged, but not to right solution."
        fi
    fi    



    if ! [ -d ${parentocc}to${occ} ]; then mkdir ${parentocc}to${occ}; fi
    cd ${parentocc}to${occ}
    ln -s ../{C,O}.nao .
    write_input_file


    # Calculate energy vs fractional occupation starting from different guesses
    wf_file_ini="../../${parentocc}/wf.dat"
    basis_file_ini=${wf_file_ini/wf/basis}
    wf_file_fin="../wf.${parentocc_final}.dat"
    basis_file_fin=${wf_file_fin/wf/basis}
    # initial solution
    fractions=(1.00 0.90 0.80 0.70 0.60 0.50 0.40 0.30 0.20 0.10 0.00)
    wf_file=$wf_file_ini
    basis_file=${wf_file/wf/basis}
    out_file_suffix="ini.out"
    data_file="total_energies.${parentocc}.ini.dat"
    xmol_angles_options="-MOM -NODIIS -SwitchToDIIS 0.01"
    # calc_total_energies_of_fractions
    calc_total_energies_of_fractions_and_olap_to_ref
    collect_total_energies_of_fractions
    # Collect overlap to references
    data_file="olap_to_ref.${parentocc}.ini.dat"
    collect_olap_to_ref
    # final solution ##########
    fractions=(0.80 0.70 0.60 0.50 0.40 0.30 0.20 0.10 0.00) 
    wf_file=$wf_file_fin
    basis_file=${wf_file/wf/basis}
    out_file_suffix="fin.out"
    data_file="total_energies.${parentocc}.fin.dat"
    xmol_angles_options="-MOM -NODIIS -SwitchToDIIS 0.01"
    # calc_total_energies_of_fractions
    calc_total_energies_of_fractions_and_olap_to_ref
    collect_total_energies_of_fractions
    data_file="olap_to_ref.${parentocc}.fin.dat"
    collect_olap_to_ref
   
    cd ..

    gnuplot -e "parentocc='$parentocc';occ='$occ';" ../../total_energies.ini_and_fin.plot
    gnuplot -e "parentocc='$parentocc';occ='$occ';" ../../olap_to_ref.ini_and_fin.plot

    cd ..
done | tee ${script_dir}/run_examples.out


############################################################
# Now we should be ready to start the investigation!
# Get the list of cases, in which either the standard method (DIIS) or the fallback (smooth) didn't converge.
cp $ref_dir/notconvergedlist.txt .
cat notconvergedlist.txt
# All cases, in which DIIS and smooth both didn't converge are cases we call scf failure. 
# 1101102 -> 1001102
# 2001002 -> 1001002
# 1001102 -> 1001002
# 1100202 -> 1000202 <- wasn't on Ludgers list. he says the exact cases might depend on numerics.
# 1001202 -> 0001202
# 0001212 -> 0001202

############################################################
# Let's look at the case where we try to converge to the occupation 1001002!
occ=1001002
cat $occ/results.txt
# This should look like this, where maybe the last case might not have converged either.
#   2001002 -> 1001002 0.00 0.0
#   1101002 -> 1001002 -35.19397133 -0.9263
# * 1011002 -> 1001002 -35.19397133 -0.9928
#   1002002 -> 1001002 -35.19397133 -0.8150
#   1001102 -> 1001002 -35.19397133 -0.525158674611791

# The directory ${parentocc}to${occ} will contain the files and plots from the analysis.
# E.g. scan_rotation.1001102.*.png, which are the plots of the energy functional 
# considering one angle between MOs 4 and 7
cd $occ

parentocc=1011002 # Converges...
. ./${parentocc}to${occ}.sh

parentocc=2001002
. ./${parentocc}to${occ}.sh

parentocc=1001102
. ./${parentocc}to${occ}.sh

cd ..

# Next is 0001202:
occ=0001202
cat $occ/results.txt
# This should look like this:
#   1001202 -> 0001202 0.00 0.0
#   0101202 -> 0001202 -19.21160386 0.9271
# * 0011202 -> 0001202 -19.21160386 0.9734
#   0002202 -> 0001202 -19.21160386 0.7635
#   0001212 -> 0001202 0.00 0.0

cd $occ

parentocc=0011202
. ./${parentocc}to${occ}.sh

parentocc=1001202
. ./${parentocc}to${occ}.sh

parentocc=0001212
. ./${parentocc}to${occ}.sh

cd ..

# Next is 1001102:
occ=1001102
cat $occ/results.txt
# This should look like this:
#   2001102 -> 1001102 -41.77707859 0.584578818083112
#   1101102 -> 1001102 0.00 0.0
#   1011102 -> 1001102 -41.77707859 0.7829
# * 1002102 -> 1001102 -41.77707859 0.9343
#   1001202 -> 1001102 -41.77707859 0.9113

cd $occ

parentocc=1001202
. ./${parentocc}to${occ}.sh

parentocc=1101102
. ./${parentocc}to${occ}.sh

cd ..

# Next is 1000202
occ=1000202
cat $occ/results.txt
# This should look like this:
#   2000202 -> 1000202 -41.81494541 0.7294
#   1100202 -> 1000202 0.00 0.0
#   1010202 -> 1000202 -41.81494540 0.478737840165354
# * 1001202 -> 1000202 -41.81494540 0.9422
#   1000212 -> 1000202 -41.81494540 0.9364

cd $occ

parentocc=1001202
. ./${parentocc}to${occ}.sh
# This case shows irregularities

parentocc=2000202
. ./${parentocc}to${occ}.sh

parentocc=1100202
. ./${parentocc}to${occ}.sh

cd ..


############################################################
# Old stuff:

# # run maketransitions in sub directory transitions if user says so
# function ask_yes_or_no() {
#     read -p "$1 ([y]es or [N]o): "
#     case $(echo $REPLY | tr '[A-Z]' '[a-z]') in
#         y|yes) echo "yes" ;;
#         *)     echo "no" ;;
#     esac
# }
# if [ $(ask_yes_or_no "Run maketransitions? This will take a quite long time.") == "yes" ]; then
#     cd transitions    
# fi

# #####################################################
# # prepare directory transitions
# #####################################################
# reference_dir=$maketransitions_dir/test_CO

# cd transitions
# cp $reference_dir/maketransitions_setup.sh .
# calculate_basis_and_atomic_transitions
# cd ..

# #####################################################
# # calculate cases of convergence failure at fixed geometry for CO

# # calculate case 1001102 to 1001002
# cd transitions

# mkdir 1001102to1001002
# mkdir 1001102to1001002/1001102
# cp $reference_dir/1001102/{wf.dat,basis.dat} 1001102to1001002/1001102/
# cd 1001102to1001002/1001102
# ln -s ../../O.1s1_2s2_2p4.nao O.nao
# ln -s ../../C.1s0_2s2_2p2.nao C.nao
# xmolecule input.in -occ 1001102 -NODIIS -RIni -RIniSh -MOM -LevelShift 0.2 -SwitchToDIIS 0.01 -population
# # This should converge!!! Then we are ready to start.
# cd ../../..

# . ./1001102to1001002.sh

# # calculate case 2001002 to 1001002
# . ./2001002to1001002.sh

############################################################
# # First we want to redo the calculation for this occupation! 
# # For this we need the atomic basis functions:
# calculate_basis_and_atomic_transitions
# # Then we determine all parents and create directories including xmolecule wf and basis files from ref_dir.
# # The alternative here would be to calculate the whole tree of configurations up to this point.
# get_parents
# i=0
# while ! [ -z ${parents[$i]+x} ]; do
#     if exclude_occ_due_to_sym ${parents[$i]}; then 
#         unset parents[$i]
#     fi
#     let i+=1
# done
# parents=(${parents[@]})
# for ((i=0; i<${#parents[@]}; i++)); do          
#     parentocc=${parents[$i]}
#     if ! [ -d "$parentocc" ]; then mkdir $parentocc; fi
#     cp $ref_dir/$parentocc/{wf.dat,basis.dat} $parentocc
# done

# # Now we will run the calculation of occ from all parents:
# set_atom_conf 
# link_basis_and_atomic_transitions 
# for ((i=0; i<${#parents[@]}; i++)); do          
#     parentocc=${parents[$i]}

#     calculate_parentocc_occ
#     # choose best method
#     analyze_parentocc_occ 
# done
# # choose best parent
# analyze_results_all_parents
