#!/bin/bash
#----------------------------------------------------------------------------
# string_functions.sh - functions that will make working with strings easier
#
# $Id$
#----------------------------------------------------------------------------

get_text_between(){ 
    ################################################################################
    # get lines of text in file $1 between searchterms $2 and $3, excluding the line containing $3
    #
    # use for example to get blocks from maketransitions output between "Different" and "Calculating"
    ################################################################################
    local file search1 search2
        if [ -z "${1}" ] || [ -z "${3}" ] || [ -z "${2}" ]
    then
        echo "Pass ${FUNCNAME[0]} a filename and two searchterms -> exit"
        return 1
    fi    
    file=$1
    search1=$2
    search2=$3
    cat $file | sed -n "/${search1}/,/${search2}/p" | grep -v "${search2}"
}

### extract numbers from a line/string
# - to operate on small files use get_..._num "$(cat file)" [$nth]
# - to get n-th number from all lines in a file containing a string
#   use get_..._num "$(cat file | grep "string") $nth
# - right now only a number starting with a numeral is matched,
#   while sometimes for 0.5 .5 is written
# - how to read lines from stdin? (instead of from input variable)
#   (this might also be possible by using a pipe and xargs with the current functions)

# extract the n-th number
get_nth_num() {
    if [ -z "${1+x}" ]; then
        echo "first argument should be a string -> exit"
    fi
    string="$1"
    if [ -z "${2+x}" ]; then
        nth=1
    else
        nth=$2
    fi
    echo "$string" | egrep -oE  "\-?[0-9]+[.]?[0-9]*((e|E)(\-|\+)?)?[0-9]*" | sed "${nth}q;d"
    # egrep with option -m doesn't seem to produce the described result from help
    # echo "$string" | egrep -oE -m ${nth} "\-?[0-9]+[.]?[0-9]*((e|E)(\-|\+)?)?[0-9]*" | tail -1
}
# extract last number
get_last_num() {
    if [ -z "${1+x}" ]; then
        echo "first argument should be a string -> exit"
    fi
    string=$1
    echo "$string" | egrep -oE "\-?[0-9]+[.]?[0-9]*((e|E)(\-|\+)?)?[0-9]*" | tail -1
}
# get nth to last number
get_nth_to_last_num() {
    if [ -z "${1+x}" ]; then
        echo "first argument should be a string -> exit"
    fi
    string=$1
    if [ -z "${2+x}" ]; then
        nth=1
    else
        nth=$2
    fi
    echo "$string" | egrep -oE "\-?[0-9]+[.]?[0-9]*((e|E)(\-|\+)?)?[0-9]*" | tail -${nth} | head -1
}

