#!/bin/bash
################################################################################
# Run maketransitions.sh from .. for CO and compare the produced statistics
# to previous values. 
#
# A copy of the script and its subscripts will be used to not interfere with ongoing changes.
# 
# You will need to check the output files.
#
# Writes:
#     - maketransitions.error - error stream of maketransitions.sh
#     - statistics.test
# Overwrites:
#     - statistics.txt
#
# $Id$
################################################################################

# record date and time of starting the script for automated backup of the used input files and created output files
# with save_files function 
date_start=$(date +%Y-%m-%d)
time_start=$(date +%k-%m-%S)


for file in maketransitions.sh maketransitions_functions.sh string_functions.sh init.sh; do
    if [ -f "../${file}" ]; then
        cp "../${file}" ${file}
    else
        echo "../${file} doesn't exist, but is necessary for execution."
        yes_exit="yes"
    fi
    if [ "${yes_exit}" == "yes" ]; then
        echo "not all files for execution were found -> exit"
        exit 1
    fi
done

. ./setup.sh
. ./init.sh

save_files maketransitions.sh maketransitions_functions.sh string_functions.sh init.sh


# prepare directory for test
echo "prepare directory"
clean_files_and_directories

# execute maketransitions.sh
# no output to screen
echo "run maketransitions.sh"
bash maketransitions.sh > /dev/null 2> maketransitions.error || { echo "maketransition.sh execution failed -> exit" ; exit 1 ; }
# bash maketransitions.sh || { echo "maketransition.sh execution failed -> exit" ; exit 1 ;} 

# overwrite reference and acceptable deviation values from setup.sh
#
# reference values for CO and acceptible deviation for tests
# states origin of reference value
# value of acceptible deviation chosen 
stats_text=" are reference values for CO"
stats_total=(5016 0) # Ludger
stats_DIIS=(4996 10) # Ludger
stats_DIIS_small=(149 10) # Ludger, ambigous value was 151
stats_DIIS_improved=(113 10) # calculated with Rev. 2693
stats_DIIS_improved_big=(56 10) # Ludger
stats_DIIS_not_converged=(20 5) # Ludger
stats_DIIS_not_converged_smooth=(15 5) # Ludger
stats_DIIS_not_converged_smooth_big=(1 5) # Ludger
stats_smooth_converged=(160 10) # calculated with Rev. 2693
stats_smooth_not_converged=(8 5) # calculated with Rev. 2693
stats_not_converged=(5 3) # Ludger
stats_different_energies=(61 20) # calculated with Rev. 2693
stats_converged_wrong=(79 20) # calculated with Rev. 2693
stats_overlap_small=(108 20) # calculated with Rev. 2693 and by summing values from Ludger

# overwrite statistics.txt
get_stats_of_output_files maketransitions.out notconvergedlist.txt converged_to_wrong_states.txt 
echo "Write statistics:"
write_stats_of_output_files | tee statistics.txt

# test stats
test_stats_of_output_files() {
    format_string="Var. %-40s %15s: Difference is: %10d. Accepted if <= %10d\n." 

    for var in stats_total stats_DIIS stats_DIIS_small stats_DIIS_improved stats_DIIS_improved_big stats_DIIS_not_converged stats_DIIS_not_converged_smooth stats_DIIS_not_converged_smooth_big stats_smooth_converged stats_smooth_not_converged stats_not_converged stats_different_energies stats_converged_wrong stats_overlap_small; do
        eval "let diff=\${${var}[2]}-\${${var}[0]}"
        eval "let dev=\${${var}[1]}"
        if [ ${diff#-} -le ${dev} ]; then
            printf "${format_string}" "\$${var}" "accepted" ${diff#-} ${dev}
        else
            printf "${format_string}" "\$${var}" "not accepted" ${diff#-} ${dev}
        fi
    done
} 

echo "test statistics:"
test_stats_of_output_files | tee statistics.test

# save output files
save_files maketransitions.error statistics.test

