#!/bin/bash
#----------------------------------------------------------------------------
# setup file - takes computer specific data like the paths to xmolecule and other programs
#            and variables that are molecule specific to setup calculation with
#            maketransitions.sh
#
#### to adapt to other molecules, you will have to change
#### the following variables:
#      occ_neutral - to the occupation of the neutral molecule     
#      atoms - array of atom names
#      atoms_conf_neutral - array of neutral configurations of each atom, like 1s2_2s2_2p2 for C
#            (K-hole configurations, like 1s1 and 1s0, are generated from this)
#      atoms_Kshell_MO_index - array of index's of molecular orbital which is equivalent to
#              atoms K-shell orbital
#      atoms_options - array of options used to calculate atoms with xatom
#### the following functions in maketransitions_functions.sh are still CO specific:
#      exclude_occ_due_to_sym - function that chooses which occupations
#            can be excluded from calculation (in case of CO because of
#            symmetry of Pi-orbitals)
#
# $Id$
#----------------------------------------------------------------------------

# paths to executables (please do not use ~ in double quotes)
xmolecule_path="/home/michael/XMOLECULE/src/xmolecule"
overlap_path="/home/michael/XMOLECULE/src/overlap"
rotate_orbitals_path="/home/michael/XMOLECULE/src/rotate_orbitals"
xatom_path="/home/michael/XATOM/src/xatom"

# options for calling xmolecule
input_file="input.in"
# occupation of neutral molecule
occ_neutral=2222222
# options for neutral molecule
optionsneutral=" -DIIS -SIni  -population"
# optionsneutral=" -DIIS -SIni  -population -pcs -fluorescence -auger -PE 540"

# first try to converge with these options
optionsionized=" -NODIIS -RIni -RIniSh -SIni -MOM -LevelShift 0.2 -SwitchToDIIS 0.01  -population "
# fall back options 
optionsionized0=" -NODIIS -SwitchToDIIS 0.01 -RIni -RIniSh -SIni -MOM   -population "                  

### arrays specifying options for atoms
atoms=("C" "O")
atoms_conf_neutral=("1s2_2s2_2p2" "1s2_2s2_2p4")
atoms_Kshell_MO_index=(1 0) # counting of MO index starts with 0
# options for calling xatom for each atom in array atoms
atoms_options[0]="-N 100 -lmax 3 -rmax 40 -latter false -silent -nao -transition auger -transition transitiondipole -PE_start 10 -PE_end 600 -dE 1" 
atoms_options[1]="-N 100 -lmax 3 -rmax 40 -latter false -silent -nao -transition auger -transition transitiondipole -PE_start 10 -PE_end 600 -dE 1 "

################################################################################
# The following variables are optional
# 
# reference values for CO and acceptible deviation for tests
# states origin of reference value
# value of acceptible deviation chosen 
stats_text=" are reference values for CO"
stats_total=(5016 0) # Ludger
stats_DIIS=(4996 10) # Ludger
stats_DIIS_small=(149 10) # Ludger, ambigous value was 151
stats_DIIS_improved=(113 10) # calculated with Rev. 2693
stats_DIIS_improved_big=(56 10) # Ludger
stats_DIIS_not_converged=(20 5) # Ludger
stats_DIIS_not_converged_smooth=(15 5) # Ludger
stats_DIIS_not_converged_smooth_big=(1 5) # Ludger
stats_smooth_converged=(160 10) # calculated with Rev. 2693
stats_smooth_not_converged=(8 5) # calculated with Rev. 2693
stats_not_converged=(5 3) # Ludger
stats_different_energies=(61 20) # calculated with Rev. 2693
stats_converged_wrong=(79 20) # calculated with Rev. 2693
stats_overlap_small=(108 20) # calculated with Rev. 2693 and by summing values from Ludger


