!
! XMOLECULE electronic structure toolkit
! Copyright 2023 Ludger Inhester
! Ludger Inhester, Center for Free-Electron Laser Science, DESY
! ludger.inhester@cfel.de
!
! This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
! See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
!

module elec_config
  !! author: Ludger Inhester
  !!
  !!  module to specify electronic configuration/ occupation pattern in molecules
  !!  copied parts from XATOM to XMOLECULE
  !!
  use constants
  implicit none
  private
  public :: occ_to_index, index_to_occ
contains

  subroutine occ_to_index(occ, i)
    !! maps index to occupation pattern
    integer, intent(in) :: occ(:)
    !integer*8, intent(out) :: i
    integer, intent(out) :: i
    integer :: j, occ_n, N_orb
    i = 0
    N_orb = size(occ)
    do j = N_orb, 1, -1
      occ_n = occ(j)
      i = 3*i + (2 - occ_n)
    end do
    i = i + 1
  end subroutine occ_to_index

  subroutine index_to_occ(i, occ)
    !! maps occupation pattern to index
    integer, intent(inout) :: occ(:)
    !integer*8, intent(in) :: i
    integer, intent(in) :: i
    integer :: j, occ_n, N_orb, k
    k = i - 1
    N_orb = size(occ)
    occ(:) = 2
    do j = 1, N_orb, 1
      if (k .le. 0) exit
      occ_n = 2 - mod(k, 3)
      occ(j) = occ_n
      k = (k - (2 - occ_n))/3
    end do

  end subroutine index_to_occ

end module elec_config
