!
! XMOLECULE electronic structure toolkit
! Copyright 2023 Ludger Inhester
! Ludger Inhester, Center for Free-Electron Laser Science, DESY
! ludger.inhester@cfel.de
!
! This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
! See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
!

module rwfile
  !! author: Yajiang Hao
  !!
  !! This module contain subroutines to read and write data in disk.
  use constants
  use read_param
  use molecular_grid
  use wavefunction
  use basis_func
  implicit none
  private
  public :: read_wf, read_basis, save_wf, save_basis
contains


  subroutine save_basis(B, filepath)
    !! writes molecular grid data to file
    character*(*), optional :: filepath
    type(Basis), intent(in) :: B
    integer ::  i, RWStatus

    if (present(filepath)) then; 
      open (unit=42, form='unformatted', file=trim(filepath), IOSTAT=RWStatus)
    else
      open (unit=42, form='unformatted', file="basis.dat", IOSTAT=RWStatus)
    end if
    if (RWStatus .eq. 0) then
      do i = 1, B%N_basis
        write (42) B%func(:, i)
      end do
    end if
    close (42)

  end subroutine save_basis

  subroutine read_basis(F, basis_ini, fp)
    !! reads molecular grid data from file
    type(WF_data), intent(in) :: F
    real(long), dimension(F%N_grid, F%N_basis), intent(inout) :: basis_ini
    integer ::  i, RWStatus
    character*(*), OPTIONAL :: fp

    if (present(fp)) then
      open (unit=42, form='unformatted', file=FP, IOSTAT=RWStatus, status='OLD', ACTION='READ')
    else
      open (unit=42, form='unformatted', file="basis.dat", IOSTAT=RWStatus, status='OLD', ACTION='READ')
    end if

    if (RWStatus .eq. 0) then
      do i = 1, F%N_basis
        read (42) basis_ini(:, i)
      end do
    end if
    close (42)

  end subroutine read_basis

  subroutine save_wf(LC, filepath)
    !! write LCAO coefficients to file
    real(long) :: LC(:,:)
    integer ::  i, RWStatus
    character*(*), optional :: filepath

    if (present(filepath)) then
      open (unit=41, file=trim(filepath), IOSTAT=RWStatus)
    else
      open (unit=41, file="wf.dat", IOSTAT=RWStatus)
    end if
    if (RWStatus .eq. 0) then
      do i = 1, size(LC,1)
        write (41, *) LC(i, :)
      end do
    end if
    close (41)
  end subroutine save_wf

  subroutine read_wf(F, wf, filepath)
    !! reads LCAO coefficients from file
    type(WF_data), intent(in) :: F
    real(long), dimension(F%N_basis, F%Nr_basis), intent(inout) :: wf
    integer ::  i, RWStatus

    character*(*), optional :: filepath
    if (present(filepath)) then
      open (unit=41, file=trim(filepath), IOSTAT=RWStatus)
    else
      open (unit=41, file="wf.dat", IOSTAT=RWStatus, status='OLD', ACTION='READ')
    end if

    if (RWStatus .eq. 0) then
      do i = 1, F%N_basis
        read (41, *) wf(i, :)
      end do
    end if
    close (41)

  end subroutine read_wf

end module RWFile
