!
! XMOLECULE electronic structure toolkit
! Copyright 2023 Ludger Inhester
! Ludger Inhester, Center for Free-Electron Laser Science, DESY
! ludger.inhester@cfel.de
!
! This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
! See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
!


program test_absorption_interface 
!----------------------------------------------------------------------------
! TEST_ABSORPTION_INTERFACE
!
! $Id: xmolecule.f90 2735 2017-09-20 16:01:25Z linhest $
!----------------------------------------------------------------------------
  use read_param
  use atom_in_mol
  use molecular_grid
  use basis_func
  use wavefunction
  use process
!  use augerrates
!  use photoionization
!  use fluorescence
  use absorption
!  use population
  use scf
!  use scf_unrestricted
!  use nuclearDerivatives
!  use scattering
  use svn  
!  use mcscf
  use gradient

  use gaussian 
!  use cubefile
  implicit none
  type(Param) :: P
  type(Atom_set), allocatable :: A(:)
  type(Grid3D) :: G
  type(Basis) :: B
  type(WF_data) :: F
  type(Process_data) :: S

  character(len=2048) :: filename
  
  integer :: i, j,k , ia, ig
  real(long),parameter :: epsilon=1e-15

  real(long), allocatable :: aCS(:,:)

  call init_time()
  !
  ! read parameters
  !
  call read_parameters( P, "static" )
  call print_parameters( P, "electronic structure of molecules, " // &
                            "Rev. " // SVNVERSION )


!  if(P%yes_HF==.FALSE.)&
!    stop "this test only works with HF: please use option -HF"
!  if(P%yes_gaussian==.FALSE.)&
!    stop "this test only works with gto basis: please use option -gto <basis_set filename>"
  !
  ! construct molecular coordinate
  !
  call construct_Atom_set( P, A )
  call construct_Grid3D( P, A, G )
  call construct_Basis( A, G, B, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym ,gto_file_path=P%gto_file_path, yes_gradient=P%yes_gradient)
  call construct_WF_data( P, A, G, B, F )

  call print_calc_info( P, A, G, B )    

  call precalculate_twoel_GTO( B ) 
  call SCF_procedure( P, A, G, B, F )

  write(6,'("# ****************************************************")') 
  write(6,'("# Now calling absorption CS without optional argument ")') 
  write(6,'("# ****************************************************")') 

  call calc_oscillator_strength( P, B, F, G, S )

  write(6,'("# *************************************************")') 
  write(6,'("# Now calling absorption CS with optional argument ")') 
  write(6,'("# --> should give same output as above")') 
  write(6,'("# *************************************************")') 

  allocate(aCS(size(F%LC,1),size(F%LC,1)))

  call calc_oscillator_strength( P, B, F, G, S, aCS )

  write(6,'("# ***************************************************************")') 
  write(6,'("# aCS array after call to calc_absorption_CS ")') 
  write(6,'("# --> should match cross section*factor given in the output above")') 
  write(6,'("# ***************************************************************")') 
  do i=1,size(F%LC,1)
    do j=1,size(F%LC,1)
      if(aCS(j,i)>epsilon)then
        write(6, '(2x,i8,x,i8,x,1es15.5)') i,j,aCS(j,i)
      endif
    end do
  end do

  ! deallocate and quit
  deallocate(aCS)

end program test_absorption_interface 
