!
! XMOLECULE electronic structure toolkit
! Copyright 2023 Ludger Inhester
! Ludger Inhester, Center for Free-Electron Laser Science, DESY
! ludger.inhester@cfel.de
!
! This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
! See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
!



! calculates the angular integral for a all angularmomenta l<=lmax

program AngularIntegration_Test
  use constants
  use stdio
  use AngularIntegration
  implicit none

  integer :: i
  type(Argument) :: clARG
  integer :: OUTPUT
  character(len=32) :: fileOut

  real(long) :: tresult
  integer :: l
  type(atomorbital) :: i1,i2,i3,i4
  integer :: l1,l2,l3,l4,lmax
  integer :: m1,m2,m3,m4
  fileOut=""
  OUTPUT = STDOUT
  call get_input(clarg)
  do i = 1, clarg%N
    select case (clarg%option(i))
    case("-o")
      fileOut = trim(clarg%string(i))
      OUTPUT = 11
    case default
      write(6,"(a,x,a,x,a)") 'command line argument ',trim(clarg%option(i)),'not understood'
      error stop
    end select
  end do
  if (OUTPUT .ne. STDOUT) open (OUTPUT, file=fileOut)

  lmax=3
  do l1=0,lmax
     do l2=0,lmax
        do l3=0,l1
           do l4=0,l2
              
              do m1=-l1,l1
                 do m2=-l2,l2
                    do m3=-m1,m1
                       do m4=-m2,m2

                          if(m1+m3/=m2+m4 ) then
                             continue
                          endif

                          i1%n=0
                          i2%n=0
                          i3%n=0
                          i4%n=0
                          
                          i1%l=l1
                          i2%l=l2
                          i3%l=l3
                          i4%l=l4
                          
                          i1%m=m1
                          i2%m=m2
                          i3%m=m3
                          i4%m=m4

                          do l=0,lmax
                             tresult = realAngularIntegral(l,i1,i2,i3,i4)
                             if( abs(tresult) > 0.00001_long )then
                                write(OUTPUT,fmt='("A_",I1,"[", 4(I2,I2,";") "]=",F7.4)') &
                                     l,l1,m1,l2,m2,l3,m3,l4,m4,tresult
                             
                             endif
                          enddo !l loop
                       enddo !m4 loop
                    enddo ! m3 loop
                 enddo!m2 loop
              enddo!m1 loop

           enddo !l4 loop
        enddo!l3 loop       
     enddo!l2 loop     
  enddo!l1 loop

end program AngularIntegration_Test



