!
! XMOLECULE electronic structure toolkit
! Copyright 2023 Ludger Inhester
! Ludger Inhester, Center for Free-Electron Laser Science, DESY
! ludger.inhester@cfel.de
!
! This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
! See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
!

program test
  use read_param
  use atom_in_mol
  use molecular_grid
  use basis_func
  use constants
  implicit none
  type(Param) :: P
  type(Atom_set), allocatable :: A(:)
  type(Grid3D) :: G
  type(Basis) :: B
  integer :: i, j, k
  call read_parameters( P, "static" )
  call construct_Atom_set( P, A )
  call construct_Grid3D( P, A, G )
  if ( P%yes_gaussian ) then
    call construct_Basis( A, G, B, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym, gto_file_path=P%gto_file_path, &
         yes_gradient=P%yes_gradient, basisSetOverlapLim=P%basisSetOverlapLim)    
  else
    call construct_Basis( A, G, B, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym, yes_gradient=P%yes_gradient, &
         basisSetOverlapLim=P%basisSetOverlapLim)
  end if
  !call dump_Basis( B )
  !
  ! testing normalization of basis functions 
  ! with multicenter integral and double-center integral
  !
  write(P%OUTPUT, "(a5,a13)", advance="no") "basis", "multicenter"
  do j = 1, P%N_nuc
    do k = j, P%N_nuc
      write(P%OUTPUT, "(7x,'(',2i2,')')", advance="no") j, k
    end do
  end do
  write(P%OUTPUT, "()")
  do i = 1, B%N_basis
    write(P%OUTPUT, "(i5,es13.5)", advance="no") &
         i, ONE - integral( G, B%func(:,i)**2 )
    if(abs(ONE - integral( G, B%func(:,i)**2 )) > 1e-6)&
         error stop
    do j = 1, P%N_nuc
      do k = j, P%N_nuc
        write(P%OUTPUT, "(es13.5)", advance="no") &
          ONE - integral_double_center( G, B%func(:,i)**2, j, k )
      end do
    end do
    write(P%OUTPUT, "()")
  end do
end program test
