!
! XMOLECULE electronic structure toolkit
! Copyright 2023 Ludger Inhester
! Ludger Inhester, Center for Free-Electron Laser Science, DESY
! ludger.inhester@cfel.de
!
! This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
! See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
!

program test_coul_pot 

  use read_param
  use atom_in_mol
  use molecular_grid
  use wavefunction
  use SCF
  use constants
  use basis_func
  use operations
  use stdio
  implicit none
  type(Param) :: P
  type(Atom_set), allocatable :: A(:)
  type(Grid3D) :: G
  type(Basis) :: B
  type(WF_data) :: F
  integer :: i
  real(long) :: xx, yy, zz
  real(long), allocatable :: v_J(:), rho1(:), rho2(:), v_JA(:), rho_A(:)
  call init_time()

  !
  ! read parameters
  !
  call read_parameters( P, "static" )
  call print_parameters( P )

  call construct_Atom_set( P, A )

  call construct_Grid3D(P, A, G )
  !call dump_Grid3D( G )
  !
  ! construct molecular coordinate
  !
  if ( P%yes_gaussian ) then
    call construct_Basis( A, G, B, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym, gto_file_path=P%gto_file_path, &
         yes_gradient=P%yes_gradient, basisSetOverlapLim=P%basisSetOverlapLim)    
  else
    call construct_Basis( A, G, B, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym, yes_gradient=P%yes_gradient, &
         basisSetOverlapLim=P%basisSetOverlapLim)
  end if

  !call dump_Basis( B )

  call construct_WF_data( P, A, G, B, F )

  !SCF process
  allocate( v_J(G%N_grid), rho1(G%N_grid), rho2(G%N_grid), v_JA(A(1)%N_r), rho_A(A(1)%N_r) )

  do i = 1, G%N_grid
    xx = G%x(i) - A(1)%x
    yy = G%y(i) - A(1)%y
    zz = G%z(i) - A(1)%z
    rho1(i) = exp(-2*sqrt(xx**2 + yy**2 + zz**2) ) / (PI)
  end do
  do i = 1, G%N_grid
    xx = G%x(i) - A(2)%x
    yy = G%y(i) - A(2)%y
    zz = G%z(i) - A(2)%z
    rho2(i) = exp(-2*sqrt(xx**2 + yy**2 + zz**2)) / (PI)
  end do

  do i = 1, A(1)%N_r
    rho_A(i) = exp(-2*A(1)%radial%r(i) ) / (PI)
  end do
  v_JA(:) = potential_J_A( A(1), rho_A )
  !v_J(:) = potential_J_AC( P, G, A, rho1 )
  !v_J(:) = potential_J_AB( P, G, A, rho1 )
  !v_J(:) = potential_J_axial( P, G, rho1 )
  !v_J(:) = potential_Jdrrv2( G, rho1 )
  !v_J(:) = potential_Jold( G, rho1 )
  !v_J(:) = potential_J_Cancel( P, G, rho1 )
  v_J(:) = potential_J_MC( P, G, A, rho1 )
  !v_J(:) = potential_J( P, G, rho1 )

  write(P%OUTPUT,*) (integral(G,rho2(:)*G%weight_nuc(1,:)*v_J(:)) + integral(G,rho2(:)*G%weight_nuc(2,:)*v_J(:)))
  write(P%OUTPUT,*) "potential_J_MC", integral(G,rho2(:)*v_J(:))
  !if(abs((integral(G,rho2(:)*G%weight_nuc(1,:)*v_J(:))+integral(G,rho2(:)*G%weight_nuc(2,:)*v_J(:))) - integral(G,rho2(:)*v_J(:)))> 1e-6 )&
  !     error stop

  !  do i = 1, A(1)%N_r
  !	print *, A(1)%r(i),  v_JA(i), v_J(i)	!rho(i), rho_A(i), rho(i)-rho_A(i)	
  !  end do
  write(P%OUTPUT,*) maxval( abs( v_JA(1:A(1)%N_r)-v_J(1:A(1)%N_r) ) )
  if (maxval( abs( v_JA(1:A(1)%N_r)-v_J(1:A(1)%N_r) ) ) > 1e-5)&
       error stop
  write(P%OUTPUT,*) HALF*integral(G,rho1(:)*v_J(:))
  write(P%OUTPUT,*) HALF*integral_A(A(1),rho_A(:)*v_JA(:))
  !if(abs(integral(G,rho1(:)*v_J(:)) - integral_A(A(1),rho_A(:)*v_JA(:))> 1e-6) )&
  !     error stop

  !if ( P%yes_print_wf ) call print_bound( P, F )
  !
  ! print footnote
  !
  call footnote( P%OUTPUT )



end program test_coul_pot
