!
! XMOLECULE electronic structure toolkit
! Copyright 2023 Ludger Inhester
! Ludger Inhester, Center for Free-Electron Laser Science, DESY
! ludger.inhester@cfel.de
!
! This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
! See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
!

program test_fluorescence_interface 
  use read_param
  use atom_in_mol
  use molecular_grid
  use basis_func
  use wavefunction
  use process
  use fluorescence
  use scf
  use gradient

  use gaussian 
  implicit none
  type(Param) :: P
  type(Atom_set), allocatable :: A(:)
  type(Grid3D) :: G
  type(Basis) :: B
  type(WF_data) :: F
  type(Process_data) :: S

  
  integer :: i, j
  real(long),parameter :: epsilon=1e-15

  real(long), allocatable :: fluo_rates(:,:)

  call init_time()
  !
  ! read parameters
  !
  call read_parameters( P, "static" )


  if(.not. P%yes_HF)&
    stop "this test only works with HF: please use option -HF"
  if(.not. P%yes_gaussian)&
    stop "this test only works with gto basis: please use option -gto <basis_set filename>"
  !
  ! construct molecular coordinate
  !
  call init_index_packed()
  call construct_Atom_set( P, A )
  call construct_Grid3D( P, A, G )
  if ( P%yes_gaussian ) then
    call construct_Basis( A, G, B, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym, gto_file_path=P%gto_file_path, &
         yes_gradient=P%yes_gradient, basisSetOverlapLim=P%basisSetOverlapLim)    
  else
    call construct_Basis( A, G, B, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym, yes_gradient=P%yes_gradient, &
         basisSetOverlapLim=P%basisSetOverlapLim)
  end if
  if ( P%yes_gaussian .and. .NOT. P%yes_direct_scf ) then
    write(P%OUTPUT,'("# calculating two electron integrals (gto): ...")')
    call precalculate_twoel_GTO( P, B ) 
    write(P%OUTPUT,'("# done")')
  end if

  call construct_WF_data( P, A, G, B, F )
  if( P%yes_HF ) then
    if(.not. associated(B%teint) .and. .not. P%yes_direct_scf) then
      write(P%OUTPUT,'("# calculating two electron integrals: (grid)...")')
      call precalculate_twoel( P, A, G, B)
    end if
  end if
  S = create_Process(P%N_occ, P%N_proc_max, spin=.False., ci=.False.)
  call SCF_procedure( P, A, G, B, F )

  call calc_Fluorescence_rates( P, B, F, G, S)
  call print_process(S, P%OUTPUT)
  allocate(fluo_rates(B%Nr_basis,B%Nr_basis))
  call calc_Fluorescence_rates( P, B, F, G, S, fluo_rates )

  write(P%OUTPUT,'("# ***************************************************************")') 
  write(P%OUTPUT,'("# fluo_rates array after call to calc_Fluorescence_rates ")') 
  write(P%OUTPUT,'("# --> should match cross section*factor given in the output above")') 
  write(P%OUTPUT,'("# ***************************************************************")') 
  do i=1,size(F%LC,1)
    do j=1,size(F%LC,1)
      if(fluo_rates(j,i)>epsilon)then
        write(P%OUTPUT, '(2x,i8,x,i8,x,1es15.5)') j,i,fluo_rates(j,i)
      endif
    end do
  end do
  do i=1,S%N_process
    write(P%OUTPUT,*) S%tag(i),S%occ_down(1,i),S%occ_up(1,i)
    write(P%OUTPUT,*) S%value(i), fluo_rates(S%occ_down(1,i),S%occ_up(1,i))
    if( abs(S%value(i)- fluo_rates(S%occ_down(1,i),S%occ_up(1,i)))>1e-10_long)&
         error stop
  end do
  ! deallocate and quit
  deallocate(fluo_rates)
  call purge_process(S)
end program test_fluorescence_interface 
