!
! XMOLECULE electronic structure toolkit
! Copyright 2023 Ludger Inhester
! Ludger Inhester, Center for Free-Electron Laser Science, DESY
! ludger.inhester@cfel.de
!
! This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
! See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
!

program test_grad 
  use read_param
  use atom_in_mol
  use molecular_grid
  use basis_func
  use wavefunction
  use scf
  use gradient
  use gaussian_cphf
  use cis
  use gaussian 
  implicit none
  type(Param) :: P
  type(Atom_set), allocatable :: A(:)
  type(Atom_set), allocatable :: A1(:)
  type(Atom_set), allocatable :: A2(:)
  type(Grid3D) :: G,G1,G2
  type(Basis) :: B,B1,B2
  type(WF_data) :: F,F1,F2

  real(long),parameter :: epsilon=1e-5
  integer :: i, k, ia, ig
  real(long), allocatable :: grad_num(:)

  integer :: Nval
  integer, allocatable :: occs(:,:)
  integer, allocatable :: gs_occ(:)
  real(long), allocatable :: energies(:)
  real(long), allocatable :: gradients(:,:)
  real(long), allocatable :: nacs(:,:,:)
  integer :: state_index
  
  call init_time()
  !
  ! read parameters
  !
  call read_parameters( P, "static" )
  call init_index_packed()

  !
  ! construct molecular coordinate
  !
  call construct_Atom_set( P, A )
  call construct_Grid3D( P, A, G )

  if ( P%yes_gaussian ) then
    call construct_Basis( A, G, B, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym ,gto_file_path=P%gto_file_path, yes_gradient=.True.)
  else
    call construct_Basis( A, G, B, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym , yes_gradient=.True.)
  end if
  if ( P%yes_gaussian .and. .NOT. P%yes_direct_scf ) then
    write(P%OUTPUT,'("# calculating two electron integrals (gto): ...")')
    call precalculate_twoel_GTO( P, B ) 
    write(P%OUTPUT,'("# done")')
  end if
  call print_calc_info( P, A, G, B )    

  call construct_WF_data( P, A, G, B, F )

  if( P%yes_HF ) then
    if(.not. associated(B%teint) .and. .not. P%yes_direct_scf) then
      write(P%OUTPUT,'("# calculating two electron integrals: (grid)...")')
      call precalculate_twoel(P, A, G, B )
    end if
    call SCF_procedure( P, A, G, B, F )
!  else if(P%yes_unrestricted) then
!    call SCF_procedure_unrestricted( P, A, G, B, F )
  else
    call SCF_procedure( P, A, G, B, F )
  end if

  if(P%yes_CIS)then
    call CIS_procedure( P, A, G, B, F )
    F%E_tot=F%E_tot+F%E_cis(P%state)
  end if


  if(P%yes_gaussian )then
    if(P%yes_valgs_occ)then
      if(.NOT. allocated(F%teint))then
        call construct_h0_matrix( G, B, F )
        F%H0=F%LC(:,:).tm. F%H0(:,:) .m. F%LC(:,:)
        call transform_twoel(P, B ,F, P%OUTPUT )
      end if
      if(count(F%N_occ==1)==1)then
        allocate(gs_occ(size(F%N_occ)))
        gs_occ=groundstate_occ(F%N_occ(:),Ncore=F%Ncore)  
        Nval=count(gs_occ==2)-F%Ncore
        allocate(energies(Nval))
        allocate(occs(size(F%N_occ),Nval))
        allocate(gradients(size(A)*3 ,Nval))
        allocate(nacs(size(A)*3 ,Nval,Nval))
        call calculate_hole_energies_gradients_nacs(P, A, B, F, occs,energies,gradients,nacs,OUTPUT=P%OUTPUT)
        write(P%OUTPUT,*) 'Occupations:'
        do i=1,size(energies)
          write(P%OUTPUT,'(i4,f10.3,x,99i1)') i,energies(i),occs(:,i)
          if(all(occs(:,i)==F%N_occ))then
            state_index=i
          end if
        end do
        write(P%OUTPUT,*) 'state_index=',state_index
        write(P%OUTPUT,*) 'Energy=',F%E_tot,energies(state_index)
!        F%E_tot=energies(state_index)
        F%grad=gradients(:,state_index)
      end if
    else if(P%yes_CIS)then
      call calculate_CIS_gradient_GTO(P,A,B,F,state=P%state,OUTPUT=P%OUTPUT)
    else
      call calculate_energy_gradient_GTO(P,A,G,B,F)
    end if
  end if
  if(.not. P%yes_gaussian .and. .not. P%yes_HF)then
    call calculate_energy_gradient( P, A, G, B, F )
  end if


  allocate(grad_num(size(A)*3))


  grad_num=ZERO
  ! Compare to numerical dF/dR
  ! Move geometry by epsilon

  call construct_Atom_set( P, A1 )
  call construct_Atom_set( P, A2 )
  do k=1,size(A,1)*3
    ig=mod(k-1,3)+1
    ia=(k-mod(k-1,3)-1)/3+1
    A1(:)%x=A(:)%x
    A1(:)%y=A(:)%y
    A1(:)%z=A(:)%z
    A2(:)%x=A(:)%x
    A2(:)%y=A(:)%y
    A2(:)%z=A(:)%z
    if(ig==1)then
      A1(ia)%x=A2(ia)%x-HALF*epsilon
    elseif(ig==2)then
      A1(ia)%y=A2(ia)%y-HALF*epsilon
    elseif(ig==3)then
      A1(ia)%z=A2(ia)%z-HALF*epsilon
    end if
    if(ig==1)then
      A2(ia)%x=A2(ia)%x+HALF*epsilon
    elseif(ig==2)then
      A2(ia)%y=A2(ia)%y+HALF*epsilon
    elseif(ig==3)then
      A2(ia)%z=A2(ia)%z+HALF*epsilon
    end if
    call construct_Grid3D( P, A1, G1 )
    call construct_Grid3D( P, A2, G2 )

    if ( P%yes_gaussian ) then
      if( associated(B2%teint) ) then
        deallocate(B2%teint)
        nullify(B2%teint)
      end if
      call construct_Basis( A1, G1, B1, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym ,gto_file_path=P%gto_file_path, yes_gradient=.True.)
      call construct_Basis( A2, G2, B2, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym ,gto_file_path=P%gto_file_path, yes_gradient=.True.)
    else
      call construct_Basis( A1, G1, B1, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym , yes_gradient=.True.)
      call construct_Basis( A2, G2, B2, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym , yes_gradient=.True.)
    end if
    if ( P%yes_gaussian .and. .NOT. P%yes_direct_scf ) then
      write(P%OUTPUT,'("# calculating two electron integrals (gto): ...")')
      call precalculate_twoel_GTO( P, B1 )
      call precalculate_twoel_GTO( P, B2 )     
      write(P%OUTPUT,'("# done")')
    end if
    call construct_WF_data(P, A1, G1, B1, F1 )
    call construct_WF_data(P, A2, G2, B2, F2 )
    F1%LC_ini = F%LC
    F1%iscfstate = 2    
    F2%LC_ini = F%LC
    F2%iscfstate = 2
    if( P%yes_HF ) then
      if(.not. associated(B2%teint) .and. .not. P%yes_direct_scf) then
        write(P%OUTPUT,'("# calculating two electron integrals: (grid)...")')        
        call precalculate_twoel(P, A1, G1, B1)
        call precalculate_twoel(P, A2, G2, B2)
      end if
      
      call SCF_procedure( P, A1, G1, B1, F1 )
      call SCF_procedure( P, A2, G2, B2, F2 )
    else
      call SCF_procedure( P, A1, G1, B1, F1 )
      call SCF_procedure( P, A2, G2, B2, F2 )
    end if
    if(P%yes_CIS)then
      call CIS_procedure( P, A1, G1, B1, F1 )
      call CIS_procedure( P, A2, G2, B2, F2 )
      F1%E_tot=F1%E_tot+F1%E_cis(P%state)
      F2%E_tot=F2%E_tot+F2%E_cis(P%state)    
    end if
    write(P%OUTPUT,*) 'Energy=',F1%E_tot  
    write(P%OUTPUT,*) 'Energy=',F2%E_tot
    grad_num(k) = ( F2%E_tot - F1%E_tot) / epsilon
    call purge_WF_data(F1)
    call purge_WF_data(F2)
  end do
  write(P%OUTPUT,'("# Differences Analytical Numerical gradients of d E / dR  larger than ",es10.1)') 10*epsilon
  write(P%OUTPUT,'("# ",a8,x,a15,a15,a15)') "coord","anal. grad","num. grad","diff"
  do k=1,size(A,1)*3
    write(P%OUTPUT,'(2x,x,i8,x,es15.5,es15.5,es15.5)') k,F%grad(k),grad_num(k),F%grad(k)-grad_num(k)
    if(abs(F%grad(k)-grad_num(k))>10*epsilon)then 
      error stop
    end if
  end do
  deallocate(grad_num)
  write(P%OUTPUT,*) 'All tests passed'
end program test_grad
