!
! XMOLECULE electronic structure toolkit
! Copyright 2023 Ludger Inhester
! Ludger Inhester, Center for Free-Electron Laser Science, DESY
! ludger.inhester@cfel.de
!
! This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
! See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
!

program test_hgrad 
!----------------------------------------------------------------------------
! TEST_HGRAD
!
! $Id: xmolecule.f90 2735 2017-09-20 16:01:25Z linhest $
!----------------------------------------------------------------------------
  use read_param
  use atom_in_mol
  use molecular_grid
  use basis_func
  use wavefunction
  use scf
  use gradient

  use gaussian 
!  use cubefile
  implicit none
  type(Param) :: P
  type(Atom_set), allocatable :: A(:)
  type(Atom_set), allocatable :: A1(:)
  type(Atom_set), allocatable :: A2(:)
  type(Grid3D) :: G,G1,G2
  type(Basis) :: B,B1,B2
  type(WF_data) :: F,F1,F2

  
  integer :: i, j,k , ia, ig
  real(long),parameter :: epsilon=1e-6
  real(long), allocatable :: h0_grad(:,:,:)
  real(long), allocatable :: h0_grad_num(:,:,:)
  real(long), allocatable :: K_matrix(:,:)
  real(long), allocatable :: J_matrix(:,:)
  real(long), allocatable :: K_matrix1(:,:)
  real(long), allocatable :: J_matrix1(:,:)
  real(long), allocatable :: K_matrix2(:,:)
  real(long), allocatable :: J_matrix2(:,:)
  real(long), allocatable :: J_grad(:,:,:)
  real(long), allocatable :: J_grad_num(:,:,:)
  real(long), allocatable :: K_grad(:,:,:)
  real(long), allocatable :: K_grad_num(:,:,:)
  real(long), allocatable :: F_grad_num(:,:,:)
  real(long), allocatable :: Rinv_grad(:,:,:)

  real(long), allocatable :: F_grad(:,:,:)
  real(long), allocatable :: DU(:,:,:)
  real(long), allocatable :: M_matrix(:,:,:)
  real(long), allocatable :: temp(:,:)
  call init_time()
  !
  ! read parameters
  !
  call read_parameters( P, "static" )

  if(.not. P%yes_HF)&
    stop "this test only works with HF: please use option -HF"
  if(.not. P%yes_gaussian)&
    stop "this test only works with gto basis: please use option -gto <basis_set filename>"
  call init_index_packed()
  !
  ! construct molecular coordinate
  !
  call construct_Atom_set( P, A )
  call construct_Grid3D( P, A, G )
  call construct_Basis( A, G, B, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym ,gto_file_path=P%gto_file_path, yes_gradient=P%yes_gradient)
  call precalculate_twoel_GTO( P, B ) 

  call print_calc_info( P, A, G, B )   
  call construct_WF_data( P, A, G, B, F )

  call SCF_procedure( P, A, G, B, F )

  allocate(Rinv_grad(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(h0_grad(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(h0_grad_num(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(J_grad(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(J_grad_num(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(K_grad(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(K_grad_num(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))


  allocate(K_matrix(size(F%LC,1),size(F%LC,1)))
  allocate(J_matrix(size(F%LC,1),size(F%LC,1)))
  allocate(K_matrix1(size(F%LC,1),size(F%LC,1)))
  allocate(J_matrix1(size(F%LC,1),size(F%LC,1)))
  allocate(K_matrix2(size(F%LC,1),size(F%LC,1)))
  allocate(J_matrix2(size(F%LC,1),size(F%LC,1)))

  allocate(F_grad(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(F_grad_num(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(M_matrix(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(temp(size(F%LC,1),size(F%LC,1)))
  allocate(DU(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))

  call construct_J_Matrix(B%teint, F%D, J_matrix)
  call construct_K_Matrix(B%teint, F%D, K_matrix)
  F%H0 = F%LC(:,:) .tm. F%H0(:,:) .m. F%LC(:,:)

  call construct_Atom_set( P, A1 )
  call construct_Atom_set( P, A2 )

  do k=1,size(B%atm,2)*3
    ig=mod(k-1,3)+1
    ia=(k-mod(k-1,3)-1)/3+1
    A1(:)%x=A(:)%x
    A1(:)%y=A(:)%y
    A1(:)%z=A(:)%z
    A2(:)%x=A(:)%x
    A2(:)%y=A(:)%y
    A2(:)%z=A(:)%z
    if(ig==1)then
      A2(ia)%x=A(ia)%x+HALF*epsilon
    elseif(ig==2)then
      A2(ia)%y=A(ia)%y+HALF*epsilon
    elseif(ig==3)then
      A2(ia)%z=A(ia)%z+HALF*epsilon
    end if
    if(ig==1)then
      A1(ia)%x=A(ia)%x-HALF*epsilon
    elseif(ig==2)then
      A1(ia)%y=A(ia)%y-HALF*epsilon
    elseif(ig==3)then
      A1(ia)%z=A(ia)%z-HALF*epsilon
    end if
    call construct_Grid3D( P, A1, G1 )
    call construct_Basis(A1,G1,B1,P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym ,gto_file_path=P%gto_file_path, yes_gradient=P%yes_gradient)
    call construct_WF_data(P, A1, G1, B1, F1 )
    call precalculate_twoel_GTO( P, B1 )
    call construct_J_Matrix(B1%teint, F%D, J_matrix1)
    call construct_K_Matrix(B1%teint, F%D, K_matrix1)

    call construct_Grid3D( P, A2, G2 )
    call construct_Basis(A2,G2,B2,P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym ,gto_file_path=P%gto_file_path, yes_gradient=P%yes_gradient)
    call construct_WF_data(P, A2, G2, B2, F2 )
    call precalculate_twoel_GTO( P, B2 )
    call construct_J_Matrix(B2%teint, F%D, J_matrix2)
    call construct_K_Matrix(B2%teint, F%D, K_matrix2)

    !F1%H0 = F%LC(:,:) .tm. (B1%H0(:,:) .m. F%LC(:,:))
    !F2%H0 = F%LC(:,:) .tm. (B2%H0(:,:) .m. F%LC(:,:))
    !h0_grad_num(:,:,k)=(F2%h0(:,:)-F1%h0(:,:))/epsilon
    h0_grad_num(:,:,k)=(B2%h0(:,:)-B1%h0(:,:))/epsilon
    J_grad_num(:,:,k)=(J_matrix2(:,:)-J_matrix1(:,:))/epsilon
    K_grad_num(:,:,k)=(K_matrix2(:,:)-K_matrix1(:,:))/epsilon
!    h0_grad_num(:,:,k)=(B2%hkin(:,:)-B%hkin(:,:))/epsilon

    call purge_WF_data(F2)
    call purge_WF_data(F1)
    if(ig==1)then
      A1(ia)%x=A(ia)%x
      A2(ia)%x=A(ia)%x
    elseif(ig==2)then
      A1(ia)%x=A(ia)%x
      A2(ia)%y=A(ia)%y
    elseif(ig==3)then
      A1(ia)%x=A(ia)%x
      A2(ia)%z=A(ia)%z
    end if


  end do

  call h0_gradient(B%env,B%atm,B%bas,B%first_bf_of_shell,h0_grad)
  ! compute gradient contribution for the point charges
  !do ai=1,size(B%atm,2)
  !  if(A(ai)%charge==0)then
  !    pos(1)=A(ai)%x
  !    pos(2)=A(ai)%y
  !    pos(3)=A(ai)%z
  !    call Rinv_gradient(B%env,B%atm,B%bas,B%first_bf_of_shell,pos,ai,Rinv_grad)
  !    h0_grad(:,:,:)=h0_grad(:,:,:)-A(ai)%fcharge * Rinv_grad(:,:,:)
  !  end if
  !end do
  !do k=1,size(DU,3)
  !  h0_grad(:,:,k) = F%LC(:,:) .tm. h0_grad(:,:,k) .m. F%LC(:,:)
  !  h0_grad_num(:,:,k) = F%LC(:,:) .tm. h0_grad_num(:,:,k) .m. F%LC(:,:)
  !end do

  write(P%OUTPUT,'("# Differences Analytical Numerical gradients of (d h0 / dR )_ij larger than ",es10.1)') 10*epsilon
  write(P%OUTPUT,'("# ",a8,x,a8,x,a8,x,a15,a15,a15)') "i","j","coord","anal. grad","num. grad","diff"
  do k=1,size(B%atm,2)*3
    do i=1,size(F%LC,1)
      do j=1,size(F%LC,1)
        write(P%OUTPUT,'(2x,i8,x,i8,x,i8,x,es15.5,es15.5,es15.5)') i,j,k,h0_grad(i,j,k),h0_grad_num(i,j,k),h0_grad(i,j,k)-h0_grad_num(i,j,k) 
        if(abs(h0_grad(i,j,k)-h0_grad_num(i,j,k))>10*epsilon)then
          error stop
        end if
      end do
    end do
  end do

  call jk_gradient(B%env,B%atm,B%bas,B%first_bf_of_shell,F%D,J_grad,K_grad)
  write(P%OUTPUT,'("# Differences Analytical Numerical gradients of (d J / dR )_ij larger than ",es10.1)') 10*epsilon
  write(P%OUTPUT,'("# ",a8,x,a8,x,a8,x,a15,a15,a15)') "i","j","coord","anal. grad","num. grad","diff"
  do k=1,size(B%atm,2)*3
    do i=1,size(F%LC,1)
      do j=1,size(F%LC,1)
        write(P%OUTPUT,'(2x,i8,x,i8,x,i8,x,es15.5,es15.5,es15.5)') i,j,k,J_grad(i,j,k),J_grad_num(i,j,k),J_grad(i,j,k)-J_grad_num(i,j,k) 
        if(abs(J_grad(i,j,k)-J_grad_num(i,j,k))>10*epsilon)then
          error stop
        end if
      end do
    end do
  end do


  write(P%OUTPUT,'("# Differences Analytical Numerical gradients of (d K / dR )_ij larger than ",es10.1)') 10*epsilon
  write(P%OUTPUT,'("# ",a8,x,a8,x,a8,x,a15,a15,a15)') "i","j","coord","anal. grad","num. grad","diff"
  do k=1,size(B%atm,2)*3
    do i=1,size(F%LC,1)
      do j=1,size(F%LC,1)
        write(P%OUTPUT,'(2x,i8,x,i8,x,i8,x,es15.5,es15.5,es15.5)') i,j,k,K_grad(i,j,k),K_grad_num(i,j,k),K_grad(i,j,k)-K_grad_num(i,j,k) 
        if(abs(K_grad(i,j,k)-K_grad_num(i,j,k))>10*epsilon)then
          error stop
        end if
      end do
    end do
  end do

!  F_grad = h0_grad + TWO * J_grad - K_grad
!
!  write(P%OUTPUT,'("# Differences Analytical Numerical gradients of (d F / dR )_ij larger than ",es10.1)') 10*epsilon
!  write(P%OUTPUT,'("# ",a8,x,a8,x,a8,x,a15,a15,a15)') "i","j","coord","anal. grad","num. grad","diff"
!  do k=1,size(B%atm,2)*3
!    do i=1,size(F%LC,1)
!      do j=1,size(F%LC,1)
!        if(abs(F_grad(i,j,k)-F_grad_num(i,j,k))>10*epsilon)then
!          write(P%OUTPUT,'(2x,i8,x,i8,x,i8,x,es15.5,es15.5,es15.5)') i,j,k,F_grad(i,j,k),F_grad_num(i,j,k),F_grad(i,j,k)-F_grad_num(i,j,k) 
!          stop
!        end if
!      end do
!    end do
!  end do




  ! write(P%OUTPUT,'("# Quantities in MO space:")')
  ! write(P%OUTPUT,'("# F  ")')
  ! do i=1,size(F%LC,1)
  !   write(P%OUTPUT,'(99f8.3)') F%H(i,:)
  ! end do

  ! write(P%OUTPUT,'("# h0  ")')
  ! do i=1,size(F%LC,1)
  !   write(P%OUTPUT,'(99f8.3)') F%H0(i,:)
  ! end do

!  write(P%OUTPUT,'("# two electron integrals ")')
!  call dump_twoel(P, B ,F, 6)

  ! DU=ZERO
  ! call construct_M_Matrix(P,F,DU,M_matrix)

  ! write(P%OUTPUT,'("# Analytical derivatives of (d X / dR )_ij for X=h0,J,K,M in MO space ")')
  ! write(P%OUTPUT,'("# ",a8,x,a8,x,a8,x,a15,a15,a15,a15)') "i","j","coord","d h0/ dR","d J/ dR","d K/ dR","M"
  ! do k=1,size(B%atm,2)*3
  !   do i=1,size(F%LC,1)
  !     do j=1,size(F%LC,1)
  !       write(P%OUTPUT,'(2x,i8,x,i8,x,i8,x,es15.5,es15.5,es15.5,es15.5)') i,j,k,h0_grad(i,j,k),J_grad(i,j,k),K_grad(i,j,k),M_matrix(i,j,k)
  !     end do
  !   end do
  ! end do




  deallocate(h0_grad)
  deallocate(h0_grad_num)
  deallocate(J_grad)
  deallocate(J_grad_num)
  deallocate(K_grad)
  deallocate(K_grad_num)
  deallocate(F_grad_num)
  deallocate(F_grad)
  deallocate(M_matrix)
  deallocate(DU)
  write(P%OUTPUT,*) "All tests passed"
end program test_hgrad
