!
! XMOLECULE electronic structure toolkit
! Copyright 2023 Ludger Inhester
! Ludger Inhester, Center for Free-Electron Laser Science, DESY
! ludger.inhester@cfel.de
!
! This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
! See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
!

program test
  use constants
  use read_param
  use atom_in_mol
  use molecular_grid
  implicit none
  type(Param) :: P
  type(Atom_set),  allocatable :: A(:)
  type(Grid3D) :: G
  integer :: i, j, k
  real(long), allocatable :: func(:)
  call read_parameters( P, "static" )
  call construct_Atom_set( P, A )
  call construct_Grid3D(P, A, G )
  !
  ! print all grid points
  !
  !call dump_Grid3D( G )
  !
  ! print r vs. weight
  !
!  do i = 1, G%N_grid
!    r = sqrt( G%x(i)**2 + G%y(i)**2 + G%z(i)**2 )
!    print *, i, G%weight(i), G%i_nuc(i)	!, G%weight_overlap(1,i),G%weight_overlap(2,i)
!    print *, G%z(i), G%weight_nuc(1,i) , G%weight_nuc(2,i)
!    if( mod(i,500) == 0) pause
!  end do
  !
  ! let's put an analytically-known function
  !
  allocate( func(G%N_grid) )
  !
  ! Testing multicenter integration: two-center Gaussian
  ! For a diatomic molecule with R,
  ! this integral should give exp(-R^2 / 2) * (pi/2)^(3/2).
  !
!  do i = 1, G%N_grid
!    func(i) = ONE
!    !do j = 1, P%N_nuc
!    do j = 1, 2
!      func(i) = func(i) * exp( -( ( G%x(i) - A(j)%x )**2 + &
!                                  ( G%y(i) - A(j)%y )**2 + &
!                                  ( G%z(i) - A(j)%z )**2 ) )
!    end do
!  end do
!  r = sqrt( ( A(1)%x - A(2)%x )**2 + &
!            ( A(1)%y - A(2)%y )**2 + &
!            ( A(1)%z - A(2)%z )**2 )
!  I1 = integral( G, func(:) )
!  I2 = exp(-r**2 / TWO) * (PI/TWO)**1.5_long
!  print *, "two-center Gaussian: ", I1, I2
!  print *, "              error: ", I1 - I2
  !
  ! Testing multicenter integration: one-center H wavefunction
  ! H wavefunction located at each atom
  ! this integral should give one.
  !
  write(P%OUTPUT, "(a)") "Test for integration with one-center H wavefunction"
  write(P%OUTPUT, "(a)") "Error = 1 - | < psi_H1s | psi_H1s > |^2"
  write(P%OUTPUT, "(a5,5a13)") "i", "multi-center", "one-center", "two-center"
  do i = 1, P%N_nuc
    do k = 1, G%N_grid
      func(k) = TWO / sqrt( FOUR * PI ) * &
                exp( -sqrt( ( G%x(k) - A(i)%x )**2 + &
                            ( G%y(k) - A(i)%y )**2 + &
                            ( G%z(k) - A(i)%z )**2 ) )
    end do
    write(P%OUTPUT, "(i5,2es13.5)", advance="no") i, &
      ONE - integral( G, func(:)**2 ), &
      ONE - integral_single_center( G, func(:)**2, i )
    do j = 1, P%N_nuc
      write(P%OUTPUT, "(es13.5)", advance="no") &
        ONE - integral_double_center( G, func(:)**2, i, j )
    end do
    write(P%OUTPUT, "()")

    if(abs(ONE - integral( G, func(:)**2 )) > 1e-5)&
         error stop
    if(abs(ONE - integral_single_center( G, func(:)**2, i )) > 1e-5)&
         error stop
    do j = 1, P%N_nuc
      if(abs(ONE - integral_double_center( G, func(:)**2, i, j )) > 1e-5)&
           error stop
    end do
  end do
end program test
