!
! XMOLECULE electronic structure toolkit
! Copyright 2023 Ludger Inhester
! Ludger Inhester, Center for Free-Electron Laser Science, DESY
! ludger.inhester@cfel.de
!
! This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
! See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
!

program test_ograd
  use read_param
  use atom_in_mol
  use molecular_grid
  use basis_func
  use wavefunction
  use scf
  use gradient

  use gaussian
  implicit none
  type(Param) :: P
  type(Atom_set), allocatable :: A(:)
  type(Atom_set), allocatable :: A2(:)
  type(Grid3D) :: G
  type(Grid3D) :: G2
  type(Basis) :: B
  type(Basis) :: B2
  type(WF_data) :: F

  real(long) :: epsilon = 1.e-5
  real(long),allocatable :: delta_o_grad(:,:,:)
  real(long),allocatable :: o_grad_num(:,:,:)
  real(long),allocatable :: o_grad_mo(:,:,:) !30.11.2017
  real(long),allocatable :: temp(:,:)
  integer :: i, j, k, m, n
  real(long),allocatable :: o_grad(:,:,:)
  call init_time()
  !
  ! read parameters
  !
  call read_parameters( P, "static" )

  call init_index_packed()
  !
  ! construct molecular coordinate
  !
  call construct_Atom_set( P, A )
  call construct_Grid3D( P, A, G )
  if ( P%yes_gaussian ) then
    call construct_Basis( A, G, B, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym ,gto_file_path=P%gto_file_path, yes_gradient=P%yes_gradient)    
  else
    call construct_Basis( A, G, B, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym , yes_gradient=P%yes_gradient)
  end if
  if ( P%yes_gaussian .and. .NOT. P%yes_direct_scf ) then
    write(P%OUTPUT,'("# calculating two electron integrals (gto): ...")')
    call precalculate_twoel_GTO( P, B )
    write(P%OUTPUT,'("# done")')
  end if
  call print_calc_info( P, A, G, B )
  call construct_WF_data( P, A, G, B, F )
  ! distorted coord
  call construct_Atom_set( P, A2 )

  allocate(o_grad(B%N_basis,B%N_basis,3*size(B%atm,2)))
  allocate(o_grad_num(B%N_basis,B%N_basis,3*size(B%atm,2)))
  allocate(delta_o_grad(B%N_basis,B%N_basis,3*size(B%atm,2)))
  allocate(o_grad_mo(size(F%LC,1),size(F%LC,1),3*size(B%atm,2)))
  allocate(temp(size(F%LC,1),size(F%LC,1)))
!!!!!!!!!!!!!!!!!!!!!!!!!!!
  do n = 1,size(B%atm,2)
    do m = 1,3
      k = 3*(n-1)+m
      if (m==1) then
          A2(n)%x=A2(n)%x+epsilon
      else if (m==2) then
          A2(n)%y=A2(n)%y+epsilon
      else if (m==3) then
          A2(n)%z=A2(n)%z+epsilon
      end if
      ! move geometry along coordinate k
      call construct_Grid3D( P, A2, G2 )
      if ( P%yes_gaussian ) then
          call construct_Basis( A2, G2, B2, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym ,gto_file_path=P%gto_file_path, yes_gradient=P%yes_gradient)    
      else
          call construct_Basis( A2, G2, B2, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym , yes_gradient=P%yes_gradient)
      end if

      o_grad_num(:,:,k) = ( B2%S(:,:) - B%S(:,:) ) / epsilon
!      do i=1,B%N_basis
!        do j=1,B%N_basis
!        o_grad_num(i,j,k) = (B2%S(i,j)-B%S(i,j))/epsilon
!        end do
!      end do
      !move geometry back
      if (m==1) then
          A2(n)%x=A2(n)%x-epsilon
      else if (m==2) then
          A2(n)%y=A2(n)%y-epsilon
      else if (m==3) then
          A2(n)%z=A2(n)%z-epsilon
      end if
    end do
  end do

!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  call overlap_gradient(B%env,B%atm,B%bas,B%first_bf_of_shell,o_grad)
  !call overlap_mo_gradient(B%N_basis, B%env,B%atm,B%bas,B%first_bf_of_shell,F%LC,o_grad_mo)
  delta_o_grad = abs(o_grad_num - o_grad)
  write(P%OUTPUT,"('#      Numerical and analytical O_grad ')")
  write(P%OUTPUT,'("# "x,a8,x,a8,x,a8,x,a15,x,a15,x,a15)') "i","j","k","o_grad","o_grad_num","Delta o_grad"
  do i=1,size(F%LC,1)
     do j=1,size(F%LC,1)
        do k=1,3*size(B%atm,2)

          write(P%OUTPUT,"(x,i8,x,i8,x,i8,x,1es15.5,x,1es15.5,x,1es15.5)") i,j,k,o_grad(i,j,k),o_grad_num(i,j,k),delta_o_grad(i,j,k)
          if ( delta_o_grad(i,j,k) > epsilon ) then
            write(P%OUTPUT,*) "Numerical and analytical O_grad differ"
            error stop
          end if
        end do !k
      end do !j
  end do !i
  write(P%OUTPUT,*) "All tests passed"
end program test_ograd
