!
! XMOLECULE electronic structure toolkit
! Copyright 2023 Ludger Inhester
! Ludger Inhester, Center for Free-Electron Laser Science, DESY
! ludger.inhester@cfel.de
!
! This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
! See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
!

program test_ugrad 
!----------------------------------------------------------------------------
! TEST_HGRAD
!
!----------------------------------------------------------------------------
  use read_param
  use atom_in_mol
  use molecular_grid
  use basis_func
  use wavefunction
!  use process
!  use augerrates
!  use photoionization
!  use fluorescence
!  use absorption
!  use population
  use scf
  use gradient
  use gaussian_cphf

  use gaussian 
!  use cubefile
  implicit none
  type(Param) :: P
  type(Atom_set), allocatable :: A(:)
  type(Atom_set), allocatable :: A2(:)
  type(Grid3D) :: G,G2
  type(Basis) :: B,B2
  type(WF_data) :: F,F2

  real(long),parameter :: epsilon=1e-5
  integer :: i, j,k , ia, ig

  real(long), allocatable :: DU(:,:,:)
  real(long), allocatable :: F_grad(:,:,:)
  real(long), allocatable :: h0_grad(:,:,:)
  real(long),allocatable :: Rinv_grad(:,:,:)
  real(long), allocatable :: J_grad(:,:,:)
  real(long), allocatable :: K_grad(:,:,:)
  real(long), allocatable :: F_grad_num(:,:,:)
  real(long), allocatable :: h0_grad_num(:,:,:)
  real(long), allocatable :: egrad_num(:,:)
  real(long), allocatable :: fh_ao(:,:)
  real(long), allocatable :: fh0_ao(:,:)
  real(long), allocatable :: temp(:)
  real(long), allocatable :: orb_grad(:,:)
  real(long), allocatable :: J_matrix(:,:)
  real(long), allocatable :: K_matrix(:,:)
  real(long) :: pos(3)
  call init_time()
  !
  ! read parameters
  !
  call read_parameters( P, "static" )
  call init_index_packed()


  if(.not.P%yes_HF)&
    stop "this test only works with HF: please use option -HF"
  if(.not.P%yes_gaussian)&
    stop "this test only works with gto basis: please use option -gto <basis_set filename>"
  !
  ! construct molecular coordinate
  !
  call construct_Atom_set( P, A )
  call construct_Grid3D( P, A, G )
  call construct_Basis( A, G, B, P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym ,gto_file_path=P%gto_file_path, yes_gradient=P%yes_gradient)
  call construct_WF_data( P, A, G, B, F )

  call print_calc_info( P, A, G, B )    

  call precalculate_twoel_GTO( P, B ) 
  call SCF_procedure( P, A, G, B, F )


  allocate(DU(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(F_grad(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(h0_grad(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(Rinv_grad(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(J_grad(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(K_grad(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(F_grad_num(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(h0_grad_num(size(F%LC,1),size(F%LC,1),size(B%atm,2)*3))
  allocate(temp(size(F%LC,1)))
  allocate(fh_ao(size(F%LC,1),size(F%LC,1)))
  allocate(fh0_ao(size(F%LC,1),size(F%LC,1)))
  allocate(J_matrix(size(F%LC,1),size(F%LC,1)))
  allocate(K_matrix(size(F%LC,1),size(F%LC,1)))
  allocate(egrad_num(size(F%LC,1),size(B%atm,2)*3))
  allocate(orb_grad(size(F%LC,1),size(B%atm,2)*3))

  ! Test Fock matrix construction
  call construct_J_Matrix(B%teint, F%D, J_matrix)
  call construct_K_Matrix(B%teint, F%D, K_matrix)

  DU(:,:,:) = ZERO

!  fh_ao = F%H0(:,:) + 0*J_matrix - 0*HALF * K_matrix 
  fh_ao = F%H
  fh0_ao = F%H0

  ! Transform quantities from F to MO space
  F%H0 = F%LC(:,:) .tm. F%H0(:,:) .m. F%LC(:,:)
  F%H  = F%LC(:,:) .tm. F%H(:,:) .m. F%LC(:,:)
  call transform_twoel( P, B, F, STDOUT )



  ! Compare to numerical dF/dR
  ! Move geometry by epsilon

  call construct_Atom_set( P, A2 )
  do k=1,size(B%atm,2)*3
    ig=mod(k-1,3)+1
    ia=(k-mod(k-1,3)-1)/3+1
    if(ig==1)then
      A2(ia)%x=A2(ia)%x+epsilon
    elseif(ig==2)then
      A2(ia)%y=A2(ia)%y+epsilon
    elseif(ig==3)then
      A2(ia)%z=A2(ia)%z+epsilon
    end if

    call construct_Grid3D( P, A2, G2 )
    call construct_Basis(A2,G2,B2,P%EPS_overlap, P%EPS_overlap_single, yes_sym=P%yes_sym ,gto_file_path=P%gto_file_path, yes_gradient=P%yes_gradient)
    call construct_WF_data(P, A2, G2, B2, F2 )
    call precalculate_twoel_GTO( P, B2 ) 
    F2%LC_ini=F%LC
    F2%iscfstate=2    
    call SCF_procedure( P, A2, G2, B2, F2 )


    call construct_J_Matrix(B2%teint, F2%D, J_matrix)
    call construct_K_Matrix(B2%teint, F2%D, K_matrix)
    F_grad_num(:,:,k) = ( F2%H0(:,:) + J_matrix - HALF * K_matrix - fh_ao(:,:) ) / epsilon
    h0_grad_num(:,:,k)=( F2%H0(:,:) - fh0_ao(:,:) ) / epsilon

!    F2%H0 = F%LC(:,:) .tm. F2%H0(:,:) .m. F%LC(:,:)
    !  Central difference approximation gradient
    do i=1,size(F2%LC,1)
        !egrad_num(i,k) = ( F2%H0(i,i) - F%H0(i,i)) / epsilon
        egrad_num(i,k) = ( F2%CE(i) - F%CE(i)) / epsilon
    end do

    ! back to R0
    call purge_WF_data(F2)
    if(ig==1)then
      A2(ia)%x=A2(ia)%x-epsilon
    elseif(ig==2)then
      A2(ia)%y=A2(ia)%y-epsilon
    elseif(ig==3)then
      A2(ia)%z=A2(ia)%z-epsilon
    end if


  end do

  !! calculate DU
  call gaussian_cphf_iteration( P, A, B, F, DU, orb_grad )
  !call gaussian_cphf_iteration( P, B, F, DU, F_grad )

  call h0_gradient(B%env,B%atm,B%bas,B%first_bf_of_shell,h0_grad)
  ! compute gradient contribution for the point charges
  do ia=1,size(B%atm,2)
    if(A(ia)%charge==0)then
      pos(1)=A(ia)%x
      pos(2)=A(ia)%y
      pos(3)=A(ia)%z
      call Rinv_gradient(B%env,B%atm,B%bas,B%first_bf_of_shell,pos,ia,Rinv_grad)
      h0_grad(:,:,:)=h0_grad(:,:,:)-A(ia)%fcharge * Rinv_grad(:,:,:)
    end if
  end do


  write(6,'("# Differences Analytical Numerical gradients of (d H0 / dR )_ij larger than ",es10.1)') 10*epsilon
  write(6,'("# ",a8,x,a8,x,a8,x,a15,a15,a15)') "i","j","coord","anal. grad","num. grad","diff"
  do k=1,size(B%atm,2)*3
    do i=1,size(F%LC,1)
      do j=1,size(F%LC,1)
        if(abs(h0_grad(i,j,k)-h0_grad_num(i,j,k))>10*epsilon)then 
          write(6,'(2x,i8,x,i8,x,i8,x,es15.5,es15.5,es15.5)') i,j,k,h0_grad(i,j,k),h0_grad_num(i,j,k),h0_grad(i,j,k)-h0_grad_num(i,j,k) 
          stop 1
        end if
      end do
    end do
  end do

  call jk_gradient(B%env,B%atm,B%bas,B%first_bf_of_shell,F%D,J_grad,K_grad)

  ! F_grad = h0_grad + J_grad - HALF * K_grad

  ! write(6,'("# Differences Analytical Numerical gradients of (d F / dR )_ij larger than ",es10.1)') 10*epsilon
  ! write(6,'("# ",a8,x,a8,x,a8,x,a15,a15,a15)') "i","j","coord","anal. grad","num. grad","diff"
  ! do k=1,size(B%atm,2)*3
  !   do i=1,size(F%LC,1)
  !     do j=1,size(F%LC,1)
  !       if(abs(F_grad(i,j,k)-F_grad_num(i,j,k))>10*epsilon)then 
  !         write(6,'(2x,i8,x,i8,x,i8,x,es15.5,es15.5,es15.5)') i,j,k,F_grad(i,j,k),F_grad_num(i,j,k),F_grad(i,j,k)-F_grad_num(i,j,k) 
  !       end if
  !     end do
  !   end do
  ! end do
  !         stop 1


  !! Write output
  write(6,'("# ----------- test orbital energy gradients ----------  ")') 
  write(6,'("# Numerical vs. analytical orbital energy gradients ")') 
  write(6,'("# Numerical (E_i(R0 + epsilon) - E_i(R0)) / epsilon ")') 
  write(6,'("# Analytical dF_ii(R)/dR | R_0 + d O_ii(r)/dR | R0 * E_i(R0)")') 
  write(6,'("# ",a8,x,a8,x,a15,x,a15,x,a15,1es15.5)') "i","coord","num. grad.","an. grad.","abs. diff. > ", 10*epsilon
  do k=1,size(B%atm,2)*3
    do i=1,size(F%LC,1)
        if( abs(egrad_num(i,k) - orb_grad(i,k)) > 10*epsilon ) then
          write(6,'(2x,i8,x,i8,x,1es15.5,x,1es15.5,x,1es15.5)') i,k,egrad_num(i,k),orb_grad(i,k),abs(egrad_num(i,k)-orb_grad(i,k))
          stop 1
        end if
    end do
  end do
  write(6,'("# ----------- end test orbital energy gradients ----------  ")') 
  write(6,'("All tests passed")') 


  deallocate(DU)
  deallocate(F_grad)
  deallocate(h0_grad)
  deallocate(J_grad)
  deallocate(K_grad)
  deallocate(temp)
  deallocate(egrad_num)
  deallocate(orb_grad)
  
end program test_ugrad 
