/*
  XMOLECULE electronic structure toolkit
  Copyright 2023 Ludger Inhester
  Ludger Inhester, Center for Free-Electron Laser Science, DESY
  ludger.inhester@cfel.de

  This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
  See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <sys/resource.h>
#include "../procmem.h"
#include "../xmolecule_interface.h"

#if defined(__INTEL_COMPILER) || defined(__INTEL_LLVM_COMPILER)
// Routines from libifcore
void for_rtl_init_(int *, char **);
int for_rtl_finish_();
// Wrappers
#define fc_init(argc, argv) \
  for_rtl_init_(&argc, argv)
#define fc_finish() \
  for_rtl_finish_()
#elif defined(__GNUC__)
// Routines from libgfortran
void _gfortran_set_args(int, char **);
// Wrappers
#define fc_init(argc, argv) \
  _gfortran_set_args(argc, argv)
#define fc_finish()
#endif

#define XMOL_OBJ_SIZE 1000
int main(int argc, char **argv){
  int i,j;
  void * xmol_data;
  int len;
  int open_shell;
  if(argc==1){
    printf("please provide xmolecule input file as argument\n");
    exit(0);
  }

  fc_init(argc, argv);
  //for_rtl_init_ (&argc, argv);
  len = strlen(argv[1]) + 1;
  xmol_data=malloc(sizeof(void *));
  xmolecule_setup(xmol_data, argv[1], &len);
  xmolecule_init(xmol_data);
  int natom=0;
  xmolecule_get_geom(xmol_data, NULL, &natom);
  double * geom=malloc(sizeof(double)*3*natom);
  xmolecule_get_geom(xmol_data, geom, &natom);  
  printf("geom:\n");
  for(i=0;i<natom;i++){
    printf("%f %f %f\n",geom[3*i+0],geom[3*i+1],geom[3*i+2]);
  }
  //  printf("write chkpt:\n");
  //  xmolecule_write_chkpt(xmol_data);

  int size_occ=0;
  xmolecule_get_occ(xmol_data,NULL,&size_occ);
  int * occ=malloc(sizeof(int)*size_occ);
  xmolecule_get_occ(xmol_data,occ,&size_occ);
  printf("occ: ");
  open_shell = 0;
  for(i=0;i<size_occ;i++){
    printf("%1d ",occ[i]);
    if(occ[i]==1) open_shell = 1;
  }
  printf("\n");


  /////////////
  double * energies;
  double * nacs;
  double * gradient;
  int * occs;
  int nstates=1;
  double energy;
  double * gradients;
  if (open_shell==0){
    xmolecule_get_energy(xmol_data,&energy);

    xmolecule_cis_energies(xmol_data,&energy,&nstates);
    nstates*=-1;
    printf("N states %d\n",nstates);
    energies=malloc(sizeof(double)*nstates);
    xmolecule_cis_energies(xmol_data, energies, &nstates);
    gradient=malloc(sizeof(double)*3*natom);
    nacs=malloc(sizeof(double)*3*natom*nstates*nstates);
    int state=1;

    int iter;
    for(iter=0;iter<5;iter++){
      size_t rss = getCurrentRSS();
      printf("MEMORY=%ld\n",rss/1024);
      xmolecule_cis_energies(xmol_data,energies,&nstates);
      xmolecule_get_cis_gradient_nacs(xmol_data, &state, gradient, &(natom), nacs, &(nstates));
      printf("gradient:\n");
      for(i=0;i<natom;i++){
        printf("%f %f %f\n",gradient[3*i+0],gradient[3*i+1],gradient[3*i+2]);
      }
      geom[0]-=0.1;
      xmolecule_set_geom(xmol_data, geom, &natom);
      xmolecule_get_geom(xmol_data, geom, &natom);
      printf("new geometry:\n");
      for(i=0;i<natom;i++){
        printf("%f %f %f\n",geom[3*i+0],geom[3*i+1],geom[3*i+2]);
      }
    }

    free(nacs);
    free(gradient);
    free(energies);
    free(occ);
  }else{

    int N_exc = 10;
    int locc = size_occ;
    occs = malloc(locc * N_exc * sizeof(int) );
    energies = malloc(N_exc * sizeof(double));
    gradients = malloc( 3 * natom * N_exc * sizeof(double));
    nacs = malloc(N_exc * N_exc * natom * 3 * sizeof(double));
    xmolecule_part_occs_energies_gradients_nacs(xmol_data,
                                                occ, energies,
                                                gradients, nacs,
                                                &N_exc, &natom, &locc);
    if (N_exc < 0)
    {
      N_exc = -N_exc;
      occs = realloc(occs, sizeof(int) * locc * N_exc);
      energies = realloc(energies, sizeof(double) * N_exc);
      gradients =
          realloc(gradients, sizeof(double) * 3 * natom * N_exc);
      nacs =
          realloc(nacs, sizeof(double) * N_exc * N_exc * natom * 3);
      xmolecule_part_occs_energies_gradients_nacs(xmol_data, occs,
                                                energies, gradients, nacs,
                                                &N_exc, &natom,
                                                &locc);
    }
    printf("%d\n", N_exc);

    int iter;
    for(iter=0;iter<10;iter++){
      size_t rss = getCurrentRSS();
      printf("MEMORY=%f mb\n",rss/1024./1024.);
      geom[0]-=0.1;
      xmolecule_set_geom(xmol_data, geom, &natom);
      xmolecule_part_occs_energies_gradients_nacs(xmol_data, occs,
                                                energies, gradients, nacs,
                                                &N_exc, &natom,
                                                &locc);
    }

    free(occs);
    free(energies);
    free(gradients);
    free(nacs);
  }
  xmolecule_purge(xmol_data);
  free(xmol_data);
  free(occ);
  free(geom);

  fc_finish();
  exit(0);

 
}
