/*
 XMOLECULE electronic structure toolkit
 Copyright 2023 Ludger Inhester
 Ludger Inhester, Center for Free-Electron Laser Science, DESY
 ludger.inhester@cfel.de

 This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
 See LICENSE.txt or https:\/\/www.gnu.org\/licenses\/gpl-3.0.html#license-text for a full license text.
*/

/*
prototypes for xmolecule_interface.h
*/

#ifndef XMOLECULEH
#define XMOLECULEH


#ifdef __INTEL_COMPILER
void for_rtl_init_(int *, char **);
int for_rtl_finish_();
#elif __GNUC__
void _gfortran_set_args(int argc, char *argv[]);
#endif

void xmolecule_purge(void *Xdata_cptr);
void xmolecule_setup(void *Xdata_cptr, char *filename, int *lenFilename);
void xmolecule_init(void *Xdata_cptr);
void xmolecule_get_energy(void *Xdata_cptr, double *energy);
void xmolecule_get_mass(void *Xdata_cptr, double *mass, int *l_mass);
void xmolecule_get_atomlabels(void *Xdata_cptr, char *labels, int *l_labels);
void xmolecule_get_ao_labels(void *Xdata_cptr, char *ao_labels, int *l_ao);
void xmolecule_get_overlapmatrix(void *Xdata_cptr, double *S_ao, int *l_ao);
void xmolecule_orbital_energies(void *Xdata_cptr, double *CE, int *l_CE);
void xmolecule_hole_occs_energies_efield(void *Xdata_cptr, double *pos, int *occs, double *energies, double *efields, double *upots, int *l_E, int *l_occ);
void xmolecule_hole_occs_energies_gradients_nacs(void *Xdata_cptr, int *occs, double *energies, double *gradients, double *nacs, int *l_E, int *l_gradient, int *l_occ);
void xmolecule_part_occs_energies_gradients_nacs(void *Xdata_cptr, int *occs, double *energies, double *gradients, double *nacs, int *l_E, int *l_gradient, int *l_occ);
void xmolecule_excitation_energies(void *Xdata_cptr, int *occs, double *energies, int *l_E, int *l_occ);
void xmolecule_cis_energies(void *Xdata_cptr, double *energies, int *l_E);
void xmolecule_set_ci_state(void *Xdata_cptr, int *state);
void xmolecule_ci_overlaps(void *Xdata_cptr, double *Olap, int *l_E);
void xmolecule_ci_energies(void *Xdata_cptr, double *energies, int *l_E);
void xmolecule_get_cis_gradient(void *Xdata_cptr, int *state, double *gradient, int *l_gradient);
void xmolecule_get_cis_gradient_nacs(void *Xdata_cptr, int *state, double *gradient, int *l_gradient, double *nacs, int *l_nacs);
void xmolecule_get_cis_transdens(void *Xdata_cptr, double *tdens, int *state, int *l_ao);
void xmolecule_get_gradient(void *Xdata_cptr, double *gradient, int *l_gradient);
void xmolecule_get_auger_ci(void *Xdata_cptr, int *f_irrep, int *f_multp, int *f_index, double *rate, double *energy, int *N_process);
void xmolecule_get_process(void *Xdata_cptr, int *final_occs, int *E_eV, double *val, unsigned char *tag, int *N_process, int *l_occ);
void xmolecule_get_population(void *Xdata_cptr, double *pop, int *l_mo, int *l_atom);
void xmolecule_get_mo_coef(void *Xdata_cptr, double *mo_coef, int *l_mo, int *l_ao);
void xmolecule_set_mo_coef(void *Xdata_cptr, double *mo_coef, int *l_ao, int *l_mo);
void xmolecule_get_nao_conf(void *Xdata_cptr, char *nao_conf_string, int *l_atom);
void xmolecule_get_mullikencharge(void *Xdata_cptr, double *charge, int *l_charge);
void xmolecule_get_loewdincharge(void *Xdata_cptr, double *charge, int *l_charge);
void xmolecule_get_spin_analysis(void *Xdata_cptr, double *center, double *rog);
void xmolecule_get_cis_mullikenpop(void *Xdata_cptr, double *charge, int *l_charge, int *state, char *type);
void xmolecule_get_cis_loewdinpop(void *Xdata_cptr, double *charge, int *l_charge, int *state, char *type);
void xmolecule_get_bondorder(void *Xdata_cptr, double *BO, int *n_atom);
void xmolecule_get_cis_oscillator_strength(void *Xdata_cptr, int *state, double *aCS, double *tdip, int *nstates);
void xmolecule_get_oscillator_strength(void *Xdata_cptr, double *aCS, double *tDip, int *nmo);
void xmolecule_get_fluorescencerates(void *Xdata_cptr, double *fluorates, int *nmo);
void xmolecule_write_molden(void *Xdata_cptr, char * argOutputfile, int *len);
void xmolecule_write_density_cube(void *Xdata_cptr);
void xmolecule_write_mo_cube(void *Xdata_cptr, int *mo);
void xmolecule_get_geom(void *Xdata_cptr, double *geom, int *l_geom);
void xmolecule_set_calculate_ci_overlap(void *Xdata_cptr, int *yes);
void xmolecule_set_reinit(void *Xdata_cptr, int *yes);
void xmolecule_set_geom(void *Xdata_cptr, double *geom, int *l_geom);
void xmolecule_write_chkpt(void *Xdata_cptr);
void xmolecule_read_chkpt(void *Xdata_cptr);
void xmolecule_get_occ(void *Xdata_cptr, int *occ, int *l_occ);
void xmolecule_set_multp(void *Xdata_cptr, int *multp_in);
void xmolecule_set_irrep(void *Xdata_cptr, int *irrep_in);
void cistate2occ(void *Xdata_cptr, int *state, int *occ, int *l_occ);
void occ2cistate(void *Xdata_cptr, int *occ_in, int *l_occ_in, int *state);
void xmolecule_set_occ(void *Xdata_cptr, int *occ_in, int *l_occ_in);

#endif