#!/usr/bin/python


import os

import unittest
import re
from subprocess import PIPE,Popen,call



class TestRateEquation(unittest.TestCase):

    xmol=""
    xmol_req=""

    xatom=""
    xatom_req=""

    def setUp(self):
        dir=os.getcwd()
        self.xatom=os.getenv("XATOM_PATH")
        print("Using xatom=",self.xatom  )
        self.xatom_req='/work/linhest/XATOM/src/xatom_req'
        print("Using xatom_req=",self.xatom_req  )

        self.xmol=dir+"/../src/xmolecule"
        print("Using xmolecule=",self.xmol)

        self.xmol_req=dir+"/../src/xmolecule_req"
        print("Using xmolecule_req=",self.xmol_req)


    def test_oxygen_req(self):
        # This Test calculates all states for the oxygen atom, calculated all transition rates and runs xmolecule_req
        # to compare the final charge state population 
#        ExpectedChargePop=[2.470E-04, 1.752E-04, 6.041E-03, 4.359E-03, 4.677E-02, 3.761E-02, 2.961E-01, 4.848E-01, 1.239E-01]
        # this is without valence optimization
        ExpectedChargePop=[2.450E-04, 1.724E-04, 5.880E-03, 4.723E-03, 5.050E-02, 4.701E-02, 3.597E-01, 4.637E-01, 6.804E-02]
# xatom_req result        [ 0.000245, 0.0001763,  0.005992,  0.004397,   0.04647,   0.03791,    0.2964,     0.484, 0.1244]

        print('Testing rate equation results for atomic oxygen (takes some time!)')



        #
        cmd=" echo ""> xatom_process.dat; for i in 0 1 2 ; do for j in 0 1 2 ; do for k in 0 1 2 3 4; do %s -A O -N 50 -rmax 20 -rm 1e-6 -latter off -conf 1s${i}_2s${j}_2p${k} -auger -pcs -fluorescence -process xatom_process.dat;  done; done; done > /dev/zero" % self.xatom
        os.system(cmd)

        cmd="%s -A O -pulse_shape gaussian -FWHM 10 -photons 1e15 -width 1 -d xatom_process.dat" % self.xatom_req
        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0].decode('UTF8')
        m=re.search('\|POP\|.*',out)
        ChargePopTxt=m.group(0).split()[1:]
        ChargePop=[]
        for p in ChargePopTxt:
                ChargePop.append(float(p))
        print(ChargePop)
        
        # set up input file
        f = open('input.in', 'w')
        f.write('system=oxygen\n')
        f.write('N_nuc= 3\n')
        f.write('O     0.0000000000        0.0000000000        0.0000000000    2s1p O.nao\n')
        f.write('average_occ_MO=3,4,5\n')
        f.write('average_rates=yes\n')

        f.close()

        # calculate all process data put them into process.dat
        o1=0
        o2=0
        o3=0    
        os.system('echo "" > process.dat')

        for o1 in range(0,3):
            for o2 in range(0,3):
                for o3 in range(0,5):
                    if ( o1+o2+o3 ==0 ):
                        continue
                    n1=o1
                    n2=o2
                    if(o3<=2):
                        n3=o3
                        n4=0
                        n5=0
                    elif( o3 <=4):
                        n3=2
                        n4=o3-2
                        n5=0
                    elif( o3 <=6):
                        n3=2
                        n4=2
                        n5=o3-4
#                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,o2,o3)
                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,2,4)
                    occstring="%1d%1d%1d%1d%1d" % (n1,n2,n3,n4,n5)
                    print('Calculating atomic configuration '+atomconf +" : "+ occstring)
                    cmd=self.xatom+' -s O -N 50 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao O.nao -transition transitiondipole -PE_start 11000 -PE_end 12000 -dE 10 -file_tdipole O.tdipole -transition auger -file_auger O.auger -conf ' + atomconf
                    os.system(cmd)
                    cmd=self.xmol+' input.in   -pcs -fluorescence -auger -occ '+ occstring + '| tee output.tmp| grep -v ^#|grep -v singular >> process.dat'
                    os.system(cmd)

        # run xmol_req
        cmd=self.xmol_req + ' input.in -N_photons 1e15 -FWHM 10 -occ 22220 '
        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0].decode('UTF8')

        # parse for POP line and compare results
        m=re.search('\|POP\|.*',out)
        ChargePopTxt=m.group(0).split()[3:]
        ChargePop=[]
        for p in ChargePopTxt:
                ChargePop.append(float(p))
        print('ChargePop        :',ChargePop)
        print('ExpectedChargePop:',ExpectedChargePop)
        for i in range(0,len(ChargePop)):
            self.assertAlmostEqual(ChargePop[i], ExpectedChargePop[i],places=2)




    def test_water_req(self):
        # This Test calculates all states for the water molecule, calculates all transition rates and runs xmolecule_req
        # to compare the final charge state population 
        ExpectedChargePop=[0.000256, 0.0001577, 0.005054, 0.003039, 0.03229, 0.02014, 0.1153, 0.08024, 0.3809, 0.3249, 0.03759]

        print('Testing rate equation results for water (takes some time!)')



        
        # set up input file
        f = open('input.in', 'w')
        f.write('system=water\n')
        f.write('N_nuc= 3\n')
        f.write('O     0.0000000000        0.0000000000        0.0000000000    2s1p O.nao\n')
        f.write('H     0.0000000000       -1.4309301866        1.1083919440    1s   H.nao\n')
        f.write('H     0.0000000000        1.4309301866        1.1083919440    1s   H.nao\n')

        f.close()

        # calculate all process data put them into process.dat
        
        os.system('echo "" > process.dat')

        cmd=self.xatom+' -s H -N 50 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao H.nao '
        os.system(cmd)

        n1old=0
        oldoccstring=""
        for nel in range(10,0,-1):
            for n1 in range(0,3):
                for n2 in range(0,3):
                    for n3 in range(0,3):
                        for n4 in range(0,3):
                            for n5 in range(0,3):
                                if ( n1+n2+n3+n4+n5 ==0 ):
                                    continue
                                if ( n1+n2+n3+n4+n5 !=nel ):
                                    continue
                                if(n1old!=n1):
                                    atomconf="1s%1d_2s%1d_2p%1d" % (n1,2,4)
                                    print('Calculating atomic configuration '+atomconf +" : ")
                                    cmd=self.xatom+' -s O -N 50 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao O.nao -transition transitiondipole -PE_start 11000 -PE_end 12000 -dE 10 -file_tdipole O.tdipole -transition auger -file_auger O.auger -conf ' + atomconf
                                    os.system(cmd)
                                    n1old=n1
                                occstring="%1d%1d%1d%1d%1d" % (n1,n2,n3,n4,n5)
                                print(oldoccstring,'->',occstring)
                                if(len(oldoccstring)==0):
                                    cmd=self.xmol+' input.in -SIni                     -pcs -fluorescence -auger -occ '+ occstring + '| tee output.tmp| grep -v ^#|grep -v singular  >> process.dat'
                                else:
                                    cmd=self.xmol+' input.in -SIni -RIni -RIniSh -MOM  -pcs -fluorescence -auger -occ '+ occstring + '| tee output.tmp| grep -v ^#|grep -v singular  >> process.dat'
                                os.system(cmd)
                                oldoccstring=occstring

        cmd='mv process.dat process.tmp'
        os.system(cmd)
        cmd='cat process.tmp | awk "\$1==\\"F\\" && \$5 < 10 {next} {print \$0}" > process.dat'
        os.system(cmd)


        # run xmol_req
        cmd=self.xmol_req + ' input.in -N_photons 1e15 -FWHM 10 -occ 22222 '
        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0].decode('UTF8')

        # parse for POP line and compare results
        m=re.search('\|POP\|.*',out)
        ChargePopTxt=m.group(0).split()[1:]

        ChargePop=[]
        for p in ChargePopTxt:
                ChargePop.append(float(p))
        print('ChargePop        :',ChargePop)
        print('ExpectedChargePop:',ExpectedChargePop)
        for i in range(0,len(ChargePop)):
            self.assertAlmostEqual(ChargePop[i], ExpectedChargePop[i],places=2)


if __name__ == "__main__": 
    unittest.main()


