#!/usr/bin/python


import os

import unittest
import re
from subprocess import PIPE,Popen,call



class TestRateEquation(unittest.TestCase):

    xmol=""
    xmol_req=""

    xatom=""
    xatom_req=""

    def setUp(self):
        dir=os.getcwd()
        self.xatom=os.getenv("XATOM_PATH")
        print "Using xatom=",self.xatom  
        self.xatom_req='/work/linhest/XATOM/src/xatom_req'
        print "Using xatom_req=",self.xatom_req  

        self.xmol=dir+"/../src/xmolecule"
        print "Using xmolecule=",self.xmol

        self.xmol_req=dir+"/../src/xmolecule_req"
        print "Using xmolecule_req=",self.xmol_req


    def test_neon_req(self):
        # This Test calculates all states for the neon atom, calculated all transition rates and runs xmolecule_req
        # to compare the final charge state population 
        # this is the result with valence optimization
#        ExpectedChargePop=[0.01576, 0.005394, 0.08142, 0.02765, 0.1792, 0.06133, 0.2298, 0.08395, 0.2216, 0.08614, 0.007715]
        # this is without valence optimization
        ExpectedChargePop= [0.01576, 0.004729, 0.06701, 0.0225, 0.148, 0.05535, 0.2148, 0.09724, 0.255, 0.1113, 0.008244]
#       xatom result      [0.01576, 0.005394, 0.08142, 0.02765, 0.1792, 0.0612, 0.2284, 0.08317, 0.2204, 0.08892, 0.008466]

        print 'Testing rate equation results for atomic neon (takes some time!)'



        #
        cmd=" echo ""> xatom_process.dat; for i in 0 1 2 ; do for j in 0 1 2 ; do for k in 0 1 2 3 4 5 6 ; do %s -A Ne -N 50 -PE 2000 -rmax 20 -rm 1e-6 -latter off -conf 1s${i}_2s${j}_2p${k} -auger -pcs -fluorescence -process xatom_process.dat;  done; done; done > /dev/zero" % self.xatom
        os.system(cmd)

        cmd="%s -A Ne -pulse_shape gaussian -FWHM 10 -photons 1e12 -d xatom_process.dat" % self.xatom_req
        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0]
        m=re.search('\|POP\|.*',out)
        ChargePopTxt=m.group(0).split()[1:]
        ChargePop=[]
        for p in ChargePopTxt:
                ChargePop.append(float(p))
        print ChargePop
        
        # set up input file
        f = open('input.in', 'w')
        f.write('system=neon\n')
        f.write('XATOM_PATH=%s\n' %self.xatom)
        f.write('N_nuc= 3\n')
        f.write('PE=2000\n')
        f.write('Ne     0.0000000000        0.0000000000        0.0000000000    2s1p Ne.nao\n')
        f.write('average_occ_MO=3,4,5\n')
        f.write('average_rates=yes\n')

        f.close()

        # calculate all process data put them into process.dat
        o1=0
        o2=0
        o3=0    
        os.system('echo "" > process.dat')

        for o1 in range(0,3):
            for o2 in range(0,3):
                for o3 in range(0,7):
                    if ( o1+o2+o3 ==0 ):
                        continue
                    n1=o1
                    n2=o2
                    if(o3<=2):
                        n3=o3
                        n4=0
                        n5=0
                    elif( o3 <=4):
                        n3=2
                        n4=o3-2
                        n5=0
                    elif( o3 <=6):
                        n3=2
                        n4=2
                        n5=o3-4
#                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,o2,o3)
                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,2,6)
                    occstring="%1d%1d%1d%1d%1d" % (n1,n2,n3,n4,n5)
                    print 'Calculating atomic configuration '+atomconf +" : "+ occstring
                    cmd=self.xatom+' -s Ne -N 50 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao Ne.nao -transition transitiondipole -PE_start 1000 -PE_end 2000 -dE 10 -file_tdipole Ne.tdipole -transition auger -file_auger Ne.auger -conf ' + atomconf
                    os.system(cmd)
                    cmd=self.xmol+' input.in   -pcs -fluorescence -auger -occ '+ occstring + '| tee output.tmp| grep -v ^#|grep -v singular >> process.dat'
                    os.system(cmd)

        # run xmol_req
        cmd=self.xmol_req + ' input.in -N_photons 1e12 -FWHM 10 -occ 22222 '
        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0]

        # parse for POP line and compare results
        m=re.search('\|POP\|.*',out)
        ChargePopTxt=m.group(0).split()[1:]
        ChargePop=[]
        for p in ChargePopTxt:
                ChargePop.append(float(p))

        print 'ChargePop        :',ChargePop
        print 'ExpectedChargePop:',ExpectedChargePop
        for i in range(0,len(ChargePop)):
            self.assertAlmostEqual(ChargePop[i], ExpectedChargePop[i],places=2)



if __name__ == "__main__": 
    unittest.main()


