#!/usr/bin/python


import os

import unittest
import re
from subprocess import PIPE,Popen,call



class TestRateEquation(unittest.TestCase):

    xmol=""
    xmol_req=""

    xatom=""
    xatom_req=""

    def setUp(self):
        dir=os.getcwd()
        self.xatom=os.getenv("XATOM_PATH")
        print "Using xatom=",self.xatom  
        self.xatom_req='/work/linhest/XATOM/src/xatom_req'
        print "Using xatom_req=",self.xatom_req  

        self.xmol=dir+"/../src/xmolecule"
        print "Using xmolecule=",self.xmol

        self.xmol_req=dir+"/../src/xmolecule_req"
        print "Using xmolecule_req=",self.xmol_req


    def test_neon_req(self):
        # This Test calculates all states for the neon atom, calculated all transition rates and runs xmolecule_req
        # to compare the final charge state population 

        # this is with valence optimization
#        ExpectedChargePop=[0.1131, 0.01892, 0.2667, 0.04357, 0.2688, 0.04324, 0.1537, 0.02553, 0.05607, 0.009877, 0.0004547]
        # this is without valence optimization
        ExpectedChargePop=[0.1131, 0.01873, 0.2628, 0.04355, 0.2749, 0.04491, 0.1576, 0.02671, 0.05053, 0.007026, 0.0001746]
# xatom result              [0.1131, 0.01892, 0.2667, 0.04357, 0.2688, 0.04324, 0.1537, 0.02553, 0.05607, 0.009877, 0.0004547]

        print 'Testing rate equation results for atomic neon (takes some time!)'



        #
        cmd=" echo ""> xatom_process.dat; for i in 0 1 2 ; do for j in 0 1 2 ; do for k in 0 1 2 3 4 5 6 ; do %s -A Ne -N 50 -PE 12000 -rmax 20 -rm 1e-6 -latter off -conf 1s${i}_2s${j}_2p${k} -auger -pcs -fluorescence -process xatom_process.dat;  done; done; done > /dev/zero" % self.xatom
        os.system(cmd)

        cmd="%s -A Ne -pulse_shape gaussian -FWHM 10 -photons 1e14 -d xatom_process.dat" % self.xatom_req
        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0]
        m=re.search('\|POP\|.*',out)
        ChargePopTxt=m.group(0).split()[1:]
        ChargePop=[]
        for p in ChargePopTxt:
                ChargePop.append(float(p))
        print ChargePop
        
        # set up input file
        f = open('input.in', 'w')
        f.write('system=neon\n')
        f.write('XATOM_PATH=%s\n' %self.xatom)
        f.write('N_nuc= 3\n')
        f.write('PE=12000\n')
        f.write('Ne     0.0000000000        0.0000000000        0.0000000000    2s1p Ne.nao\n')
        f.write('average_occ_MO=3,4,5\n')
        f.write('average_rates=yes\n')

        f.close()

        # calculate all process data put them into process.dat
        o1=0
        o2=0
        o3=0    
        os.system('echo "" > process.dat')

        for o1 in range(0,3):
            for o2 in range(0,3):
                for o3 in range(0,7):
                    if ( o1+o2+o3 ==0 ):
                        continue
                    n1=o1
                    n2=o2
                    if(o3<=2):
                        n3=o3
                        n4=0
                        n5=0
                    elif( o3 <=4):
                        n3=2
                        n4=o3-2
                        n5=0
                    elif( o3 <=6):
                        n3=2
                        n4=2
                        n5=o3-4
#                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,o2,o3)
                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,2,6)
                    occstring="%1d%1d%1d%1d%1d" % (n1,n2,n3,n4,n5)
                    print 'Calculating atomic configuration '+atomconf +" : "+ occstring
                    cmd=self.xatom+' -s Ne -N 50 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao Ne.nao -transition transitiondipole -PE_start 10000 -PE_end 12000 -dE 10 -file_tdipole Ne.tdipole -transition auger -file_auger Ne.auger -conf ' + atomconf
                    os.system(cmd)
                    cmd=self.xmol+' input.in   -pcs -fluorescence -auger -occ '+ occstring + '| tee output.tmp| grep -v ^#|grep -v singular >> process.dat'
                    os.system(cmd)

        # run xmol_req
        cmd=self.xmol_req + ' input.in -N_photons 1e14 -FWHM 10 '
        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0]

        # parse for POP line and compare results
        m=re.search('\|POP\|.*',out)
        ChargePopTxt=m.group(0).split()[1:]
        ChargePop=[]
        for p in ChargePopTxt:
                ChargePop.append(float(p))

        print 'ChargePop        :',ChargePop
        print 'ExpectedChargePop:',ExpectedChargePop
        for i in range(0,len(ChargePop)):
            self.assertAlmostEqual(ChargePop[i], ExpectedChargePop[i],places=2)



if __name__ == "__main__": 
    unittest.main()


