#!/usr/bin/python


import os

import unittest
import re
from subprocess import PIPE,Popen,call

import sys
lib_path = os.path.abspath(os.path.join('..',  'src'))
sys.path.append(lib_path)
import rate_solver as RS


class TestRateSolver(unittest.TestCase):


    xmol=""
    xmol_req=""

    xatom=""
    xatom_req=""

    def setUp(self):
        dir=os.getcwd()
        self.xatom=os.getenv("XATOM_PATH")
        print "Using xatom=",self.xatom  
        self.xatom_req='/work/linhest/XATOM/src/xatom_req'
        print "Using xatom_req=",self.xatom_req  

        self.xmol=dir+"/../src/xmolecule"
        print "Using xmolecule=",self.xmol

        self.xmol_req=dir+"/../src/xmolecule_req"
        print "Using xmolecule_req=",self.xmol_req


    def test_oxygen_req(self):
        # This Test calculates all states for the oxygen atom, calculated all transition rates and runs xmolecule_req
        # to compare the final charge state population 
#        ExpectedChargePop=[2.470E-04, 1.752E-04, 6.041E-03, 4.359E-03, 4.677E-02, 3.761E-02, 2.961E-01, 4.848E-01, 1.239E-01]
        # this is without valence optimization
        ExpectedChargePop=[2.450E-04, 1.724E-04, 5.880E-03, 4.723E-03, 5.050E-02, 4.701E-02, 3.597E-01, 4.637E-01, 6.804E-02]
        print 'Testing rate equation results for atomic oxygen (takes some time!)'


        # set up input file
        f = open('input.in', 'w')
        f.write('system=oxygen\n')
        f.write('N_nuc= 3\n')
        f.write('O     0.0000000000        0.0000000000        0.0000000000    2s1p O.nao\n')
        f.write('average_occ_MO=3,4,5\n')
        f.write('average_rates=yes\n')

        f.close()

        # calculate all process data put them into process.dat
        o1=0
        o2=0
        o3=0    
        os.system('echo "" > process.dat')

        for o1 in range(0,3):
            for o2 in range(0,3):
                for o3 in range(0,5):
                    if ( o1+o2+o3 ==0 ):
                        continue
                    n1=o1
                    n2=o2
                    if(o3<=2):
                        n3=o3
                        n4=0
                        n5=0
                    elif( o3 <=4):
                        n3=2
                        n4=o3-2
                        n5=0
                    elif( o3 <=6):
                        n3=2
                        n4=2
                        n5=o3=2
#                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,o2,o3)
                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,2,4)
                    occstring="%1d%1d%1d%1d%1d" % (n1,n2,n3,n4,n5)
                    print 'Calculating atomic configuration '+atomconf +" : "+ occstring
                    cmd=self.xatom+' -s O -N 50 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao O.nao -transition transitiondipole -PE_start 11000 -PE_end 12000 -dE 10 -file_tdipole O.tdipole -transition auger -file_auger O.auger -conf ' + atomconf
                    os.system(cmd)
                    cmd=self.xmol+' input.in   -pcs -fluorescence -auger -occ '+ occstring + '| tee output.tmp| grep -v ^#|grep -v singular >> process.dat'
                    os.system(cmd)

        # run xmol_req
        #cmd=self.xmol_req + ' input.in -N_photons 1e15 -FWHM 10 -occ 22220 '
        #p=Popen(cmd,stdout=PIPE, shell=True)
        #out=p.communicate()[0]

        R=RS.rate_solver()
        R.startocc="22220"
        R.run()


        # parse for POP line and compare results
#        m=re.search('\|POP\|.*',out)
#        ChargePopTxt=m.group(0).split()[3:]
        N=len(R.statelist)
        ChargePop=R.pq(R.C_end[:N])[::-1].tolist()
        print 'ChargePop        :',ChargePop
        print 'ExpectedChargePop:',ExpectedChargePop
        for i in range(0,len(ChargePop)):
            self.assertAlmostEqual(ChargePop[i], ExpectedChargePop[i],places=2)


    def test_neon_req(self):
        # This Test calculates all states for the oxygen atom, calculated all transition rates and runs xmolecule_req
        # to compare the final charge state population 
        ExpectedChargePop=[3.4017169797790643e-10, 0.008297753754207878, 0.3860450726413414, 0.2866356745861758, 0.12432872831487828, 0.10531543272637747, 0.02895662997075155, 0.02163700498795218, 0.006170397979155755, 0.014017663280678737, 0.018595641418309546]
        # this is without valence optimization
        #ExpectedChargePop=[3.396642879520738e-10, 0.00829782748066162, 0.38602738737965997, 0.2830257597060457, 0.11402346425868977, 0.11522426037367176, 0.02995830654947043, 0.024493558380346343, 0.010249825834165192, 0.019616751582525807, 0.009082858115099136]
        # without valence option & with polarization basis functions
        # ExpectedChargePop=[3.402263184834573e-10, 0.008297796963359663, 0.3860741270054718, 0.2875462941183997, 0.12764467013961844, 0.10547864686581683, 0.027439105436187262, 0.019502418440709397, 0.004984546249296653, 0.016159107303648088, 0.016873287137265772]

        print 'Testing rate equation results for atomic neon (takes some time!)'


        # set up input file
        f = open('input.in', 'w')
        f.write('system=neon\n')
        f.write('N_nuc= 1\n')
        f.write('Ne     0.0000000000        0.0000000000        0.0000000000    2s2p1d Ne.nao\n')
#        f.write('Ne     0.0000000000        0.0000000000        0.0000000000    2s1p Ne.nao\n')
        f.write('average_occ_MO=3,4,5\n')
        f.close()

        # calculate all process data put them into process.dat
        o1=0
        o2=0
        o3=0    
        os.system('echo "" > process.dat')

        for o1 in range(0,3):
            for o2 in range(0,3):
                for o3 in range(0,7):
                    if ( o1+o2+o3 ==0 ):
                        continue
                    n1=o1
                    n2=o2
                    if(o3<=2):
                        n3=o3
                        n4=0
                        n5=0
                    elif( o3 <=4):
                        n3=2
                        n4=o3-2
                        n5=0
                    elif( o3 <=6):
                        n3=2
                        n4=2
                        n5=o3-4
#                   for fully relaxed atomic configuration
#                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,o2,o3)
#                   for core hole relaxed atomic configuration
                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,2,6)
                    occstring="%1d%1d%1d%1d%1d" % (n1,n2,n3,n4,n5)
                    print 'Calculating atomic configuration '+atomconf +" : "+ occstring
#                    cmd=self.xatom+' -s Ne -N 50 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao Ne.nao -transition transitiondipole -PE_start 11000 -PE_end 12000 -dE 10 -file_tdipole Ne.tdipole -transition auger -file_auger Ne.auger -conf ' + atomconf
                    cmd=self.xatom+' -s Ne -N 50 -silent -rmax 20 -rm 1e-6 -latter false -polarization -nao -file_nao Ne.nao -transition transitiondipole -PE_start 11000 -PE_end 12000 -dE 10 -file_tdipole Ne.tdipole -transition auger -file_auger Ne.auger -conf ' + atomconf
                    os.system(cmd)
                    cmd=self.xmol+' input.in   -pcs -fluorescence -auger -occ '+ occstring + '| tee output.tmp| grep -v ^#|grep -v singular >> process.dat'
                    os.system(cmd)

        R=RS.rate_solver()
        R.startocc="22222"
        R.run()

        N=len(R.statelist)
        ChargePop=R.pq(R.C_end[:N])[::-1].tolist()
        print 'ChargePop        :',ChargePop
        print 'ExpectedChargePop:',ExpectedChargePop
        for i in range(0,len(ChargePop)):
            self.assertAlmostEqual(ChargePop[i], ExpectedChargePop[i],places=2)




    def test_water_req(self):
        # This Test calculates all states for the water molecule, calculates all transition rates and runs xmolecule_req
        # to compare the final charge state population 
        ExpectedChargePop=[0.000256, 0.0001577, 0.005054, 0.003039, 0.03229, 0.02014, 0.1153, 0.08024, 0.3809, 0.3249, 0.03759]

        print 'Testing rate equation results for water (takes some time!)'



        
        # set up input file
        f = open('input.in', 'w')
        f.write('system=water\n')
        f.write('N_nuc= 3\n')
        f.write('O     0.0000000000        0.0000000000        0.0000000000    2s1p O.nao\n')
        f.write('H     0.0000000000       -1.4309301866        1.1083919440    1s   H.nao\n')
        f.write('H     0.0000000000        1.4309301866        1.1083919440    1s   H.nao\n')

        f.close()

        # calculate all process data put them into process.dat
        
        os.system('echo "" > process.dat')

        cmd=self.xatom+' -s H -N 50 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao H.nao '
        os.system(cmd)

        n1old=0
        oldoccstring=""
        for nel in range(10,0,-1):
            for n1 in range(0,3):
                for n2 in range(0,3):
                    for n3 in range(0,3):
                        for n4 in range(0,3):
                            for n5 in range(0,3):
                                if ( n1+n2+n3+n4+n5 ==0 ):
                                    continue
                                if ( n1+n2+n3+n4+n5 !=nel ):
                                    continue
                                if(n1old!=n1):
                                    atomconf="1s%1d_2s%1d_2p%1d" % (n1,2,4)
                                    print 'Calculating atomic configuration '+atomconf +" : "
                                    cmd=self.xatom+' -s O -N 50 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao O.nao -transition transitiondipole -PE_start 11000 -PE_end 12000 -dE 10 -file_tdipole O.tdipole -transition auger -file_auger O.auger -conf ' + atomconf
                                    os.system(cmd)
                                    n1old=n1
                                occstring="%1d%1d%1d%1d%1d" % (n1,n2,n3,n4,n5)
                                print oldoccstring,'->',occstring
                                if(len(oldoccstring)==0):
                                    cmd=self.xmol+' input.in -SIni                     -pcs -fluorescence -auger -occ '+ occstring + '| tee output.tmp| grep -v ^#|grep -v singular |grep -v WARNING >> process.dat'
                                else:
                                    cmd=self.xmol+' input.in -SIni -RIni -RIniSh -MOM  -pcs -fluorescence -auger -occ '+ occstring + '| tee output.tmp| grep -v ^#|grep -v singular |grep -v WARNING >> process.dat'
                                os.system(cmd)
                                oldoccstring=occstring

        cmd='mv process.dat process.tmp'
        os.system(cmd)
        cmd='cat process.tmp | awk "\$1==\\"F\\" && \$5 < 10 {next} {print \$0}" > process.dat'
        os.system(cmd)


        R=RS.rate_solver()
        R.startocc="22222"
        R.run()


        # parse for POP line and compare results
#        m=re.search('\|POP\|.*',out)
#        ChargePopTxt=m.group(0).split()[3:]
        N=len(R.statelist)
        ChargePop=R.pq(R.C_end[:N])[::-1].tolist()
        print 'ChargePop        :',ChargePop
        print 'ExpectedChargePop:',ExpectedChargePop
        for i in range(0,len(ChargePop)):
            self.assertAlmostEqual(ChargePop[i], ExpectedChargePop[i],places=2)


if __name__ == "__main__": 
    unittest.main()


