#!/usr/bin/python


import os

import unittest
import re
from subprocess import PIPE,Popen,call

import sys
lib_path = os.path.abspath(os.path.join('..',  'src'))
sys.path.append(lib_path)
import rate_solver as RS


class TestRateSolver(unittest.TestCase):


    xmol=""
    xmol_req=""

    xatom=""
    xatom_req=""

    def setUp(self):
        dir=os.getcwd()
        self.xatom=os.getenv("XATOM_PATH")
        print "Using xatom=",self.xatom  
        self.xatom_req='/work/linhest/XATOM/src/xatom_req'
        print "Using xatom_req=",self.xatom_req  

        self.xmol=dir+"/../src/xmolecule"
        print "Using xmolecule=",self.xmol

        self.xmol_req=dir+"/../src/xmolecule_req"
        print "Using xmolecule_req=",self.xmol_req


    def test_oxygen_req(self):
        # This Test calculates all states for the oxygen atom, calculated all transition rates and runs xmolecule_req
        # to compare the final charge state population 
#        ExpectedChargePop=[2.470E-04, 1.752E-04, 6.041E-03, 4.359E-03, 4.677E-02, 3.761E-02, 2.961E-01, 4.848E-01, 1.239E-01]
        # this is without valence optimization
        ExpectedChargePop=[0.00024498173584777547, 0.0001708668722657607, 0.006080223074720327, 0.005525958180088638, 0.0699504509231747, 0.18971569975621744, 0.4249344428610209, 0.26099471178563277, 0.042382664811031444]
        print 'Testing rate equation results for atomic oxygen (takes some time!)'


        # set up input file
        f = open('input.in', 'w')
        f.write('system=oxygen\n')
        f.write('N_nuc= 3\n')
        f.write('O     0.0000000000        0.0000000000        0.0000000000    2s1p O.nao\n')
        f.write('average_occ_MO=3,4,5\n')
        f.write('process_spin=yes\n')
        f.write('all_occs=yes\n')

        f.close()

        # calculate all process data put them into process.dat
        o1=0
        o2=0
        o3=0    
        os.system('echo "" > process.dat')

        for o1 in range(0,3):
            for o2 in range(0,3):
                for o3 in range(0,5):
                    if ( o1+o2+o3 ==0 ):
                        continue
                    n1=o1
                    n2=o2
                    if(o3<=2):
                        n3=o3
                        n4=0
                        n5=0
                    elif( o3 <=4):
                        n3=2
                        n4=o3-2
                        n5=0
                    elif( o3 <=6):
                        n3=2
                        n4=2
                        n5=o3-4
#                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,o2,o3)
                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,2,4)
                    occstring="%1d%1d%1d%1d%1d" % (n1,n2,n3,n4,n5)
                    print 'Calculating atomic configuration '+atomconf +" : "+ occstring
                    cmd=self.xatom+' -s O -N 50 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao O.nao -transition transitiondipole -PE_start 11000 -PE_end 12000 -dE 10 -file_tdipole O.tdipole -transition auger -file_auger O.auger -conf ' + atomconf
                    os.system(cmd)
                    cmd=self.xmol+' input.in   -pcs -fluorescence -auger -occ '+ occstring + '| tee output.tmp| grep -v ^#|grep -v singular >> process.dat'
                    os.system(cmd)

        R=RS.rate_solver()
        R.startocc="22211"
        R.run()

        N=len(R.statelist)
        ChargePop=R.pq(R.C_end[:N])[::-1].tolist()
        print 'ChargePop        :',ChargePop
        print 'ExpectedChargePop:',ExpectedChargePop
        for i in range(0,len(ChargePop)):
            self.assertAlmostEqual(ChargePop[i], ExpectedChargePop[i],places=2)




    def test_neon_req(self):
        # This Test calculates all states for the oxygen atom, calculated all transition rates and runs xmolecule_req
        # to compare the final charge state population 
        # ExpectedChargePop=[3.416447164276165e-10, 3.960353558993479e-09, 1.9899210159399748e-07, 9.36949618678834e-07, 1.487283941392715e-05, 6.0995808247271115e-05, 0.0012431785564136076, 0.024428407868576495, 0.22203065475910447, 0.4233791287250862, 0.3288416211994401]
        # this is without valence optimization
        ExpectedChargePop=[3.407223661944444e-10, 3.8068692465581266e-09, 1.9250062102219844e-07, 1.2251194025747525e-06, 2.106424469689511e-05, 0.00015858743668773596, 0.0033550954192700863, 0.05022600496655527, 0.30913849756224643, 0.46656097382453066, 0.1705383547783975]
        # without valence option & with polarization basis functions
        # ExpectedChargePop=[3.4080376891157357e-10, 3.8382692826982924e-09, 1.8818012652929812e-07, 8.025088098197036e-07, 1.2366125386160618e-05, 4.4990986761541e-05, 0.001040592064573003, 0.020521741909483582, 0.2007592171546151, 0.4755430932796751, 0.3020770036114965]

        print 'Testing rate equation results for atomic neon (takes some time!)'


        # set up input file
        f = open('input.in', 'w')
        f.write('system=neon\n')
        f.write('N_nuc= 1\n')
#        f.write('Ne     0.0000000000        0.0000000000        0.0000000000    2s2p1d Ne.nao\n')
        f.write('Ne     0.0000000000        0.0000000000        0.0000000000    2s1p Ne.nao\n')
        f.write('average_occ_MO=3,4,5\n')
        f.write('process_spin=yes\n')
        f.write('all_occs=yes\n')

        f.close()

        # calculate all process data put them into process.dat
        o1=0
        o2=0
        o3=0    
        os.system('echo "" > process.dat')

        for o1 in range(0,3):
            for o2 in range(0,3):
                for o3 in range(0,7):
                    if ( o1+o2+o3 ==0 ):
                        continue
                    n1=o1
                    n2=o2
                    if(o3<=2):
                        n3=o3
                        n4=0
                        n5=0
                    elif( o3 <=4):
                        n3=2
                        n4=o3-2
                        n5=0
                    elif( o3 <=6):
                        n3=2
                        n4=2
                        n5=o3-4
#                   for fully relaxed atomic configuration
#                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,o2,o3)
#                   for core hole relaxed atomic configuration
                    atomconf="1s%1d_2s%1d_2p%1d" % (o1,2,6)
                    occstring="%1d%1d%1d%1d%1d" % (n1,n2,n3,n4,n5)
                    print 'Calculating atomic configuration '+atomconf +" : "+ occstring
                    cmd=self.xatom+' -s Ne -N 50 -silent -rmax 20 -rm 1e-6 -latter false -nao -file_nao Ne.nao -transition transitiondipole -PE_start 11000 -PE_end 12000 -dE 10 -file_tdipole Ne.tdipole -transition auger -file_auger Ne.auger -conf ' + atomconf
                    os.system(cmd)
                    cmd=self.xmol+' input.in   -pcs -fluorescence -auger -occ '+ occstring + '| tee output.tmp| grep -v ^#|grep -v singular >> process.dat'
                    os.system(cmd)

        R=RS.rate_solver()
        R.startocc="22222"
        R.run()

        N=len(R.statelist)
        ChargePop=R.pq(R.C_end[:N])[::-1].tolist()
        print 'ChargePop        :',ChargePop
        print 'ExpectedChargePop:',ExpectedChargePop
        for i in range(0,len(ChargePop)):
            self.assertAlmostEqual(ChargePop[i], ExpectedChargePop[i],places=2)



    def test_water_req(self):
        # This Test calculates all states for the water molecule, calculates all transition rates and runs xmolecule_req
        # to compare the final charge state population 
        ExpectedChargePop=[0.00025604784123971723, 0.00015770223908854423, 0.005055406794092078, 0.0031043686592142003, 0.03183451496477675, 0.020066817242131312, 0.1158179351459192, 0.1513494348694407, 0.34690277933410923, 0.2818894872156127, 0.04356550569437534]

        print 'Testing rate equation results for water (takes some time!)'



        
        # set up input file
        f = open('input.in', 'w')
        f.write('system=water\n')
        f.write('N_nuc= 3\n')
        f.write('O     0.0000000000        0.0000000000        0.0000000000    2s1p O.nao\n')
        f.write('H     0.0000000000       -1.4309301866        1.1083919440    1s   H.nao\n')
        f.write('H     0.0000000000        1.4309301866        1.1083919440    1s   H.nao\n')
        f.write('process_spin=yes\n')

        f.close()

        # calculate all process data put them into process.dat
        
        os.system('echo "" > process.dat')

        cmd=self.xatom+' -s H -N 50 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao H.nao '
        os.system(cmd)

        n1old=0
        oldoccstring=""
        for nel in range(10,0,-1):
            for n1 in range(0,3):
                for n2 in range(0,3):
                    for n3 in range(0,3):
                        for n4 in range(0,3):
                            for n5 in range(0,3):
                                if ( n1+n2+n3+n4+n5 ==0 ):
                                    continue
                                if ( n1+n2+n3+n4+n5 !=nel ):
                                    continue
                                if(n1old!=n1):
                                    atomconf="1s%1d_2s%1d_2p%1d" % (n1,2,4)
                                    print 'Calculating atomic configuration '+atomconf +" : "
                                    cmd=self.xatom+' -s O -N 50 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao O.nao -transition transitiondipole -PE_start 11000 -PE_end 12000 -dE 10 -file_tdipole O.tdipole -transition auger -file_auger O.auger -conf ' + atomconf
                                    os.system(cmd)
                                    n1old=n1
                                occstring="%1d%1d%1d%1d%1d" % (n1,n2,n3,n4,n5)
                                print oldoccstring,'->',occstring
                                if(len(oldoccstring)==0):
                                    cmd=self.xmol+' input.in -SIni                     -pcs -fluorescence -auger -occ '+ occstring + '| tee output.tmp| grep -v ^#|grep -v singular |grep -v WARNING >> process.dat'
                                else:
                                    cmd=self.xmol+' input.in -SIni -RIni -RIniSh -MOM  -pcs -fluorescence -auger -occ '+ occstring + '| tee output.tmp| grep -v ^#|grep -v singular |grep -v WARNING >> process.dat'
                                os.system(cmd)
                                oldoccstring=occstring

        cmd='mv process.dat process.tmp'
        os.system(cmd)
        cmd='cat process.tmp | awk "\$1==\\"F\\" && \$5 < 10 {next} {print \$0}" > process.dat'
        os.system(cmd)


        R=RS.rate_solver()
        R.startocc="22222"
        R.run()


        # parse for POP line and compare results
#        m=re.search('\|POP\|.*',out)
#        ChargePopTxt=m.group(0).split()[3:]
        N=len(R.statelist)
        ChargePop=R.pq(R.C_end[:N])[::-1].tolist()
        print 'ChargePop        :',ChargePop
        print 'ExpectedChargePop:',ExpectedChargePop
        for i in range(0,len(ChargePop)):
            self.assertAlmostEqual(ChargePop[i], ExpectedChargePop[i],places=2)


if __name__ == "__main__": 
    unittest.main()


