#!/usr/bin/env python
import os
from sys import exit,stdout
import unittest
import re

import random
import numpy as np
from math import sqrt,acos
import pickle
import copy
from python_xmolecule_interface import xmol

class TestOxygenRates(unittest.TestCase):

    xmol=""
    xmol_req=""
    xatom=""

    def setUp(self):
        dir=os.getcwd()
        self.xatom=os.getenv("XATOM_PATH")
        if not os.path.isfile(self.xatom):
             print('Please specify xatom path!')
             exit(-1)
        print("Using xatom=",self.xatom)  


    def test_water(self):
        print('Checking various interface interaction  ')
        # set up input file
        f = open('input.in', 'w')
        f.write('MOM=yes\n')
        f.write('SCF_energy_conv=1e-8\n')
        f.write('SCF_density_conv=1e-8\n')
        f.write('XATOM_PATH=%s\n' % self.xatom)
        f.write('O       0.0000000000        0.0000000000  0.0000000000         \n')
        f.write('H      -1.4309301866        1.1083919440  0.0000000000        \n')
        f.write('H       1.4309301866        1.1083919440  0.0000000000        \n')
        f.close()

        
        XM=xmol()
        XM.init()

        g=XM.get_geom()
        E1=XM.energy
        stdout.flush()
        try:
            with open( "XM.pickle", "wb" ) as f:
                pickle.dump(XM,f)
                f.close()
        except:
            print("pickling failed (should only work for python>=3.4 )")

        g[0]=g[0]+0.1
        XM.set_geom(g)        
        E2=XM.energy        
        print(E1,E2)

        occ=XM.get_occ()
        occ[0]=1
        XM.set_occ(occ)
        E3=XM.energy        
        print(E2,E3)

        print("process")
        P=XM.process()
        for p in P:
            print(p)

        print("BO")
        BO=XM.bondorder()
        print(BO)

        print("mulliken charge")
        Q=XM.mullikencharge()
        print(Q)

        print("gradient")
        G=XM.gradient()
        print(G)

        print("orbital energies")
        eps=XM.orbital_energies()
        print(eps)

        print("valence excitation energies")
        exc=XM.excitation_energies()
        for e in exc:
            print(e)

        try:        
            with open( "XM.pickle", "rb" ) as f:
                XM=pickle.load(f)
                f.close()
                stdout.flush()
        except:
            print("un-pickling failed (should only work for python>=3.4 )")


        XM.write_density_cube()
        XM.write_mo_cube(5)


            
if __name__ == "__main__": 
    unittest.main()


