#!/usr/bin/python


import os

import unittest
import re
from subprocess import PIPE,Popen,call,STDOUT

import numpy

class Testxmoleculemc(unittest.TestCase):

    xmol=""
    xmol_req=""
    xFSSH=""
    xatom=""

    def setUp(self):
        dir=os.getcwd()
        self.xatom=os.getenv("XATOM_PATH")
        print "Using xatom=",self.xatom  

        self.xmol=dir+"/../src/xmolecule"
        print "Using xmolecule=",self.xmol

        self.xmol_req=dir+"/../src/xmolecule_req"
        print "Using xmolecule_req=",self.xmol_req

        self.xmol_mc=dir+"/../src/xmolecule_mc"
        print "Using xmolecule_mc=",self.xmol_mc

        self.xFSSH=dir+"/../src/xFSSH"
        print "Using xFSSH=",self.xFSSH

    def test_water_xmolecule_mc(self):
# from rate eq pulse end
        ExpectedChargePop_Pulse_End=[2.560E-04, 1.578E-04, 5.054E-03, 3.054E-03, 3.235E-02, 2.359E-02, 1.237E-01, 2.037E-01, 2.895E-01, 2.810E-01, 3.759E-02]
        print 'Testing MC results for H2O (takes really long time ( a day) !)'
        # set up input file
        f = open('input.in', 'w')
        f.write('system=water\n')
        f.write('N_nuc= 3\n')
        f.write('O     0.0000000000        0.0000000000        0.0000000000    2s1p O.nao\n')
        f.write('H     0.0000000000       -1.4309301866        1.1083919440    1s   H.nao\n')
        f.write('H     0.0000000000        1.4309301866        1.1083919440    1s   H.nao\n')

        f.write('SCF_max_iter= 100\n')
        f.write('SCF_energy_conv= 1E-7\n')
        f.write('SCF_density_conv= 1E-7\n')
        f.write('MC_ratesum_cutoff=1e-15\n')
#        f.write('Fluorescence_Energy_Threshold_in_eV=0\n')
        f.write('MOM= yes\n')        
#        f.write('average_occ_MO=3,4,5\n')
        f.write('XATOM_PATH=/work/linhest/XATOM/src/xatom\n')
        f.write('N_photons=1E+15\n')
        f.write('FWHM=10\n')
        f.write('MC_conv=0.005\n')
        f.write('MC_step=50\n')
        f.write('N_MC_max=4000\n')
#        f.write('average_rates=yes\n')
#        f.write('MC_cache=yes\n')
        f.write('MC_cache=no\n')
#        f.write('seed=2059237394, 698163386\n')
#        f.write('pathway=yes\n')
#        f.write('adapt_all_AO=yes\n')                
        f.close()
        

        cmd=self.xatom+' -s O -N 50 -silent -rmax 20 -latter false  -nao -file_nao O.nao '
        os.system(cmd)
        cmd=self.xatom+' -s H -N 50 -silent -rmax 20 -latter false  -nao -file_nao H.nao ' 
        os.system(cmd)

        cmd=self.xmol + ' input.in -SIni > /dev/zero'
        os.system(cmd)

        # run xmol_req
        cmd=self.xmol_mc + ' input.in -RIni'
        proc=Popen(cmd,stdout=PIPE,stderr=STDOUT, shell=True)
        
        maxdevhist=[]
        ntrajhist=[]
        print "#","N_traj","Estimated Deviation","Actual Deviation"
        while True:
            line = proc.stdout.readline()
            if line != '':
                m=re.search('.*CheckConvergence.*',line)
                if(m!=None):
                    print m.group(0)
                    ChargePopTxt=m.group(0).split()[3:]
                    N_traj=m.group(0).split()[1]
                    dev=m.group(0).split()[2]
                    diff=m.group(0).split()[3]
                    ChargePop=[]
                    Deviation=[]
                    Deviation2=[]

                    for p in ChargePopTxt[1:]:
                        ChargePop.append(float(p))

                    for i in range(0,len(ChargePop)):
#                        Deviation.append(abs(ChargePop[i]-ExpectedChargePop_Final[i]))
                        Deviation2.append(abs(ChargePop[i]-ExpectedChargePop_Pulse_End[i]))

                    print N_traj,dev,max(Deviation2)
                    ntrajhist.append(N_traj)
                    maxdevhist.append(max(Deviation2))


            else:
                break
        # the MC procedure stops at estimated convergence 0.005 
        # we expect the true convergence to be at least 0.05
        self.assertTrue( maxdevhist[-1]  < 0.05)


    def test_oxygen_xmolecule_mc(self):
        # with full optimization
        # final charge pop
        ExpectedChargePop=[2.470E-04, 1.752E-04, 6.041E-03, 4.359E-03, 4.677E-02, 3.761E-02, 2.961E-01, 4.848E-01, 1.239E-01]

        # This Test calculates xFSSH MC trajectories for atomic oxygen
        # and compares the final charge state population with
#        ExpectedChargePop=[2.470E-04, 1.752E-04, 6.041E-03, 4.359E-03, 4.677E-02, 3.761E-02, 2.961E-01, 4.848E-01, 1.239E-01]

        # with core occupation only relaxation
        # final charge pop
        ExpectedChargePop_Final=    [ 2.450E-04, 1.724E-04, 5.880E-03, 4.723E-03, 5.050E-02, 4.701E-02, 3.597E-01, 4.637E-01, 6.804E-02]

        # At Pulse end

        ExpectedChargePop_Pulse_End=[ 2.450E-04, 1.737E-04, 5.895E-03, 6.504E-03, 5.647E-02, 1.639E-01, 3.009E-01, 3.978E-01, 6.804E-02]


        print 'Testing MC results for atomic oxygen (takes some time!)'

        
        # set up input file
        f = open('input.in', 'w')
        f.write('system=oxygen\n')
        f.write('N_nuc= 3\n')
        f.write('O     0.0000000000        0.0000000000        0.0000000000    2s1p O.nao\n')

        f.write('SCF_max_iter= 100\n')
        f.write('SCF_energy_conv= 1E-7\n')
        f.write('SCF_density_conv= 1E-7\n')
        f.write('MC_ratesum_cutoff=1e-15\n')
#        f.write('Fluorescence_Energy_Threshold_in_eV=0\n')
        f.write('MOM= yes\n')        
        f.write('average_occ_MO=3,4,5\n')
        f.write('XATOM_PATH=/work/linhest/XATOM/src/xatom\n')
        f.write('N_photons=1E+15\n')
        f.write('FWHM=10\n')
        f.write('MC_conv=0.005\n')
        f.write('MC_step=50\n')
        f.write('N_MC_max=4000\n')
        f.write('average_rates=yes\n')
#        f.write('MC_cache=yes\n')
        f.write('MC_cache=no\n')
#        f.write('seed=2059237394, 698163386\n')
#        f.write('pathway=yes\n')
#        f.write('adapt_all_AO=yes\n')                
        f.close()
        

        cmd=self.xatom+' -s O -N 50 -silent -rmax 20 -latter false  -nao -file_nao O.nao ' 
        os.system(cmd)

        cmd=self.xmol + ' input.in -SIni > /dev/zero'
        os.system(cmd)

        # run xmol_req
        cmd=self.xmol_mc + ' input.in -RIni'
        proc=Popen(cmd,stdout=PIPE,stderr=STDOUT, shell=True)
        
        maxdevhist=[]
        ntrajhist=[]
        print "#","N_traj","Estimated Deviation","Actual Deviation"
        while True:
            line = proc.stdout.readline()
            if line != '':
                m=re.search('.*CheckConvergence.*',line)
                if(m!=None):
#                    print m.group(0)
                    ChargePopTxt=m.group(0).split()[3:]
                    N_traj=m.group(0).split()[1]
                    dev=m.group(0).split()[2]
                    diff=m.group(0).split()[3]
                    ChargePop=[]
                    Deviation=[]
                    Deviation2=[]

                    for p in ChargePopTxt[1:]:
                        ChargePop.append(float(p))

                    for i in range(0,len(ChargePop)):
#                        Deviation.append(abs(ChargePop[i]-ExpectedChargePop_Final[i]))
                        Deviation2.append(abs(ChargePop[i]-ExpectedChargePop_Pulse_End[i]))

                    print N_traj,dev,max(Deviation2)
                    ntrajhist.append(N_traj)
                    maxdevhist.append(max(Deviation2))


            else:
                break
        # the MC procedure stops at estimated convergence 0.005 
        # we expect the true convergence to be at least 0.05

if __name__ == "__main__": 
    unittest.main()


