#!/usr/bin/python


import os
from sys import exit
import unittest
import re
from python_xmolecule_interface import xmol
import random


class TestArgonRates(unittest.TestCase):

    xmol=""
    xatom=""

    def setUp(self):
        dir=os.getcwd()
        self.xatom=os.getenv("XATOM_PATH")
        if not os.path.isfile(self.xatom):
             print('Please specify xatom path!')
             exit(-1)
        print("Using xatom=",self.xatom  )

    def test_ArgonRates(self):
        # This Test calculates all states for the oxygen atom, calculated all transition rates 
        # to compare the final charge state population 
        def atom_conf_to_occ_array(atom_conf):        
            occ_array=[]
            for n in atom_conf:
                for i in range(1,n,2):
                    occ_array.append(2)
                if(n%2==1):
                    occ_array.append(1)
            return(occ_array)

        def occ_string_to_occ_array(occ_string):        
            occ_array=[]
            for s in occ_string:
                occ_array.append(int(s))
            return(occ_array)

        def occ_array_to_atom_array(occ_array):        
            atom_array=[]

            atom_array.append(sum(occ_array[0:1])) #1s
            atom_array.append(sum(occ_array[1:2])) #2s
            atom_array.append(sum(occ_array[2:5])) # 2p
            atom_array.append(sum(occ_array[5:6])) # 3s
            atom_array.append(sum(occ_array[6:9])) # 3p

            return(atom_array)

        print('Testing rates for Argon atom for several holes, this may take some time...')

        ##############################
        # set up input file
        f = open('input.in', 'w')
        f.write('system=Argon\n')
        f.write('N_nuc= 1\n')
        f.write('Ar     0.0000000000        0.0000000000        0.0000000000   \n')
        f.write('average_occ_MO=3,4,5:7,8,9\n')
        f.write('average_rates=yes\n')
        f.write('MOM=yes\n')
        f.write('N=50\n')
        f.write('rmax=50\n')
        f.write('XATOM_PATH=%s\n' % self.xatom)
        f.close()
        
        os.system("rm Ar.nao* Ar.auger.* Ar.tdipole*")

        atomholeconfs=["1s0","1s1","1s1_2p1","1s2","1s1_2s1","1s1_2p1","1s2_2p2","1s2_2p2_3p1","1s2_2p2_3s1_3p1","2p1_3p2","3s1_3p2"]
        occstrings=[[2,2,2,2,2,2,2,2,2],
                    [1,2,2,2,2,2,2,2,2],
                    [1,2,2,2,1,2,2,2,2],
                    [0,2,2,2,2,2,2,2,2],
                    [1,1,2,2,2,2,2,2,2],
                    [1,2,1,2,2,2,2,2,2],
                    [0,2,0,2,2,2,2,2,2],
                    [0,2,1,1,2,2,2,2,1],
                    [0,2,2,2,0,1,2,2,1],
                    [2,2,2,2,1,2,2,2,0],
                    [2,2,2,2,2,1,2,1,1]]

#        atomholeconfs=["1s0"]
#        occstrings=[[2,2,2,2,2,2,2,2,2]]

        XM=xmol()
        XM.init()


        #################################
        # loop over different hole configurations
        for n in range(0,len(atomholeconfs)):
            atomholeconf=atomholeconfs[n]
            occstring=occstrings[n]
            
            
            XM.set_occ(occstring)
            process1=XM.process()

            # decode p[0] into utf8
            process1 = [ (p[0].decode('UTF8'), p[1], p[2], p[3]) for p in process1]
            
            # compare with xatom process data
            os.system('rm xatom_process.dat')
            cmd=self.xatom+' -s Ar -N 50 -silent -rmax 40 -latter false -auger -pcs -fluorescence -process xatom_process.dat -hole ' +atomholeconf
            print('  Running',cmd )
            os.system(cmd)
            # f1 = open('process.dat', 'r')
            f2 = open('xatom_process.dat', 'r')        
            # process1=[]
            process2=[]
            # while True :
            #     line1=f1.readline()
            #     if not line1: break
            #     tag=line1.split()[0]
            #     rate=float(line1.split()[3])
            #     # skip rate smaller than 1e-20, in xatom some transitions are excluded due to symmetry, which in 
            #     if rate<1e-20: continue 
            #     energy=int(line1.split()[4])            
            #     iniocc=occ_string_to_occ_array(line1.split()[6])
            #     finalocc=occ_string_to_occ_array(line1.split()[8])
            #     iniconf=occ_array_to_atom_array(iniocc)        
            #     finalconf=occ_array_to_atom_array(finalocc)        
            #     process1.append((tag,rate,energy,iniconf,finalconf))

            XM.set_geom([random.random(),random.random(),random.random()])

            while True :
                line2=f2.readline()
                if not line2: break
                if len(line2.split()) < 1: break
                tag=line2.split()[0]
                if tag=="V": tag="P"
                rate=float(line2.split()[3])
                energy=int(line2.split()[4])
                iniconf=list(map(int,line2.split()[6:11]))
                finalconf=list(map(int,line2.split()[12:17]))
                process2.append((tag,finalconf,rate,energy))
            f2.close
            # f1.close
                
            Nprocess_xmolecule=0
            lifetime1=0.0
            lifetime2=0.0
            for p in process1:
                if(p[0]=="F" or p[0]=="A"):
                    lifetime1+=p[2]
                # skip rate smaller than 1e-20, in xatom some transitions are excluded due to symmetry, which in 
                if p[0]=="F" and p[2]<1e-20: continue 
                Nprocess_xmolecule=Nprocess_xmolecule+1            

            for p in process2:
                if(p[0]=="F" or p[0]=="A"):
                    lifetime2+=p[2]
                    

            Nprocess_xatom=len(process2)
            iniconf=occ_array_to_atom_array(occstring)
            self.assertEqual(Nprocess_xmolecule,Nprocess_xatom)        
#            self.assertAlmostEqual(lifetime1,lifetime2,places=2,msg="Lifetime for %s %e != %e" % (str(iniconf),lifetime1,lifetime2))
            print('lifetime xmolecule %e lifetime xatom %e'  % (lifetime1,lifetime2))
            for p1 in process1:
                if p1[0]=="F" and p1[2]<1e-20: continue 
                (tag1,finalocc1,rate1,energy1)=p1
                finalconf1=occ_array_to_atom_array(finalocc1)
                found=False
                for p2 in process2:
                    (tag2,finalconf2,rate2,energy2)=p2                    
                    if(tag1==tag2 and finalconf1==finalconf2 ):
                        if tag1=="A" or tag1=="F":
                            print('relative difference in rate: %10.5e absolute: %10.5e trans_energy diff %d'% (abs(2.*(rate1-rate2)/(rate1+rate2)),abs(rate1-rate2) ,abs(energy1-energy2)))
                        if tag1=="P":
                            print('relative difference in cs  : %10.5e absolute: %10.5e trans_energy diff %d'% (abs(2.*(rate1-rate2)/(rate1+rate2)),abs(rate1-rate2) ,abs(energy1-energy2)))
                        self.assertAlmostEqual(rate1,rate2,places=2,msg="Rate for %s %s->%s %e != %e" % (tag1,str(iniconf),str(finalconf2),rate1,rate2))
                        
                        self.assertTrue(abs(energy1-energy2)<=20,msg="Energy for %s %s ->%s %d != %d" % (tag1,str(iniconf),str(finalconf2),energy1,energy2))
                        found=True
                self.assertTrue(found,msg="Matching Process for %s %s->%s not found" % (tag1,str(iniconf),str(finalconf1)) )

    
        #  end of loop
        ####################################






            
if __name__ == "__main__": 
    unittest.main()


