#!/usr/bin/python


import os

import unittest
import re
from subprocess import PIPE,Popen,call



class TestMCSCF(unittest.TestCase):
    xmol=""
    xatom=""

    def setUp(self):
        dir=os.getcwd()
        self.xatom=os.getenv("XATOM_PATH")
        if not os.path.isfile(self.xatom):
            print('Please specify xatom path!')
            exit(-1)
        print("Using xatom=",self.xatom  )

        self.xmol=dir+"/../src/xmolecule"
        print("Using xmolecule=",self.xmol)

    def test_CO(self, mmap=False):
        print('Testing CO 6-31G/CASSCF(6,6) ground state potential energy curve')
        ExpectedEnergies=[]
        distances=[]        
        distances.append(1.00)
        ExpectedEnergies.append(-112.7209072273)# values from GAMESS calculation
        distances.append(1.20)
        ExpectedEnergies.append(-112.7960432805)
        distances.append(1.50)
        ExpectedEnergies.append(-112.6950140992)
        distances.append(2.00)
        ExpectedEnergies.append(-112.5325850118)
        distances.append(3.00)
        ExpectedEnergies.append(-112.4581835320)
        distances.append(3.50)
        ExpectedEnergies.append(-112.4561845129)

        dist=1.0
        f = open('input_HF.in', 'w')
        f.write('C      0.0                 0.0                 0.0\n')
        f.write("O      0.0                 0.0                 %f \n" % dist)
        f.write("unit=angstrom\n")
        f.write("SCF_max_iter= 100\n")
        f.write("SCF_energy_conv= 1E-10\n")
        f.write("SCF_density_conv= 1E-10\n")
        f.write("HF=yes\n")
        f.write("sym=yes\n")
        f.write("irrep_MO=1,1,1,1,3,4,1,3,4,1,3,4,1,1,3,4,1,1\n")
        f.write("SIni=yes\n")
        f.write("gto=6-31G.gbs\n")
        f.close()
        cmd=self.xmol+' input_HF.in '
        p=Popen(cmd, stdout=PIPE,shell=True)
        p.wait()

        for n in range(0,len(distances)):
            dist=distances[n]

            f = open('input.in', 'w')
            f.write('C      0.0                 0.0                 0.0\n')
            f.write("O      0.0                 0.0                 %f \n" % dist)
            f.write("SCF_max_iter= 100\n")
            f.write("unit=angstrom\n")
            f.write("SCF_energy_conv= 1E-10\n")
            f.write("SCF_density_conv= 1E-10\n")
            f.write("MCSCF=yes\n")
            f.write("RIni=yes\n")
            f.write("SIni=yes\n")
            f.write("sym=yes\n")
            if mmap:
                f.write('mmap_teints=yes\n')
            f.write("irrep_MO=1,1,1,1,3,4,1,3,4,1,3,4,1,1,3,4,1,1\n")
            f.write("state_irrep=1\n")
            f.write("active_space_MO=5,6,7,8,9,10\n")
            f.write("gto=6-31G.gbs\n")
            f.close()
            cmd=self.xmol+' input.in '

            p=Popen(cmd,stdout=PIPE, shell=True)
            out=p.communicate()[0].decode('UTF8')
            m=re.search('#CI_energy=.*',out)
            CIEnergytxt=m.group(0).split()[1]
            CIEnergy=float(CIEnergytxt)
            print("\n")
            print("%30s:%f Angstrom" %("Bondlength ", dist))
            print("%30s:%f Hartree" %("CASSCF Energy (XMOLECULE)",CIEnergy))
            print("%30s:%f Hartree" %("CASSCF Energy (GAMESSUS)",ExpectedEnergies[n]))
            self.assertAlmostEqual(CIEnergy, ExpectedEnergies[n],places=6)

    def test_CO_mmap(self):
        self.test_CO(mmap=True)

if __name__ == "__main__": 
    unittest.main()


