#!/usr/bin/python


import os

import unittest
import re
from subprocess import PIPE,Popen,call



class TestOrbitalEnergies(unittest.TestCase):
    xmol=""
    xatom=""

    def setUp(self):
        dir=os.getcwd()
        self.xatom=os.getenv("XATOM_PATH")
        if not os.path.isfile(self.xatom):
            print('Please specify xatom path!')
            exit(-1)
        print("Using xatom=",self.xatom)

        self.xmol=dir+"/../src/xmolecule"
        print("Using xmolecule=",self.xmol)

    def test_OrbitalEnergiesHe(self):
        ExpectedOrbitalEnergies = [-24.87]
        f = open('input.in', 'w')
        f.write('He      0.0                 0.0                 0.0  \n')
        f.write('gto=6-31G\n')
        f.write('HF=yes\n')
        f.close()
        cmd=self.xmol+' input.in'

        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0].decode('UTF8')

        m=re.search('\|SCF\|.*',out)
        OrbitalEnergiestxt=m.group(0).split()[1:]
        OrbitalEnergies=[]
        for e in OrbitalEnergiestxt:
            OrbitalEnergies.append(float(e))

        print("OrbitalEnergies       ",["%3.2f" % e for e in OrbitalEnergies])

        for i in range(0,len(ExpectedOrbitalEnergies)):
            self.assertAlmostEqual(OrbitalEnergies[i], ExpectedOrbitalEnergies[i],places=1)

    def test_OrbitalEnergiesHePcharge(self):
        ExpectedOrbitalEnergies = [-74.79]
        f = open('input.in', 'w')
        f.write('He      0.0                 0.0                 0.0  \n')
        f.write('X      1.0                 0.0                 0.0  1.0 \n')
        f.write('X      0.0                 1.0                 0.0  1.0 \n')
        f.write('gto=6-31G_star_star\n')
        f.write('HF=yes\n')
        f.close()
        cmd=self.xmol+' input.in'

        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0].decode('UTF8')

        m=re.search('\|SCF\|.*',out)
        OrbitalEnergiestxt=m.group(0).split()[1:]
        OrbitalEnergies=[]
        for e in OrbitalEnergiestxt:
            OrbitalEnergies.append(float(e))

        print("OrbitalEnergies       ",["%3.2f" % e for e in OrbitalEnergies])

        for i in range(0,len(ExpectedOrbitalEnergies)):
            self.assertAlmostEqual(OrbitalEnergies[i], ExpectedOrbitalEnergies[i],places=1)

    def test_OrbitalEnergiesIonizedCO(self):
        print('Testing CO double core ionized')
        ExpectedOrbitalEnergies=[]
        distances=[]
        
        distances.append(2.00)
        ExpectedOrbitalEnergies.append( [-625.54999999999995, -371.63, -73.909999999999997, -50.310000000000002, -50.310000000000002, -49.950000000000003, -43.020000000000003])

        distances.append(2.50)
        ExpectedOrbitalEnergies.append([-619.24000000000001, -368.56999999999999, -62.969999999999999, -47.700000000000003, -43.869999999999997, -43.869999999999997, -42.090000000000003])

        distances.append(3.00)
        ExpectedOrbitalEnergies.append( [-615.37, -366.41000000000003, -56.380000000000003, -46.369999999999997, -40.719999999999999, -40.049999999999997, -40.049999999999997])

        print("Creating atomic orbital basis functions")
        cmd=self.xatom+" -s C -conf 1s1_2s2_2p2 -N 50 -lmax 3 -rmax 20 -latter false -silent -nao -file_nao C_1s.nao"
        p1=Popen(cmd,shell=True)
        cmd=self.xatom+" -s O -conf 1s1_2s2_2p4 -N 50 -lmax 3 -rmax 20 -latter false -silent -nao -file_nao O_1s.nao"
        p2=Popen(cmd,shell=True)        
        p1.wait()
        p2.wait()


        for n in range(0,len(distances)):
            dist=distances[n]
            f = open('input.in', 'w')
            f.write('system=CO\n')
            f.write('N_nuc= 2\n')
            f.write('C      0.0                 0.0                 0.0    2s1p C_1s.nao\n')
            f.write("O      0.0                 0.0                 %f   2s1p O_1s.nao\n" % dist)
            f.write("lmax_ang=8\n")
            f.write("lmax_MP=8\n")
            f.write("SCF_max_iter= 100\n")
            f.write("SCF_energy_conv= 1E-5\n")
            f.write("SCF_density_conv= 1E-4\n")
            f.close()

            cmd=self.xmol+' input.in -occ 1122222'

            p=Popen(cmd,stdout=PIPE, shell=True)
            out=p.communicate()[0].decode('UTF8')
            m=re.search('\|SCF\|.*',out)
            OrbitalEnergiestxt=m.group(0).split()[1:]
            OrbitalEnergies=[]
            for e in OrbitalEnergiestxt:
                OrbitalEnergies.append(float(e))

            print("Bondlength ", dist)

            print("OrbitalEnergies       ",["%3.2f" % e for e in OrbitalEnergies])
            print("ExpectedOrbitalEnergies",["%3.2f" % e for e in ExpectedOrbitalEnergies[n]])
            for i in range(0,len(ExpectedOrbitalEnergies[n])):
                self.assertAlmostEqual(OrbitalEnergies[i], ExpectedOrbitalEnergies[n][i],places=1)

    
    def test_OrbitalEnergiesCO(self):
        print('Testing CO')
        ExpectedOrbitalEnergies=[]
        distances=[]
        distances.append(2.00)
        ExpectedOrbitalEnergies.append([-538.33, -296.28, -39.16, -19.24, -18.71, -18.71, -14.01])

        distances.append(2.50)
        ExpectedOrbitalEnergies.append([-535.37, -294.99000000000001, -32.829999999999998, -18.82, -15.09, -15.09, -13.720000000000001])

        distances.append(3.00)
        ExpectedOrbitalEnergies.append( [-533.47000000000003, -294.47000000000003, -28.84, -18.780000000000001, -13.25, -12.84, -12.84])

        print("Creating atomic orbital basis functions")
        cmd=self.xatom+" -s C -N 50 -lmax 3 -rmax 20 -latter false -silent -nao -file_nao C.nao"
        p1=Popen(cmd,shell=True)
        cmd=self.xatom+" -s O -N 50 -lmax 3 -rmax 20 -latter false -silent -nao -file_nao O.nao"
        p2=Popen(cmd,shell=True)    
        p1.wait()
        p2.wait()


        for n in range(0,len(distances)):
            dist=distances[n]
            f = open('input.in', 'w')
            f.write('system=CO\n')
            f.write('N_nuc= 2\n')
            f.write('C      0.0                 0.0                 0.0    2s1p C.nao\n')
            f.write("O      0.0                 0.0                 %f   2s1p O.nao\n" % dist)
            f.write("lmax_ang=8\n")
            f.write("lmax_MP=8\n")
            f.write("SCF_max_iter= 100\n")
            f.write("SCF_energy_conv= 1E-5\n")
            f.write("SCF_density_conv= 1E-4\n")
            f.close()

            cmd=self.xmol+' input.in '

            p=Popen(cmd,stdout=PIPE, shell=True)
            out=p.communicate()[0].decode('UTF8')


            m=re.search('\|SCF\|.*',out)
            OrbitalEnergiestxt=m.group(0).split()[1:]
            OrbitalEnergies=[]
            for e in OrbitalEnergiestxt:
                OrbitalEnergies.append(float(e))

            print("Bondlength ", dist)
            print("OrbitalEnergies       ",["%3.2f" % e for e in OrbitalEnergies])
            print("ExpectedOrbitalEnergies",["%3.2f" % e for e in ExpectedOrbitalEnergies[n]])
            for i in range(0,len(ExpectedOrbitalEnergies)):
                self.assertAlmostEqual(OrbitalEnergies[i], ExpectedOrbitalEnergies[n][i],places=1)


    def test_OrbitalEnergiesCO_RHF(self):
        print('Testing CO RHF')
        ExpectedOrbitalEnergies=[]
        ExpectedDipole = []
        distances=[]        
        distances.append(2.00)
        ExpectedOrbitalEnergies.append([-562.86   ,-309.44 ,   -43.92 ,   -21.80  ,  -18.36  ,  -18.36 ,   -14.76])
        ExpectedDipole.append([0,0,-0.22638])
        distances.append(2.50)
        ExpectedOrbitalEnergies.append([-562.78, -311.85, -38.82, -21.50, -15.46, -15.46, -15.14])
        ExpectedDipole.append([0,0,-1.51633])
        distances.append(3.00)
        ExpectedOrbitalEnergies.append( [-562.10, -313.50, -35.10, -21.80, -14.05, -13.53, -13.53])
        ExpectedDipole.append([0,0,-2.68010])


        for n in range(0,len(distances)):
            for direct in [False,True]:
                dist=distances[n]
                f = open('input.in', 'w')
                f.write('C      0.0                 0.0                 0.0\n')
                f.write("O      0.0                 0.0                 %f \n" % dist)
                f.write("lmax_ang=8\n")
                f.write("lmax_MP=8\n")
                f.write("SCF_max_iter= 100\n")
                f.write("SCF_energy_conv= 1E-10\n")
                f.write("SCF_density_conv= 1E-10\n")
                f.write("HF=yes\n")
                f.write("gto=6-31G\n")
                if direct:
                    f.write("direct=yes")
                f.close()

                cmd=self.xmol+' input.in '

                p=Popen(cmd,stdout=PIPE, shell=True)
                out=p.communicate()[0].decode('UTF8')
                m=re.search('\|SCF\|.*',out)
                OrbitalEnergiestxt=m.group(0).split()[1:]
                OrbitalEnergies=[]
                for e in OrbitalEnergiestxt:
                    OrbitalEnergies.append(float(e)) 
                m=re.search('\|DIPOLE_DEBEYE\|.*',out)
                dipole = [float(d) for d in m.group(0).split()[1:]]
                
                print("Bondlength ", dist)
                print("OrbitalEnergies       ",["%3.2f" % e for e in OrbitalEnergies])
                print("ExpectedOrbitalEnergies",["%3.2f" % e for e in ExpectedOrbitalEnergies[n]])
                print("dipole       ",["%3.5f" % d for d in dipole])
                print("ExpectedDipole",["%3.5f" % e for e in ExpectedDipole[n]])
                
                for i in range(0,len(ExpectedOrbitalEnergies)):
                    self.assertAlmostEqual(OrbitalEnergies[i], ExpectedOrbitalEnergies[n][i],places=2)
                for i in range(0,3):
                    self.assertAlmostEqual(dipole[i], ExpectedDipole[n][i],places=2)


    def test_OrbitalEnergiesCO_ROHF(self):
        print('Testing CO RHF')
        ExpectedOrbitalEnergies=[]
        ExpectedDipole = []
        distances=[]        
        distances.append(2.00)
        ExpectedOrbitalEnergies.append([-574.38, -322.23, -54.21, -30.99, -28.11, -28.11, -20.61])
        ExpectedDipole.append([0,0,-0.40806])
        distances.append(2.50)
        ExpectedOrbitalEnergies.append([-574.42, -324.20, -48.44, -28.57, -25.24, -25.24, -20.73])
        ExpectedDipole.append([0,0,-0.51830])
        distances.append(3.00)
        ExpectedOrbitalEnergies.append([-572.84, -325.74, -43.32, -28.39, -23.56, -23.56, -18.89])
        ExpectedDipole.append([0,0,0.07601])
        for n in range(0,len(distances)):
            for direct in [False,True]:
                dist=distances[n]
                f = open('input.in', 'w')
                f.write('C      0.0                 0.0                 0.0\n')
                f.write("O      0.0                 0.0                 %f \n" % dist)
                f.write("lmax_ang=8\n")
                f.write("lmax_MP=8\n")
                f.write("SCF_max_iter= 100\n")
                f.write("SCF_energy_conv= 1E-10\n")
                f.write("SCF_density_conv= 1E-10\n")
                f.write("HF=yes\n")
                f.write("occ=2222221\n")
                f.write("gto=6-31G\n")
                f.write("coreini=yes\n")
                if direct:
                    f.write("direct=yes")

                f.close()
                cmd=self.xmol+' input.in '
                p=Popen(cmd,stdout=PIPE, shell=True)
                out=p.communicate()[0].decode('UTF8')
                m=re.search('\|SCF\|.*',out)
                OrbitalEnergiestxt=m.group(0).split()[1:]
                OrbitalEnergies=[]
                for e in OrbitalEnergiestxt:
                    OrbitalEnergies.append(float(e))
                m=re.search('\|DIPOLE_DEBEYE\|.*',out)
                dipole = [float(d) for d in m.group(0).split()[1:]]

                print("Bondlength ", dist)
                print("OrbitalEnergies       ",["%3.2f" % e for e in OrbitalEnergies])
                print("ExpectedOrbitalEnergies",["%3.2f" % e for e in ExpectedOrbitalEnergies[n]])
                for i in range(0,len(ExpectedOrbitalEnergies[n])):
                    self.assertAlmostEqual(OrbitalEnergies[i], ExpectedOrbitalEnergies[n][i],places=1)
                print("dipole       ",["%3.5f" % d for d in dipole])
                print("ExpectedDipole",["%3.5f" % e for e in ExpectedDipole[n]])
                for i in range(0,3):
                    self.assertAlmostEqual(dipole[i], ExpectedDipole[n][i],places=2)


    def test_OrbitalEnergiesBenzene(self):
        ExpectedOrbitalEnergies=[-266.03, -266.03, -266.01, -266.01, -266.01, -266.01, -20.98, -18.18, -18.17, -14.64, -14.64, -12.85, -10.97, -10.82, -10.14, -10.13, -8.91, -8.07, -8.07, -6.13, -6.13]

        print('Testing Benzene')

        f = open('input.in', 'w')
        f.write('C  -2.2862613440  1.3199736024  0.0000000000  3s2p  C_6-31G.nao\n')
        f.write('C  -2.2862613440 -1.3199736024  0.0000000000  3s2p  C_6-31G.nao\n')
        f.write('C   0.0000000000 -2.6399472048  0.0000000000  3s2p  C_6-31G.nao\n')
        f.write('C   2.2862613440 -1.3199736024  0.0000000000  3s2p  C_6-31G.nao\n')
        f.write('C   2.2862613440  1.3199736024  0.0000000000  3s2p  C_6-31G.nao\n')
        f.write('C   0.0000000000  2.6399472048  0.0000000000  3s2p  C_6-31G.nao\n')
        f.write('H  -4.0602823154  2.3442050878  0.0000000000  2s    H_6-31G.nao\n')
        f.write('H  -4.0602823154 -2.3442050878  0.0000000000  2s    H_6-31G.nao\n')
        f.write('H   0.0000000000 -4.6884101755  0.0000000000  2s    H_6-31G.nao\n')
        f.write('H   4.0602823154 -2.3442050878  0.0000000000  2s    H_6-31G.nao\n')
        f.write('H   4.0602823154  2.3442050878  0.0000000000  2s    H_6-31G.nao\n')
        f.write('H   0.0000000000  4.6884101755  0.0000000000  2s    H_6-31G.nao\n')
        f.write('exchange= LDA\n')
        f.write('correlation= VWN\n')
        f.write('lmax_ang= 8\n')
        f.write('lmax_MP= 8\n')
        f.write('SCF_mixing= 0.4\n')
        f.write('SCF_energy_conv= 1.0e-5\n')
        f.write('SCF_density_conv= 1.0e-4\n')
        f.write('XATOM_PATH=%s\n' % self.xatom)
        f.close()

        cmd=self.xmol+' input.in '

        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0].decode('UTF8')
        p.wait()

        m=re.search('\|SCF\|.*',out)
        OrbitalEnergiestxt=m.group(0).split()[1:]
        OrbitalEnergies=[]
        for e in OrbitalEnergiestxt:
            OrbitalEnergies.append(float(e))

            
        print("OrbitalEnergies       ",["%3.2f" % e for e in OrbitalEnergies])
        print("ExpectedOrbitalEnergies",["%3.2f" % e for e in ExpectedOrbitalEnergies])

        for i in range(0,len(ExpectedOrbitalEnergies)):
            self.assertAlmostEqual(OrbitalEnergies[i], ExpectedOrbitalEnergies[i],places=1)



    def test_OrbitalEnergiesBenzene_HF(self, mmap=False):
        ExpectedOrbitalEnergies=[-305.84, -305.82, -305.82, -305.79, -305.79, -305.77, -31.30, -27.57, -27.57, -22.33, -22.33, -19.27, -17.36, -16.75, -15.92, -15.92, -13.55, -13.25, -13.25, -9.05, -9.05]
        ExpectedDipole = [0,0,0]
        print('Testing Benzene HF')

        f = open('input.in', 'w')
        f.write('C  -2.2862613440  1.3199736024  0.0000000000  \n')
        f.write('C  -2.2862613440 -1.3199736024  0.0000000000  \n')
        f.write('C   0.0000000000 -2.6399472048  0.0000000000  \n')
        f.write('C   2.2862613440 -1.3199736024  0.0000000000  \n')
        f.write('C   2.2862613440  1.3199736024  0.0000000000  \n')
        f.write('C   0.0000000000  2.6399472048  0.0000000000  \n')
        f.write('H  -4.0602823154  2.3442050878  0.0000000000  \n')
        f.write('H  -4.0602823154 -2.3442050878  0.0000000000  \n')
        f.write('H   0.0000000000 -4.6884101755  0.0000000000  \n')
        f.write('H   4.0602823154 -2.3442050878  0.0000000000  \n')
        f.write('H   4.0602823154  2.3442050878  0.0000000000  \n')
        f.write('H   0.0000000000  4.6884101755  0.0000000000  \n')
        f.write('exchange= LDA\n')
        f.write('correlation= VWN\n')
        if mmap:
            f.write('mmap_teints=yes\n')
        f.write('lmax_ang= 8\n')
        f.write('lmax_MP= 8\n')
        f.write('SCF_mixing= 0.4\n')
        f.write('SCF_energy_conv= 1.0e-5\n')
        f.write('SCF_density_conv= 1.0e-4\n')
        f.write("HF=yes\n")
        f.write("gto=6-31G.gbs\n")

        f.close()

        cmd=self.xmol+' input.in '

        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0].decode('UTF8')
        p.wait()

        m=re.search('\|SCF\|.*',out)
        OrbitalEnergiestxt=m.group(0).split()[1:]
        OrbitalEnergies=[]
        for e in OrbitalEnergiestxt:
            OrbitalEnergies.append(float(e))
        m=re.search('\|DIPOLE_DEBEYE\|.*',out)
        dipole = [float(d) for d in m.group(0).split()[1:]]

            
        print("OrbitalEnergies       ",["%3.2f" % e for e in OrbitalEnergies])
        print("ExpectedOrbitalEnergies",["%3.2f" % e for e in ExpectedOrbitalEnergies])
        print("dipole       ",["%3.5f" % d for d in dipole])
        print("ExpectedDipole",["%3.5f" % e for e in ExpectedDipole])
        for i in range(0,3):
            self.assertAlmostEqual(dipole[i], ExpectedDipole[i],places=2)

        for i in range(0,len(ExpectedOrbitalEnergies)):
            self.assertAlmostEqual(OrbitalEnergies[i], ExpectedOrbitalEnergies[i],places=1)

    def test_OrbitalEnergiesBenzene_HF_mmap(self):
        self.test_OrbitalEnergiesBenzene_HF(mmap=True)

if __name__ == "__main__": 
    unittest.main()


