#!/usr/bin/env python
import os
from sys import exit,stdout
import unittest
import re

import random
import numpy as np
from math import sqrt,acos
import pickle
import copy
from python_xmolecule_interface import xmol

class TestUpdateNrbasis(unittest.TestCase):

    xmol=""
    xatom=""

    def setUp(self):
        dir=os.getcwd()
        self.xatom=os.getenv("XATOM_PATH")
        if not os.path.isfile(self.xatom):
             print('Please specify xatom path!')
             exit(-1)
        print("Using xatom=",self.xatom)  


    def test_water(self, mmap=False):
        print('Checking water without sym')
        # set up input file
        f = open('input.in', 'w')
        f.write('MOM=yes\n')
        f.write('SCF_energy_conv=1e-8\n')
        f.write('SCF_density_conv=1e-8\n')
        f.write('gto=6-31++G_star_star\n')
        if mmap:
            f.write('mmap_teints=yes\n')
        f.write('basissetoverlaplim=4.0e-2\n')
        f.write('sym=no\n')
        f.write('verbose=no\n')
        f.write('O    0.0000000000     0.0000000000        0.0000000000         \n')
        f.write('H    0.0000000000    -1.4309301866        1.1083919440        \n')
        f.write('H    0.0000000000     1.4309301866        1.1083919440        \n')
        f.close()

        
        XM=xmol()
        XM.init()
        E0=XM.energy
        g=XM.get_geom()
        XM.set_geom(g)
        gOld = g[:]
        g = [x*1.1 for x in g]
        XM.set_geom(g)        
        g = [x*0.8 for x in g]
        XM.set_geom(g)        
        XM.set_geom(gOld)

        E1=XM.energy
        self.assertAlmostEqual(E0,E1, places=8)
        stdout.flush()

    def test_water_mmap(self):
        self.test_water(mmap=True)

    def test_waterSym(self, mmap=False):
        print('Checking water with symmetry  ')
        # set up input file
        f = open('input.in', 'w')
        f.write('MOM=yes\n')
        f.write('SCF_energy_conv=1e-8\n')
        f.write('SCF_density_conv=1e-8\n')
        f.write('gto=6-31++G_star_star\n')
        f.write('basissetoverlaplim=4.0e-2\n')
        f.write('sym=yes\n')
        f.write('verbose=no\n')
        if mmap:
            f.write('mmap_teints=yes\n')
        f.write('O    0.0000000000     0.0000000000        0.0000000000         \n')
        f.write('H    0.0000000000    -1.4309301866        1.1083919440        \n')
        f.write('H    0.0000000000     1.4309301866        1.1083919440        \n')
        f.close()

        
        XM=xmol()
        XM.init()
        E0=XM.energy
        g=XM.get_geom()
        XM.set_geom(g)
        gOld = g[:]
        g = [x*1.1 for x in g]
        XM.set_geom(g)        
        g = [x*0.8 for x in g]
        XM.set_geom(g)        
        XM.set_geom(gOld)

        E1=XM.energy
        self.assertAlmostEqual(E0,E1, places=8)
        stdout.flush()
        
    def test_waterSym_mmap(self):
        self.test_waterSym(mmap=True)
            
if __name__ == "__main__": 
    unittest.main()


