#!/usr/bin/python


import os
from sys import exit, stdout
import unittest
import re
from python_xmolecule_interface import xmol
import random
import numpy as np
from math import sqrt, acos


class TestInterface(unittest.TestCase):

    xmol = ""
    xatom = ""

    def setUp(self):
        dir = os.getcwd()
        self.xatom = os.getenv("XATOM_PATH")
        if not os.path.isfile(self.xatom):
            print('Please specify xatom path!')
            exit(-1)
        print("Using xatom=" + self.xatom)

    def test_He(self):
        print('Checking He')
        # set up input file
        f = open('test.in', 'w')
        f.write('HF=yes\n')
        f.write('gto=6-31G\n')
        f.write('He       0.0000000000        0.0000000000  0.0000000000         \n')
        f.write('X        100.0               0             0       1.0 \n')
        f.write('X        100.0               0             1.      1.0  \n')
        f.close()
        XM = xmol(input='test.in')
        XM.init()
        E1 = XM.energy
        Eref = -1.85516043
        self.assertAlmostEqual(E1, Eref)
        g = XM.gradient()
        Gref= np.array([0.,0.,0., 0,0,1, 0,0,-1])
        np.testing.assert_almost_equal(g,Gref)

        XM.set_geom(np.array([0,0,0,1,0,0,1,0,1.]))
        g = XM.gradient()
        Gref= np.array([ 0.6777716,  0.       ,  0.0850087,
                         -0.5927629,  0.       ,   1.     ,
                         -0.0850087,  0.       , -1.0850087])
        np.testing.assert_almost_equal(g, Gref, decimal=4)

    def test_water(self):
        print('Checking various interface interaction  ')
        # set up input file
        f = open('test.in', 'w')
        f.write('HF=yes\n')
        f.write('SCF_energy_conv=1e-8\n')
        f.write('SCF_density_conv=1e-8\n')
        f.write('gto=6-31G.gbs\n')
        f.write('occ=22222\n')
        f.write('mmap_teints=yes\n')
        f.write('O       0.0000000000        0.0000000000  0.0000000000         \n')
        f.write('H      -1.4309301866        1.1083919440  0.0000000000        \n')
        f.write('H       1.4309301866        1.1083919440  0.0000000000        \n')
        f.close()
        XM = xmol(input='test.in')
        XM.init()

        g = XM.get_geom()
        E1 = XM.energy
        Eref = -75.98397312209923
        self.assertAlmostEqual(E1, Eref)
        
        aCs,tdip = XM.os()
        aCsRef = 137.035999084 / (2.*np.pi**2) * np.array([[0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 1.53103023e-03, 3.33947323e-03, 5.47660975e-04,
                            1.71300705e-02, 1.01159253e-02, 7.95983954e-03, 1.90713427e-02,
                            5.87691005e-04],
                           [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 1.62983575e-03, 2.48185686e-03, 4.76400016e-04,
                            2.56807481e-02, 1.18819144e-02, 1.06624089e-02, 2.47278404e-02,
                            2.60624766e-03],
                           [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 9.07280552e-02, 4.88658584e-02, 9.59922595e-02,
                            0.00000000e+00, 5.06340654e-02, 4.34510059e-02, 8.36051566e-04,
                            3.34093182e-03],
                           [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 3.99846206e-02, 4.48661452e-02, 7.86032155e-02,
                            2.82579840e-05, 9.06741805e-04, 1.11609785e-02, 3.29413299e-03,
                            1.14145256e-02],
                           [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 4.72091035e-03, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 3.02324198e-02, 1.26064203e-03, 0.00000000e+00,
                            1.09284413e-01],
                           [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00],
                           [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00],
                           [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00],
                           [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00],
                           [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00],
                           [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00],
                           [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00],
                           [0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 0.00000000e+00,
                            0.00000000e+00]])
        np.testing.assert_almost_equal(aCs, aCsRef)
            
        tdipRef = np.array([[[-3.80389550e-18, -2.21372869e-01, -4.90818768e-18],
                          [ 1.41005947e-16,  1.13467324e-02, -1.78670174e-16],
                          [-4.80618197e-02, -1.64791487e-16,  5.05992573e-16],
                          [-8.50776936e-16, -5.10913014e-02,  2.27245659e-17],
                          [ 2.67165267e-16,  9.85093503e-17,  5.81414720e-02],
                          [-1.34823374e-15,  1.95937597e-02,  1.23785388e-16],
                          [ 2.88710365e-02, -1.47726137e-15,  2.12223515e-17],
                          [ 1.14850739e-02, -2.32767922e-16,  3.21729972e-16],
                          [ 4.77751624e-16,  2.37020470e-16, -6.40741782e-02],
                          [-1.60562959e-15, -4.92133250e-02,  8.35735670e-16],
                          [-9.82305308e-16,  4.36258860e-02,  1.20695123e-15],
                          [ 6.72754711e-02,  7.14258709e-17,  4.93719337e-16],
                          [ 2.45317346e-16, -1.17253840e-02, -1.26844869e-16]],
                         
                         [[ 1.41091271e-16,  1.13467324e-02, -1.78670174e-16],
                          [-1.66534778e-15,  1.38430532e-01,  1.31316156e-16],
                          [-8.19453410e-01,  5.17564075e-15,  5.31562178e-15],
                          [-5.51882372e-15, -6.75114965e-01,  7.63985816e-16],
                          [ 6.00122778e-15,  7.26258957e-16,  6.43402269e-01],
                          [-7.19847225e-16, -7.37611577e-02,  3.31417856e-17],
                          [-8.83412107e-02,  2.59527718e-15,  4.51418687e-16],
                          [-3.20597708e-02,  4.61345195e-17, -2.06564809e-15],
                          [-1.52986885e-15, -1.01343990e-15,  2.30323583e-01],
                          [ 4.48483924e-15,  1.55976200e-01, -2.66037323e-15],
                          [ 2.36171956e-15, -1.46924346e-01, -4.39845358e-15],
                          [-2.16950971e-01,  7.97409903e-16, -1.55991358e-15],
                          [-3.49682552e-15,  6.66771655e-02, -3.02422024e-16]],
                         
                         [[-4.80618197e-02, -1.63589682e-16,  5.05992573e-16],
                          [-8.19453410e-01,  5.17676038e-15,  5.31562178e-15],
                          [-2.92323179e-15,  1.88651804e-01, -3.69750815e-29],
                          [ 1.65758471e-01,  4.26999050e-15,  1.51954986e-15],
                          [-2.00402270e-16, -4.37036884e-15, -2.22254593e-15],
                          [ 7.19136457e-01,  1.02848788e-16, -1.37972917e-15],
                          [-1.74729246e-14,  5.02025011e-01,  2.03985664e-29],
                          [-1.67524944e-14,  5.31837147e-01, -1.24753657e-28],
                          [-1.08536593e-14,  3.06341115e-15, -3.18141098e-16],
                          [-3.72830894e-01, -1.24674579e-14, -2.45414779e-15],
                          [-3.42778146e-01, -5.01962528e-15, -5.53908912e-16],
                          [-8.92971305e-15, -4.56472178e-02, -9.60358301e-29],
                          [ 8.50293049e-02,  6.01827225e-15,  4.00813822e-15]],

                         [[-8.51039191e-16, -5.10913014e-02,  2.27245659e-17],
                          [-5.51103589e-15, -6.75114965e-01,  7.63985816e-16],
                          [ 1.65758471e-01,  4.26104763e-15,  1.51954986e-15],
                          [ 2.09400840e-15, -4.01600169e-01,  4.19057461e-16],
                          [-2.34367325e-15,  1.63959124e-16,  1.81883151e-01],
                          [ 1.27876191e-14,  5.21931526e-01, -2.29513015e-16],
                          [ 5.21087419e-01, -3.10000157e-15,  1.14729733e-16],
                          [ 5.02964213e-01,  2.47795005e-15, -2.49301840e-16],
                          [ 2.49729888e-15, -4.73154450e-15,  9.23532384e-03],
                          [-1.22941051e-14, -5.19785848e-02, -4.78138172e-16],
                          [-7.53510393e-15, -1.80875159e-01, -2.50443784e-16],
                          [-9.40283036e-02, -9.91348029e-16, -2.32102818e-17],
                          [ 5.39401394e-15, -1.62280242e-01,  6.08458515e-16]],

                         [[ 2.67165267e-16,  9.85093503e-17,  5.81414720e-02],
                          [ 6.00122778e-15,  7.26258957e-16,  6.43402269e-01],
                          [-2.00402270e-16, -4.37036884e-15, -2.20986583e-15],
                          [-2.34367325e-15,  1.63959124e-16,  1.81883151e-01],
                          [ 5.70788426e-30, -2.21678389e-01, -3.80101080e-16],
                          [-7.25403705e-15, -6.93761794e-16, -1.86724589e-01],
                          [-7.23323812e-16, -5.05194912e-15,  2.88527624e-15],
                          [-4.74811240e-16, -3.49589375e-15, -1.61165792e-14],
                          [ 8.22692302e-29,  8.15206124e-17, -3.45381746e-15],
                          [ 3.25557137e-15,  2.91011152e-17, -3.05357696e-01],
                          [ 2.46138811e-15,  1.60852718e-16, -6.18286501e-02],
                          [ 1.34804806e-16,  3.70001801e-16, -1.22793736e-14],
                          [-1.58158188e-15,  1.55253029e-16,  5.08853112e-01]],

                         [[-1.34904544e-15,  1.95937597e-02,  1.23785388e-16],
                          [-7.10782227e-16, -7.37611577e-02,  3.31417856e-17],
                          [ 7.19136457e-01,  7.13987480e-17, -1.37972917e-15],
                          [ 1.27578926e-14,  5.21931526e-01, -2.29513015e-16],
                          [-7.25403705e-15, -6.93761794e-16, -1.86724589e-01],
                          [ 5.91692189e-14,  1.17263936e+00, -3.46890128e-16],
                          [ 1.88637632e+00,  4.32226544e-16, -2.55362886e-16],
                          [-7.04500543e-01,  4.85192697e-15,  3.82992795e-15],
                          [-6.24666863e-15,  1.30181500e-14, -6.06237568e-01],
                          [ 2.41814993e-14,  1.11559061e-01,  6.69303840e-15],
                          [ 1.81105439e-14,  6.67335922e-01,  1.15419589e-14],
                          [-1.19722852e-01,  2.53601479e-17,  4.48562091e-15],
                          [-4.35611228e-15,  3.84664597e-01,  1.35015536e-15]],

                         [[ 2.88710365e-02, -1.47044560e-15,  2.12223515e-17],
                          [-8.83412107e-02,  2.62438736e-15,  4.51418687e-16],
                          [-1.73772466e-14,  5.02025011e-01,  2.02566065e-29],
                          [ 5.21087419e-01, -3.14260231e-15,  1.14729733e-16],
                          [-7.23323812e-16, -5.05194912e-15,  2.92677987e-15],
                          [ 1.88637632e+00,  6.37771206e-16, -2.55362886e-16],
                          [-5.27098029e-14,  1.14255679e+00,  7.91067234e-30],
                          [ 1.21627267e-14, -2.52112494e-01, -7.30431161e-29],
                          [ 5.94658359e-15, -4.75038807e-15,  9.24645388e-15],
                          [ 3.40373315e-01,  9.69761840e-15, -2.66626321e-16],
                          [ 1.36786711e-01,  1.55676417e-15, -1.04887291e-17],
                          [ 3.52682124e-15, -4.06766556e-01, -7.96293925e-29],
                          [ 7.29297422e-01, -1.52505240e-14,  5.06506077e-16]],

                         [[ 1.14850739e-02, -2.33718806e-16,  3.21729972e-16],
                          [-3.20597708e-02,  3.89156375e-17, -2.06564809e-15],
                          [-1.66864482e-14,  5.31837147e-01, -1.24642138e-28],
                          [ 5.02964213e-01,  2.46368312e-15, -2.49301840e-16],
                          [-4.74811240e-16, -3.49589375e-15, -1.61156831e-14],
                          [-7.04500543e-01,  4.88926912e-15,  3.82992795e-15],
                          [ 1.23032010e-14, -2.52112494e-01, -7.37744845e-29],
                          [-6.53060745e-14,  2.07225756e-01,  2.72139455e-28],
                          [-1.37678701e-14,  6.75533787e-15, -1.98760482e-14],
                          [-1.19747142e+00,  5.83401740e-15,  2.34116138e-15],
                          [-7.54593735e-02,  7.96124009e-15, -6.55930375e-16],
                          [-3.55612367e-14,  6.35556151e-01,  2.52476512e-28],
                          [ 1.15541706e-01,  4.22077854e-15, -5.51704695e-15]],

                         [[ 4.77751624e-16,  2.37020470e-16, -6.40741782e-02],
                          [-1.52986885e-15, -1.01343990e-15,  2.30323583e-01],
                          [-1.08536593e-14,  3.06341115e-15, -2.97449011e-16],
                          [ 2.49729888e-15, -4.73154450e-15,  9.23532384e-03],
                          [ 8.22692302e-29,  9.57871924e-17, -3.45381746e-15],
                          [-6.24666863e-15,  1.30181500e-14, -6.06237568e-01],
                          [ 5.94658359e-15, -4.75038807e-15,  9.31934266e-15],
                          [-1.37678701e-14,  6.75533787e-15, -1.98217041e-14],
                          [-4.00365844e-28, -2.21678389e-01, -1.66908879e-17],
                          [-1.06918779e-14,  2.16888228e-14, -3.36528106e-01],
                          [-4.55026316e-15,  1.64329668e-15,  1.20532941e-01],
                          [-1.71926933e-14,  5.28192099e-15, -1.57752840e-14],
                          [-3.96827400e-15, -6.48201176e-15,  8.30572211e-01]],

                         [[-1.60596017e-15, -4.92133250e-02,  8.35735670e-16],
                          [ 4.54263345e-15,  1.55976200e-01, -2.66037323e-15],
                          [-3.72830894e-01, -1.24404971e-14, -2.45414779e-15],
                          [-1.22805578e-14, -5.19785848e-02, -4.78138172e-16],
                          [ 3.25557137e-15,  2.91011152e-17, -3.05357696e-01],
                          [ 2.40590150e-14,  1.11559061e-01,  6.69303840e-15],
                          [ 3.40373315e-01,  9.58967595e-15, -2.66626321e-16],
                          [-1.19747142e+00,  5.93158962e-15,  2.34116138e-15],
                          [-1.06918779e-14,  2.16888228e-14, -3.36528106e-01],
                          [ 6.94942277e-14,  5.79726848e-01,  7.56865119e-15],
                          [ 3.55015506e-14,  6.98730466e-01,  5.05404483e-15],
                          [-4.24765667e-01,  4.45212129e-15,  2.43917428e-15],
                          [-1.48046383e-14,  2.03985876e-01, -8.72182271e-15]],

                         [[-9.81775812e-16,  4.36258860e-02,  1.20695123e-15],
                          [ 2.36737078e-15, -1.46924346e-01, -4.39845358e-15],
                          [-3.42778146e-01, -5.03046494e-15, -5.53908912e-16],
                          [-7.55372990e-15, -1.80875159e-01, -2.50443784e-16],
                          [ 2.46138811e-15,  1.60852718e-16, -6.18286501e-02],
                          [ 1.80098309e-14,  6.67335922e-01,  1.15419589e-14],
                          [ 1.36786711e-01,  1.61004423e-15, -1.04887291e-17],
                          [-7.54593735e-02,  7.93809823e-15, -6.55930375e-16],
                          [-4.55026316e-15,  1.64329668e-15,  1.20532941e-01],
                          [ 3.55412804e-14,  6.98730466e-01,  5.05404483e-15],
                          [ 1.65291775e-14, -5.01793776e-01, -4.56762939e-15],
                          [-6.43455799e-01,  6.04124874e-15, -9.16776129e-16],
                          [-3.10025742e-15, -5.16949021e-01, -1.60064313e-14]],

                         [[ 6.72754711e-02,  6.58697319e-17,  4.93719337e-16],
                          [-2.16950971e-01,  7.87707268e-16, -1.55991358e-15],
                          [-9.02482172e-15, -4.56472178e-02, -9.60152902e-29],
                          [-9.40283036e-02, -1.03964845e-15, -2.32102818e-17],
                          [ 1.34804806e-16,  3.70001801e-16, -1.22448078e-14],
                          [-1.19722852e-01, -7.16975236e-17,  4.48562091e-15],
                          [ 3.78216472e-15, -4.06766556e-01, -8.01728320e-29],
                          [-3.54260412e-14,  6.35556151e-01,  2.51980774e-28],
                          [-1.71926933e-14,  5.28192099e-15, -1.57824742e-14],
                          [-4.24765667e-01,  4.40650446e-15,  2.43917428e-15],
                          [-6.43455799e-01,  5.94154580e-15, -9.16776129e-16],
                          [-1.34745226e-14, -1.21957379e-01,  2.30124769e-28],
                          [-7.10020329e-01,  2.31702273e-15, -6.08172705e-15]],

                         [[ 2.44377095e-16, -1.17253840e-02, -1.26844869e-16],
                          [-3.52000667e-15,  6.66771655e-02, -3.02422024e-16],
                          [ 8.50293049e-02,  6.08451541e-15,  4.00813822e-15],
                          [ 5.42494360e-15, -1.62280242e-01,  6.08458515e-16],
                          [-1.58158188e-15,  1.55253029e-16,  5.08853112e-01],
                          [-4.31538873e-15,  3.84664597e-01,  1.35015536e-15],
                          [ 7.29297422e-01, -1.56174640e-14,  5.06506077e-16],
                          [ 1.15541706e-01,  4.52095316e-15, -5.51704695e-15],
                          [-3.96827400e-15, -6.48201176e-15,  8.30572211e-01],
                          [-1.48302488e-14,  2.03985876e-01, -8.72182271e-15],
                          [-3.14090329e-15, -5.16949021e-01, -1.60064313e-14],
                          [-7.10020329e-01,  2.48144598e-15, -6.08172705e-15],
                          [-1.96474304e-14,  4.03355912e-01, -3.00258200e-15]]])

        np.testing.assert_almost_equal(np.abs(tdip), np.abs(tdipRef))

        pop = XM.population()
        popRef = np.array([[ 9.99696500e-01,  1.51749924e-04,  1.51749924e-04],
                           [ 8.71670672e-01,  6.41646641e-02,  6.41646641e-02],
                           [ 6.31250288e-01,  1.84374856e-01,  1.84374856e-01],
                           [ 8.93285914e-01,  5.33570428e-02,  5.33570428e-02],
                           [ 1.00000000e+00,  1.68182689e-30,  9.19504747e-31],
                           [-1.23120977e-01,  5.61560489e-01,  5.61560489e-01],
                           [ 5.60233735e-02,  4.71988313e-01,  4.71988313e-01],
                           [ 2.26066514e-01,  3.86966743e-01,  3.86966743e-01],
                           [ 1.00000000e+00, -2.02098716e-31,  1.22179732e-29],
                           [ 3.32541475e-01,  3.33729263e-01,  3.33729263e-01],
                           [ 8.38786904e-01,  8.06065478e-02,  8.06065478e-02],
                           [ 1.08665982e+00, -4.33299123e-02, -4.33299123e-02],
                           [ 1.18713951e+00, -9.35697560e-02, -9.35697560e-02]])
        np.testing.assert_almost_equal(pop, popRef, decimal=5)

        BO = XM.bondorder()
        BOref = np.array([[15.976865673350186, 0.8033739118501921, 0.8033739118502013], [0.8033739118501921, 0.4038626396145137, 0.000956700010022243], [0.8033739118502013, 0.000956700010022243, 0.40386263961452273]])
        
        np.testing.assert_almost_equal(np.array(BO), BOref, decimal=5)

        Q = XM.mullikencharge()
        Qref = np.array([-0.7918067485252767, 0.39590337426263555, 0.39590337426262634])
        np.testing.assert_almost_equal(np.array(Q), Qref)

        G = XM.gradient()
        Gref = np.array([-2.813721478034381e-15, -0.023861965701611254, 3.357395966197374e-17, 0.004387447474420991, 0.011930982850797855, 1.9584013746242618e-16, -0.004387447474424322, 0.01193098285080052, -2.294140971244024e-16])
        np.testing.assert_almost_equal(np.array(G), Gref)

        eps = XM.orbital_energies()
        epsRef = np.array([-20.560529616873264, -1.356116188049412, -0.7098185045688243, -0.5606117873983336, -0.5013673414344274, 0.20363234561305937, 0.2997156558379725, 1.0572161749017732, 1.1644438724890211, 1.1868228153097833, 1.215664892455666, 1.3793363616953176, 1.6961866335423852])
        np.testing.assert_almost_equal(np.array(eps), epsRef)

        
        coef = XM.mo_coef()
        aoOverlap = XM.ao_overlap()        
        #np.testing.assert_almost_equal(coef.T @ aoOverlap @ coef, np.eye(coef.shape[1]))
        np.testing.assert_almost_equal(np.matmul(np.matmul(coef.T,aoOverlap),coef), np.eye(coef.shape[1]))

        coefInv = np.linalg.inv(coef)
        np.testing.assert_almost_equal(np.matmul(coefInv.T,coefInv), aoOverlap)

        nao_conf = XM.nao_conf()

        ao_labels = XM.ao_labels()
        ao_labelsRef = ['O1 1s     ', 'O1 2s     ', 'O1 3s     ', 'O1 2p1-   ', 'O1 2p0    ', 'O1 2p1+   ', 'O1 3p1-   ', 'O1 3p0    ', 'O1 3p1+   ', 'H2 1s     ', 'H2 2s     ', 'H3 1s     ', 'H3 2s     ']        
        for a,b in zip(ao_labels,ao_labelsRef):
            self.assertEqual(a, b)
        
        atom_labels = XM.atom_labels()
        atom_labelsRef = ['O1    ', 'H2    ', 'H3    ']
        for a,b in zip(atom_labels, atom_labelsRef):
            self.assertEqual(a, b)
            
        mass = XM.mass()
        massRef = np.array([15.9994, 1.0079, 1.0079])
        np.testing.assert_almost_equal(mass, massRef)


        occ = XM.get_occ()
        occ[0] = 1
        XM.set_occ(occ)

        fl = XM.fluorescencerates()
        flRef = np.zeros((13,13))
        flRef[0,:] = np.array([0.0000000e+00, 7.1645665e-07, 1.4413382e-05, 1.5953203e-05, 1.9134346e-05, 0.0000000e+00, 0.0000000e+00, 0.0000000e+00,
             0.0000000e+00, 0.0000000e+00, 0.0000000e+00, 0.0000000e+00,
             0.0000000e+00])
        np.testing.assert_almost_equal(fl, flRef)
        occ[0] = 2
        XM.set_occ(occ)
        
        CISenergies = XM.CIS_energies()
        CISenergiesRef = np.array([-75.98397312209924, -75.63775722459289, -75.56658914602484, -75.54793249978995, -75.47135294989867, -75.41311789830962, -75.27249409383968, -74.85206434843282, -74.85141097709896, -74.82623885705607, -74.80135811676575, -74.77020443814605, -74.74101217460502, -74.70460818789968, -74.68898674366412, -74.65729249181523, -74.64222982302402, -74.6225521142167, -74.59056943095632, -74.58701881707071, -74.57935095762562, -74.53607249242683, -74.44672022920503, -74.27224761273087, -74.21339383339225, -74.10992386445542, -74.06073057424626, -73.99477491383963, -73.99327555361238, -73.9410665621575, -73.82820348807188, -73.77425397623543, -73.34033839825092])
        np.testing.assert_almost_equal(CISenergies, CISenergiesRef)

        grad = XM.CIS_energy_gradient(1)
        gradRef = np.array([-1.80688797e-14,  1.13918560e-01, -2.11508162e-17,  9.38982241e-02, -5.69592799e-02, -6.63215518e-16, -9.38982241e-02, -5.69592799e-02,  6.84366334e-16])
        np.testing.assert_almost_equal(grad, gradRef)

        energies, grad, nac_array = XM.CIS_energy_gradient_nacs(1)
        np.testing.assert_almost_equal(energies, CISenergiesRef)
        np.testing.assert_almost_equal(grad, gradRef)
        nac_array12Ref = np.array([ 1.15271248e+00, -3.19341212e-14,  1.57361063e-15, -5.76356239e-01,
                                    4.08287870e-01, -5.04037899e-16, -5.76356239e-01, -4.08287870e-01,
                                    -1.06957273e-15])
        np.testing.assert_almost_equal(np.abs(nac_array12Ref), np.abs(nac_array[1,2]))

        for i in range(nac_array.shape[1]):
            np.testing.assert_almost_equal(nac_array[i,i,:], np.zeros(nac_array.shape[2]))
            for j in range(i+1, nac_array.shape[1]):
                np.testing.assert_almost_equal(nac_array[i,j,:], -nac_array[j,i,:])

        tdens = XM.CIS_transdens(1)
        tdensRef=np.array([[-8.29215458e-18,  2.53509190e-17,  9.08522151e-18,
                            5.55947141e-17,  5.85653858e-02,  1.54608723e-16,
                            4.59047579e-17,  4.65205887e-02,  1.01032442e-16,
                            3.89700062e-17, -7.70877641e-17, -6.82033140e-17,
                            9.52613454e-17],
                           [ 2.10077551e-17, -6.26752043e-17, -2.47476351e-17,
                             -1.26444974e-16, -1.35761930e-01, -3.30797975e-16,
                             -1.04831417e-16, -1.07840575e-01, -2.19495069e-16,
                             -7.76405792e-17,  1.85330453e-16,  1.42071337e-16,
                             -2.27711531e-16],
                           [ 8.77119649e-17, -2.71331629e-16, -9.36339561e-17,
                             -6.20934476e-16, -6.44554559e-01, -1.76601533e-15,
                             -5.11408588e-16, -5.11992828e-01, -1.14627325e-15,
                             -4.57713615e-16,  8.33441408e-16,  7.88866274e-16,
                             -1.03183880e-15],
                           [-2.20376163e-17,  7.35283574e-17,  6.67892427e-18,
                            1.72061010e-16,  2.13363607e-01,  5.42104948e-16,
                            1.44574284e-16,  1.69482374e-01,  3.56801386e-16,
                            1.36999104e-16, -2.81138394e-16, -2.31428388e-16,
                            3.57118819e-16],
                           [ 7.77288465e-04, -1.77635404e-03, -2.36474138e-03,
                             -2.25313618e-03, -6.13457239e-17,  1.19316580e-16,
                             -1.69144751e-03, -4.65240126e-17,  5.85347880e-17,
                             3.61899020e-04,  3.79861006e-04,  3.61899020e-04,
                             3.79861006e-04],
                           [ 5.95226472e-18, -1.28448201e-17, -1.04112282e-17,
                             1.62964189e-17, -1.45343257e-14, -2.84292185e-17,
                             1.11617363e-17, -1.15451368e-14, -1.51503498e-17,
                             -2.30461485e-17, -8.45612647e-18,  6.54150736e-18,
                             5.46273299e-18],
                           [-4.04602620e-17,  1.18679695e-16,  6.02169126e-17,
                            2.56576482e-16,  2.37527418e-01,  6.66693073e-16,
                            2.09162825e-16,  1.88676557e-01,  4.30886673e-16,
                            1.71786027e-16, -3.07768334e-16, -3.04135334e-16,
                            3.71833176e-16],
                           [-8.38782212e-04,  1.91688702e-03,  2.55182355e-03,
                            2.43138890e-03,  3.01774706e-17, -1.28756092e-16,
                            1.82526327e-03,  2.15914977e-17, -6.31656600e-17,
                            -3.90529995e-04, -4.09913011e-04, -3.90529995e-04,
                            -4.09913011e-04],
                           [ 5.70970900e-18, -1.17552926e-17, -4.23849782e-18,
                             4.06882069e-17, -2.59940743e-14, -1.71661465e-17,
                             2.87158649e-17, -2.06480265e-14, -9.14809266e-18,
                             -2.50814727e-17, -8.92809602e-18, -7.21583786e-18,
                             -5.23602543e-19],
                           [-9.10861118e-18,  2.84861314e-17,  5.95771083e-18,
                            5.67418020e-17,  7.19194478e-02,  2.03544090e-16,
                            4.79855018e-17,  5.71281996e-02,  1.31360920e-16,
                            5.82407158e-17, -8.96860020e-17, -8.76092623e-17,
                            1.15263664e-16],
                           [-9.57193274e-17,  2.79288334e-16,  1.12338223e-16,
                            4.94674896e-16,  5.72830142e-01,  1.57926068e-15,
                            4.15059695e-16,  4.55019549e-01,  1.02392174e-15,
                            4.39293150e-16, -7.32721910e-16, -6.78731392e-16,
                            9.20214597e-16],
                           [-1.05533217e-17,  3.20613410e-17,  1.31310411e-17,
                            7.30384839e-17,  7.19194478e-02,  1.81703081e-16,
                            5.98326073e-17,  5.71281996e-02,  1.19721524e-16,
                            4.22239556e-17, -9.73287230e-17, -8.08950329e-17,
                            1.18314235e-16],
                           [-6.42706619e-17,  2.12074475e-16,  6.39461671e-17,
                            6.09611453e-16,  5.72830142e-01,  1.50368160e-15,
                            4.94757560e-16,  4.55019549e-01,  9.83644530e-16,
                            3.46888539e-16, -7.62917681e-16, -6.92477212e-16,
                            9.27022118e-16]])
        np.testing.assert_almost_equal(np.abs(tdens), np.abs(tdensRef))

if __name__ == "__main__":
    unittest.main()
    
