#!/usr/bin/python


import os

import unittest
import re
from subprocess import PIPE,Popen,call



class testgradients(unittest.TestCase):

    xmol=""
    xatom=""
    def setUp(self):
        print(os.getcwd())
        dir=os.getcwd()
        self.xatom=os.getenv("XATOM_PATH")
        print("Using xatom=",self.xatom  )

        self.xmol=os.path.realpath(dir+"/../src/xmolecule")
        print("Using xmolecule=",self.xmol)



    def test_water_gradient(self):
        # This Test calculates all states for the water molecule, calculates all transition rates and runs xmolecule_req
        # to compare the final charge state population 

        print('Testing gradient results for water HFS with nao basis ')
        expectedgrad=[[0.0, 0.0, 0.0157959], [0.0, 0.02500334, -0.0087847], [0.0, -0.02500334, -0.0087847]]

        lmax=20
        # set up input file
        f = open('input.in', 'w')
        f.write('system=water\n')
        f.write('lmax=%d\n' % lmax)
        f.write('O     0.0000000000        0.0000000000        0.0000000000    2s1p O.nao\n')
        f.write('H     0.0000000000       -1.4309301866        1.1083919440    1s   H.nao\n')
        f.write('H     0.0000000000        1.4309301866        1.1083919440    1s   H.nao\n')
        f.close()
        # calculate all process data put them into process.dat        
        cmd=self.xatom+' -s O -N 100 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao O.nao '
        os.system(cmd)
        cmd=self.xatom+' -s H -N 100 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao H.nao '
        os.system(cmd)

        cmd=self.xmol + ' input.in -gradient '
        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0].decode('UTF8')

        # parse for POP line and compare results
        m=re.findall(r'\|GRAD\| .*',out)
        grad=[]
        for mi in m:
            gradtxt=mi.split()[2:]
            grad.append([float(gradtxt[0]),float(gradtxt[1]),float(gradtxt[2])])

        print(grad)
        for j in range(0,len(grad)):
            self.assertAlmostEqual( 
                sum([ grad[i][j] for i in [0,1,2]]), 0.,places=2)

        for i in range(0,len(grad)):
            for j in range(0,len(grad[i])):
                self.assertAlmostEqual(grad[i][j], expectedgrad[i][j],places=2)


    def test_water_gradient_finiteT(self):
        # This Test calculates all states for the water molecule, calculates all transition rates and runs xmolecule_req
        # to compare the final charge state population 

        print('Testing gradient results for water HFS with nao basis and Temp 10eV'        )
        expectedgrad=[[0.0, 0.0, 0.54422236], [0.0, 0.35224552, -0.27273471], [0.0, -0.35224552, -0.27273471]]


        lmax=20
        temp=10
        # set up input file
        f = open('input.in', 'w')
        f.write('system=water\n')
        f.write('lmax=%d\n' % lmax)
        f.write('T_eV=%f\n' % temp)
        f.write('O     0.0000000000        0.0000000000        0.0000000000    2s1p O.nao\n')
        f.write('H     0.0000000000       -1.4309301866        1.1083919440    1s   H.nao\n')
        f.write('H     0.0000000000        1.4309301866        1.1083919440    1s   H.nao\n')
        f.close()
        # calculate all process data put them into process.dat        
        cmd=self.xatom+' -s O -N 100 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao O.nao '
        os.system(cmd)
        cmd=self.xatom+' -s H -N 100 -silent -rmax 20 -rm 1e-6 -latter false  -nao -file_nao H.nao '
        os.system(cmd)

        cmd=self.xmol + ' input.in -gradient '
        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0].decode('UTF8')

        # parse for POP line and compare results
        m=re.findall(r'\|GRAD\| .*',out)
        grad=[]
        for mi in m:
            gradtxt=mi.split()[2:]
            grad.append([float(gradtxt[0]),float(gradtxt[1]),float(gradtxt[2])])

        print(grad)
        for j in range(0,len(grad)):
            self.assertAlmostEqual( 
                sum([ grad[i][j] for i in [0,1,2]]), 0.,places=2)

        for i in range(0,len(grad)):
            for j in range(0,len(grad[i])):
                self.assertAlmostEqual(grad[i][j], expectedgrad[i][j],places=2)



    def test_water_gradient_GTO_HFS(self, mmap=False):
        # This Test calculates all states for the water molecule, calculates all transition rates and runs xmolecule_req
        # to compare the final charge state population 

        print('Testing gradient results for water HFS with STO-3G basis ')
        expectedgrad=[[0.0, 0.0, -0.00810526], [0.0, -0.01575399, 0.00421141], [0.0, 0.01575399, 0.00421141]]
        lmax=20
        # set up input file
        f = open('input.in', 'w')
        f.write('system=water\n')
        f.write('gto=STO-3G.gbs\n')
        if mmap:
            f.write('mmap_teints=yes\n')
        f.write('lmax=%d\n' % lmax)
        f.write('O     0.0000000000        0.0000000000        0.0000000000  \n')
        f.write('H     0.0000000000       -1.4309301866        1.1083919440  \n')
        f.write('H     0.0000000000        1.4309301866        1.1083919440  \n')
        f.close()

        cmd=self.xmol + ' input.in -gradient '
        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0].decode('UTF8')

        # parse for POP line and compare results
        m=re.findall(r'\|GRAD\| .*',out)
        grad=[]
        for mi in m:
            gradtxt=mi.split()[2:]
            grad.append([float(gradtxt[0]),float(gradtxt[1]),float(gradtxt[2])])

        print(grad)
        for j in range(0,len(grad)):
            self.assertAlmostEqual( 
                sum([ grad[i][j] for i in [0,1,2]]), 0.,places=2)

        for i in range(0,len(grad)):
            for j in range(0,len(grad[i])):
                self.assertAlmostEqual(grad[i][j], expectedgrad[i][j],places=2)

    def test_water_gradient_GTO_HFS_mmap(self):
        self.test_water_gradient_GTO_HFS(mmap=True)

    def test_water_gradient_GTO_HFS_finiteT(self, mmap=False):
        # This Test calculates all states for the water molecule, calculates all transition rates and runs xmolecule_req
        # to compare the final charge state population 

        print('Testing gradient results for water HFS with STO-3G basis and finiteT=10eV')
        expectedgrad=[[0.0, 0.0, 0.52413321], [0.0, 0.33919155, -0.26210766], [0.0, -0.33919155, -0.26210766]]

        lmax=20
        temp=10.
        # set up input file
        f = open('input.in', 'w')
        f.write('system=water\n')
        f.write('gto=STO-3G.gbs\n')
        f.write('lmax=%d\n' % lmax)
        f.write('T_eV=%f\n' % temp)
        if mmap:
            f.write('mmap_teints=yes\n')
        f.write('O     0.0000000000        0.0000000000        0.0000000000  \n')
        f.write('H     0.0000000000       -1.4309301866        1.1083919440  \n')
        f.write('H     0.0000000000        1.4309301866        1.1083919440  \n')
        f.close()

        cmd=self.xmol + ' input.in -gradient '
        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0].decode('UTF8')

        # parse for POP line and compare results
        m=re.findall(r'\|GRAD\| .*',out)
        grad=[]
        for mi in m:
            gradtxt=mi.split()[2:]
            grad.append([float(gradtxt[0]),float(gradtxt[1]),float(gradtxt[2])])

        print(grad)
        for j in range(0,len(grad)):
            self.assertAlmostEqual( 
                sum([ grad[i][j] for i in [0,1,2]]), 0.,places=2)

        for i in range(0,len(grad)):
            for j in range(0,len(grad[i])):
                self.assertAlmostEqual(grad[i][j], expectedgrad[i][j],places=2)

    def test_water_gradient_GTO_HFS_finiteT_mmap(self):
        self.test_water_gradient_GTO_HFS_finiteT(mmap=True)

    def test_water_gradient_GTO_HF(self, mmap=False):
        # This Test calculates all states for the water molecule, calculates all transition rates and runs xmolecule_req
        # to compare the final charge state population 
        print('Testing gradient results for water HF with STO-3G basis ')
        expectedgrad=[[0.0, 0.0, 0.06136688], [0.0, 0.02360645, -0.03068344], [0.0, -0.02360645, -0.03068344]]
        lmax=20
        # set up input file
        f = open('input.in', 'w')
        f.write('system=water\n')
        f.write('gto=STO-3G.gbs\n')
        f.write('HF=yes\n')
        if mmap:
            f.write('mmap_teints=yes\n')
        f.write('lmax=%d\n' % lmax)
        f.write('O     0.0000000000        0.0000000000        0.0000000000   \n')
        f.write('H     0.0000000000       -1.4309301866        1.1083919440   \n')
        f.write('H     0.0000000000        1.4309301866        1.1083919440   \n')
        f.close()

        cmd=self.xmol + ' input.in -gradient '
        p=Popen(cmd,stdout=PIPE, shell=True)
        out=p.communicate()[0].decode('UTF8')

        # parse for POP line and compare results
        m=re.findall(r'\|GRAD\| .*',out)
        grad=[]
        for mi in m:
            gradtxt=mi.split()[2:]
            grad.append([float(gradtxt[0]),float(gradtxt[1]),float(gradtxt[2])])

        print(grad)
        print(expectedgrad)
        print(sum([ grad[i][0] for i in [0,1,2]]))
        print(sum([ grad[i][1] for i in [0,1,2]]))
        print(sum([ grad[i][2] for i in [0,1,2]]))

        for i in range(0,len(grad)):
            for j in range(0,len(grad[i])):
                self.assertAlmostEqual(grad[i][j], expectedgrad[i][j],places=2)

    def test_water_gradient_GTO_HF_mmap(self):
        self.test_water_gradient_GTO_HF(mmap=True)

if __name__ == "__main__": 
    unittest.main()


