!
! XMOLECULE electronic structure toolkit
! Copyright 2023 Ludger Inhester
! Ludger Inhester, Center for Free-Electron Laser Science, DESY
! ludger.inhester@cfel.de
!
! This file is part of XMOLECULE, which is distributed under the terms of the GNU General Public License version 3.
! See LICENSE.txt or https://www.gnu.org/licenses/gpl-3.0.html#license-text for a full license text.
!

module packed_index
  !! author: Ludger Inhester
  !!
  !! tools for packing 2-index and 4-index quantities by exploitinh index symmetry
  implicit none
  integer(8), parameter, private :: MAX_INDEX_PACKED = 1024*1024/2
  integer(8), private :: SQ_HALF(MAX_INDEX_PACKED)
  interface index_packed
    !! index for the upper triangular matrix in a packed storage scheme
    !! SEE: http://www.netlib.org/lapack/lug/node123.html
    !! optimization:
    !! use precalculated array sq_half(i)= \( i(i-1)/2 \)
    module procedure index_packed4
    module procedure index_packed8
  end interface index_packed
  private
  public :: index_packed, init_index_packed
contains
  subroutine init_index_packed
    !! initialize index_packed: precompute index values
    integer(8) :: k
    SQ_HALF(1) = 0
    do k = 2, size(SQ_HALF)
      SQ_HALF(k) = SQ_HALF(k - 1) + (k - 1)
    end do
  end subroutine init_index_packed

  elemental function index_packed4(i, j) result(ij)
    integer, intent(in) :: i, j
    integer(8) :: ij
    ij = min(i, j) + SQ_HALF(max(i, j))
  end function index_packed4

  elemental function index_packed8(i, j) result(ij)
    integer(8), intent(in) :: i, j
    integer(8) :: ij
    ij = min(i, j) + SQ_HALF(max(i, j))
  end function index_packed8

end module packed_index
